@@ -21,12 +21,12 @@
   limitations under the License.
  */
 
-#include "libts.h"
-
-#include "ink_args.h"
-#include "I_Version.h"
-#include "Tokenizer.h"
-#include "TextBuffer.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_memory.h"
+#include "ts/ink_args.h"
+#include "ts/I_Version.h"
+#include "ts/Tokenizer.h"
+#include "ts/TextBuffer.h"
 #include "mgmtapi.h"
 #include <stdio.h>
 #include <string.h>
@@ -282,7 +282,7 @@ handleArgInvocation()
         printf("Set %s, restart required\n", SetVar);
         break;
       case TS_ACTION_RECONFIGURE:
-        // printf("Set %s, reconfiguration required\n", SetVar);
+        printf("Set %s, please wait 10 seconds for traffic server to sync configuration, restart is not required\n", SetVar);
         break;
       case TS_ACTION_DYNAMIC:
       default:
@@ -308,7 +308,8 @@ main(int /* argc ATS_UNUSED */, const char **argv)
   TSMgmtError status;
 
   // build the application information structure
-  appVersionInfo.setup(PACKAGE_NAME, "traffic_line", PACKAGE_VERSION, __DATE__, __TIME__, BUILD_MACHINE, BUILD_PERSON, "");
+  appVersionInfo.setup(PACKAGE_NAME, "traffic_line [DEPRECATED]", PACKAGE_VERSION, __DATE__, __TIME__, BUILD_MACHINE, BUILD_PERSON,
+                       "");
 
   program_name = appVersionInfo.AppStr;
 
