@@ -28,6 +28,7 @@
 
 from docutils import nodes
 from docutils.parsers import rst
+from docutils.parsers.rst import directives
 from sphinx.domains import Domain, ObjType, std
 from sphinx.roles import XRefRole
 from sphinx.locale import l_, _
@@ -49,6 +50,7 @@ class TSConfVar(std.Target):
         'class' : rst.directives.class_option,
         'reloadable' : rst.directives.flag,
         'deprecated' : rst.directives.flag,
+        'overridable' : rst.directives.flag,
         'metric' : rst.directives.unchanged,
     }
     required_arguments = 3
@@ -117,6 +119,8 @@ class TSConfVar(std.Target):
             fl.append(self.make_field('Metric', self.options['metric']))
         if ('reloadable' in self.options):
             fl.append(self.make_field('Reloadable', 'Yes'))
+        if ('overridable' in self.options):
+            fl.append(self.make_field('Overridable', 'Yes'))
         if ('deprecated' in self.options):
             fl.append(self.make_field('Deprecated', 'Yes'))
 
@@ -139,6 +143,123 @@ class TSConfVarRef(XRefRole):
     def process_link(self, env, ref_node, explicit_title_p, title, target):
         return title, target
 
+def metrictypes(typename):
+    return directives.choice(typename.lower(), ('counter','gauge','derivative','flag','text'))
+
+def metricunits(unitname):
+    return directives.choice(unitname.lower(), ('ratio','percent','kbits','mbits','bytes','kbytes','mbytes','nanoseconds','microseconds','milliseconds','seconds'))
+
+class TSStat(std.Target):
+    """
+    Description of a traffic server statistic.
+
+    Argument is the JSON stat group ("global", etc.) in which the statistic is
+    returned, then the statistic name as used by traffic_line/stats_over_http,
+    followed by the value type of the statistic ('string', 'integer'), and
+    finally an example value.
+
+    Descriptive text should follow, indented.
+
+    Then the bulk description (if any) undented. This should be considered
+    equivalent to the Doxygen short and long description.
+    """
+
+    option_spec = {
+        'type': metrictypes,
+        'unit': metricunits,
+        'introduced' : rst.directives.unchanged,
+        'deprecated' : rst.directives.unchanged,
+        'ungathered' : rst.directives.flag
+    }
+    required_arguments = 3
+    optional_arguments = 1 # example value is optional
+    final_argument_whitespace = True
+    has_content = True
+
+    def make_field(self, tag, value):
+        field = nodes.field();
+        field.append(nodes.field_name(text=tag))
+        body = nodes.field_body()
+        if (isinstance(value, basestring)):
+            body.append(sphinx.addnodes.compact_paragraph(text=value))
+        else:
+            body.append(value)
+        field.append(body)
+        return field
+
+    # External entry point
+    def run(self):
+        env = self.state.document.settings.env
+        stat_example = None
+        stat_group, stat_name, stat_type = self.arguments[0:3]
+        if (len(self.arguments) > 3):
+            stat_example = self.arguments[3]
+
+        # First, make a generic desc() node to be the parent.
+        node = sphinx.addnodes.desc()
+        node.document = self.state.document
+        node['objtype'] = 'stat'
+
+        # Next, make a signature node. This creates a permalink and a
+        # highlighted background when the link is selected.
+        title = sphinx.addnodes.desc_signature(stat_name, '')
+        title['ids'].append(nodes.make_id('stat-'+stat_name))
+        title['names'].append(stat_name)
+        title['first'] = False
+        title['objtype'] = 'stat'
+        self.add_name(title)
+        title.set_class('ts-stat-title')
+
+        # Finally, add a desc_name() node to display the name of the
+        # configuration variable.
+        title += sphinx.addnodes.desc_name(stat_name, stat_name)
+
+        node.append(title)
+
+        # This has to be a distinct node before the title. if nested then
+        # the browser will scroll forward to just past the title.
+        anchor = nodes.target('', '', names=[stat_name])
+        # Second (optional) arg is 'msgNode' - no idea what I should pass for that
+        # or if it even matters, although I now think it should not be used.
+        self.state.document.note_explicit_target(title)
+        env.domaindata['ts']['stat'][stat_name] = env.docname
+
+        fl = nodes.field_list()
+        fl.append(self.make_field('Collection', stat_group))
+        if ('type' in self.options):
+            fl.append(self.make_field('Type', self.options['type']))
+        if ('unit' in self.options):
+            fl.append(self.make_field('Units', self.options['unit']))
+        fl.append(self.make_field('Datatype', stat_type))
+        if ('introduced' in self.options and len(self.options['introduced']) > 0):
+            fl.append(self.make_field('Introduced', self.options['introduced']))
+        if ('deprecated' in self.options):
+            if (len(self.options['deprecated']) > 0):
+                fl.append(self.make_field('Deprecated', self.options['deprecated']))
+            else:
+                fl.append(self.make_field('Deprecated', 'Yes'))
+        if ('ungathered' in self.options):
+            fl.append(self.make_field('Gathered', 'No'))
+        if (stat_example):
+            fl.append(self.make_field('Example', stat_example))
+
+        # Get any contained content
+        nn = nodes.compound();
+        self.state.nested_parse(self.content, self.content_offset, nn)
+
+        # Create an index node so that Sphinx adds this statistic to the
+        # index. nodes.make_id() specifies the link anchor name that is
+        # implicitly generated by the anchor node above.
+        indexnode = sphinx.addnodes.index(entries=[])
+        indexnode['entries'].append(
+            ('single', _('%s') % stat_name, nodes.make_id(stat_name), '')
+        )
+
+        return [ indexnode, node, fl, nn ]
+
+class TSStatRef(XRefRole):
+    def process_link(self, env, ref_node, explicit_title_p, title, target):
+        return title, target
 
 class TrafficServerDomain(Domain):
     """
@@ -150,23 +271,28 @@ class TrafficServerDomain(Domain):
     data_version = 2
 
     object_types = {
-        'cv': ObjType(l_('configuration variable'), 'cv')
+        'cv': ObjType(l_('configuration variable'), 'cv'),
+        'stat': ObjType(l_('statistic'), 'stat')
     }
 
     directives = {
-        'cv' : TSConfVar
+        'cv' : TSConfVar,
+        'stat' : TSStat
     }
 
     roles = {
-        'cv' : TSConfVarRef()
+        'cv' : TSConfVarRef(),
+        'stat' : TSStatRef()
     }
 
     initial_data = {
-        'cv' : {} # full name -> docname
+        'cv' : {}, # full name -> docname
+        'stat' : {}
     }
 
     dangling_warnings = {
-        'cv' : "No definition found for configuration variable '%(target)s'"
+        'cv' : "No definition found for configuration variable '%(target)s'",
+        'stat' : "No definition found for statistic '%(target)s'"
     }
 
     def clear_doc(self, docname):
@@ -174,12 +300,18 @@ class TrafficServerDomain(Domain):
         for var, doc in cv_list.items():
             if doc == docname:
                 del cv_list[var]
+        stat_list = self.data['stat']
+        for var, doc in stat_list.items():
+            if doc == docname:
+                del stat_list[var]
 
     def find_doc(self, key, obj_type):
         zret = None
 
         if obj_type == 'cv' :
             obj_list = self.data['cv']
+        elif obj_type == 'stat' :
+            obj_list = self.data['stat']
         else:
             obj_list = None
 
@@ -196,6 +328,8 @@ class TrafficServerDomain(Domain):
     def get_objects(self):
         for var, doc in self.data['cv'].iteritems():
             yield var, var, 'cv', doc, var, 1
+        for var, doc in self.data['stat'].iteritems():
+            yield var, var, 'stat', doc, var, 1
 
 # These types are ignored as missing references for the C++ domain.
 # We really need to do better with this. Editing this file for each of
