@@ -37,7 +37,7 @@
 #include <stdio.h>
 
 #include "ts/ts.h"
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 
 #define TS_NULL_MUTEX NULL
 #define STATE_BUFFER_DATA 0
@@ -270,7 +270,7 @@ transformable(TSHttpTxn txnp)
 
   if (TSHandleMLocRelease(bufp, TS_NULL_MLOC, hdr_loc) == TS_ERROR) {
     TSError("[bnull-transform] Error releasing MLOC while checking "
-            "header status\n");
+            "header status");
   }
 
   return retv;
@@ -315,8 +315,9 @@ TSPluginInit(int argc ATS_UNUSED, const char *argv[] ATS_UNUSED)
   info.vendor_name = "MyCompany";
   info.support_email = "ts-api-support@MyCompany.com";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
-    TSError("[bnull-transform] Plugin registration failed.\n");
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[bnull-transform] Plugin registration failed.");
+
     goto Lerror;
   }
 
@@ -327,5 +328,5 @@ TSPluginInit(int argc ATS_UNUSED, const char *argv[] ATS_UNUSED)
   return;
 
 Lerror:
-  TSError("[bnull-transform] Plugin disabled\n");
+  TSError("[bnull-transform] Plugin disabled");
 }
