@@ -44,7 +44,7 @@
 #include <sys/time.h>
 #include <sys/resource.h>
 
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 #include "ts/ts.h"
 #include "ts/remap.h"
 
@@ -145,8 +145,8 @@ TSPluginInit(int argc ATS_UNUSED, const char *argv[] ATS_UNUSED)
   info.vendor_name = (char *)"Apache";
   info.support_email = (char *)"";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
-    TSError("Plugin registration failed. \n");
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[remap] Plugin registration failed.");
   }
   TSDebug("debug-remap", "TSPluginInit: Remap plugin started\n");
 }
