@@ -53,7 +53,7 @@
 #include <ctype.h>
 
 #include "ts/ts.h"
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 
 static int init_buffer_status;
 
@@ -88,7 +88,7 @@ modify_header(TSHttpTxn txnp)
     return; /* caller reenables */
 
   if (TSHttpTxnServerRespGet(txnp, &resp_bufp, &resp_loc) != TS_SUCCESS) {
-    TSError("couldn't retrieve server response header\n");
+    TSError("[response_header-1] Couldn't retrieve server response header");
     return; /* caller reenables */
   }
 
@@ -136,8 +136,8 @@ modify_header(TSHttpTxn txnp)
 
     /* Get the cached HTTP header */
     if (TSHttpTxnCachedRespGet(txnp, &cached_bufp, &cached_loc) != TS_SUCCESS) {
-      TSError("STATUS 304, TSHttpTxnCachedRespGet():");
-      TSError("couldn't retrieve cached response header\n");
+      TSError("[response_header-1] STATUS 304, TSHttpTxnCachedRespGet():");
+      TSError("[response_header-1] Couldn't retrieve cached response header");
       TSHandleMLocRelease(resp_bufp, TS_NULL_MLOC, resp_loc);
       return; /* Caller reenables */
     }
@@ -145,7 +145,7 @@ modify_header(TSHttpTxn txnp)
     /* Get the cached MIME field name for this HTTP header */
     cached_field_loc = TSMimeHdrFieldFind(cached_bufp, cached_loc, (const char *)mimehdr1_name, strlen(mimehdr1_name));
     if (TS_NULL_MLOC == cached_field_loc) {
-      TSError("Can't find header %s in cached document", mimehdr1_name);
+      TSError("[response_header-1] Can't find header %s in cached document", mimehdr1_name);
       TSHandleMLocRelease(resp_bufp, TS_NULL_MLOC, resp_loc);
       TSHandleMLocRelease(cached_bufp, TS_NULL_MLOC, cached_loc);
       return; /* Caller reenables */
@@ -154,7 +154,7 @@ modify_header(TSHttpTxn txnp)
     /* Get the cached MIME value for this name in this HTTP header */
     chkptr = TSMimeHdrFieldValueStringGet(cached_bufp, cached_loc, cached_field_loc, -1, &chklength);
     if (NULL == chkptr || !chklength) {
-      TSError("Could not find value for cached MIME field name %s", mimehdr1_name);
+      TSError("[response_header-1] Could not find value for cached MIME field name %s", mimehdr1_name);
       TSHandleMLocRelease(resp_bufp, TS_NULL_MLOC, resp_loc);
       TSHandleMLocRelease(cached_bufp, TS_NULL_MLOC, cached_loc);
       TSHandleMLocRelease(cached_bufp, cached_loc, cached_field_loc);
@@ -238,15 +238,15 @@ TSPluginInit(int argc, const char *argv[])
   info.vendor_name = "MyCompany";
   info.support_email = "ts-api-support@MyCompany.com";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
-    TSError("Plugin registration failed.\n");
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[response_header-1] Plugin registration failed.");
   }
 
   init_buffer_status = 0;
   if (argc > 1) {
-    TSError("usage: %s \n", argv[0]);
-    TSError("warning: too many args %d\n", argc);
-    TSError("warning: ignoring unused arguments beginning with %s\n", argv[1]);
+    TSError("[response_header-1] usage: %s", argv[0]);
+    TSError("[response_header-1] warning: too many args %d", argc);
+    TSError("[response_header-1] warning: ignoring unused arguments beginning with %s", argv[1]);
   }
 
   /*
@@ -287,12 +287,12 @@ TSPluginInit(int argc, const char *argv[])
    */
 
   if (TS_NULL_MLOC == (chk_field_loc = TSMimeHdrFieldGet(hdr_bufp, hdr_loc, 0))) {
-    TSError("couldn't retrieve header field from init buffer");
-    TSError("marking init buffer as corrupt; no more plugin processing");
+    TSError("[response_header-1] Couldn't retrieve header field from init buffer");
+    TSError("[response_header-1] Marking init buffer as corrupt; no more plugin processing");
     init_buffer_status = 0;
     /* bail out here and reenable transaction */
   } else {
     if (field_loc != chk_field_loc)
-      TSError("retrieved buffer field loc is %p when it should be %p", chk_field_loc, field_loc);
+      TSError("[response_header-1] Retrieved buffer field loc is %p when it should be %p", chk_field_loc, field_loc);
   }
 }
