@@ -31,7 +31,7 @@
 
 #include <stdio.h>
 #include "ts/ts.h"
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 
 static INKStat transaction_count;
 static INKStat session_count;
@@ -97,8 +97,9 @@ TSPluginInit(int argc, const char *argv[])
   info.vendor_name = "MyCompany";
   info.support_email = "ts-api-support@MyCompany.com";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
-    TSError("[PluginInit] Plugin registration failed.\n");
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[session-1] Plugin registration failed.\n");
+
     goto error;
   }
 
@@ -110,5 +111,5 @@ TSPluginInit(int argc, const char *argv[])
   TSHttpHookAdd(TS_HTTP_SSN_START_HOOK, contp);
 
 error:
-  TSError("[PluginInit] Plugin not initialized");
+  TSError("[session-1] Plugin not initialized");
 }
