@@ -23,16 +23,13 @@
 
 #include "P_AIO.h"
 #include "InkAPIInternal.h"
-#include "I_Layout.h"
+#include "ts/I_Layout.h"
 #include <iostream>
 #include <fstream>
 
 using std::cout;
 using std::endl;
 
-Diags *diags;
-#define DIAGS_LOG_FILE "diags.log"
-
 // Necessary for AIO
 #if defined(solaris)
 int net_config_poll_timeout = 30;
@@ -40,92 +37,7 @@ int net_config_poll_timeout = 30;
 int net_config_poll_timeout = 10;
 #endif
 
-
-//////////////////////////////////////////////////////////////////////////////
-//
-//      void reconfigure_diags()
-//
-//      This function extracts the current diags configuration settings from
-//      records.config, and rebuilds the Diags data structures.
-//
-//////////////////////////////////////////////////////////////////////////////
-
-static void
-reconfigure_diags()
-{
-  int i;
-  DiagsConfigState c;
-
-  // initial value set to 0 or 1 based on command line tags
-  c.enabled[DiagsTagType_Debug] = (diags->base_debug_tags != NULL);
-  c.enabled[DiagsTagType_Action] = (diags->base_action_tags != NULL);
-
-  c.enabled[DiagsTagType_Debug] = 1;
-  c.enabled[DiagsTagType_Action] = 1;
-  diags->show_location = 1;
-
-  // read output routing values
-  for (i = 0; i < DiagsLevel_Count; i++) {
-    c.outputs[i].to_stdout = 0;
-    c.outputs[i].to_stderr = 1;
-    c.outputs[i].to_syslog = 1;
-    c.outputs[i].to_diagslog = 1;
-  }
-
-  //////////////////////////////
-  // clear out old tag tables //
-  //////////////////////////////
-
-  diags->deactivate_all(DiagsTagType_Debug);
-  diags->deactivate_all(DiagsTagType_Action);
-
-  //////////////////////////////////////////////////////////////////////
-  //                     add new tag tables
-  //////////////////////////////////////////////////////////////////////
-
-  if (diags->base_debug_tags)
-    diags->activate_taglist(diags->base_debug_tags, DiagsTagType_Debug);
-  if (diags->base_action_tags)
-    diags->activate_taglist(diags->base_action_tags, DiagsTagType_Action);
-
-////////////////////////////////////
-// change the diags config values //
-////////////////////////////////////
-#if !defined(__GNUC__) && !defined(hpux)
-  diags->config = c;
-#else
-  memcpy(((void *)&diags->config), ((void *)&c), sizeof(DiagsConfigState));
-#endif
-}
-
-static void
-init_diags(const char *bdt, const char *bat)
-{
-  FILE *diags_log_fp;
-  char diags_logpath[500];
-  strcpy(diags_logpath, DIAGS_LOG_FILE);
-
-  diags_log_fp = fopen(diags_logpath, "w");
-  if (diags_log_fp) {
-    int status;
-    status = setvbuf(diags_log_fp, NULL, _IOLBF, 512);
-    if (status != 0) {
-      fclose(diags_log_fp);
-      diags_log_fp = NULL;
-    }
-  }
-
-  diags = new Diags(bdt, bat, diags_log_fp);
-
-  if (diags_log_fp == NULL) {
-    Warning("couldn't open diags log file '%s', "
-            "will not log to this file",
-            diags_logpath);
-  }
-
-  Status("opened %s", diags_logpath);
-  reconfigure_diags();
-}
+#include "diags.i"
 
 #define MAX_DISK_THREADS 200
 #ifdef DISK_ALIGN
@@ -332,11 +244,11 @@ int
 AIO_Device::do_fd(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
 {
   if (!time_start) {
-    time_start = ink_get_hrtime();
+    time_start = Thread::get_hrtime();
     fprintf(stderr, "Starting the aio_testing \n");
   }
-  if ((ink_get_hrtime() - time_start) > (run_time * HRTIME_SECOND)) {
-    time_end = ink_get_hrtime();
+  if ((Thread::get_hrtime() - time_start) > (run_time * HRTIME_SECOND)) {
+    time_end = Thread::get_hrtime();
     ink_atomic_increment(&n_accessors, -1);
     if (n_accessors <= 0)
       dump_summary();
