@@ -31,7 +31,7 @@
 extern int cache_config_compatibility_4_2_0_fixup;
 
 Action *
-Cache::open_read(Continuation *cont, CacheKey *key, CacheFragType type, char *hostname, int host_len)
+Cache::open_read(Continuation *cont, const CacheKey *key, CacheFragType type, const char *hostname, int host_len)
 {
   if (!CacheProcessor::IsCacheReady(type)) {
     cont->handleEvent(CACHE_EVENT_OPEN_READ_FAILED, (void *)-ECACHE_NOT_READY);
@@ -93,8 +93,8 @@ Lcallreturn:
 
 #ifdef HTTP_CACHE
 Action *
-Cache::open_read(Continuation *cont, CacheKey *key, CacheHTTPHdr *request, CacheLookupHttpConfig *params, CacheFragType type,
-                 char *hostname, int host_len)
+Cache::open_read(Continuation *cont, const CacheKey *key, CacheHTTPHdr *request, CacheLookupHttpConfig *params, CacheFragType type,
+                 const char *hostname, int host_len)
 {
   if (!CacheProcessor::IsCacheReady(type)) {
     cont->handleEvent(CACHE_EVENT_OPEN_READ_FAILED, (void *)-ECACHE_NOT_READY);
@@ -300,7 +300,7 @@ CacheVC::openReadFromWriter(int event, Event *e)
     // before the open_write, but the reader could not get the volume
     // lock. If we don't reset the clock here, we won't choose any writer
     // and hence fail the read request.
-    start_time = ink_get_hrtime();
+    start_time = Thread::get_hrtime();
     f.read_from_writer_called = 1;
   }
   cancel_trigger();
@@ -336,7 +336,11 @@ CacheVC::openReadFromWriter(int event, Event *e)
       return openReadStartHead(event, e);
     } else if (ret == EVENT_CONT) {
       ink_assert(!write_vc);
-      VC_SCHED_WRITER_RETRY();
+      if (writer_lock_retry < cache_config_read_while_writer_max_retries) {
+        VC_SCHED_WRITER_RETRY();
+      } else {
+        return openReadFromWriterFailure(CACHE_EVENT_OPEN_READ_FAILED, (Event *)-err);
+      }
     } else
       ink_assert(write_vc);
   } else {
@@ -364,7 +368,8 @@ CacheVC::openReadFromWriter(int event, Event *e)
   // allow reading from unclosed writer for http requests only.
   ink_assert(frag_type == CACHE_FRAG_TYPE_HTTP || write_vc->closed);
   if (!write_vc->closed && !write_vc->fragment) {
-    if (!cache_config_read_while_writer || frag_type != CACHE_FRAG_TYPE_HTTP) {
+    if (!cache_config_read_while_writer || frag_type != CACHE_FRAG_TYPE_HTTP ||
+        writer_lock_retry >= cache_config_read_while_writer_max_retries) {
       MUTEX_RELEASE(lock);
       return openReadFromWriterFailure(CACHE_EVENT_OPEN_READ_FAILED, (Event *)-err);
     }
@@ -561,31 +566,13 @@ CacheVC::openReadReadDone(int event, Event *e)
           Warning("Middle: Doc checksum does not match for %s", key.toHexStr(tmpstring));
         else
           Warning("Middle: Doc magic does not match for %s", key.toHexStr(tmpstring));
-#if TS_USE_INTERIM_CACHE == 1
-        if (dir_ininterim(&dir)) {
-          dir_delete(&key, vol, &dir);
-          goto Lread;
-        }
-#endif
         goto Lerror;
       }
       if (doc->key == key)
         goto LreadMain;
-#if TS_USE_INTERIM_CACHE == 1
-      else if (dir_ininterim(&dir)) {
-        dir_delete(&key, vol, &dir);
-        last_collision = NULL;
-      }
-#endif
     }
-#if TS_USE_INTERIM_CACHE == 1
-    if (last_collision && dir_get_offset(&dir) != dir_get_offset(last_collision))
-      last_collision = 0;
-  Lread:
-#else
     if (last_collision && dir_offset(&dir) != dir_offset(last_collision))
       last_collision = 0; // object has been/is being overwritten
-#endif
     if (dir_probe(&key, vol, &dir, &last_collision)) {
       int ret = do_read_call(&key);
       if (ret == EVENT_RETURN)
@@ -595,11 +582,7 @@ CacheVC::openReadReadDone(int event, Event *e)
       if (writer_done()) {
         last_collision = NULL;
         while (dir_probe(&earliest_key, vol, &dir, &last_collision)) {
-#if TS_USE_INTERIM_CACHE == 1
-          if (dir_get_offset(&dir) == dir_get_offset(&earliest_dir)) {
-#else
           if (dir_offset(&dir) == dir_offset(&earliest_dir)) {
-#endif
             DDebug("cache_read_agg", "%p: key: %X ReadRead complete: %d", this, first_key.slice32(1), (int)vio.ndone);
             doc_len = vio.ndone;
             goto Ldone;
@@ -608,8 +591,14 @@ CacheVC::openReadReadDone(int event, Event *e)
         DDebug("cache_read_agg", "%p: key: %X ReadRead writer aborted: %d", this, first_key.slice32(1), (int)vio.ndone);
         goto Lerror;
       }
-      DDebug("cache_read_agg", "%p: key: %X ReadRead retrying: %d", this, first_key.slice32(1), (int)vio.ndone);
-      VC_SCHED_WRITER_RETRY(); // wait for writer
+      if (writer_lock_retry < cache_config_read_while_writer_max_retries) {
+        DDebug("cache_read_agg", "%p: key: %X ReadRead retrying: %d", this, first_key.slice32(1), (int)vio.ndone);
+        VC_SCHED_WRITER_RETRY(); // wait for writer
+      } else {
+        DDebug("cache_read_agg", "%p: key: %X ReadRead retries exhausted, bailing..: %d", this, first_key.slice32(1),
+               (int)vio.ndone);
+        goto Ldone;
+      }
     }
     // fall through for truncated documents
   }
@@ -845,11 +834,7 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
     next_CacheKey(&key, &doc->key);
     vol->begin_read(this);
     if (vol->within_hit_evacuate_window(&earliest_dir) &&
-        (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)
-#if TS_USE_INTERIM_CACHE == 1
-        && !dir_ininterim(&dir)
-#endif
-          ) {
+        (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)) {
       DDebug("cache_hit_evac", "dir: %" PRId64 ", write: %" PRId64 ", phase: %d", dir_offset(&earliest_dir),
              offset_to_vol_offset(vol, vol->header->write_pos), vol->header->phase);
       f.hit_evacuate = 1;
@@ -858,13 +843,6 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
   Lread:
     if (dir_probe(&key, vol, &earliest_dir, &last_collision) || dir_lookaside_probe(&key, vol, &earliest_dir, NULL)) {
       dir = earliest_dir;
-#if TS_USE_INTERIM_CACHE == 1
-      if (dir_ininterim(&dir) && alternate.get_frag_offset_count() > 1) {
-        dir_delete(&key, vol, &dir);
-        last_collision = NULL;
-        goto Lread;
-      }
-#endif
       if ((ret = do_read_call(&key)) == EVENT_RETURN)
         goto Lcallreturn;
       return ret;
@@ -885,12 +863,7 @@ CacheVC::openReadStartEarliest(int /* event ATS_UNUSED */, Event * /* e ATS_UNUS
           // finds that the directory entry has been overwritten
           // (cannot assert on the return value)
           dir_delete(&first_key, vol, &first_dir);
-        }
-#if TS_USE_INTERIM_CACHE == 1
-        else if (dir_ininterim(&first_dir))
-          dir_delete(&first_key, vol, &first_dir);
-#endif
-        else {
+        } else {
           buf = NULL;
           last_collision = NULL;
           write_len = 0;
@@ -1024,12 +997,6 @@ CacheVC::openReadStartHead(int event, Event *e)
       // a directory entry which is nolonger valid may have been overwritten
       if (!dir_valid(vol, &dir))
         last_collision = NULL;
-#if TS_USE_INTERIM_CACHE == 1
-      if (dir_ininterim(&dir)) {
-        dir_delete(&key, vol, &dir);
-        last_collision = NULL;
-      }
-#endif
       goto Lread;
     }
     doc = (Doc *)buf->data();
@@ -1052,17 +1019,7 @@ CacheVC::openReadStartHead(int event, Event *e)
       goto Lread;
     }
     if (!(doc->first_key == key))
-#if TS_USE_INTERIM_CACHE == 1
-    {
-      if (dir_ininterim(&dir)) {
-        dir_delete(&key, vol, &dir);
-        last_collision = NULL;
-      }
       goto Lread;
-    }
-#else
-      goto Lread;
-#endif
     if (f.lookup)
       goto Lookup;
     earliest_dir = dir;
@@ -1152,11 +1109,7 @@ CacheVC::openReadStartHead(int event, Event *e)
       goto Learliest;
 
     if (vol->within_hit_evacuate_window(&dir) &&
-        (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)
-#if TS_USE_INTERIM_CACHE == 1
-        && !f.read_from_interim
-#endif
-        ) {
+        (!cache_config_hit_evacuate_size_limit || doc_len <= (uint64_t)cache_config_hit_evacuate_size_limit)) {
       DDebug("cache_hit_evac", "dir: %" PRId64 ", write: %" PRId64 ", phase: %d", dir_offset(&dir),
              offset_to_vol_offset(vol, vol->header->write_pos), vol->header->phase);
       f.hit_evacuate = 1;
