@@ -166,7 +166,7 @@ CacheVC::updateVector(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
    - total_len. The total number of bytes for the document so far.
      Doc->total_len and alternate's total len is set to this value.
    - first_key. Doc's first_key is set to this value.
-   - pin_in_cache. Doc's pinned value is set to this + ink_get_hrtime().
+   - pin_in_cache. Doc's pinned value is set to this + Thread::get_hrtime().
    - earliest_key. If f.use_first_key, Doc's key is set to this value.
    - key. If !f.use_first_key, Doc's key is set to this value.
    - blocks. Used only if write_len is set. Data to be written
@@ -538,11 +538,7 @@ evacuate_fragments(CacheKey *key, CacheKey *earliest_key, int force, Vol *vol)
   while (dir_probe(key, vol, &dir, &last_collision)) {
     // next fragment cannot be a head...if it is, it must have been a
     // directory collision.
-    if (dir_head(&dir)
-#if TS_USE_INTERIM_CACHE == 1
-        || dir_ininterim(&dir)
-#endif
-          )
+    if (dir_head(&dir))
       continue;
     EvacuationBlock *b = evacuation_block_exists(&dir, vol);
     if (!b) {
@@ -615,7 +611,7 @@ Vol::evacuateDocReadDone(int event, Event *e)
   }
   if (!b)
     goto Ldone;
-  if ((b->f.pinned && !b->readers) && doc->pinned < (uint32_t)(ink_get_based_hrtime() / HRTIME_SECOND))
+  if ((b->f.pinned && !b->readers) && doc->pinned < (uint32_t)(Thread::get_hrtime() / HRTIME_SECOND))
     goto Ldone;
 
   if (dir_head(&b->dir) && b->f.evacuate_head) {
@@ -732,9 +728,6 @@ agg_copy(char *p, CacheVC *vc)
     dir_set_approx_size(&vc->dir, vc->agg_len);
     dir_set_offset(&vc->dir, offset_to_vol_offset(vol, o));
     ink_assert(vol_offset(vol, &vc->dir) < (vol->skip + vol->len));
-#if TS_USE_INTERIM_CACHE == 1
-    dir_set_indisk(&vc->dir);
-#endif
     dir_set_phase(&vc->dir, vol->header->phase);
 
     // fill in document header
@@ -752,7 +745,7 @@ agg_copy(char *p, CacheVC *vc)
     doc->checksum = DOC_NO_CHECKSUM;
     if (vc->pin_in_cache) {
       dir_set_pinned(&vc->dir, 1);
-      doc->pinned = (uint32_t)(ink_get_based_hrtime() / HRTIME_SECOND) + vc->pin_in_cache;
+      doc->pinned = (uint32_t)(Thread::get_hrtime() / HRTIME_SECOND) + vc->pin_in_cache;
     } else {
       dir_set_pinned(&vc->dir, 0);
       doc->pinned = 0;
@@ -864,9 +857,6 @@ agg_copy(char *p, CacheVC *vc)
     vc->dir = vc->overwrite_dir;
     dir_set_offset(&vc->dir, offset_to_vol_offset(vc->vol, o));
     dir_set_phase(&vc->dir, vc->vol->header->phase);
-#if TS_USE_INTERIM_CACHE == 1
-    dir_set_indisk(&vc->dir);
-#endif
     return l;
   }
 }
@@ -1494,31 +1484,9 @@ CacheVC::openWriteStartDone(int event, Event *e)
 
       if (doc->magic != DOC_MAGIC || !doc->hlen || this->load_http_info(write_vector, doc, buf) != doc->hlen) {
         err = ECACHE_BAD_META_DATA;
-#if TS_USE_INTERIM_CACHE == 1
-        if (dir_ininterim(&dir)) {
-          dir_delete(&first_key, vol, &dir);
-          last_collision = NULL;
-          goto Lcollision;
-        }
-#endif
         goto Lfailure;
       }
       ink_assert(write_vector->count() > 0);
-#if TS_USE_INTERIM_CACHE == 1
-    Lagain:
-      if (dir_ininterim(&dir)) {
-        dir_delete(&first_key, vol, &dir);
-        last_collision = NULL;
-        if (dir_probe(&first_key, vol, &dir, &last_collision)) {
-          goto Lagain;
-        } else {
-          if (f.update) {
-            // fail update because vector has been GC'd
-            goto Lfailure;
-          }
-        }
-      }
-#endif
       od->first_dir = dir;
       first_dir = dir;
       if (doc->single_fragment()) {
@@ -1605,8 +1573,8 @@ CacheVC::openWriteStartBegin(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED
 
 // main entry point for writing of of non-http documents
 Action *
-Cache::open_write(Continuation *cont, CacheKey *key, CacheFragType frag_type, int options, time_t apin_in_cache, char *hostname,
-                  int host_len)
+Cache::open_write(Continuation *cont, const CacheKey *key, CacheFragType frag_type, int options, time_t apin_in_cache,
+                  const char *hostname, int host_len)
 {
   if (!CacheProcessor::IsCacheReady(frag_type)) {
     cont->handleEvent(CACHE_EVENT_OPEN_WRITE_FAILED, (void *)-ECACHE_NOT_READY);
@@ -1618,7 +1586,7 @@ Cache::open_write(Continuation *cont, CacheKey *key, CacheFragType frag_type, in
   intptr_t res = 0;
   CacheVC *c = new_CacheVC(cont);
   ProxyMutex *mutex = cont->mutex;
-  MUTEX_LOCK(lock, c->mutex, this_ethread());
+  SCOPED_MUTEX_LOCK(lock, c->mutex, this_ethread());
   c->vio.op = VIO::WRITE;
   c->base_stat = cache_write_active_stat;
   c->vol = key_to_vol(key, hostname, host_len);
@@ -1673,8 +1641,8 @@ Cache::open_write(Continuation *cont, CacheKey *key, CacheFragType frag_type, in
 #ifdef HTTP_CACHE
 // main entry point for writing of http documents
 Action *
-Cache::open_write(Continuation *cont, CacheKey *key, CacheHTTPInfo *info, time_t apin_in_cache, CacheKey * /* key1 ATS_UNUSED */,
-                  CacheFragType type, char *hostname, int host_len)
+Cache::open_write(Continuation *cont, const CacheKey *key, CacheHTTPInfo *info, time_t apin_in_cache,
+                  const CacheKey * /* key1 ATS_UNUSED */, CacheFragType type, const char *hostname, int host_len)
 {
   if (!CacheProcessor::IsCacheReady(type)) {
     cont->handleEvent(CACHE_EVENT_OPEN_WRITE_FAILED, (void *)-ECACHE_NOT_READY);
@@ -1804,150 +1772,4 @@ Lcallreturn:
   return &c->_action;
 }
 
-#if TS_USE_INTERIM_CACHE == 1
-int
-InterimCacheVol::aggWrite(int /* event ATS_UNUSED */, void * /* ATS_UNUSED e */)
-{
-  ink_assert(!is_io_in_progress());
-  MigrateToInterimCache *mts;
-  Doc *doc;
-  uint64_t old_off, new_off;
-  ink_assert(this_ethread() == mutex.m_ptr->thread_holding && vol->mutex.m_ptr == mutex.m_ptr);
-Lagain:
-
-  while ((mts = agg.head) != NULL) {
-    doc = (Doc *)mts->buf->data();
-    uint32_t agg_len = dir_approx_size(&mts->dir);
-    ink_assert(agg_len == mts->agg_len);
-    ink_assert(agg_len <= AGG_SIZE && agg_buf_pos <= AGG_SIZE);
-
-    if (agg_buf_pos + agg_len > AGG_SIZE || header->agg_pos + agg_len > (skip + len))
-      break;
-    mts = agg.dequeue();
-
-    if (!mts->notMigrate) {
-      old_off = dir_get_offset(&mts->dir);
-      Dir old_dir = mts->dir;
-      doc->sync_serial = header->sync_serial;
-      doc->write_serial = header->write_serial;
-
-      memcpy(agg_buffer + agg_buf_pos, doc, doc->len);
-      off_t o = header->write_pos + agg_buf_pos;
-      dir_set_offset(&mts->dir, offset_to_vol_offset(this, o));
-      ink_assert(this == mts->interim_vol);
-      ink_assert(vol_offset(this, &mts->dir) < mts->interim_vol->skip + mts->interim_vol->len);
-      dir_set_phase(&mts->dir, header->phase);
-      dir_set_ininterim(&mts->dir);
-      dir_set_index(&mts->dir, (this - vol->interim_vols));
-
-      agg_buf_pos += agg_len;
-      header->agg_pos = header->write_pos + agg_buf_pos;
-      new_off = dir_get_offset(&mts->dir);
-
-      if (mts->rewrite)
-        dir_overwrite(&mts->key, vol, &mts->dir, &old_dir);
-      else
-        dir_insert(&mts->key, vol, &mts->dir);
-      DDebug("cache_insert",
-             "InterimCache: WriteDone: key: %X, first_key: %X, write_len: %d, write_offset: %" PRId64 ", dir_last_word: %X",
-             doc->key.slice32(0), doc->first_key.slice32(0), mts->agg_len, o, mts->dir.w[4]);
-
-      if (mts->copy) {
-        mts->interim_vol->vol->ram_cache->fixup(&mts->key, (uint32_t)(old_off >> 32), (uint32_t)old_off, (uint32_t)(new_off >> 32),
-                                                (uint32_t)new_off);
-      } else {
-        mts->vc->f.ram_fixup = 1;
-        mts->vc->dir_off = new_off;
-      }
-      vol->set_migrate_done(mts);
-    } else
-      vol->set_migrate_failed(mts);
-
-    mts->buf = NULL;
-    migrateToInterimCacheAllocator.free(mts);
-  }
-
-  if (!agg_buf_pos) {
-    if (header->write_pos + AGG_SIZE > (skip + len)) {
-      header->write_pos = start;
-      header->phase = !(header->phase);
-
-      header->cycle++;
-      header->agg_pos = header->write_pos;
-      dir_clean_interimvol(this);
-      goto Lagain;
-    }
-    return EVENT_CONT;
-  }
-
-  if (agg.head == NULL && agg_buf_pos < (AGG_SIZE / 2) && !sync && header->write_pos + AGG_SIZE <= (skip + len))
-    return EVENT_CONT;
-
-  for (mts = agg.head; mts != NULL; mts = mts->link.next) {
-    if (!mts->copy) {
-      Ptr<IOBufferData> buf = mts->buf;
-      doc = (Doc *)buf->data();
-      mts->buf = new_IOBufferData(iobuffer_size_to_index(mts->agg_len, MAX_BUFFER_SIZE_INDEX), MEMALIGNED);
-      mts->copy = true;
-      memcpy(mts->buf->data(), buf->data(), doc->len);
-      buf = NULL;
-    }
-  }
-  // set write limit
-
-  io.aiocb.aio_fildes = fd;
-  io.aiocb.aio_offset = header->write_pos;
-  io.aiocb.aio_buf = agg_buffer;
-  io.aiocb.aio_nbytes = agg_buf_pos;
-  io.action = this;
-  /*
-    Callback on AIO thread so that we can issue a new write ASAP
-    as all writes are serialized in the volume.  This is not necessary
-    for reads proceed independently.
-   */
-  io.thread = AIO_CALLBACK_THREAD_AIO;
-  SET_HANDLER(&InterimCacheVol::aggWriteDone);
-  ink_aio_write(&io);
-  return EVENT_CONT;
-}
-
-int
-InterimCacheVol::aggWriteDone(int event, void *e)
-{
-  ink_release_assert(this_ethread() == mutex.m_ptr->thread_holding && vol->mutex.m_ptr == mutex.m_ptr);
-  if (io.ok()) {
-    header->last_write_pos = header->write_pos;
-    header->write_pos += io.aiocb.aio_nbytes;
-    ink_assert(header->write_pos >= start);
-    DDebug("cache_agg", "Write: %" PRIu64 ", last Write: %" PRIu64 "\n", header->write_pos, header->last_write_pos);
-    ink_assert(header->write_pos == header->agg_pos);
-    agg_buf_pos = 0;
-    header->write_serial++;
-  } else {
-    // delete all the directory entries that we inserted
-    // for fragments is this aggregation buffer
-    Debug("cache_disk_error", "Write error on disk %s\n \
-               write range : [%" PRIu64 " - %" PRIu64 " bytes]  [%" PRIu64 " - %" PRIu64 " blocks] \n",
-          "InterimCache ID", (uint64_t)io.aiocb.aio_offset, (uint64_t)(io.aiocb.aio_offset + io.aiocb.aio_nbytes),
-          (uint64_t)io.aiocb.aio_offset / CACHE_BLOCK_SIZE,
-          (uint64_t)(io.aiocb.aio_offset + io.aiocb.aio_nbytes) / CACHE_BLOCK_SIZE);
-    Dir del_dir;
-    dir_clear(&del_dir);
-    dir_set_ininterim(&del_dir);
-    dir_set_index(&del_dir, (this - vol->interim_vols));
-    for (int done = 0; done < agg_buf_pos;) {
-      Doc *doc = (Doc *)(agg_buffer + done);
-      dir_set_offset(&del_dir, header->write_pos + done);
-      dir_delete(&doc->key, vol, &del_dir);
-      done += this->round_to_approx_size(doc->len);
-    }
-    agg_buf_pos = 0;
-  }
-  set_io_not_in_progress();
-  sync = false;
-  if (agg.head)
-    aggWrite(event, e);
-  return EVENT_CONT;
-}
-#endif
 #endif
