@@ -24,7 +24,7 @@
 #ifndef _P_CACHE_H__
 #define _P_CACHE_H__
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "P_EventSystem.h"
 #include "P_AIO.h"
 #include "I_RecProcess.h"
@@ -32,7 +32,7 @@
 #ifdef HTTP_CACHE
 #include "HTTP.h"
 #include "MIME.h"
-#include "MimeTable.h"
+#include "ts/MimeTable.h"
 #include "HttpTransactCache.h"
 #endif
 
