@@ -42,7 +42,8 @@ struct RamCacheLRU : public RamCache {
   // returns 1 on found/stored, 0 on not found/stored, if provided auxkey1 and auxkey2 must match
   int get(INK_MD5 *key, Ptr<IOBufferData> *ret_data, uint32_t auxkey1 = 0, uint32_t auxkey2 = 0);
   int put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool copy = false, uint32_t auxkey1 = 0, uint32_t auxkey2 = 0);
-  int fixup(INK_MD5 *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2);
+  int fixup(const INK_MD5 *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2);
+  int64_t size() const;
 
   void init(int64_t max_bytes, Vol *vol);
 
@@ -60,6 +61,19 @@ struct RamCacheLRU : public RamCache {
   RamCacheLRU() : bytes(0), objects(0), seen(0), bucket(0), nbuckets(0), ibuckets(0), vol(NULL) {}
 };
 
+int64_t
+RamCacheLRU::size() const
+{
+  int64_t s = 0;
+  forl_LL(RamCacheLRUEntry, e, lru)
+  {
+    s += sizeof(*e);
+    s += sizeof(*e->data);
+    s += e->data->block_size();
+  }
+  return s;
+}
+
 ClassAllocator<RamCacheLRUEntry> ramCacheLRUEntryAllocator("RamCacheLRUEntry");
 
 static const int bucket_sizes[] = {127,     251,      509,      1021,     2039,      4093,      8191,     16381,
@@ -198,7 +212,7 @@ RamCacheLRU::put(INK_MD5 *key, IOBufferData *data, uint32_t len, bool, uint32_t
 }
 
 int
-RamCacheLRU::fixup(INK_MD5 *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2)
+RamCacheLRU::fixup(const INK_MD5 *key, uint32_t old_auxkey1, uint32_t old_auxkey2, uint32_t new_auxkey1, uint32_t new_auxkey2)
 {
   if (!max_bytes)
     return 0;
