@@ -21,9 +21,12 @@
   limitations under the License.
  */
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "P_Cache.h"
-#include "I_Layout.h"
+#include "ts/I_Layout.h"
+#include "ts/ink_file.h"
+#include "ts/Tokenizer.h"
+#include "ts/SimpleTokenizer.h"
 
 #if HAVE_LINUX_MAJOR_H
 #include <linux/major.h>
@@ -68,12 +71,7 @@ span_file_typename(mode_t st_mode)
 }
 
 Ptr<ProxyMutex> tmp_p;
-Store::Store()
-  : n_disks(0), disk(NULL)
-#if TS_USE_INTERIM_CACHE == 1
-    ,
-    n_interim_disks(0), interim_disk(NULL)
-#endif
+Store::Store() : n_disks_in_config(0), n_disks(0), disk(NULL)
 {
 }
 
@@ -355,6 +353,7 @@ Store::read_config()
 
     // parse
     Debug("cache_init", "Store::read_config: \"%s\"", path);
+    ++n_disks_in_config;
 
     int64_t size = -1;
     int volume_num = -1;
@@ -383,6 +382,7 @@ Store::read_config()
     }
 
     char *pp = Layout::get()->relative(path);
+
     ns = new Span;
     Debug("cache_init", "Store::read_config - ns = new Span; ns->init(\"%s\",%" PRId64 "), forced volume=%d%s%s", pp, size,
           volume_num, seed ? " id=" : "", seed ? seed : "");
@@ -434,50 +434,6 @@ Lfail:
   return err;
 }
 
-#if TS_USE_INTERIM_CACHE == 1
-const char *
-Store::read_interim_config()
-{
-  char p[PATH_NAME_MAX];
-  Span *sd = NULL;
-  Span *ns;
-  int interim_store = 0;
-  REC_ReadConfigString(p, "proxy.config.cache.interim.storage", PATH_NAME_MAX);
-
-  char *n = p;
-  int sz = strlen(p);
-
-  const char *err = NULL;
-  for (int len = 0; n < p + sz; n += len + 1) {
-    char *e = strpbrk(n, " \t\n");
-    len = e ? e - n : strlen(n);
-    n[len] = '\0';
-    ns = new Span;
-    if ((err = ns->init(n, -1))) {
-      RecSignalWarning(REC_SIGNAL_SYSTEM_ERROR, "could not initialize storage \"%s\" [%s]", n, err);
-      Debug("cache_init", "Store::read_interim_config - could not initialize storage \"%s\" [%s]", n, err);
-      delete ns;
-      continue;
-    }
-    ns->link.next = sd;
-    sd = ns;
-    interim_store++;
-  }
-
-  n_interim_disks = interim_store;
-  interim_disk = (Span **)ats_malloc(interim_store * sizeof(Span *));
-  {
-    int i = 0;
-    while (sd) {
-      ns = sd;
-      sd = sd->link.next;
-      ns->link.next = NULL;
-      interim_disk[i++] = ns;
-    }
-  }
-  return NULL;
-}
-#endif
 
 int
 Store::write_config_data(int fd) const
