@@ -25,7 +25,9 @@
 #ifndef _EThread_h_
 #define _EThread_h_
 
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_rand.h"
+#include "ts/I_Version.h"
 #include "I_Thread.h"
 #include "I_PriorityEventQueue.h"
 #include "I_ProtectedQueue.h"
