@@ -31,7 +31,7 @@
 #ifndef _I_ProxyAllocator_h_
 #define _I_ProxyAllocator_h_
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 
 class EThread;
 
@@ -49,7 +49,6 @@ template <class C>
 inline C *
 thread_alloc(ClassAllocator<C> &a, ProxyAllocator &l)
 {
-#if TS_USE_FREELIST && !TS_USE_RECLAIMABLE_FREELIST
   if (l.freelist) {
     C *v = (C *)l.freelist;
     l.freelist = *(C **)l.freelist;
@@ -57,9 +56,6 @@ thread_alloc(ClassAllocator<C> &a, ProxyAllocator &l)
     *(void **)v = *(void **)&a.proto.typeObject;
     return v;
   }
-#else
-  (void)l;
-#endif
   return a.alloc();
 }
 
@@ -67,7 +63,6 @@ template <class C>
 inline C *
 thread_alloc_init(ClassAllocator<C> &a, ProxyAllocator &l)
 {
-#if TS_USE_FREELIST && !TS_USE_RECLAIMABLE_FREELIST
   if (l.freelist) {
     C *v = (C *)l.freelist;
     l.freelist = *(C **)l.freelist;
@@ -75,9 +70,6 @@ thread_alloc_init(ClassAllocator<C> &a, ProxyAllocator &l)
     memcpy((void *)v, (void *)&a.proto.typeObject, sizeof(C));
     return v;
   }
-#else
-  (void)l;
-#endif
   return a.alloc();
 }
 
@@ -98,9 +90,7 @@ template <class C>
 inline void
 thread_freeup(ClassAllocator<C> &a, ProxyAllocator &l)
 {
-#if !TS_USE_RECLAIMABLE_FREELIST
   C *head = (C *)l.freelist;
-#endif
   C *tail = (C *)l.freelist;
   size_t count = 0;
   while (l.freelist && l.allocated > thread_freelist_low_watermark) {
@@ -108,11 +98,8 @@ thread_freeup(ClassAllocator<C> &a, ProxyAllocator &l)
     l.freelist = *(C **)l.freelist;
     --(l.allocated);
     ++count;
-#if TS_USE_RECLAIMABLE_FREELIST
-    a.free(tail);
-#endif
   }
-#if !TS_USE_RECLAIMABLE_FREELIST
+
   if (unlikely(count == 1)) {
     a.free(tail);
   } else if (count > 0) {
@@ -120,7 +107,6 @@ thread_freeup(ClassAllocator<C> &a, ProxyAllocator &l)
   }
 
   ink_assert(l.allocated >= thread_freelist_low_watermark);
-#endif
 }
 
 void *thread_alloc(Allocator &a, ProxyAllocator &l);
@@ -128,7 +114,6 @@ void thread_freeup(Allocator &a, ProxyAllocator &l);
 
 #define THREAD_ALLOC(_a, _t) thread_alloc(::_a, _t->_a)
 #define THREAD_ALLOC_INIT(_a, _t) thread_alloc_init(::_a, _t->_a)
-#if TS_USE_FREELIST && !TS_USE_RECLAIMABLE_FREELIST
 #define THREAD_FREE(_p, _a, _t)                            \
   do {                                                     \
     *(char **)_p = (char *)_t->_a.freelist;                \
@@ -137,12 +122,5 @@ void thread_freeup(Allocator &a, ProxyAllocator &l);
     if (_t->_a.allocated > thread_freelist_high_watermark) \
       thread_freeup(::_a, _t->_a);                         \
   } while (0)
-#else /* !TS_USE_FREELIST || TS_USE_RECLAIMABLE_FREELIST */
-#define THREAD_FREE(_p, _a, _t) \
-  do {                          \
-    (void) _t;                  \
-    thread_free(::_a, _p);      \
-  } while (0)
-#endif
 
 #endif /* _ProxyAllocator_h_ */
