@@ -25,8 +25,8 @@
 #if !defined(_P_IOBuffer_h)
 #define _P_IOBuffer_h
 
-#include "libts.h"
-#include "ink_resource.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_resource.h"
 
 
 // TODO: I think we're overly aggressive here on making MIOBuffer 64-bit
@@ -477,7 +477,7 @@ IOBufferBlock::set(IOBufferData *d, int64_t len, int64_t offset)
   data = d;
   _start = buf() + offset;
   _end = _start + len;
-  _buf_end = _start + d->block_size();
+  _buf_end = buf() + d->block_size();
 }
 
 TS_INLINE void
