@@ -76,7 +76,7 @@ EThread::schedule_in(Continuation *cont, ink_hrtime t, int callback_event, void
   Event *e = ::eventAllocator.alloc();
   e->callback_event = callback_event;
   e->cookie = cookie;
-  return schedule(e->init(cont, ink_get_based_hrtime() + t, 0));
+  return schedule(e->init(cont, get_hrtime() + t, 0));
 }
 
 TS_INLINE Event *
@@ -85,7 +85,7 @@ EThread::schedule_every(Continuation *cont, ink_hrtime t, int callback_event, vo
   Event *e = ::eventAllocator.alloc();
   e->callback_event = callback_event;
   e->cookie = cookie;
-  return schedule(e->init(cont, ink_get_based_hrtime() + t, t));
+  return schedule(e->init(cont, get_hrtime() + t, t));
 }
 
 TS_INLINE Event *
@@ -126,7 +126,7 @@ EThread::schedule_in_local(Continuation *cont, ink_hrtime t, int callback_event,
   Event *e = EVENT_ALLOC(eventAllocator, this);
   e->callback_event = callback_event;
   e->cookie = cookie;
-  return schedule_local(e->init(cont, ink_get_based_hrtime() + t, 0));
+  return schedule_local(e->init(cont, get_hrtime() + t, 0));
 }
 
 TS_INLINE Event *
@@ -135,7 +135,7 @@ EThread::schedule_every_local(Continuation *cont, ink_hrtime t, int callback_eve
   Event *e = EVENT_ALLOC(eventAllocator, this);
   e->callback_event = callback_event;
   e->cookie = cookie;
-  return schedule_local(e->init(cont, ink_get_based_hrtime() + t, t));
+  return schedule_local(e->init(cont, get_hrtime() + t, t));
 }
 
 TS_INLINE Event *
