@@ -29,6 +29,7 @@
 
 **************************************************************************/
 #include "P_EventSystem.h"
+#include "ts/ink_string.h"
 
 ///////////////////////////////////////////////
 // Common Interface impl                     //
@@ -101,3 +102,9 @@ Thread::start(const char *name, size_t stacksize, ThreadFunction f, void *a)
 
   return tid;
 }
+
+ink_hrtime
+Thread::get_hrtime()
+{
+  return Thread::cur_time;
+}
