@@ -27,7 +27,7 @@
 #include <alloca.h>
 #include <hwloc.h>
 #endif
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 
 EventType
 EventProcessor::spawn_event_threads(int n_threads, const char *et_name, size_t stacksize)
@@ -80,11 +80,9 @@ EventProcessor::start(int n_event_threads, size_t stacksize)
   n_ethreads = n_event_threads;
   n_thread_groups = 1;
 
-  int first_thread = 1;
-
   for (i = 0; i < n_event_threads; i++) {
     EThread *t = new EThread(REGULAR, i);
-    if (first_thread && !i) {
+    if (i == 0) {
       ink_thread_setspecific(Thread::thread_data_key, t);
       global_mutex = t->mutex;
       t->cur_time = ink_get_based_hrtime_internal();
@@ -97,7 +95,7 @@ EventProcessor::start(int n_event_threads, size_t stacksize)
   n_threads_for_type[ET_CALL] = n_event_threads;
 
 #if TS_USE_HWLOC
-  int affinity = 0;
+  int affinity = 1;
   REC_ReadConfigInteger(affinity, "proxy.config.exec_thread.affinity");
   hwloc_obj_t obj;
   hwloc_obj_type_t obj_type;
@@ -135,10 +133,14 @@ EventProcessor::start(int n_event_threads, size_t stacksize)
   Debug("iocore_thread", "Affinity: %d %ss: %d PU: %d", affinity, obj_name, obj_count, ink_number_of_processors());
 
 #endif
-  for (i = first_thread; i < n_ethreads; i++) {
-    snprintf(thr_name, MAX_THREAD_NAME_LENGTH, "[ET_NET %d]", i);
-    ink_thread tid = all_ethreads[i]->start(thr_name, stacksize);
-    (void)tid;
+  for (i = 0; i < n_ethreads; i++) {
+    ink_thread tid;
+    if (i > 0) {
+      snprintf(thr_name, MAX_THREAD_NAME_LENGTH, "[ET_NET %d]", i);
+      tid = all_ethreads[i]->start(thr_name, stacksize);
+    } else {
+      tid = ink_thread_self();
+    }
 #if TS_USE_HWLOC
     if (obj_count > 0) {
       obj = hwloc_get_obj_by_type(ink_get_topology(), obj_type, i % obj_count);
@@ -154,6 +156,8 @@ EventProcessor::start(int n_event_threads, size_t stacksize)
     } else {
       Warning("hwloc returned an unexpected value -- CPU affinity disabled");
     }
+#else
+    (void)tid;
 #endif // TS_USE_HWLOC
   }
 
