@@ -24,8 +24,9 @@
 #define _HOSTDB_CC_
 
 #include "P_HostDB.h"
-#include "I_Layout.h"
+#include "ts/I_Layout.h"
 #include "Show.h"
+#include "ts/Tokenizer.h"
 
 #include <vector>
 #include <algorithm>
@@ -35,7 +36,7 @@
 //#define Warning
 //#define Note
 
-#include "ink_apidefs.h"
+#include "ts/ink_apidefs.h"
 
 HostDBProcessor hostDBProcessor;
 int HostDBProcessor::hostdb_strict_round_robin = 0;
@@ -46,7 +47,7 @@ int hostdb_enable = true;
 int hostdb_migrate_on_demand = true;
 int hostdb_cluster = false;
 int hostdb_cluster_round_robin = false;
-int hostdb_lookup_timeout = 120;
+int hostdb_lookup_timeout = 30;
 int hostdb_insert_timeout = 160;
 int hostdb_re_dns_on_reload = false;
 int hostdb_ttl_mode = TTL_OBEY;
@@ -238,6 +239,7 @@ HostDBCache::HostDBCache()
   max_hits = (1 << HOST_DB_HITS_BITS) - 1;
   version.ink_major = HOST_DB_CACHE_MAJOR_VERSION;
   version.ink_minor = HOST_DB_CACHE_MINOR_VERSION;
+  hosts_file_ptr = new RefCountedHostsFileMap();
 }
 
 
@@ -388,7 +390,7 @@ int
 HostDBSyncer::sync_event(int, void *)
 {
   SET_HANDLER(&HostDBSyncer::wait_event);
-  start_time = ink_get_hrtime();
+  start_time = Thread::get_hrtime();
   hostDBProcessor.cache()->sync_partitions(this);
   return EVENT_DONE;
 }
@@ -397,7 +399,7 @@ HostDBSyncer::sync_event(int, void *)
 int
 HostDBSyncer::wait_event(int, void *)
 {
-  ink_hrtime next_sync = HRTIME_SECONDS(hostdb_sync_frequency) - (ink_get_hrtime() - start_time);
+  ink_hrtime next_sync = HRTIME_SECONDS(hostdb_sync_frequency) - (Thread::get_hrtime() - start_time);
 
   SET_HANDLER(&HostDBSyncer::sync_event);
   if (next_sync > HRTIME_MSECONDS(100))
@@ -524,7 +526,7 @@ HostDBProcessor::start(int, size_t)
   //
   // Set up hostdb_current_interval
   //
-  hostdb_current_interval = (unsigned int)(ink_get_based_hrtime() / HOST_DB_TIMEOUT_INTERVAL);
+  hostdb_current_interval = (unsigned int)(Thread::get_hrtime() / HOST_DB_TIMEOUT_INTERVAL);
 
   HostDBContinuation *b = hostDBContAllocator.alloc();
   SET_CONTINUATION_HANDLER(b, (HostDBContHandler)&HostDBContinuation::backgroundEvent);
@@ -659,6 +661,16 @@ db_mark_for(IpAddr const &ip)
 HostDBInfo *
 probe(ProxyMutex *mutex, HostDBMD5 const &md5, bool ignore_timeout)
 {
+  // If we have an entry in our hosts file, we don't need to bother with DNS
+  // Make a local copy/reference so we don't have to worry about updates.
+  Ptr<RefCountedHostsFileMap> current_host_file_map = hostDB.hosts_file_ptr;
+
+  ts::ConstBuffer hname(md5.host_name, md5.host_len);
+  HostsFileMap::iterator find_result = current_host_file_map->hosts_file_map.find(hname);
+  if (find_result != current_host_file_map->hosts_file_map.end()) {
+    return &(find_result->second);
+  }
+
   ink_assert(this_ethread() == hostDB.lock_for_bucket((int)(fold_md5(md5.hash) % hostDB.buckets))->thread_holding);
   if (hostdb_enable) {
     uint64_t folded_md5 = fold_md5(md5.hash);
@@ -854,6 +866,25 @@ HostDBProcessor::getbyname_re(Continuation *cont, const char *ahostname, int len
   return getby(cont, ahostname, len, 0, force_dns, opt.host_res_style, opt.timeout);
 }
 
+Action *
+HostDBProcessor::getbynameport_re(Continuation *cont, const char *ahostname, int len, Options const &opt)
+{
+  bool force_dns = false;
+  EThread *thread = this_ethread();
+  ProxyMutex *mutex = thread->mutex;
+
+  if (opt.flags & HOSTDB_FORCE_DNS_ALWAYS)
+    force_dns = true;
+  else if (opt.flags & HOSTDB_FORCE_DNS_RELOAD) {
+    force_dns = (hostdb_re_dns_on_reload ? true : false);
+    if (force_dns)
+      HOSTDB_INCREMENT_DYN_STAT(hostdb_re_dns_on_reload_stat);
+  }
+  sockaddr sa;
+  ats_ip4_set(&sa, INADDR_ANY, htons(opt.port));
+  return getby(cont, ahostname, len, &sa, force_dns, opt.host_res_style, opt.timeout);
+}
+
 
 /* Support SRV records */
 Action *
@@ -967,7 +998,7 @@ HostDBProcessor::getbyname_imm(Continuation *cont, process_hostdb_info_pfn proce
       loop = false; // loop only on explicit set for retry
       // find the partition lock
       ProxyMutex *bucket_mutex = hostDB.lock_for_bucket((int)(fold_md5(md5.hash) % hostDB.buckets));
-      MUTEX_LOCK(lock, bucket_mutex, thread);
+      SCOPED_MUTEX_LOCK(lock, bucket_mutex, thread);
       // do a level 1 probe for immediate result.
       HostDBInfo *r = probe(bucket_mutex, md5, false);
       if (r) {
@@ -1002,6 +1033,34 @@ HostDBProcessor::getbyname_imm(Continuation *cont, process_hostdb_info_pfn proce
   return &c->action;
 }
 
+Action *
+HostDBProcessor::iterate(Continuation *cont)
+{
+  ink_assert(cont->mutex->thread_holding == this_ethread());
+  EThread *thread = cont->mutex->thread_holding;
+  ProxyMutex *mutex = thread->mutex;
+
+  HOSTDB_INCREMENT_DYN_STAT(hostdb_total_lookups_stat);
+
+
+  HostDBContinuation *c = hostDBContAllocator.alloc();
+  HostDBContinuation::Options copt;
+  copt.cont = cont;
+  copt.force_dns = false;
+  copt.timeout = 0;
+  copt.host_res_style = HOST_RES_NONE;
+  c->init(HostDBMD5(), copt);
+  c->current_iterate_pos = 0;
+  SET_CONTINUATION_HANDLER(c, (HostDBContHandler)&HostDBContinuation::iterateEvent);
+
+  if (thread->mutex == cont->mutex) {
+    thread->schedule_in(c, HOST_DB_RETRY_PERIOD);
+  } else {
+    dnsProcessor.thread->schedule_imm(c);
+  }
+
+  return &c->action;
+}
 
 static void
 do_setby(HostDBInfo *r, HostDBApplicationInfo *app, const char *hostname, IpAddr const &ip, bool is_srv = false)
@@ -1233,6 +1292,7 @@ HostDBContinuation::lookup_done(IpAddr const &ip, char const *aname, bool around
     }
     i = insert(hostdb_ip_fail_timeout_interval); // currently ... 0
     i->round_robin = false;
+    i->round_robin_elt = false;
     i->is_srv = is_srv();
     i->reverse_dns = !is_byname() && !is_srv();
 
@@ -1264,6 +1324,7 @@ HostDBContinuation::lookup_done(IpAddr const &ip, char const *aname, bool around
       ttl_seconds = 1;
 
     i = insert(ttl_seconds);
+    i->round_robin_elt = false; // only true for elements explicitly added as RR elements.
     if (is_byname()) {
       ip_text_buffer b;
       Debug("hostdb", "done %s TTL %d", ip.toString(b, sizeof b), ttl_seconds);
@@ -1303,6 +1364,20 @@ HostDBContinuation::lookup_done(IpAddr const &ip, char const *aname, bool around
       }
     }
   }
+
+  if (aname) {
+    const size_t s_size = strlen(aname) + 1;
+    void *host_dest = hostDB.alloc(&i->hostname_offset, s_size);
+    if (host_dest) {
+      ink_strlcpy((char *)host_dest, aname, s_size);
+      *((char *)host_dest + s_size) = '\0';
+    } else {
+      Warning("Out of room in hostdb for hostname (data area full!)");
+      hostDB.delete_block(i);
+      return NULL;
+    }
+  }
+
   if (from_cont)
     do_put_response(from, i, from_cont);
   ink_assert(!i->round_robin || !i->reverse_dns);
@@ -1504,6 +1579,7 @@ HostDBContinuation::dnsEvent(int event, HostEnt *e)
 
             memset(&item, 0, sizeof(item));
             item.round_robin = 0;
+            item.round_robin_elt = 1;
             item.reverse_dns = 0;
             item.is_srv = 1;
             item.data.srv.srv_weight = t->weight;
@@ -1522,6 +1598,7 @@ HostDBContinuation::dnsEvent(int event, HostEnt *e)
             item.md5_low = r->md5_low;
             item.md5_low_low = r->md5_low_low;
             item.full = 1;
+            item.hostname_offset = 0;
 
             item.app.allotment.application1 = 0;
             item.app.allotment.application2 = 0;
@@ -1547,14 +1624,17 @@ HostDBContinuation::dnsEvent(int event, HostEnt *e)
           for (ii = 0; ii < nn; ++ii) {
             if (is_addr_valid(af, e->ent.h_addr_list[ii])) {
               HostDBInfo &item = rr_data->info[i];
+              memset(&item, 0, sizeof(item));
               ip_addr_set(item.ip(), af, e->ent.h_addr_list[ii]);
               item.full = 1;
               item.round_robin = 0;
+              item.round_robin_elt = 1;
               item.reverse_dns = 0;
               item.is_srv = 0;
               item.md5_high = r->md5_high;
               item.md5_low = r->md5_low;
               item.md5_low_low = r->md5_low_low;
+              item.hostname_offset = 0;
               if (!restore_info(&item, old_r, old_info, old_rr_data)) {
                 item.app.allotment.application1 = 0;
                 item.app.allotment.application2 = 0;
@@ -1569,6 +1649,7 @@ HostDBContinuation::dnsEvent(int event, HostEnt *e)
         ink_assert(!"out of room in hostdb data area");
         Warning("out of room in hostdb for round-robin DNS data");
         r->round_robin = 0;
+        r->round_robin_elt = 0;
       }
     }
     if (!failed && !rr && !is_srv())
@@ -1748,6 +1829,62 @@ HostDBContinuation::make_put_message(HostDBInfo *r, Continuation *c, char *buf,
   return len;
 }
 
+int
+HostDBContinuation::iterateEvent(int event, Event *e)
+{
+  Debug("hostdb", "iterateEvent event=%d eventp=%p", event, e);
+  ink_assert(!link.prev && !link.next);
+  EThread *t = e ? e->ethread : this_ethread();
+
+  MUTEX_TRY_LOCK_FOR(lock, action.mutex, t, action.continuation);
+  if (!lock.is_locked()) {
+    Debug("hostdb", "iterateEvent event=%d eventp=%p: reschedule due to not getting action mutex", event, e);
+    mutex->thread_holding->schedule_in(this, HOST_DB_RETRY_PERIOD);
+    return EVENT_CONT;
+  }
+
+  if (action.cancelled) {
+    hostdb_cont_free(this);
+    return EVENT_DONE;
+  }
+
+  // let's iterate through another record and then reschedule ourself.
+  if (current_iterate_pos < hostDB.buckets) {
+    // do 100 at a time
+    int end = min(current_iterate_pos + 100, hostDB.buckets);
+    for (; current_iterate_pos < end; ++current_iterate_pos) {
+      ProxyMutex *bucket_mutex = hostDB.lock_for_bucket(current_iterate_pos);
+      MUTEX_TRY_LOCK_FOR(lock_bucket, bucket_mutex, t, this);
+      if (!lock_bucket.is_locked()) {
+        // we couldn't get the bucket lock, let's just reschedule and try later.
+        Debug("hostdb", "iterateEvent event=%d eventp=%p: reschedule due to not getting bucket mutex", event, e);
+        mutex->thread_holding->schedule_in(this, HOST_DB_RETRY_PERIOD);
+        return EVENT_CONT;
+      }
+
+      for (unsigned int l = 0; l < hostDB.levels; ++l) {
+        HostDBInfo *r =
+          reinterpret_cast<HostDBInfo *>(hostDB.data + hostDB.level_offset[l] + hostDB.bucketsize[l] * current_iterate_pos);
+        if (!r->deleted && !r->failed()) {
+          action.continuation->handleEvent(EVENT_INTERVAL, static_cast<void *>(r));
+        }
+      }
+    }
+
+    // And reschedule ourselves to pickup the next bucket after HOST_DB_RETRY_PERIOD.
+    Debug("hostdb", "iterateEvent event=%d eventp=%p: completed current iteration %d of %d", event, e, current_iterate_pos,
+          hostDB.buckets);
+    mutex->thread_holding->schedule_in(this, HOST_DB_ITERATE_PERIOD);
+    return EVENT_CONT;
+  } else {
+    Debug("hostdb", "iterateEvent event=%d eventp=%p: completed FINAL iteration %d", event, e, current_iterate_pos);
+    // if there are no more buckets, then we're done.
+    action.continuation->handleEvent(EVENT_DONE, NULL);
+    hostdb_cont_free(this);
+  }
+
+  return EVENT_DONE;
+}
 
 //
 // Build the put message and send it
@@ -1766,7 +1903,6 @@ HostDBContinuation::do_put_response(ClusterMachine *m, HostDBInfo *r, Continuati
   clusterProcessor.invoke_remote(m->pop_ClusterHandler(), PUT_HOSTINFO_CLUSTER_FUNCTION, (char *)&msg, len);
 }
 
-
 //
 // Probe state
 //
@@ -1888,10 +2024,11 @@ HostDBContinuation::do_dns()
       return;
     }
   }
-  if (hostdb_lookup_timeout)
+  if (hostdb_lookup_timeout) {
     timeout = mutex->thread_holding->schedule_in(this, HRTIME_SECONDS(hostdb_lookup_timeout));
-  else
+  } else {
     timeout = NULL;
+  }
   if (set_check_pending_dns()) {
     DNSProcessor::Options opt;
     opt.timeout = dns_lookup_timeout;
@@ -2121,23 +2258,37 @@ HostDBContinuation::backgroundEvent(int /* event ATS_UNUSED */, Event * /* e ATS
   if (hostdb_hostfile_check_interval && // enabled
       (hostdb_current_interval - hostdb_hostfile_check_timestamp) * (HOST_DB_TIMEOUT_INTERVAL / HRTIME_SECOND) >
         hostdb_hostfile_check_interval) {
+    bool update_p = false; // do we need to reparse the file and update?
     struct stat info;
     char path[sizeof(hostdb_hostfile_path)];
 
     REC_ReadConfigString(path, "proxy.config.hostdb.host_file.path", sizeof(path));
     if (0 != strcasecmp(hostdb_hostfile_path, path)) {
-      Debug("hostdb", "Update host file '%s' <- '%s'", path, hostdb_hostfile_path);
+      Debug("hostdb", "Update host file '%s' -> '%s'", (*hostdb_hostfile_path ? hostdb_hostfile_path : "*-none-*"),
+            (*path ? path : "*-none-*"));
       // path to hostfile changed
       hostdb_hostfile_update_timestamp = 0; // never updated from this file
-      memcpy(hostdb_hostfile_path, path, sizeof(hostdb_hostfile_path));
-    }
-    hostdb_hostfile_check_timestamp = hostdb_current_interval;
-    if (0 == stat(hostdb_hostfile_path, &info)) {
-      if (info.st_mtime > (time_t)hostdb_hostfile_update_timestamp) {
-        ParseHostFile(hostdb_hostfile_path);
+      if ('\0' != *path) {
+        memcpy(hostdb_hostfile_path, path, sizeof(hostdb_hostfile_path));
+      } else {
+        hostdb_hostfile_path[0] = 0; // mark as not there
       }
+      update_p = true;
     } else {
-      Debug("hostdb", "Failed to stat host file '%s'", hostdb_hostfile_path);
+      hostdb_hostfile_check_timestamp = hostdb_current_interval;
+      if (*hostdb_hostfile_path) {
+        if (0 == stat(hostdb_hostfile_path, &info)) {
+          if (info.st_mtime > (time_t)hostdb_hostfile_update_timestamp) {
+            update_p = true; // same file but it's changed.
+          }
+        } else {
+          Debug("hostdb", "Failed to stat host file '%s'", hostdb_hostfile_path);
+        }
+      }
+    }
+    if (update_p) {
+      Debug("hostdb", "Updating from host file");
+      ParseHostFile(hostdb_hostfile_path);
     }
   }
 
@@ -2177,6 +2328,17 @@ HostDBInfo::hostname()
   return (char *)hostDB.ptr(&data.hostname_offset, hostDB.ptr_to_partition((char *)this));
 }
 
+/*
+ * The perm_hostname exists for all records not just reverse dns records.
+ */
+char *
+HostDBInfo::perm_hostname()
+{
+  if (hostname_offset == 0)
+    return NULL;
+
+  return (char *)hostDB.ptr(&hostname_offset, hostDB.ptr_to_partition((char *)this));
+}
 
 HostDBRoundRobin *
 HostDBInfo::rr()
@@ -2236,13 +2398,17 @@ struct ShowHostDB;
 typedef int (ShowHostDB::*ShowHostDBEventHandler)(int event, Event *data);
 struct ShowHostDB : public ShowCont {
   char *name;
+  uint16_t port;
   IpEndpoint ip;
   bool force;
+  bool output_json;
+  int records_seen;
 
   int
   showMain(int event, Event *e)
   {
     CHECK_SHOW(begin("HostDB"));
+    CHECK_SHOW(show("<a href=\"./showall\">Show all HostDB records<a/><hr>"));
     CHECK_SHOW(show("<form method = GET action = \"./name\">\n"
                     "Lookup by name (e.g. trafficserver.apache.org):<br>\n"
                     "<input type=text name=name size=64 maxlength=256>\n"
@@ -2263,35 +2429,158 @@ struct ShowHostDB : public ShowCont {
   showLookup(int /* event ATS_UNUSED */, Event * /* e ATS_UNUSED */)
   {
     SET_HANDLER(&ShowHostDB::showLookupDone);
-    if (name)
-      hostDBProcessor.getbyname_re(this, name, 0,
-                                   HostDBProcessor::Options().setFlags(force ? HostDBProcessor::HOSTDB_FORCE_DNS_ALWAYS : 0));
-    else
+    if (name) {
+      HostDBProcessor::Options opts;
+      opts.port = port;
+      opts.flags = HostDBProcessor::HOSTDB_DO_NOT_FORCE_DNS;
+      hostDBProcessor.getbynameport_re(this, name, strlen(name), opts);
+    } else
       hostDBProcessor.getbyaddr_re(this, &ip.sa);
     return EVENT_CONT;
   }
 
+  int
+  showAll(int event, Event *e)
+  {
+    if (!output_json) {
+      CHECK_SHOW(begin("HostDB All Records"));
+      CHECK_SHOW(show("<hr>"));
+    } else {
+      CHECK_SHOW(show("["));
+    }
+    SET_HANDLER(&ShowHostDB::showAllEvent);
+    hostDBProcessor.iterate(this);
+    return EVENT_CONT;
+  }
+
+  int
+  showAllEvent(int event, Event *e)
+  {
+    if (event == EVENT_INTERVAL) {
+      HostDBInfo *r = reinterpret_cast<HostDBInfo *>(e);
+      if (output_json && records_seen++ > 0) {
+        CHECK_SHOW(show(",")); // we need to seperate records
+      }
+      showOne(r, false, event, e);
+      if (r->round_robin) {
+        HostDBRoundRobin *rr_data = r->rr();
+        if (rr_data) {
+          if (!output_json) {
+            CHECK_SHOW(show("<table border=1>\n"));
+            CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "Total", rr_data->rrcount));
+            CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "Good", rr_data->good));
+            CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "Current", rr_data->current));
+            CHECK_SHOW(show("</table>\n"));
+          } else {
+            CHECK_SHOW(show(",\"%s\":\"%d\",", "rr_total", rr_data->rrcount));
+            CHECK_SHOW(show("\"%s\":\"%d\",", "rr_good", rr_data->good));
+            CHECK_SHOW(show("\"%s\":\"%d\",", "rr_current", rr_data->current));
+            CHECK_SHOW(show("\"rr_records\":["));
+          }
+
+          for (int i = 0; i < rr_data->rrcount; i++) {
+            showOne(&rr_data->info[i], true, event, e, rr_data);
+            if (output_json) {
+              CHECK_SHOW(show("}")); // we need to seperate records
+              if (i < (rr_data->rrcount - 1))
+                CHECK_SHOW(show(","));
+            }
+          }
+
+          if (!output_json) {
+            CHECK_SHOW(show("<br />\n<br />\n"));
+          } else {
+            CHECK_SHOW(show("]"));
+          }
+        }
+      }
+
+      if (output_json) {
+        CHECK_SHOW(show("}"));
+      }
+
+    } else if (event == EVENT_DONE) {
+      if (output_json) {
+        CHECK_SHOW(show("]"));
+        return completeJson(event, e);
+      } else {
+        return complete(event, e);
+      }
+    } else {
+      ink_assert(!"unexpected event");
+    }
+    return EVENT_CONT;
+  }
+
 
   int
-  showOne(HostDBInfo *r, bool rr, int event, Event *e)
+  showOne(HostDBInfo *r, bool rr, int event, Event *e, HostDBRoundRobin *hostdb_rr = NULL)
   {
     ip_text_buffer b;
-    CHECK_SHOW(show("<table border=1>\n"));
-    CHECK_SHOW(show("<tr><td>%s</td><td>%s%s</td></tr>\n", "Type", r->round_robin ? "Round-Robin" : "",
-                    r->reverse_dns ? "Reverse DNS" : "DNS"));
-    CHECK_SHOW(show("<tr><td>%s</td><td>%u</td></tr>\n", "App1", r->app.allotment.application1));
-    CHECK_SHOW(show("<tr><td>%s</td><td>%u</td></tr>\n", "App2", r->app.allotment.application2));
-    if (!rr) {
-      CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Stale", r->is_ip_stale() ? "Yes" : "No"));
-      CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Timed-Out", r->is_ip_timeout() ? "Yes" : "No"));
-      CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "TTL", r->ip_time_remaining()));
-    }
-    if (r->reverse_dns) {
-      CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Hostname", r->hostname() ? r->hostname() : "<none>"));
+    if (!output_json) {
+      CHECK_SHOW(show("<table border=1>\n"));
+      CHECK_SHOW(show("<tr><td>%s</td><td>%s%s %s</td></tr>\n", "Type", r->round_robin ? "Round-Robin" : "",
+                      r->reverse_dns ? "Reverse DNS" : "", r->is_srv ? "SRV" : "DNS"));
+
+      if (r->perm_hostname()) {
+        CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Hostname", r->perm_hostname()));
+      } else if (rr && r->is_srv && hostdb_rr) {
+        CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Hostname", r->srvname(hostdb_rr)));
+      }
+
+      // Let's display the MD5.
+      CHECK_SHOW(show("<tr><td>%s</td><td>%0.16llx %0.8x %0.8x</td></tr>\n", "MD5 (high, low, low low)", r->md5_high, r->md5_low,
+                      r->md5_low_low));
+      CHECK_SHOW(show("<tr><td>%s</td><td>%u</td></tr>\n", "App1", r->app.allotment.application1));
+      CHECK_SHOW(show("<tr><td>%s</td><td>%u</td></tr>\n", "App2", r->app.allotment.application2));
+      CHECK_SHOW(show("<tr><td>%s</td><td>%u</td></tr>\n", "LastFailure", r->app.http_data.last_failure));
+      if (!rr) {
+        CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Stale", r->is_ip_stale() ? "Yes" : "No"));
+        CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "Timed-Out", r->is_ip_timeout() ? "Yes" : "No"));
+        CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "TTL", r->ip_time_remaining()));
+      }
+
+      if (rr && r->is_srv) {
+        CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "Weight", r->data.srv.srv_weight));
+        CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "Priority", r->data.srv.srv_priority));
+        CHECK_SHOW(show("<tr><td>%s</td><td>%d</td></tr>\n", "Port", r->data.srv.srv_port));
+        CHECK_SHOW(show("<tr><td>%s</td><td>%x</td></tr>\n", "Key", r->data.srv.key));
+      } else if (!r->is_srv) {
+        CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "IP", ats_ip_ntop(r->ip(), b, sizeof b)));
+      }
+
+      CHECK_SHOW(show("</table>\n"));
     } else {
-      CHECK_SHOW(show("<tr><td>%s</td><td>%s</td></tr>\n", "IP", ats_ip_ntop(r->ip(), b, sizeof b)));
+      CHECK_SHOW(show("{"));
+      CHECK_SHOW(show("\"%s\":\"%s%s%s\",", "type", (r->round_robin && !r->is_srv) ? "roundrobin" : "",
+                      r->reverse_dns ? "reversedns" : "", r->is_srv ? "srv" : "dns"));
+
+      if (r->perm_hostname()) {
+        CHECK_SHOW(show("\"%s\":\"%s\",", "hostname", r->perm_hostname()));
+      } else if (rr && r->is_srv && hostdb_rr) {
+        CHECK_SHOW(show("\"%s\":\"%s\",", "hostname", r->srvname(hostdb_rr)));
+      }
+
+      CHECK_SHOW(show("\"%s\":\"%u\",", "app1", r->app.allotment.application1));
+      CHECK_SHOW(show("\"%s\":\"%u\",", "app2", r->app.allotment.application2));
+      CHECK_SHOW(show("\"%s\":\"%u\",", "lastfailure", r->app.http_data.last_failure));
+      if (!rr) {
+        CHECK_SHOW(show("\"%s\":\"%s\",", "stale", r->is_ip_stale() ? "yes" : "no"));
+        CHECK_SHOW(show("\"%s\":\"%s\",", "timedout", r->is_ip_timeout() ? "yes" : "no"));
+        CHECK_SHOW(show("\"%s\":\"%d\",", "ttl", r->ip_time_remaining()));
+      }
+
+      if (rr && r->is_srv) {
+        CHECK_SHOW(show("\"%s\":\"%d\",", "weight", r->data.srv.srv_weight));
+        CHECK_SHOW(show("\"%s\":\"%d\",", "priority", r->data.srv.srv_priority));
+        CHECK_SHOW(show("\"%s\":\"%d\",", "port", r->data.srv.srv_port));
+        CHECK_SHOW(show("\"%s\":\"%x\",", "key", r->data.srv.key));
+      } else if (!r->is_srv) {
+        CHECK_SHOW(show("\"%s\":\"%s\",", "ip", ats_ip_ntop(r->ip(), b, sizeof b)));
+      }
+      // Let's display the MD5.
+      CHECK_SHOW(show("\"%s\":\"%0.16llx %0.8x %0.8x\"", "md5", r->md5_high, r->md5_low, r->md5_low_low));
     }
-    CHECK_SHOW(show("</table>\n"));
     return EVENT_CONT;
   }
 
@@ -2319,11 +2608,11 @@ struct ShowHostDB : public ShowCont {
           CHECK_SHOW(show("</table>\n"));
 
           for (int i = 0; i < rr_data->rrcount; i++)
-            showOne(&rr_data->info[i], true, event, e);
+            showOne(&rr_data->info[i], true, event, e, rr_data);
         }
       }
     } else {
-      if (name) {
+      if (!name) {
         ip_text_buffer b;
         CHECK_SHOW(show("<H2>%s Not Found</H2>\n", ats_ip_ntop(&ip.sa, b, sizeof b)));
       } else {
@@ -2334,7 +2623,7 @@ struct ShowHostDB : public ShowCont {
   }
 
 
-  ShowHostDB(Continuation *c, HTTPHdr *h) : ShowCont(c, h), name(0), force(0)
+  ShowHostDB(Continuation *c, HTTPHdr *h) : ShowCont(c, h), name(0), port(0), force(0), output_json(false), records_seen(0)
   {
     ats_ip_invalidate(&ip);
     SET_HANDLER(&ShowHostDB::showMain);
@@ -2372,9 +2661,25 @@ register_ShowHostDB(Continuation *c, HTTPHdr *h)
     char *gn = NULL;
     if (s->sarg)
       gn = (char *)memchr(s->sarg, '=', strlen(s->sarg));
-    if (gn)
+    if (gn) {
       s->name = gn + 1;
+      char *pos = strstr(s->name, "%3A");
+      if (pos != NULL) {
+        s->port = atoi(pos + 3);
+        *pos = '\0'; // Null terminate name
+      } else {
+        s->port = 0;
+      }
+    }
     SET_CONTINUATION_HANDLER(s, &ShowHostDB::showLookup);
+  } else if (STR_LEN_EQ_PREFIX(path, path_len, "showall")) {
+    int query_len = 0;
+    const char *query = h->url_get()->query_get(&query_len);
+    if (query && query_len && strstr(query, "json")) {
+      s->output_json = true;
+    }
+    Debug("hostdb", "dumping all hostdb records");
+    SET_CONTINUATION_HANDLER(s, &ShowHostDB::showAll);
   }
   this_ethread()->schedule_imm(s);
   return &s->action;
@@ -2512,19 +2817,6 @@ struct HostDBFileContinuation : public Continuation {
 
   HostDBFileContinuation() : Continuation(0) {}
 
-  int insertEvent(int event, void *data);
-  int removeEvent(int event, void *data);
-
-  /// Set the current entry in the HostDB.
-  HostDBInfo *setDBEntry();
-  /// Create and schedule an update continuation.
-  static void scheduleUpdate(int idx,       ///< Pair index to process.
-                             Keys *keys = 0 ///< Key table if any.
-                             );
-  /// Create and schedule a remove continuation.
-  static void scheduleRemove(int idx,   ///< Index of current new key to check against.
-                             Keys *keys ///< new valid keys
-                             );
   /// Finish update
   static void finish(Keys *keys ///< Valid keys from update.
                      );
@@ -2547,21 +2839,7 @@ HostDBFileContinuation::destroy()
 // proceeding at a time in any case so we might as well make these
 // globals.
 int HostDBFileUpdateActive = 0;
-// Contents of the host file. We keep this around because other data
-// points in to it (to minimize allocations).
-ats_scoped_str HostFileText;
-// Accumulated pairs of <IP address, host>
-std::vector<HostFilePair> HostFilePairs;
-// Entries from last update.
-HostDBFileContinuation::Keys HostFileKeys;
-/// Ordering operator for HostFilePair.
-/// We want to group first by name and then by address family.
-bool
-CmpHostFilePair(HostFilePair const &lhs, HostFilePair const &rhs)
-{
-  int zret = strcasecmp(lhs.name, rhs.name);
-  return zret < 0 || (0 == zret && lhs.ip < rhs.ip);
-}
+
 // Actual ordering doesn't matter as long as it's consistent.
 bool
 CmpMD5(INK_MD5 const &lhs, INK_MD5 const &rhs)
@@ -2569,197 +2847,27 @@ CmpMD5(INK_MD5 const &lhs, INK_MD5 const &rhs)
   return lhs[0] < rhs[0] || (lhs[0] == rhs[0] && lhs[1] < rhs[1]);
 }
 
-/// Finish current update.
-void
-HostDBFileContinuation::finish(Keys *keys)
-{
-  HostFilePairs.clear();
-  HostFileKeys.clear();
-  if (keys) {
-    std::swap(*keys, HostFileKeys); // put the new keys in place and dump the previous.
-    delete keys;
-  }
-
-  HostFileText = 0;
-  HostDBFileUpdateActive = 0;
-}
-
-HostDBInfo *
-HostDBFileContinuation::setDBEntry()
-{
-  HostDBInfo *r;
-  uint64_t folded_md5 = fold_md5(md5);
-
-  // remove the old one to prevent buildup
-  if (0 != (r = hostDB.lookup_block(folded_md5, 3))) {
-    hostDB.delete_block(r);
-    Debug("hostdb", "Update host file entry %s - %" PRIx64 ".%" PRIx64, name, md5[0], md5[1]);
-  } else {
-    Debug("hostdb", "Add host file entry %s - %" PRIx64 ".%" PRIx64, name, md5[0], md5[1]);
-  }
-
-  r = hostDB.insert_block(folded_md5, NULL, 0);
-  r->md5_high = md5[1];
-  r->ip_timeout_interval = 0; // special value - no timeout.
-  r->ip_timestamp = hostdb_current_interval;
-  keys->push_back(md5);
-  return r;
-}
-
-int
-HostDBFileContinuation::insertEvent(int, void *)
-{
-  int n = HostFilePairs.size();
-  HostFilePair const &first = HostFilePairs[idx];
-  int last = idx + 1;
-  HostDBInfo *r = 0;
-
-  ink_assert(idx < n);
-
-  // Get the set of addresses for this name.
-  while (last < n && 0 == strcasecmp(first.name, HostFilePairs[last].name) && first.ip.family() == HostFilePairs[last].ip.family())
-    ++last;
-  // Address set is now (idx, last].
-
-  int k = last - idx; // # of addrs for this name
-  ink_assert(k > 0);
-
-  r = this->setDBEntry();
-  r->reverse_dns = false;
-  r->is_srv = false;
-  if (k > 1) {
-    // multiple entries, need round robin
-    int s = HostDBRoundRobin::size(k, false);
-    HostDBRoundRobin *rr_data = static_cast<HostDBRoundRobin *>(hostDB.alloc(&r->app.rr.offset, s));
-    if (rr_data) {
-      int dst = 0; // index of destination RR item.
-      for (int src = idx; src < last; ++src, ++dst) {
-        HostDBInfo &item = rr_data->info[dst];
-        item.data.ip.assign(HostFilePairs[src].ip);
-        item.full = 1;
-        item.round_robin = false;
-        item.reverse_dns = false;
-        item.is_srv = false;
-        item.md5_high = r->md5_high;
-        item.md5_low = r->md5_low;
-        item.md5_low_low = r->md5_low_low;
-      }
-      r->round_robin = true;
-      rr_data->good = k;
-      rr_data->current = 0;
-    }
-  } else {
-    r->data.ip.assign(HostFilePairs[idx].ip);
-    r->round_robin = false;
-  }
-
-  if (last < n) { // more entries to process
-    // We create a new continuation rather than using this one to
-    // avoid mutex issues - it is critical that the continuation use
-    // the bucket mutex to guarantee a data lock for update. It's
-    // unclear if changing the mutex to this continuation will do that
-    // properly. Because we use a class allocator we should achieve
-    // efficient allocations quickly even for large host files.
-    this->scheduleUpdate(last, keys);
-  } else {
-    std::sort(keys->begin(), keys->end(), &CmpMD5);
-    //    keys->qsort(&CmpMD5);
-    // Switch to removing dead entries.
-    this->scheduleRemove(keys->size() - 1, keys);
-  }
-  // This continuation is done.
-  this->destroy();
-  return EVENT_DONE;
-}
-
-int
-HostDBFileContinuation::removeEvent(int, void *)
-{
-  HostDBInfo *r;
-  uint64_t folded_md5 = fold_md5(md5);
-  Debug("hostdb", "Remove host file entry %" PRIx64 ".%" PRIx64, md5[0], md5[1]);
-  if (0 != (r = hostDB.lookup_block(folded_md5, 3)))
-    hostDB.delete_block(r);
-  this->scheduleRemove(idx, keys);
-  this->destroy();
-  return EVENT_DONE;
-}
 
 void
-HostDBFileContinuation::scheduleUpdate(int idx, Keys *keys)
-{
-  HostDBFileContinuation *c = hostDBFileContAllocator.alloc();
-  HostFilePair &pair = HostFilePairs[idx];
-  HostDBMD5 md5;
-
-  md5.set_host(pair.name, strlen(pair.name));
-  md5.db_mark = db_mark_for(pair.ip);
-  md5.refresh();
-
-  SET_CONTINUATION_HANDLER(c, &HostDBFileContinuation::insertEvent);
-  c->idx = idx;
-  c->name = pair.name;
-  c->keys = keys ? keys : new Keys;
-  c->md5 = md5.hash;
-  c->mutex = hostDB.lock_for_bucket(fold_md5(md5.hash) % hostDB.buckets);
-  eventProcessor.schedule_imm(c, ET_CALL, EVENT_IMMEDIATE, 0);
-}
-
-void
-HostDBFileContinuation::scheduleRemove(int idx, Keys *keys)
-{
-  bool targetp = false; // Have a valid target?
-  INK_MD5 md5;
-
-  // See if we have a target.
-  if (HostFileKeys.size() > 0) {
-    md5 = HostFileKeys.back();
-    HostFileKeys.pop_back();
-    targetp = true;
-    // Move backwards through the valid items and the previous
-    // keys, removing if we have a prev key that's bigger than
-    // the current valid key (which means it's not currently valid).
-    while (idx >= 0) {
-      if (CmpMD5((*keys)[idx], md5)) { // prev is not valid, remove
-        break;
-      } else if (CmpMD5(md5, (*keys)[idx])) {
-        --idx; // prev is smaller, skip current and keep checking
-      } else if (!HostFileKeys.empty()) {
-        md5 = HostFileKeys.back();
-        HostFileKeys.pop_back(); // match, move to next potential target
-        --idx;
-      } else { // ran out of things to remove.
-        targetp = false;
-        break;
-      }
-    }
-  }
-
-  if (targetp) {
-    HostDBFileContinuation *c = hostDBFileContAllocator.alloc();
-    SET_CONTINUATION_HANDLER(c, &HostDBFileContinuation::removeEvent);
-    c->md5 = md5;
-    c->idx = idx;
-    c->keys = keys;
-    c->mutex = hostDB.lock_for_bucket(fold_md5(c->md5) % hostDB.buckets);
-    eventProcessor.schedule_imm(c, ET_CALL, EVENT_IMMEDIATE, 0);
-  } else {
-    self::finish(keys);
-  }
-}
-
-void
-ParseHostLine(char *l)
+ParseHostLine(RefCountedHostsFileMap *map, char *l)
 {
   Tokenizer elts(" \t");
   int n_elts = elts.Initialize(l, SHARE_TOKS);
   // Elements should be the address then a list of host names.
   // Don't use RecHttpLoadIp because the address *must* be literal.
-  HostFilePair item;
-  if (n_elts > 1 && 0 == item.ip.load(elts[0]) && !item.ip.isLoopback()) {
+  IpAddr ip;
+  if (n_elts > 1 && 0 == ip.load(elts[0])) {
     for (int i = 1; i < n_elts; ++i) {
-      item.name = elts[i];
-      HostFilePairs.push_back(item);
+      ts::ConstBuffer name(elts[i], strlen(elts[i]));
+      // If we don't have an entry already (host files only support single IPs for a given name)
+      if (map->hosts_file_map.find(name) == map->hosts_file_map.end()) {
+        HostsFileMap::mapped_type &item = map->hosts_file_map[name];
+        item.round_robin = false;
+        item.round_robin_elt = false;
+        item.reverse_dns = false;
+        item.is_srv = false;
+        ats_ip_set(item.ip(), ip);
+      }
     }
   }
 }
@@ -2768,7 +2876,8 @@ ParseHostLine(char *l)
 void
 ParseHostFile(char const *path)
 {
-  bool success = false;
+  Ptr<RefCountedHostsFileMap> parsed_hosts_file_ptr;
+
   // Test and set for update in progress.
   if (0 != ink_atomic_swap(&HostDBFileUpdateActive, 1)) {
     Debug("hostdb", "Skipped load of host file because update already in progress");
@@ -2776,57 +2885,60 @@ ParseHostFile(char const *path)
   }
   Debug("hostdb", "Loading host file '%s'", path);
 
-  ats_scoped_fd fd(open(path, O_RDONLY));
-  if (fd >= 0) {
-    struct stat info;
-    if (0 == fstat(fd, &info)) {
-      // +1 in case no terminating newline
-      int64_t size = info.st_size + 1;
-      HostFileText = static_cast<char *>(ats_malloc(size));
-      if (HostFileText) {
-        char *base = HostFileText;
-        char *limit;
+  if (*path) {
+    ats_scoped_fd fd(open(path, O_RDONLY));
+    if (fd >= 0) {
+      struct stat info;
+      if (0 == fstat(fd, &info)) {
+        // +1 in case no terminating newline
+        int64_t size = info.st_size + 1;
 
-        size = read(fd, HostFileText, info.st_size);
-        limit = HostFileText + size;
-        *limit = 0;
+        parsed_hosts_file_ptr = new RefCountedHostsFileMap;
+        parsed_hosts_file_ptr->HostFileText = static_cast<char *>(ats_malloc(size));
+        if (parsed_hosts_file_ptr->HostFileText) {
+          char *base = parsed_hosts_file_ptr->HostFileText;
+          char *limit;
 
-        // We need to get a list of all name/addr pairs so that we can
-        // group names for round robin records. Also note that the
-        // pairs have pointer back in to the text storage for the file
-        // so we need to keep that until we're done with @a pairs.
-        while (base < limit) {
-          char *spot = strchr(base, '\n');
+          size = read(fd, parsed_hosts_file_ptr->HostFileText, info.st_size);
+          limit = parsed_hosts_file_ptr->HostFileText + size;
+          *limit = 0;
 
-          // terminate the line.
-          if (0 == spot)
-            spot = limit; // no trailing EOL, grab remaining
-          else
-            *spot = 0;
+          // We need to get a list of all name/addr pairs so that we can
+          // group names for round robin records. Also note that the
+          // pairs have pointer back in to the text storage for the file
+          // so we need to keep that until we're done with @a pairs.
+          while (base < limit) {
+            char *spot = strchr(base, '\n');
 
-          while (base < spot && isspace(*base))
-            ++base;                        // skip leading ws
-          if (*base != '#' && base < spot) // non-empty non-comment line
-            ParseHostLine(base);
-          base = spot + 1;
-        }
+            // terminate the line.
+            if (0 == spot)
+              spot = limit; // no trailing EOL, grab remaining
+            else
+              *spot = 0;
 
-        hostdb_hostfile_update_timestamp = hostdb_current_interval;
-        success = true;
-        if (!HostFilePairs.empty()) {
-          // Need to sort by name so multiple address hosts are
-          // contiguous.
-          std::sort(HostFilePairs.begin(), HostFilePairs.end(), &CmpHostFilePair);
-          HostDBFileContinuation::scheduleUpdate(0);
-        } else if (!HostFileKeys.empty()) {
-          HostDBFileContinuation::scheduleRemove(-1, 0);
-        } else {
-          // Nothing in new data, nothing in old data, just clean up.
-          HostDBFileContinuation::finish(0);
+            while (base < spot && isspace(*base))
+              ++base;                        // skip leading ws
+            if (*base != '#' && base < spot) // non-empty non-comment line
+              ParseHostLine(parsed_hosts_file_ptr, base);
+            base = spot + 1;
+          }
+
+          hostdb_hostfile_update_timestamp = hostdb_current_interval;
         }
       }
     }
   }
-  if (!success)
-    HostDBFileUpdateActive = 0;
+
+  // Rotate the host file maps down. We depend on two assumptions here -
+  // 1) Host files are not updated frequently. Even if updated via traffic_ctl that will be at least 5-10 seconds between reloads.
+  // 2) The HostDB clients (essentially the HttpSM) copies the HostDB record or data over to local storage during event processing,
+  //    which means the data only has to be valid until the event chaining rolls back up to the event processor. This will certainly
+  //    be less than 1s
+  // The combination of these means keeping one file back in the rotation is sufficient to keep any outstanding references valid
+  // until
+  // dropped.
+  hostDB.prev_hosts_file_ptr = hostDB.hosts_file_ptr;
+  hostDB.hosts_file_ptr = parsed_hosts_file_ptr;
+  // Mark this one as completed, so we can allow another update to happen
+  HostDBFileUpdateActive = 0;
 }
