@@ -32,13 +32,26 @@
 #include "P_Net.h"
 
 RecRawStatBlock *net_rsb = NULL;
+
+// All in milli-seconds
 int net_config_poll_timeout = -1; // This will get set via either command line or records.config.
+int net_event_period = 10;
+int net_accept_period = 10;
+int net_retry_delay = 10;
+int net_throttle_delay = 50; /* milliseconds */
 
 static inline void
 configure_net(void)
 {
   REC_RegisterConfigUpdateFunc("proxy.config.net.connections_throttle", change_net_connections_throttle, NULL);
   REC_ReadConfigInteger(fds_throttle, "proxy.config.net.connections_throttle");
+
+  REC_EstablishStaticConfigInt32(net_retry_delay, "proxy.config.net.retry_delay");
+  REC_EstablishStaticConfigInt32(net_throttle_delay, "proxy.config.net.throttle_delay");
+
+  // These are not reloadable
+  REC_ReadConfigInteger(net_event_period, "proxy.config.net.event_period");
+  REC_ReadConfigInteger(net_accept_period, "proxy.config.net.accept_period");
 }
 
 
@@ -112,12 +125,12 @@ register_net_stats()
                      (int)inactivity_cop_lock_acquire_failure_stat, RecRawStatSyncSum);
 
   RecRegisterRawStat(net_rsb, RECT_PROCESS, "proxy.process.net.dynamic_keep_alive_timeout_in_total", RECD_INT, RECP_NON_PERSISTENT,
-                     (int)keep_alive_lru_timeout_total_stat, RecRawStatSyncSum);
-  NET_CLEAR_DYN_STAT(keep_alive_lru_timeout_total_stat);
+                     (int)keep_alive_queue_timeout_total_stat, RecRawStatSyncSum);
+  NET_CLEAR_DYN_STAT(keep_alive_queue_timeout_total_stat);
 
   RecRegisterRawStat(net_rsb, RECT_PROCESS, "proxy.process.net.dynamic_keep_alive_timeout_in_count", RECD_INT, RECP_NON_PERSISTENT,
-                     (int)keep_alive_lru_timeout_count_stat, RecRawStatSyncSum);
-  NET_CLEAR_DYN_STAT(keep_alive_lru_timeout_count_stat);
+                     (int)keep_alive_queue_timeout_count_stat, RecRawStatSyncSum);
+  NET_CLEAR_DYN_STAT(keep_alive_queue_timeout_count_stat);
 
   RecRegisterRawStat(net_rsb, RECT_PROCESS, "proxy.process.net.default_inactivity_timeout_applied", RECD_INT, RECP_NON_PERSISTENT,
                      (int)default_inactivity_timeout_stat, RecRawStatSyncSum);
