@@ -33,6 +33,7 @@
 
 #include "ProxyConfig.h"
 #include "SSLSessionCache.h"
+#include "ts/ink_inet.h"
 
 struct SSLCertLookup;
 
@@ -46,6 +47,7 @@ struct SSLCertLookup;
 
 
 typedef void (*init_ssl_ctx_func)(void *, bool);
+typedef void (*load_ssl_file_func)(const char *, unsigned int);
 
 struct SSLConfigParams : public ConfigInfo {
   enum SSL_SESSION_CACHE_MODE {
@@ -92,12 +94,21 @@ struct SSLConfigParams : public ConfigInfo {
   static int ssl_ocsp_cache_timeout;
   static int ssl_ocsp_request_timeout;
   static int ssl_ocsp_update_period;
+  static int ssl_handshake_timeout_in;
 
   static size_t session_cache_number_buckets;
   static size_t session_cache_max_bucket_size;
   static bool session_cache_skip_on_lock_contention;
 
+  // TS-3435 Wiretracing for SSL Connections
+  static int ssl_wire_trace_enabled;
+  static char *ssl_wire_trace_addr;
+  static IpAddr *ssl_wire_trace_ip;
+  static int ssl_wire_trace_percentage;
+  static char *ssl_wire_trace_server_name;
+
   static init_ssl_ctx_func init_ssl_ctx_cb;
+  static load_ssl_file_func load_ssl_file_cb;
 
   void initialize();
   void cleanup();
