@@ -29,8 +29,8 @@
    SSL Configurations
  ****************************************************************************/
 
-#include "libts.h"
-#include "I_Layout.h"
+#include "ts/ink_platform.h"
+#include "ts/I_Layout.h"
 
 #include <string.h>
 #include "P_Net.h"
@@ -48,11 +48,19 @@ bool SSLConfigParams::ssl_ocsp_enabled = false;
 int SSLConfigParams::ssl_ocsp_cache_timeout = 3600;
 int SSLConfigParams::ssl_ocsp_request_timeout = 10;
 int SSLConfigParams::ssl_ocsp_update_period = 60;
+int SSLConfigParams::ssl_handshake_timeout_in = 0;
 size_t SSLConfigParams::session_cache_number_buckets = 1024;
 bool SSLConfigParams::session_cache_skip_on_lock_contention = false;
 size_t SSLConfigParams::session_cache_max_bucket_size = 100;
-
 init_ssl_ctx_func SSLConfigParams::init_ssl_ctx_cb = NULL;
+load_ssl_file_func SSLConfigParams::load_ssl_file_cb = NULL;
+
+// TS-3534 Wiretracing for SSL Connections
+int SSLConfigParams::ssl_wire_trace_enabled = 0;
+char *SSLConfigParams::ssl_wire_trace_addr = NULL;
+IpAddr *SSLConfigParams::ssl_wire_trace_ip = NULL;
+int SSLConfigParams::ssl_wire_trace_percentage = 0;
+char *SSLConfigParams::ssl_wire_trace_server_name = NULL;
 
 static ConfigUpdateHandler<SSLCertificateConfig> *sslCertUpdate;
 
@@ -72,6 +80,7 @@ SSLConfigParams::SSLConfigParams()
   ssl_session_cache_skip_on_contention = 0;
   ssl_session_cache_timeout = 0;
   ssl_session_cache_auto_clear = 1;
+  configExitOnLoadError = 0;
 }
 
 SSLConfigParams::~SSLConfigParams()
@@ -95,6 +104,7 @@ SSLConfigParams::cleanup()
   ats_free_null(cipherSuite);
   ats_free_null(client_cipherSuite);
   ats_free_null(dhparamsFile);
+  ats_free_null(ssl_wire_trace_ip);
 
   clientCertLevel = client_verify_depth = verify_depth = clientVerify = 0;
 }
@@ -255,7 +265,9 @@ SSLConfigParams::initialize()
   SSLConfigParams::session_cache_skip_on_lock_contention = ssl_session_cache_skip_on_contention;
   SSLConfigParams::session_cache_number_buckets = ssl_session_cache_num_buckets;
 
-  session_cache = new SSLSessionCache();
+  if (ssl_session_cache == SSL_SESSION_CACHE_MODE_SERVER_ATS_IMPL) {
+    session_cache = new SSLSessionCache();
+  }
 
   // SSL record size
   REC_EstablishStaticConfigInt32(ssl_maxrecord, "proxy.config.ssl.max_record_size");
@@ -266,6 +278,8 @@ SSLConfigParams::initialize()
   REC_EstablishStaticConfigInt32(ssl_ocsp_request_timeout, "proxy.config.ssl.ocsp.request_timeout");
   REC_EstablishStaticConfigInt32(ssl_ocsp_update_period, "proxy.config.ssl.ocsp.update_period");
 
+  REC_ReadConfigInt32(ssl_handshake_timeout_in, "proxy.config.ssl.handshake_timeout_in");
+
   // ++++++++++++++++++++++++ Client part ++++++++++++++++++++
   client_verify_depth = 7;
   REC_ReadConfigInt32(clientVerify, "proxy.config.ssl.client.verify.server");
@@ -291,6 +305,27 @@ SSLConfigParams::initialize()
   ats_free(ssl_client_ca_cert_filename);
 
   REC_ReadConfigInt32(ssl_allow_client_renegotiation, "proxy.config.ssl.allow_client_renegotiation");
+
+  // SSL Wire Trace configurations
+  REC_ReadConfigInteger(ssl_wire_trace_enabled, "proxy.config.ssl.wire_trace_enabled");
+  if (ssl_wire_trace_enabled) {
+    // wire trace specific source ip
+    REC_ReadConfigStringAlloc(ssl_wire_trace_addr, "proxy.config.ssl.wire_trace_addr");
+    if (ssl_wire_trace_addr) {
+      ssl_wire_trace_ip = new IpAddr();
+      ssl_wire_trace_ip->load(ssl_wire_trace_addr);
+    } else {
+      ssl_wire_trace_ip = NULL;
+    }
+    // wire trace percentage of requests
+    REC_ReadConfigInteger(ssl_wire_trace_percentage, "proxy.config.ssl.wire_trace_percentage");
+    REC_ReadConfigStringAlloc(ssl_wire_trace_server_name, "proxy.config.ssl.wire_trace_server_name");
+  } else {
+    ssl_wire_trace_addr = NULL;
+    ssl_wire_trace_ip = NULL;
+    ssl_wire_trace_percentage = 0;
+    ssl_wire_trace_server_name = NULL;
+  }
 }
 
 void
@@ -325,7 +360,6 @@ SSLCertificateConfig::startup()
 {
   sslCertUpdate = new ConfigUpdateHandler<SSLCertificateConfig>();
   sslCertUpdate->attach("proxy.config.ssl.server.multicert.filename");
-  sslCertUpdate->attach("proxy.config.ssl.server.multicert.exit_on_load_fail");
   sslCertUpdate->attach("proxy.config.ssl.server.ticket_key.filename");
   sslCertUpdate->attach("proxy.config.ssl.server.cert.path");
   sslCertUpdate->attach("proxy.config.ssl.server.private_key.path");
@@ -333,7 +367,7 @@ SSLCertificateConfig::startup()
 
   // Exit if there are problems on the certificate loading and the
   // proxy.config.ssl.server.multicert.exit_on_load_fail is true
-  SSLConfigParams *params = SSLConfig::acquire();
+  SSLConfig::scoped_config params;
   if (!reconfigure() && params->configExitOnLoadError) {
     Error("Problems loading ssl certificate file, %s.  Exiting.", params->configFilePath);
     _exit(1);
@@ -357,10 +391,17 @@ SSLCertificateConfig::reconfigure()
   }
 
   SSLParseCertificateConfiguration(params, lookup);
-  configid = configProcessor.set(configid, lookup);
+
   if (!lookup->is_valid) {
     retStatus = false;
   }
+  // If there are errors in the certificate configs and we had wanted to exit on error
+  // we won't want to reset the config
+  if (lookup->is_valid || !params->configExitOnLoadError) {
+    configid = configProcessor.set(configid, lookup);
+  } else {
+    delete lookup;
+  }
 
   return retStatus;
 }
