@@ -19,10 +19,10 @@
   limitations under the License.
  */
 
-#include "ink_config.h"
+#include "ts/ink_config.h"
 
 #include "P_Net.h"
-#include "I_Layout.h"
+#include "ts/I_Layout.h"
 #include "I_RecHttp.h"
 #include "P_SSLUtils.h"
 #include "P_OCSPStapling.h"
@@ -129,7 +129,7 @@ SSLNetProcessor::allocate_vc(EThread *t)
   SSLNetVConnection *vc;
 
   if (t) {
-    vc = THREAD_ALLOC(sslNetVCAllocator, t);
+    vc = THREAD_ALLOC_INIT(sslNetVCAllocator, t);
   } else {
     if (likely(vc = sslNetVCAllocator.alloc())) {
       vc->from_accept_thread = true;
