@@ -22,6 +22,8 @@
  */
 
 #include "P_Net.h"
+#include "ts/InkErrno.h"
+#include "ts/ink_sock.h"
 
 // For Stat Pages
 #include "StatPages.h"
@@ -156,12 +158,21 @@ UnixNetProcessor::accept_internal(Continuation *cont, int fd, AcceptOptions cons
         Debug("iocore_net_accept", "Created accept thread #%d for port %d", accept_threads, ats_ip_port_host_order(&accept_ip));
         snprintf(thr_name, MAX_THREAD_NAME_LENGTH, "[ACCEPT %d:%d]", accept_threads - 1, ats_ip_port_host_order(&accept_ip));
         na->init_accept_loop(thr_name);
+#if !TS_USE_POSIX_CAP
+      } else if (fd == ts::NO_FD && opt.local_port < 1024 && 0 != geteuid()) {
+        // TS-2054 - we can fail to bind a privileged port if we waited for cache and we tried
+        // to open the socket in do_listen and we're not using libcap (POSIX_CAP) and so have reduced
+        // privilege. Mention this to the admin.
+        Warning("Failed to open reserved port %d due to lack of process privilege. Use POSIX capabilities if possible or disable "
+                "wait_for_cache.",
+                opt.local_port);
+#endif // TS_USE_POSIX_CAP
       }
     } else {
-      na->init_accept_per_thread();
+      na->init_accept_per_thread(opt.f_inbound_transparent);
     }
   } else {
-    na->init_accept();
+    na->init_accept(NULL, opt.f_inbound_transparent);
   }
 
 #ifdef TCP_DEFER_ACCEPT
@@ -213,7 +224,7 @@ UnixNetProcessor::connect_re_internal(Continuation *cont, sockaddr const *target
 
   NET_SUM_GLOBAL_DYN_STAT(net_connections_currently_open_stat, 1);
   vc->id = net_next_connection_number();
-  vc->submit_time = ink_get_hrtime();
+  vc->submit_time = Thread::get_hrtime();
   vc->setSSLClientConnection(true);
   ats_ip_copy(&vc->server_addr, target);
   vc->mutex = cont->mutex;
@@ -452,7 +463,7 @@ UnixNetProcessor::allocate_vc(EThread *t)
   UnixNetVConnection *vc;
 
   if (t) {
-    vc = THREAD_ALLOC(netVCAllocator, t);
+    vc = THREAD_ALLOC_INIT(netVCAllocator, t);
   } else {
     if (likely(vc = netVCAllocator.alloc())) {
       vc->from_accept_thread = true;
