@@ -24,7 +24,7 @@
 #include <cstring>
 #include <vector>
 #include <zlib.h>
-#include <inttypes.h>
+#include <cinttypes>
 #include "atscppapi/TransformationPlugin.h"
 #include "atscppapi/GzipInflateTransformation.h"
 #include "logging_internal.h"
