@@ -20,9 +20,20 @@
  * @file Plugin.cc
  */
 #include "atscppapi/Plugin.h"
-
+#include <ts/ts.h>
 const std::string atscppapi::HOOK_TYPE_STRINGS[] = {
   std::string("HOOK_READ_REQUEST_HEADERS_PRE_REMAP"), std::string("HOOK_READ_REQUEST_HEADERS_POST_REMAP"),
   std::string("HOOK_SEND_REQUEST_HEADERS"), std::string("HOOK_READ_RESPONSE_HEADERS"), std::string("HOOK_SEND_RESPONSE_HEADERS"),
   std::string("HOOK_OS_DNS"), std::string("HOOK_READ_REQUEST_HEADERS"), std::string("HOOK_READ_CACHE_HEADERS"),
   std::string("HOOK_CACHE_LOOKUP_COMPLETE"), std::string("HOOK_SELECT_ALT")};
+
+void
+atscppapi::RegisterGlobalPlugin(std::string name, std::string vendor, std::string email)
+{
+  TSPluginRegistrationInfo info;
+  info.plugin_name = const_cast<char *>(name.c_str());
+  info.vendor_name = const_cast<char *>(vendor.c_str());
+  info.support_email = const_cast<char *>(email.c_str());
+  if (TSPluginRegister(&info) != TS_SUCCESS)
+    TSError("[Plugin.cc] Plugin registration failed.");
+}
