@@ -25,13 +25,11 @@
 #include <cstring>
 #include <map>
 #include <string>
-#include <ts/ts.h>
-#include <ink_memory.h>
+#include "ts/ink_memory.h"
 #include "atscppapi/shared_ptr.h"
 #include "logging_internal.h"
 #include "utils_internal.h"
 #include "atscppapi/noncopyable.h"
-
 using std::map;
 using std::string;
 using namespace atscppapi;
@@ -54,13 +52,20 @@ struct atscppapi::TransactionState : noncopyable {
   TSMBuffer client_response_hdr_buf_;
   TSMLoc client_response_hdr_loc_;
   Response client_response_;
+  TSMBuffer cached_response_hdr_buf_;
+  TSMLoc cached_response_hdr_loc_;
+  Response cached_response_;
+  TSMBuffer cached_request_hdr_buf_;
+  TSMLoc cached_request_hdr_loc_;
+  Request cached_request_;
   map<string, shared_ptr<Transaction::ContextValue> > context_values_;
 
   TransactionState(TSHttpTxn txn, TSMBuffer client_request_hdr_buf, TSMLoc client_request_hdr_loc)
     : txn_(txn), client_request_hdr_buf_(client_request_hdr_buf), client_request_hdr_loc_(client_request_hdr_loc),
       client_request_(txn, client_request_hdr_buf, client_request_hdr_loc), server_request_hdr_buf_(NULL),
       server_request_hdr_loc_(NULL), server_response_hdr_buf_(NULL), server_response_hdr_loc_(NULL), client_response_hdr_buf_(NULL),
-      client_response_hdr_loc_(NULL){};
+      client_response_hdr_loc_(NULL), cached_response_hdr_buf_(NULL), cached_response_hdr_loc_(NULL), cached_request_hdr_buf_(NULL),
+      cached_request_hdr_loc_(NULL){};
 };
 
 Transaction::Transaction(void *raw_txn)
@@ -95,9 +100,65 @@ Transaction::~Transaction()
     LOG_DEBUG("Releasing client response");
     TSHandleMLocRelease(state_->client_response_hdr_buf_, NULL_PARENT_LOC, state_->client_response_hdr_loc_);
   }
+  if (state_->cached_request_hdr_buf_ && state_->cached_request_hdr_loc_) {
+    LOG_DEBUG("Releasing cached request");
+    TSHandleMLocRelease(state_->cached_request_hdr_buf_, NULL_PARENT_LOC, state_->cached_request_hdr_loc_);
+  }
+  if (state_->cached_response_hdr_buf_ && state_->cached_response_hdr_loc_) {
+    LOG_DEBUG("Releasing cached response");
+    TSHandleMLocRelease(state_->cached_response_hdr_buf_, NULL_PARENT_LOC, state_->cached_response_hdr_loc_);
+  }
   delete state_;
 }
 
+bool
+Transaction::configIntSet(TSOverridableConfigKey conf, int value)
+{
+  return TS_SUCCESS == TSHttpTxnConfigIntSet(state_->txn_, conf, static_cast<TSMgmtInt>(value));
+}
+bool
+Transaction::configIntGet(TSOverridableConfigKey conf, int *value)
+{
+  return TS_SUCCESS == TSHttpTxnConfigIntGet(state_->txn_, conf, reinterpret_cast<TSMgmtInt *>(value));
+}
+
+bool
+Transaction::configFloatSet(TSOverridableConfigKey conf, float value)
+{
+  return TS_SUCCESS == TSHttpTxnConfigFloatSet(state_->txn_, conf, static_cast<TSMgmtFloat>(value));
+}
+
+bool
+Transaction::configFloatGet(TSOverridableConfigKey conf, float *value)
+{
+  return TS_SUCCESS == TSHttpTxnConfigFloatGet(state_->txn_, conf, value);
+}
+
+bool
+Transaction::configStringSet(TSOverridableConfigKey conf, std::string const &value)
+{
+  return TS_SUCCESS == TSHttpTxnConfigStringSet(state_->txn_, conf, const_cast<TSMgmtString>(value.data()), value.length());
+}
+
+bool
+Transaction::configStringGet(TSOverridableConfigKey conf, std::string &value)
+{
+  const char *svalue;
+  int length;
+  bool zret = TS_SUCCESS == TSHttpTxnConfigStringGet(state_->txn_, conf, &svalue, &length);
+  if (zret)
+    value.assign(svalue, length);
+  else
+    value.clear();
+  return zret;
+}
+
+bool
+Transaction::configFind(std::string const &name, TSOverridableConfigKey *conf, TSRecordDataType *type)
+{
+  return TS_SUCCESS == TSHttpTxnConfigFind(name.data(), name.length(), conf, type);
+}
+
 void
 Transaction::resume()
 {
@@ -128,7 +189,7 @@ Transaction::setErrorBody(const std::string &page)
 bool
 Transaction::isInternalRequest() const
 {
-  return TSHttpIsInternalRequest(state_->txn_) == TS_SUCCESS;
+  return TSHttpTxnIsInternal(state_->txn_) == TS_SUCCESS;
 }
 
 void *
@@ -192,6 +253,18 @@ Transaction::getClientResponse()
   return state_->client_response_;
 }
 
+Request &
+Transaction::getCachedRequest()
+{
+  return state_->cached_request_;
+}
+
+Response &
+Transaction::getCachedResponse()
+{
+  return state_->cached_response_;
+}
+
 string
 Transaction::getEffectiveUrl()
 {
@@ -215,6 +288,12 @@ Transaction::setCacheUrl(const string &cache_url)
   return (res == TS_SUCCESS);
 }
 
+void
+Transaction::setSkipRemapping(int flag)
+{
+  TSSkipRemappingSet(state_->txn_, flag);
+}
+
 const sockaddr *
 Transaction::getIncomingAddress() const
 {
@@ -303,6 +382,30 @@ Transaction::setTimeout(Transaction::TimeoutType type, int time_ms)
   }
 }
 
+
+Transaction::CacheStatus
+Transaction::getCacheStatus()
+{
+  int obj_status = TS_ERROR;
+
+  if (TSHttpTxnCacheLookupStatusGet(state_->txn_, &obj_status) == TS_ERROR) {
+    return CACHE_LOOKUP_NONE;
+  }
+
+  switch (obj_status) {
+  case TS_CACHE_LOOKUP_MISS:
+    return CACHE_LOOKUP_MISS;
+  case TS_CACHE_LOOKUP_HIT_STALE:
+    return CACHE_LOOKUP_HIT_STALE;
+  case TS_CACHE_LOOKUP_HIT_FRESH:
+    return CACHE_LOOKUP_HIT_FRESH;
+  case TS_CACHE_LOOKUP_SKIPPED:
+    return CACHE_LOOKUP_SKIPED;
+  default:
+    return CACHE_LOOKUP_NONE;
+  }
+}
+
 void
 Transaction::redirectTo(std::string const &url)
 {
@@ -380,3 +483,25 @@ Transaction::initClientResponse()
     state_->client_response_.init(state_->client_response_hdr_buf_, state_->client_response_hdr_loc_);
   }
 }
+
+void
+Transaction::initCachedRequest()
+{
+  static initializeHandles initializeCachedRequestHandles(TSHttpTxnCachedReqGet);
+  if (initializeCachedRequestHandles(state_->txn_, state_->cached_request_hdr_buf_, state_->cached_request_hdr_loc_,
+                                     "cached request")) {
+    LOG_DEBUG("Initializing cached request");
+    state_->cached_request_.init(state_->cached_request_hdr_buf_, state_->cached_request_hdr_loc_);
+  }
+}
+
+void
+Transaction::initCachedResponse()
+{
+  static initializeHandles initializeCachedResponseHandles(TSHttpTxnCachedRespGet);
+  if (initializeCachedResponseHandles(state_->txn_, state_->cached_response_hdr_buf_, state_->cached_response_hdr_loc_,
+                                      "cached response")) {
+    LOG_DEBUG("Initializing cached response");
+    state_->cached_response_.init(state_->cached_response_hdr_buf_, state_->cached_response_hdr_loc_);
+  }
+}
