@@ -23,9 +23,8 @@
 #pragma once
 #ifndef ATSCPPAPI_PLUGININIT_H_
 #define ATSCPPAPI_PLUGININIT_H_
-
+#include <ts/apidefs.h>
 #include <atscppapi/utils.h>
-
 extern "C" {
 
 /**
@@ -36,12 +35,6 @@ extern "C" {
  * @param argv Array of pointers pointing to arguments
  */
 void TSPluginInit(int argc, const char *argv[]);
-
-enum TsReturnCode {
-  TS_ERROR = -1,
-  TS_SUCCESS = 0,
-};
-
 /**
  * Invoked for remap plugins - listed in remap.config. The arguments provided as @pparam
  * in the remap.config line are provided in this invocation.
@@ -52,7 +45,7 @@ enum TsReturnCode {
  * @param errbuf Not used
  * @param errbuf_size Not used
  */
-TsReturnCode TSRemapNewInstance(int argc, char *argv[], void **instance_handle, char *errbuf, int errbuf_size);
+TSReturnCode TSRemapNewInstance(int argc, char *argv[], void **instance_handle, char *errbuf, int errbuf_size);
 }
 
 
