@@ -30,7 +30,7 @@
 #include "atscppapi/shared_ptr.h"
 #include "atscppapi/ClientRequest.h"
 #include "atscppapi/Response.h"
-
+#include <ts/apidefs.h>
 namespace atscppapi
 {
 // forward declarations
@@ -224,6 +224,21 @@ public:
    */
   Response &getClientResponse();
 
+  /**
+   * Returns a Request object which is the cached request
+   *
+   * @return Request object
+   */
+  Request &getCachedRequest();
+
+  /**
+   * Returns a Response object which is the cached response
+   *
+   * @return Response object
+   */
+  Response &getCachedResponse();
+
+
   /**
    * Returns the Effective URL for this transaction taking into account host.
    */
@@ -235,6 +250,12 @@ public:
    */
   bool setCacheUrl(const std::string &);
 
+  /**
+   * Ability to skip the remap phase of the State Machine
+   * This only really makes sense in TS_HTTP_READ_REQUEST_HDR_HOOK
+   */
+  void setSkipRemapping(int);
+
   /**
    * The available types of timeouts you can set on a Transaction.
    */
@@ -254,6 +275,19 @@ public:
    */
   void setTimeout(TimeoutType type, int time_ms);
 
+  /**
+   * Represents different states of an object served out of the cache
+   */
+  enum CacheStatus {
+    CACHE_LOOKUP_MISS = 0,  /**< The object was not found in the cache */
+    CACHE_LOOKUP_HIT_STALE, /**< The object was found in cache but stale */
+    CACHE_LOOKUP_HIT_FRESH, /**< The object was found in cache and was fresh */
+    CACHE_LOOKUP_SKIPED,    /**< Cache lookup was not performed */
+    CACHE_LOOKUP_NONE
+  };
+
+  CacheStatus getCacheStatus();
+
   /**
    * Returns the TSHttpTxn related to the current Transaction
    *
@@ -308,6 +342,14 @@ public:
    */
   void redirectTo(std::string const &url);
 
+  bool configIntSet(TSOverridableConfigKey conf, int value);
+  bool configIntGet(TSOverridableConfigKey conf, int *value);
+  bool configFloatSet(TSOverridableConfigKey conf, float value);
+  bool configFloatGet(TSOverridableConfigKey conf, float *value);
+  bool configStringSet(TSOverridableConfigKey conf, std::string const &value);
+  bool configStringGet(TSOverridableConfigKey conf, std::string &value);
+  bool configFind(std::string const &name, TSOverridableConfigKey *conf, TSRecordDataType *type);
+
 private:
   TransactionState *state_;          //!< The internal TransactionState object tied to the current Transaction
   friend class TransactionPlugin;    //!< TransactionPlugin is a friend so it can call addPlugin()
@@ -341,6 +383,22 @@ private:
    */
   void initClientResponse();
 
+  /**
+   * Used to initialize the Request object for the cache.
+   *
+   * @private
+   */
+
+  void initCachedRequest();
+
+  /**
+   * Used to initialize the Response object for the cache.
+   *
+   * @private
+   */
+
+  void initCachedResponse();
+
   /**
    * Returns a list of TransactionPlugin pointers bound to the current Transaction
    *
