@@ -21,9 +21,9 @@
   limitations under the License.
  */
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "Rollback.h"
-#include "ParseRules.h"
+#include "ts/ParseRules.h"
 #include "P_RecCore.h"
 #include "P_RecLocal.h"
 #include "P_RecMessage.h"
@@ -95,7 +95,7 @@ sync_thr(void *data)
     } else {
       // If we didn't sync to disk, check whether we need to update ....
       if (configFiles->isConfigStale()) {
-        RecSetRecordInt("proxy.node.config.reconfigure_required", 1);
+        RecSetRecordInt("proxy.node.config.reconfigure_required", 1, REC_SOURCE_DEFAULT);
       }
     }
 
@@ -114,16 +114,16 @@ config_update_thr(void * /* data */)
   while (true) {
     switch (RecExecConfigUpdateCbs(REC_LOCAL_UPDATE_REQUIRED)) {
     case RECU_RESTART_TS:
-      RecSetRecordInt("proxy.node.config.restart_required.proxy", 1);
+      RecSetRecordInt("proxy.node.config.restart_required.proxy", 1, REC_SOURCE_DEFAULT);
       break;
     case RECU_RESTART_TM:
-      RecSetRecordInt("proxy.node.config.restart_required.proxy", 1);
-      RecSetRecordInt("proxy.node.config.restart_required.manager", 1);
+      RecSetRecordInt("proxy.node.config.restart_required.proxy", 1, REC_SOURCE_DEFAULT);
+      RecSetRecordInt("proxy.node.config.restart_required.manager", 1, REC_SOURCE_DEFAULT);
       break;
     case RECU_RESTART_TC:
-      RecSetRecordInt("proxy.node.config.restart_required.proxy", 1);
-      RecSetRecordInt("proxy.node.config.restart_required.manager", 1);
-      RecSetRecordInt("proxy.node.config.restart_required.cop", 1);
+      RecSetRecordInt("proxy.node.config.restart_required.proxy", 1, REC_SOURCE_DEFAULT);
+      RecSetRecordInt("proxy.node.config.restart_required.manager", 1, REC_SOURCE_DEFAULT);
+      RecSetRecordInt("proxy.node.config.restart_required.cop", 1, REC_SOURCE_DEFAULT);
       break;
     case RECU_NULL:
     case RECU_DYNAMIC:
@@ -209,9 +209,14 @@ RecLocalInitMessage()
 int
 RecLocalStart(FileManager *configFiles)
 {
-  ink_thread_create(sync_thr, configFiles);
-  ink_thread_create(config_update_thr, NULL);
-
+  RecInt disable_modification = 0;
+  RecGetRecordInt("proxy.config.disable_configuration_modification", &disable_modification);
+  if (disable_modification == 1) {
+    RecDebug(DL_Debug, "Disable configuration modification");
+  } else {
+    ink_thread_create(sync_thr, configFiles);
+    ink_thread_create(config_update_thr, NULL);
+  }
   return REC_ERR_OKAY;
 }
 
