@@ -21,8 +21,10 @@
   limitations under the License.
  */
 
-#include "libts.h"
-#include "TextBuffer.h"
+#include <stdarg.h>
+#include "ts/ink_platform.h"
+#include "ts/ink_memory.h"
+#include "ts/TextBuffer.h"
 
 /****************************************************************************
  *
