@@ -25,7 +25,7 @@
 #define _ink_defs_h
 
 
-#include "ink_config.h"
+#include "ts/ink_config.h"
 #include <stddef.h>
 #include <sys/mman.h>
 
