@@ -21,8 +21,14 @@
   limitations under the License.
  */
 
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_defs.h"
+#include "ts/ink_inet.h"
+#include "ts/ParseRules.h"
+#include "ts/ink_code.h"
+#include "ts/ink_assert.h"
 #include "ts/TestBox.h"
+#include "ts/TextBuffer.h"
 
 #if defined(darwin)
 extern "C" {
@@ -629,3 +635,35 @@ REGRESSION_TEST(Ink_Inet)(RegressionTest *t, int /* atype */, int *pstatus)
     }
   }
 }
+
+int
+ats_tcp_somaxconn()
+{
+  int fd;
+  int value = 0;
+
+/* Darwin version ... */
+#if HAVE_SYSCTLBYNAME
+  if (sysctlbyname("kern.ipc.somaxconn", NULL, NULL, &value, sizeof(value)) == 0) {
+    return value;
+  }
+#endif
+
+  fd = open("/proc/sys/net/ipv4/tcp_max_syn_backlog", O_RDONLY);
+  if (fd != -1) {
+    textBuffer text(0);
+    text.slurp(fd);
+    if (!text.empty()) {
+      value = strtoul(text.bufPtr(), NULL, 10);
+    }
+    close(fd);
+  }
+
+  // Default to the compatible value we used before detection. SOMAXCONN is the right
+  // macro to use, but most systems set this to 128, which is just too small.
+  if (value <= 0) {
+    return 1024;
+  }
+
+  return value;
+}
