@@ -69,8 +69,9 @@
 #ifndef _ink_resolver_h_
 #define _ink_resolver_h_
 
-#include "ink_platform.h"
-#include <ts/ink_inet.h>
+#include "ts/ink_platform.h"
+#include "ts/ink_inet.h"
+
 #include <resolv.h>
 #include <arpa/nameser.h>
 
@@ -265,7 +266,7 @@ struct ts_imp_res_state {
 };
 typedef ts_imp_res_state *ink_res_state;
 
-int ink_res_init(ink_res_state, IpEndpoint const *pHostList, size_t pHostListSize, const char *pDefDomain = NULL,
+int ink_res_init(ink_res_state, IpEndpoint const *pHostList, size_t pHostListSize, int dnsSearch, const char *pDefDomain = NULL,
                  const char *pSearchList = NULL, const char *pResolvConf = NULL);
 
 int ink_res_mkquery(ink_res_state, int, const char *, int, int, const unsigned char *, int, const unsigned char *, unsigned char *,
