@@ -27,7 +27,10 @@
 
 
 ***************************************************************************/
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_assert.h"
+#include "ts/ink_string.h"
+#include "ts/ink_inet.h"
 
 //
 // Compilation options
