@@ -24,8 +24,8 @@
 
 #include <stdint.h>
 #include <stdio.h>
-#include <ink_assert.h>
-#include "Vec.h"
+#include "ts/ink_assert.h"
+#include "ts/Vec.h"
 
 static void
 test_append()
@@ -123,10 +123,66 @@ test_basic()
   ink_assert(uf.find(1) == uf.find(3));
 }
 
+static bool
+compare(void *a, void *b)
+{
+  return a < b;
+}
+
+static void
+test_sort()
+{
+  Vec<void *> v;
+  for (long i = 1; i <= 1000; ++i)
+    v.add(reinterpret_cast<void *>(static_cast<intptr_t>(((i * 149) % 1000) + 1)));
+  v.qsort(&compare);
+  for (int i = 0; i < 1000; ++i)
+    ink_assert(reinterpret_cast<void *>(static_cast<intptr_t>(i + 1)) == v[i]);
+
+
+  v.clear();
+  for (long i = 1; i <= 1000000; ++i) {
+    v.add(reinterpret_cast<void *>(static_cast<intptr_t>(((i * 51511) % 1000000) + 1)));
+  }
+  v.qsort(&compare);
+
+  for (long i = 0; i < 1000000; ++i) {
+    ink_assert(reinterpret_cast<void *>(static_cast<intptr_t>(i + 1)) == v[i]);
+  }
+
+  v.clear();
+  for (long i = 1; i <= 1000000; ++i) {
+    // This should be every number 1..500000 twice.
+    v.add(reinterpret_cast<void *>(static_cast<intptr_t>(((i * 199999) % 500000) + 1)));
+  }
+  v.qsort(&compare);
+
+  for (long i = 0; i < 1000000; ++i) {
+    ink_assert(reinterpret_cast<void *>(static_cast<intptr_t>((i / 2) + 1)) == v[i]);
+  }
+
+  // Very long array, already sorted. This is what broke before.
+  v.clear();
+  for (long i = 1; i <= 10000000; ++i)
+    v.add(reinterpret_cast<void *>(static_cast<intptr_t>(i)));
+  v.qsort(&compare);
+  for (long i = 0; i < 10000000; ++i)
+    ink_assert(reinterpret_cast<void *>(static_cast<intptr_t>(i + 1)) == v[i]);
+
+  // very long, reverse sorted.
+  v.clear();
+  for (long i = 10000000; i >= 1; --i)
+    v.add(reinterpret_cast<void *>(static_cast<intptr_t>(i)));
+  v.qsort(&compare);
+  for (long i = 0; i < 10000000; ++i)
+    ink_assert(reinterpret_cast<void *>(static_cast<intptr_t>(i + 1)) == v[i]);
+}
+
 int
 main(int /* argc ATS_UNUSED */, char ** /* argv ATS_UNUSED */)
 {
   test_append();
   test_basic();
+  test_sort();
   printf("test_Vec PASSED\n");
 }
