@@ -24,8 +24,8 @@
 #include <errno.h>
 #include <openssl/md5.h>
 #include <TsException.h>
-#include "ink_memory.h"
-#include "ink_string.h"
+#include "ts/ink_memory.h"
+#include "ts/ink_string.h"
 
 namespace wccp
 {
