@@ -22,12 +22,15 @@
  */
 
 
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_string.h"
+#include "ts/ink_file.h"
+#include "ts/ink_time.h"
 #include "LocalManager.h"
 #include "ClusterCom.h"
 #include "MgmtUtils.h"
 #include "Alarms.h"
-#include "Diags.h"
+#include "ts/Diags.h"
 
 #include "P_RecCore.h"
 
@@ -71,8 +74,8 @@ Alarms::Alarms()
 Alarms::~Alarms()
 {
   ink_hash_table_destroy(cblist);
-  ink_hash_table_destroy_and_xfree_values(local_alarms);
-  ink_hash_table_destroy_and_xfree_values(remote_alarms);
+  ink_hash_table_destroy_and_free_values(local_alarms);
+  ink_hash_table_destroy_and_free_values(remote_alarms);
   ink_mutex_destroy(&mutex);
 } /* End Alarms::Alarms */
 
@@ -410,6 +413,7 @@ Alarms::constructAlarmMessage(const AppVersionInfo &version, char *ip, char *mes
       if (max >= 1) {
         message[0] = '\0';
       }
+      ink_mutex_release(&mutex);
       return;
     }
     ink_strlcpy(&message[n], "alarm: none\n", max - n);
