@@ -101,8 +101,6 @@ public:
     internal_ticker = 0;
   }
 
-  void syslogThrInit();
-
   volatile bool run_proxy;
   volatile time_t manager_started_at;
   volatile time_t proxy_started_at;
@@ -128,8 +126,6 @@ public:
   volatile int watched_process_fd;
   volatile pid_t proxy_launch_pid;
 
-  int mgmt_sync_key;
-
   Alarms *alarm_keeper;
   VMap *virt_map;
   FileManager *configFiles;
