@@ -21,20 +21,19 @@
   limitations under the License.
  */
 
-#include "libts.h"
-
-#include "ink_platform.h"
-#include "ink_defs.h"
-#include "ink_assert.h"
-#include "ink_error.h"
-#include "ink_file.h"
-#include "ink_string.h"
-#include "ink_time.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_memory.h"
+#include "ts/ink_defs.h"
+#include "ts/ink_assert.h"
+#include "ts/ink_error.h"
+#include "ts/ink_file.h"
+#include "ts/ink_string.h"
+#include "ts/ink_time.h"
 
 #include "MgmtUtils.h"
 #include "MultiFile.h"
 #include "ExpandingArray.h"
-#include "TextBuffer.h"
+#include "ts/TextBuffer.h"
 #include "WebMgmtUtils.h"
 
 /****************************************************************************
@@ -126,6 +125,7 @@ MultiFile::WalkFiles(ExpandingArray *fileList)
     } else {
       if (stat(records_config_filePath, &records_config_fileInfo) < 0) {
         delete[] filePath;
+        delete[] records_config_filePath;
         continue;
       }
       // Ignore ., .., and any dot files
