@@ -21,7 +21,7 @@
   limitations under the License.
  */
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "ProxyConfig.h"
 #include "P_EventSystem.h"
 #include "ts/TestBox.h"
@@ -154,6 +154,9 @@ ConfigProcessor::set(unsigned int id, ConfigInfo *info, unsigned timeout_secs)
     old_info = infos[idx];
   } while (!ink_atomic_cas(&infos[idx], old_info, info));
 
+  Debug("config", "Set for slot %d 0x%" PRId64 " was 0x%" PRId64 " with ref count %d", id, (int64_t)info, (int64_t)old_info,
+        (old_info) ? old_info->refcount() : 0);
+
   if (old_info) {
     // The ConfigInfoReleaser now takes our refcount, but
     // someother thread might also have one ...
@@ -204,6 +207,7 @@ ConfigProcessor::release(unsigned int id, ConfigInfo *info)
 
   if (info->refcount_dec() == 0) {
     // When we release, we should already have replaced this object in the index.
+    Debug("config", "Release config %d 0x%" PRId64, id, (int64_t)info);
     ink_release_assert(info != this->infos[idx]);
     delete info;
   }
