@@ -93,13 +93,13 @@
  * reset_stats - resets all statistics to default values
  */
 
-#include "ink_config.h"
-#include "ink_defs.h"
+#include "ts/ink_config.h"
+#include "ts/ink_defs.h"
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <strings.h>
-#include "ink_string.h"
+#include "ts/ink_string.h"
 
 #include "mgmtapi.h"
 #include "CfgContextUtils.h"
@@ -758,20 +758,6 @@ print_storage_ele(TSStorageEle *ele)
     printf("pathname=%s, size=%d\n", ele->pathname, ele->size);
 }
 
-void
-print_update_ele(TSUpdateEle *ele)
-{
-  if (!ele) {
-    printf("can't print ele\n");
-    return;
-  }
-
-  printf("url: %s\n", ele->url);
-  printf("headers: ");
-  print_string_list(ele->headers);
-  printf("%d: %d: %d\n", ele->offset_hour, ele->interval, ele->recursion_depth);
-}
-
 void
 print_vaddrs_ele(TSVirtIpAddrEle *ele)
 {
@@ -841,11 +827,9 @@ print_ele_list(TSFileNameT file, TSCfgContext ctx)
     case TS_FNAME_STORAGE:
       print_storage_ele((TSStorageEle *)ele);
       break;
-    case TS_FNAME_UPDATE_URL:
-      print_update_ele((TSUpdateEle *)ele);
-      break;
     case TS_FNAME_VADDRS:
       print_vaddrs_ele((TSVirtIpAddrEle *)ele);
+      break;
     default:
       printf("[print_ele_list] invalid file type \n");
       return;
@@ -1548,8 +1532,6 @@ test_cfg_context_get(char *args)
     file = TS_FNAME_STORAGE;
   } else if (strcmp(name, "splitdns.config") == 0) {
     file = TS_FNAME_SPLIT_DNS;
-  } else if (strcmp(name, "update.config") == 0) {
-    file = TS_FNAME_UPDATE_URL;
   } else if (strcmp(name, "vaddrs.config") == 0) {
     file = TS_FNAME_VADDRS;
   } else {
@@ -1617,8 +1599,6 @@ test_cfg_context_move(char *args)
     file = TS_FNAME_STORAGE;
   } else if (strcmp(name, "splitdns.config") == 0) {
     file = TS_FNAME_SPLIT_DNS;
-  } else if (strcmp(name, "update.config") == 0) {
-    file = TS_FNAME_UPDATE_URL;
   } else if (strcmp(name, "vaddrs.config") == 0) {
     file = TS_FNAME_VADDRS;
   } else {
