@@ -22,8 +22,8 @@
  */
 
 
-#include "libts.h"
-#include "ink_platform.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_string.h"
 
 #include "CfgContextImpl.h"
 #include "CfgContextUtils.h"
@@ -2279,145 +2279,6 @@ StorageObj::getCfgEleCopy()
   return (TSCfgEle *)copy_storage_ele(m_ele);
 }
 
-//--------------------------------------------------------------------------
-// UpdateObj
-//--------------------------------------------------------------------------
-UpdateObj::UpdateObj(TSUpdateEle *ele)
-{
-  m_ele = ele;
-  m_valid = true;
-  m_valid = isValid();
-}
-
-UpdateObj::UpdateObj(TokenList *tokens)
-{
-  Token *token;
-
-  m_ele = TSUpdateEleCreate();
-  m_ele->cfg_ele.error = TS_ERR_OKAY;
-  m_valid = true;
-
-  if (!tokens || tokens->length < 5) {
-    goto FORMAT_ERR;
-  }
-
-  m_ele->cfg_ele.type = get_rule_type(tokens, TS_FNAME_UPDATE_URL);
-  if (m_ele->cfg_ele.type == TS_TYPE_UNDEFINED) {
-    goto FORMAT_ERR;
-  }
-  // URL
-  token = tokens->first();
-  if (strcmp(token->name, "") == 0) {
-    goto FORMAT_ERR;
-  }
-  m_ele->url = ats_strdup(token->name);
-
-  // Request_headers
-  token = tokens->next(token);
-  m_ele->headers = string_to_string_list(token->name, ";");
-
-  // Offset_hour
-  token = tokens->next(token);
-  if (strcmp(token->name, "") == 0) {
-    goto FORMAT_ERR;
-  }
-  m_ele->offset_hour = ink_atoi(token->name);
-
-  // Interval
-  token = tokens->next(token);
-  if (strcmp(token->name, "") == 0) {
-    goto FORMAT_ERR;
-  }
-  m_ele->interval = ink_atoi(token->name);
-
-  // Recursion_depth
-  token = tokens->next(token);
-  if (strcmp(token->name, "") == 0) {
-    goto FORMAT_ERR;
-  }
-  m_ele->recursion_depth = ink_atoi(token->name);
-  return;
-
-FORMAT_ERR:
-  m_ele->cfg_ele.error = TS_ERR_INVALID_CONFIG_RULE;
-  m_valid = false;
-}
-
-UpdateObj::~UpdateObj()
-{
-  TSUpdateEleDestroy(m_ele);
-}
-
-char *
-UpdateObj::formatEleToRule()
-{
-  if (!isValid()) {
-    m_ele->cfg_ele.error = TS_ERR_INVALID_CONFIG_RULE;
-    return NULL;
-  }
-
-  char *list_str;
-  char buf[MAX_RULE_SIZE];
-  memset(buf, 0, MAX_RULE_SIZE);
-
-  list_str = string_list_to_string(m_ele->headers, ";");
-  if (list_str) {
-    snprintf(buf, sizeof(buf), "%s\\%s\\%d\\%d\\%d\\", m_ele->url, list_str, m_ele->offset_hour, m_ele->interval,
-             m_ele->recursion_depth);
-    ats_free(list_str);
-  } else {
-    snprintf(buf, sizeof(buf), "%s\\\\%d\\%d\\%d\\", m_ele->url, m_ele->offset_hour, m_ele->interval, m_ele->recursion_depth);
-  }
-
-  return ats_strdup(buf);
-}
-
-bool
-UpdateObj::isValid()
-{
-  if (m_ele->cfg_ele.error != TS_ERR_OKAY) {
-    m_valid = false;
-  }
-  // check url
-  if (!m_ele->url || strcmp(m_ele->url, "") == 0 || strstr(m_ele->url, "\\") || (!strstr(m_ele->url, "http"))) {
-    m_valid = false;
-  }
-  // bug 49322: check that there are no "\" in the url or headers
-  char *list_str = string_list_to_string(m_ele->headers, ";");
-  if (list_str) {
-    if (strstr(list_str, "\\"))
-      m_valid = false;
-    ats_free(list_str);
-  }
-  // offset hour range is 00-23
-  if (m_ele->offset_hour < 0 || m_ele->offset_hour > 23)
-    m_valid = false;
-
-  if (m_ele->interval < 0)
-    m_valid = false;
-
-  // optional - default is 0
-  if (m_ele->recursion_depth < 0) {
-    m_valid = false;
-  }
-  // recursion depth can only be specified for http
-  if (m_ele->recursion_depth > 0) {
-    if (!m_ele->url || !strstr(m_ele->url, "http")) {
-      m_valid = false;
-    }
-  }
-
-  if (!m_valid)
-    m_ele->cfg_ele.error = TS_ERR_INVALID_CONFIG_RULE;
-
-  return m_valid;
-}
-
-TSCfgEle *
-UpdateObj::getCfgEleCopy()
-{
-  return (TSCfgEle *)copy_update_ele(m_ele);
-}
 
 //--------------------------------------------------------------------------
 // VirtIpAddrObj
