@@ -29,15 +29,17 @@
  *
  ***************************************************************************/
 
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_file.h"
+#include "ts/ParseRules.h"
 #include "MgmtUtils.h"
 #include "LocalManager.h"
 #include "ClusterCom.h"
 #include "FileManager.h"
 #include "Rollback.h"
 #include "WebMgmtUtils.h"
-#include "Diags.h"
-#include "ink_hash_table.h"
+#include "ts/Diags.h"
+#include "ts/ink_hash_table.h"
 #include "ExpandingArray.h"
 //#include "I_AccCrypto.h"
 
@@ -45,8 +47,8 @@
 #include "CoreAPIShared.h"
 #include "CfgContextUtils.h"
 #include "EventCallback.h"
-#include "I_Layout.h"
-#include "ink_cap.h"
+#include "ts/I_Layout.h"
+#include "ts/ink_cap.h"
 
 // global variable
 CallbackTable *local_event_callbacks;
@@ -360,7 +362,7 @@ ServerBacktrace(unsigned /* options */, char **trace)
   // Unfortunately, we need to be privileged here. We either need to be root or to be holding
   // the CAP_SYS_PTRACE capability. Even though we are the parent traffic_manager, it is not
   // traceable without privilege because the process credentials do not match.
-  ElevateAccess access(true, ElevateAccess::TRACE_PRIVILEGE);
+  ElevateAccess access(ElevateAccess::TRACE_PRIVILEGE);
   threadlist threads(threads_for_process(lmgmt->watched_process_pid));
   textBuffer text(0);
 
@@ -413,8 +415,8 @@ Reconfigure()
 {
   configFiles->rereadConfig();                              // TM rereads
   lmgmt->signalEvent(MGMT_EVENT_PLUGIN_CONFIG_UPDATE, "*"); // TS rereads
-  RecSetRecordInt("proxy.node.config.reconfigure_time", time(NULL));
-  RecSetRecordInt("proxy.node.config.reconfigure_required", 0);
+  RecSetRecordInt("proxy.node.config.reconfigure_time", time(NULL), REC_SOURCE_DEFAULT);
+  RecSetRecordInt("proxy.node.config.reconfigure_required", 0, REC_SOURCE_DEFAULT);
 
   return TS_ERR_OKAY;
 }
@@ -563,6 +565,12 @@ MgmtConfigRecordDescribe(const char * /* rec_name */, unsigned /* flags */, TSCo
   return TS_ERR_NOT_SUPPORTED;
 }
 
+TSMgmtError
+MgmtConfigRecordDescribeMatching(const char *, unsigned, TSList)
+{
+  return TS_ERR_NOT_SUPPORTED;
+}
+
 /*-------------------------------------------------------------------------
  * reads the RecordsConfig info to determine which type of action is needed
  * when the record rec_name is changed; if the rec_name is invalid,
