@@ -21,7 +21,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
  */
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_sock.h"
+#include "ts/ink_string.h"
+#include "ts/ink_memory.h"
 
 #include "CoreAPIShared.h"
 #include "MgmtSocket.h"
