@@ -21,7 +21,8 @@
   limitations under the License.
  */
 
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_string.h"
 #include "GenericParser.h"
 
 /***************************************************************************
@@ -216,8 +217,6 @@ Rule::parse(const char *const_rule, TSFileNameT filetype)
     return splitdnsParse(rule);
   case TS_FNAME_STORAGE: /* storage.config */
     return storageParse(rule);
-  case TS_FNAME_UPDATE_URL: /* update.config */
-    return updateParse(rule);
   case TS_FNAME_VADDRS: /* vaddrs.config */
     return vaddrsParse(rule);
   default:
@@ -624,6 +623,7 @@ Rule::socksParse(char *rule)
         // Every token must have a '=' sign
         if (numSubRuleTok < 2) {
           setErrorHint("'=' is expected in space-delimited token");
+          delete m_tokenList;
           return NULL;
         }
 
@@ -943,8 +943,6 @@ RuleList::parse(char *fileBuf, const char *filename)
     m_filetype = TS_FNAME_SOCKS; /* socks.config */
   } else if (strstr(filename, "splitdns.config")) {
     m_filetype = TS_FNAME_SPLIT_DNS; /* splitdns.config */
-  } else if (strstr(filename, "update.config")) {
-    m_filetype = TS_FNAME_UPDATE_URL; /* update.config */
   } else if (strstr(filename, "vaddrs.config")) {
     m_filetype = TS_FNAME_VADDRS; /* vaddrs.config */
   } else if (strstr(filename, "plugin.config")) {
