@@ -30,9 +30,9 @@
  *
  ***************************************************************************/
 
-#include "ink_platform.h"
-#include "ink_sock.h"
-#include "Diags.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_sock.h"
+#include "ts/Diags.h"
 #include "MgmtUtils.h"
 #include "MgmtSocket.h"
 #include "MgmtMarshall.h"
