@@ -50,6 +50,9 @@ extern "C" {
 #define TS_RES_MEM_PATH TS_RES_PATH("memory/")
 #endif
 
+#define TM_OPT_BIND_STDOUT "bind_stdout"
+#define TM_OPT_BIND_STDERR "bind_stderr"
+
 /***************************************************************************
  * Error and Return Values
  ***************************************************************************/
@@ -313,7 +316,6 @@ typedef enum {
   TS_FNAME_SOCKS,           /* socks.config */
   TS_FNAME_SPLIT_DNS,       /* splitdns.config */
   TS_FNAME_STORAGE,         /* storage.config */
-  TS_FNAME_UPDATE_URL,      /* update.config */
   TS_FNAME_VADDRS,          /* vaddrs.config */
   TS_FNAME_VSCAN,           /* vscan.config */
   TS_FNAME_VS_TRUSTED_HOST, /* trusted-host.config */
@@ -354,10 +356,9 @@ typedef enum {
   TS_SOCKS_BYPASS, /* socks.config */
   TS_SOCKS_AUTH,
   TS_SOCKS_MULTIPLE,
-  TS_SPLIT_DNS,  /* splitdns.config */
-  TS_STORAGE,    /* storage.config */
-  TS_UPDATE_URL, /* update.config */
-  TS_VADDRS,     /* vaddrs.config */
+  TS_SPLIT_DNS, /* splitdns.config */
+  TS_STORAGE,   /* storage.config */
+  TS_VADDRS,    /* vaddrs.config */
   TS_TYPE_UNDEFINED,
   TS_TYPE_COMMENT /* for internal use only */
 } TSRuleTypeT;
@@ -399,6 +400,7 @@ typedef union {/* record value */
 
 typedef struct {
   char *rec_name;        /* record name */
+  TSInt rec_class;       /* record class (RecT) */
   TSRecordT rec_type;    /* record type {TS_REC_INT...} */
   TSRecordValueT valueT; /* record value */
 } TSRecordEle;
@@ -419,12 +421,18 @@ typedef struct {
   TSInt rec_update;     /* update_required bitmask */
   TSInt rec_updatetype; /* update type (RecUpdateT) */
   TSInt rec_checktype;  /* syntax check type (RecCheckT) */
+  TSInt rec_source;     /* source of data */
   char *rec_checkexpr;  /* syntax check expression */
 } TSConfigRecordDescription;
 
 /* Free (the contents of) a TSConfigRecordDescription */
 tsapi void TSConfigRecordDescriptionFree(TSConfigRecordDescription *val);
 
+/* Heap-allocate a TSConfigRecordDescription. */
+tsapi TSConfigRecordDescription *TSConfigRecordDescriptionCreate(void);
+/* Free and destroy a heap-allocated TSConfigRecordDescription. */
+tsapi void TSConfigRecordDescriptionDestroy(TSConfigRecordDescription *);
+
 /*--- events --------------------------------------------------------------*/
 
 /* Note: Each event has a format String associated with it from which the
@@ -678,16 +686,6 @@ typedef struct {
   int size;       /* size of the named pathname (in bytes); optional if raw disk partitions */
 } TSStorageEle;
 
-/* update.config */
-typedef struct {
-  TSCfgEle cfg_ele;
-  char *url;            /* url to update (HTTP based URLs) */
-  TSStringList headers; /* list of headers, separated by semicolons (can be NULL) */
-  int offset_hour;      /* offset hour to start update; must be 00-23 hrs  */
-  int interval;         /* in secs, frequency of updates starting at offset_hour */
-  int recursion_depth;  /* starting at given URL, the depth to which referenced URLs are recursively updated */
-} TSUpdateEle;
-
 /* vaddrs.config */
 typedef struct {
   TSCfgEle cfg_ele;
@@ -848,8 +846,6 @@ tsapi TSSplitDnsEle *TSSplitDnsEleCreate();
 tsapi void TSSplitDnsEleDestroy(TSSplitDnsEle *ele);
 tsapi TSStorageEle *TSStorageEleCreate();
 tsapi void TSStorageEleDestroy(TSStorageEle *ele);
-tsapi TSUpdateEle *TSUpdateEleCreate();
-tsapi void TSUpdateEleDestroy(TSUpdateEle *ele);
 tsapi TSVirtIpAddrEle *TSVirtIpAddrEleCreate();
 tsapi void TSVirtIpAddrEleDestroy(TSVirtIpAddrEle *ele);
 /*--- Ele helper operations -------------------------------------*/
@@ -946,7 +942,7 @@ tsapi TSMgmtError TSActionDo(TSActionNeedT action);
  */
 tsapi TSMgmtError TSBounce(unsigned options);
 
-/* TSStorageDeviceOp: Request an operation on a storage device.
+/* TSStorageDeviceCmdOffline: Request to make a cache storage device offline.
  * @arg dev Target device, specified by path to device.
  * @return Success.
  */
@@ -1120,6 +1116,7 @@ tsapi TSMgmtError TSRecordSetString(const char *rec_name, const char *string_val
  * Output: TSMgmtError
  */
 tsapi TSMgmtError TSConfigRecordDescribe(const char *rec_name, unsigned flags, TSConfigRecordDescription *val);
+tsapi TSMgmtError TSConfigRecordDescribeMatchMlt(const char *rec_regex, unsigned flags, TSList list);
 
 /* TSRecordSetMlt: sets a set of records
  * Input:  rec_list     - list of record names the user wants to set;
