@@ -24,7 +24,7 @@
 #ifndef _MGMT_SOCKET_H_
 #define _MGMT_SOCKET_H_
 
-#include "ink_platform.h"
+#include "ts/ink_platform.h"
 
 //-------------------------------------------------------------------------
 // defines
@@ -46,7 +46,7 @@ bool mgmt_transient_error();
 // mgmt_accept
 //-------------------------------------------------------------------------
 
-int mgmt_accept(int s, struct sockaddr *addr, int *addrlen);
+int mgmt_accept(int s, struct sockaddr *addr, socklen_t *addrlen);
 
 //-------------------------------------------------------------------------
 // mgmt_fopen
