@@ -20,9 +20,10 @@
   See the License for the specific language governing permissions and
   limitations under the License.
  */
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_sock.h"
 #include "MgmtUtils.h"
-#include "Diags.h"
+#include "ts/Diags.h"
 
 #include "LocalManager.h"
 
