@@ -27,9 +27,10 @@
 
 #include "ts/ts.h"
 #include "ts/remap.h"
-#include "ink_defs.h"
 
-#include <memory>
+#include "ts/ink_defs.h"
+#include "ts/ink_memory.h"
+
 #include <string>
 #include <vector>
 
@@ -42,6 +43,7 @@
 #define TOKENCOUNT 10
 #define OVECOUNT 30
 #define PLUGIN_NAME "cacheurl"
+#define DEFAULT_CONFIG "cacheurl.config"
 
 struct regex_info {
   pcre *re;          /* Compiled regular expression */
@@ -86,7 +88,7 @@ regex_substitute(char **buf, char *str, regex_info *info)
     case PCRE_ERROR_NOMATCH:
       break;
     default:
-      TSError("[%s] Matching error: %d\n", PLUGIN_NAME, matchcount);
+      TSError("[%s] Matching error: %d", PLUGIN_NAME, matchcount);
       break;
     }
     return 0;
@@ -95,7 +97,7 @@ regex_substitute(char **buf, char *str, regex_info *info)
   /* Verify the replacement has the right number of matching groups */
   for (i = 0; i < info->tokcount; i++) {
     if (info->tokens[i] >= matchcount) {
-      TSError("[%s] Invalid reference int replacement: $%d\n", PLUGIN_NAME, info->tokens[i]);
+      TSError("[%s] Invalid reference int replacement: $%d", PLUGIN_NAME, info->tokens[i]);
       return 0;
     }
   }
@@ -142,7 +144,7 @@ regex_compile(regex_info **buf, char *pattern, char *replacement)
   /* Precompile the regular expression */
   info->re = pcre_compile(pattern, 0, &reerror, &reerroffset, NULL);
   if (!info->re) {
-    TSError("[%s] Compilation of regex '%s' failed at char %d: %s\n", PLUGIN_NAME, pattern, reerroffset, reerror);
+    TSError("[%s] Compilation of regex '%s' failed at char %d: %s", PLUGIN_NAME, pattern, reerroffset, reerror);
     status = 0;
   }
 
@@ -155,12 +157,12 @@ regex_compile(regex_info **buf, char *pattern, char *replacement)
       if (replacement[i] == '$') {
         if (tokcount >= TOKENCOUNT) {
           TSError("[%s] Error: too many tokens in replacement "
-                  "string: %s\n",
+                  "string: %s",
                   PLUGIN_NAME, replacement);
           status = 0;
           break;
         } else if (replacement[i + 1] < '0' || replacement[i + 1] > '9') {
-          TSError("[%s] Error: Invalid replacement token $%c in %s: should be $0 - $9\n", PLUGIN_NAME, replacement[i + 1],
+          TSError("[%s] Error: Invalid replacement token $%c in %s: should be $0 - $9", PLUGIN_NAME, replacement[i + 1],
                   replacement);
           status = 0;
           break;
@@ -204,7 +206,7 @@ load_config_file(const char *config_file)
   char buffer[1024];
   std::string path;
   TSFile fh;
-  std::auto_ptr<pr_list> prl(new pr_list());
+  ats_scoped_obj<pr_list> prl(new pr_list());
 
   /* locations in a config file line, end of line, split start, split end */
   char *eol, *spstart, *spend;
@@ -213,10 +215,10 @@ load_config_file(const char *config_file)
   regex_info *info = 0;
 
   if (config_file == NULL) {
-    /* Default config file of plugins/cacheurl.config */
-    path = TSPluginDirGet();
-    path += "/cacheurl.config";
-  } else if (*config_file != '/') {
+    config_file = DEFAULT_CONFIG;
+  }
+
+  if (*config_file != '/') {
     // Relative paths are relative to the config directory
     path = TSConfigDirGet();
     path += "/";
@@ -230,7 +232,7 @@ load_config_file(const char *config_file)
   fh = TSfopen(path.c_str(), "r");
 
   if (!fh) {
-    TSError("[%s] Unable to open %s. No patterns will be loaded\n", PLUGIN_NAME, path.c_str());
+    TSError("[%s] Unable to open %s. No patterns will be loaded", PLUGIN_NAME, path.c_str());
     return NULL;
   }
 
@@ -260,7 +262,7 @@ load_config_file(const char *config_file)
       spstart = strstr(buffer, "\t");
     }
     if (!spstart) {
-      TSError("[%s] ERROR: Invalid format on line %d. Skipping\n", PLUGIN_NAME, lineno);
+      TSError("[%s] ERROR: Invalid format on line %d. Skipping", PLUGIN_NAME, lineno);
       TSfclose(fh);
       return NULL;
     }
@@ -271,7 +273,7 @@ load_config_file(const char *config_file)
     }
     if (*spend == 0) {
       /* We reached the end of the string without any non-whitepace */
-      TSError("[%s] ERROR: Invalid format on line %d. Skipping\n", PLUGIN_NAME, lineno);
+      TSError("[%s] ERROR: Invalid format on line %d. Skipping", PLUGIN_NAME, lineno);
       TSfclose(fh);
       return NULL;
     }
@@ -283,7 +285,7 @@ load_config_file(const char *config_file)
     TSDebug(PLUGIN_NAME, "Adding pattern/replacement pair: '%s' -> '%s'", buffer, spend);
     retval = regex_compile(&info, buffer, spend);
     if (!retval) {
-      TSError("[%s] Error precompiling regex/replacement. Skipping.\n", PLUGIN_NAME);
+      TSError("[%s] Error precompiling regex/replacement. Skipping.", PLUGIN_NAME);
       TSfclose(fh);
       return NULL;
     }
@@ -293,7 +295,7 @@ load_config_file(const char *config_file)
   TSfclose(fh);
 
   if (prl->pr.empty()) {
-    TSError("[%s] No regular expressions loaded.\n", PLUGIN_NAME);
+    TSError("[%s] No regular expressions loaded.", PLUGIN_NAME);
   }
 
   TSDebug(PLUGIN_NAME, "loaded %u regexes", (unsigned)prl->pr.size());
@@ -311,7 +313,7 @@ rewrite_cacheurl(pr_list *prl, TSHttpTxn txnp)
 
   url = TSHttpTxnEffectiveUrlStringGet(txnp, &url_length);
   if (!url) {
-    TSError("[%s] couldn't retrieve request url\n", PLUGIN_NAME);
+    TSError("[%s] couldn't retrieve request url", PLUGIN_NAME);
     ok = 0;
   }
 
@@ -327,7 +329,7 @@ rewrite_cacheurl(pr_list *prl, TSHttpTxn txnp)
       TSDebug(PLUGIN_NAME, "Rewriting cache URL for %s to %s", url, newurl);
       if (TSCacheUrlSet(txnp, newurl, strlen(newurl)) != TS_SUCCESS) {
         TSError("[%s] Unable to modify cache url from "
-                "%s to %s\n",
+                "%s to %s",
                 PLUGIN_NAME, url, newurl);
         ok = 0;
       }
@@ -370,8 +372,8 @@ handle_hook(TSCont contp, TSEvent event, void *edata)
 static void
 initialization_error(const char *msg)
 {
-  TSError("[%s] %s\n", PLUGIN_NAME, msg);
-  TSError("[%s] Unable to initialize plugin (disabled).\n", PLUGIN_NAME);
+  TSError("[%s] %s", PLUGIN_NAME, msg);
+  TSError("[%s] Unable to initialize plugin (disabled).", PLUGIN_NAME);
 }
 
 TSReturnCode
@@ -439,7 +441,7 @@ TSPluginInit(int argc, const char *argv[])
   info.vendor_name = (char *)"Apache Software Foundation";
   info.support_email = (char *)"dev@trafficserver.apache.org";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
     TSDebug(PLUGIN_NAME, "ERROR, Plugin registration failed");
     initialization_error("Plugin registration failed.");
     return;
