@@ -21,263 +21,55 @@
     See the License for the specific language governing permissions and
     limitations under the License.
 */
-
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <stdarg.h>
 #include <getopt.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
+
+#include <string>
+#include <unordered_map>
 
 #include "ts/ts.h"
 #include "ts/remap.h"
-#include "ink_defs.h"
+#include "headers.h"
+#include "rules.h"
+#include "configs.h"
 
-#include <string>
-#include <set>
-#include <map>
-
-// Some wonkiness around compiler version and the unordered map (hash)
-#if HAVE_UNORDERED_MAP
-#include <unordered_map>
-typedef std::unordered_map<std::string, bool> OutstandingRequests;
-#else
-typedef std::map<std::string, bool> OutstandingRequests;
-#endif
-
-// Constants
-const char PLUGIN_NAME[] = "background_fetch";
-
-// Types
-typedef struct {
-  bool exclude;
-  std::string ruleField;
-  std::string ruleValue;
-} BgFetchRuleStruct;
-
-typedef std::map<uint32_t, BgFetchRuleStruct> BgFetchRuleMap;
 
 // Global config, if we don't have a remap specific config.
-static BgFetchRuleMap gBgFetchRuleMap;
+static BgFetchConfig *gConfig;
 
 
 ///////////////////////////////////////////////////////////////////////////
-// Read a config file
-//
-static bool
-read_config(char *config_file, BgFetchRuleMap *ri)
-{
-  char file_path[1024];
-  TSFile file;
+// Hold the global ackground fetch state. This is currently shared across all
+// configurations, as a singleton. ToDo: Would it ever make sense to do this
+// per remap rule? Probably not.
+typedef std::unordered_map<std::string, bool> OutstandingRequests;
 
-  if (config_file == NULL) {
-    TSError("%s: invalid config file", PLUGIN_NAME);
-    return false;
-  }
-
-  TSDebug(PLUGIN_NAME, "trying to open config file in this path: %s", config_file);
-
-  file = TSfopen(config_file, "r");
-  if (file == NULL) {
-    TSDebug(PLUGIN_NAME, "Failed to open config file %s, trying rel path", config_file);
-    snprintf(file_path, sizeof(file_path), "%s/%s", TSInstallDirGet(), config_file);
-    file = TSfopen(file_path, "r");
-    if (file == NULL) {
-      TSError("%s: invalid config file", PLUGIN_NAME);
-      return false;
-    }
-  }
-
-  BgFetchRuleMap *bgFetchRuleMapP = ri;
-  char buffer[8192];
-  uint32_t index = 0;
-
-  memset(buffer, 0, sizeof(buffer));
-  while (TSfgets(file, buffer, sizeof(buffer) - 1) != NULL) {
-    char *eol = 0;
-    // make sure line was not bigger than buffer
-    if ((eol = strchr(buffer, '\n')) == NULL && (eol = strstr(buffer, "\r\n")) == NULL) {
-      TSError("%s: exclusion line too long, did not get a good line in cfg, skipping, line: %s", PLUGIN_NAME, buffer);
-      memset(buffer, 0, sizeof(buffer));
-      continue;
-    }
-    // make sure line has something useful on it
-    if (eol - buffer < 2 || buffer[0] == '#') {
-      memset(buffer, 0, sizeof(buffer));
-      continue;
-    }
-
-    char *savePtr = NULL;
-    char *cfg = strtok_r(buffer, "\n\r\n", &savePtr);
-
-    if (cfg != NULL) {
-      TSDebug(PLUGIN_NAME, "setting background_fetch exclusion criterion based on string: %s", cfg);
-      char *cfg_type = strtok_r(buffer, " ", &savePtr);
-      char *cfg_name = NULL;
-      char *cfg_value = NULL;
-      bool exclude = false;
-
-      if (cfg_type) {
-        if (!strcmp(cfg_type, "exclude")) {
-          exclude = true;
-        } else if (strcmp(cfg_type, "include")) {
-          TSError("%s: invalid specifier %s, skipping config line", PLUGIN_NAME, cfg_type);
-          memset(buffer, 0, sizeof(buffer));
-          continue;
-        }
-        cfg_name = strtok_r(NULL, " ", &savePtr);
-        if (cfg_name) {
-          cfg_value = strtok_r(NULL, " ", &savePtr);
-          if (cfg_value) {
-            if (!strcmp(cfg_name, "Content-Length")) {
-              if ((cfg_value[0] != '<') && (cfg_value[0] != '>')) {
-                TSError("%s: invalid content-len condition %s, skipping config value", PLUGIN_NAME, cfg_value);
-                memset(buffer, 0, sizeof(buffer));
-                continue;
-              }
-            }
-            TSDebug(PLUGIN_NAME, "adding background_fetch exclusion rule %d for %s: %s", exclude, cfg_name, cfg_value);
-            BgFetchRuleStruct ruleS = {exclude, cfg_name, cfg_value};
-            bgFetchRuleMapP->insert(std::make_pair(index++, ruleS));
-          } else {
-            TSError("%s: invalid value %s, skipping config line", PLUGIN_NAME, cfg_name);
-          }
-        }
-      }
-      memset(buffer, 0, sizeof(buffer));
-    }
-  }
-  TSfclose(file);
-
-  TSDebug(PLUGIN_NAME, "Done parsing config");
-
-  return true;
-}
-
-
-///////////////////////////////////////////////////////////////////////////
-// Remove a header (fully) from an TSMLoc / TSMBuffer. Return the number
-// of fields (header values) we removed.
-int
-remove_header(TSMBuffer bufp, TSMLoc hdr_loc, const char *header, int len)
-{
-  TSMLoc field = TSMimeHdrFieldFind(bufp, hdr_loc, header, len);
-  int cnt = 0;
-
-  while (field) {
-    TSMLoc tmp = TSMimeHdrFieldNextDup(bufp, hdr_loc, field);
-
-    ++cnt;
-    TSMimeHdrFieldDestroy(bufp, hdr_loc, field);
-    TSHandleMLocRelease(bufp, hdr_loc, field);
-    field = tmp;
-  }
-
-  return cnt;
-}
-
-
-///////////////////////////////////////////////////////////////////////////
-// Set a header to a specific value. This will avoid going to through a
-// remove / add sequence in case of an existing header.
-// but clean.
-bool
-set_header(TSMBuffer bufp, TSMLoc hdr_loc, const char *header, int len, const char *val, int val_len)
-{
-  if (!bufp || !hdr_loc || !header || len <= 0 || !val || val_len <= 0) {
-    return false;
-  }
-
-  bool ret = false;
-  TSMLoc field_loc = TSMimeHdrFieldFind(bufp, hdr_loc, header, len);
-
-  if (!field_loc) {
-    // No existing header, so create one
-    if (TS_SUCCESS == TSMimeHdrFieldCreateNamed(bufp, hdr_loc, header, len, &field_loc)) {
-      if (TS_SUCCESS == TSMimeHdrFieldValueStringSet(bufp, hdr_loc, field_loc, -1, val, val_len)) {
-        TSMimeHdrFieldAppend(bufp, hdr_loc, field_loc);
-        ret = true;
-      }
-      TSHandleMLocRelease(bufp, hdr_loc, field_loc);
-    }
-  } else {
-    TSMLoc tmp = NULL;
-    bool first = true;
-
-    while (field_loc) {
-      if (first) {
-        first = false;
-        if (TS_SUCCESS == TSMimeHdrFieldValueStringSet(bufp, hdr_loc, field_loc, -1, val, val_len)) {
-          ret = true;
-        }
-      } else {
-        TSMimeHdrFieldDestroy(bufp, hdr_loc, field_loc);
-      }
-      tmp = TSMimeHdrFieldNextDup(bufp, hdr_loc, field_loc);
-      TSHandleMLocRelease(bufp, hdr_loc, field_loc);
-      field_loc = tmp;
-    }
-  }
-
-  return ret;
-}
-
-
-///////////////////////////////////////////////////////////////////////////
-// Dump a header on stderr, useful together with TSDebug().
-void
-dump_headers(TSMBuffer bufp, TSMLoc hdr_loc)
-{
-  TSIOBuffer output_buffer;
-  TSIOBufferReader reader;
-  TSIOBufferBlock block;
-  const char *block_start;
-  int64_t block_avail;
-
-  output_buffer = TSIOBufferCreate();
-  reader = TSIOBufferReaderAlloc(output_buffer);
-
-  /* This will print  just MIMEFields and not the http request line */
-  TSMimeHdrPrint(bufp, hdr_loc, output_buffer);
-
-  /* We need to loop over all the buffer blocks, there can be more than 1 */
-  block = TSIOBufferReaderStart(reader);
-  do {
-    block_start = TSIOBufferBlockReadStart(block, reader, &block_avail);
-    if (block_avail > 0) {
-      TSDebug(PLUGIN_NAME, "Headers are:\n%.*s", static_cast<int>(block_avail), block_start);
-    }
-    TSIOBufferReaderConsume(reader, block_avail);
-    block = TSIOBufferReaderStart(reader);
-  } while (block && block_avail != 0);
-
-  /* Free up the TSIOBuffer that we used to print out the header */
-  TSIOBufferReaderFree(reader);
-  TSIOBufferDestroy(output_buffer);
-}
-
-
-///////////////////////////////////////////////////////////////////////////
-// Struct to hold configurations and state. This can be global, or per
-// remap rule. This also holds the list of currently outstanding URLs,
-// such that we can avoid sending more than one background fill per URL at
-// any given time.
-class BGFetchConfig
+class BgFetchState
 {
 public:
-  BGFetchConfig() : log(NULL) { _lock = TSMutexCreate(); }
-
-  ~BGFetchConfig()
+  static BgFetchState &
+  getInstance()
   {
-    // ToDo: Destroy mutex ? TS-1432
+    static BgFetchState _instance;
+    return _instance;
   }
 
+  ~BgFetchState() { TSMutexDestroy(_lock); }
+
   void
-  create_log(const char *log_name)
+  createLog(const char *log_name)
   {
     TSDebug(PLUGIN_NAME, "Creating log name %s\n", log_name);
-    TSAssert(TS_SUCCESS == TSTextLogObjectCreate(log_name, TS_LOG_MODE_ADD_TIMESTAMP, &log));
+    TSAssert(TS_SUCCESS == TSTextLogObjectCreate(log_name, TS_LOG_MODE_ADD_TIMESTAMP, &_log));
+  }
+
+  TSTextLogObject
+  getLog()
+  {
+    return _log;
   }
 
   bool
@@ -294,7 +86,7 @@ public:
     }
     TSMutexUnlock(_lock);
 
-    TSDebug(PLUGIN_NAME, "BGFetchConfig.acquire(): ret = %d, url = %s\n", ret, url.c_str());
+    TSDebug(PLUGIN_NAME, "BgFetchState.acquire(): ret = %d, url = %s\n", ret, url.c_str());
 
     return ret;
   }
@@ -316,30 +108,32 @@ public:
     return ret;
   }
 
-  TSTextLogObject log;
-
 private:
+  BgFetchState() : _log(NULL), _lock(TSMutexCreate()) {}
+
+  BgFetchState(BgFetchState const &);   // Don't Implement
+  void operator=(BgFetchState const &); // Don't implement
+
+  TSTextLogObject _log;
   OutstandingRequests _urls;
   TSMutex _lock;
 };
 
-BGFetchConfig *gConfig;
-
 
 //////////////////////////////////////////////////////////////////////////////
-// Hold and manage some state for the background fetch continuation
+// Hold and manage some state for the TXN background fetch continuation.
 // This is necessary, because the TXN is likely to not be available
 // during the time we fetch from origin.
-static int cont_bg_fetch(TSCont contp, TSEvent event, void *edata);
-
-struct BGFetchData {
-  BGFetchData(BGFetchConfig *cfg = gConfig)
-    : hdr_loc(TS_NULL_MLOC), url_loc(TS_NULL_MLOC), vc(NULL), _bytes(0), _cont(NULL), _config(cfg)
+struct BgFetchData {
+  BgFetchData()
+    : hdr_loc(TS_NULL_MLOC), url_loc(TS_NULL_MLOC), vc(NULL), req_io_buf(NULL), resp_io_buf(NULL), req_io_buf_reader(NULL),
+      resp_io_buf_reader(NULL), r_vio(NULL), w_vio(NULL), _bytes(0), _cont(NULL)
   {
     mbuf = TSMBufferCreate();
+    memset(&client_ip, 0, sizeof(client_ip));
   }
 
-  ~BGFetchData()
+  ~BgFetchData()
   {
     TSHandleMLocRelease(mbuf, TS_NULL_MLOC, hdr_loc);
     TSHandleMLocRelease(mbuf, TS_NULL_MLOC, url_loc);
@@ -347,14 +141,14 @@ struct BGFetchData {
     TSMBufferDestroy(mbuf);
 
     if (vc) {
-      TSError("%s: Destroyed BGFetchDATA while VC was alive", PLUGIN_NAME);
+      TSError("[%s] Destroyed BgFetchDATA while VC was alive", PLUGIN_NAME);
       TSVConnClose(vc);
       vc = NULL;
     }
 
     // If we got schedule, also clean that up
     if (_cont) {
-      release_url();
+      releaseUrl();
 
       TSContDestroy(_cont);
       _cont = NULL;
@@ -366,23 +160,24 @@ struct BGFetchData {
   }
 
   bool
-  acquire_url() const
+  acquireUrl() const
   {
-    return _config->acquire(_url);
+    return BgFetchState::getInstance().acquire(_url);
   }
   bool
-  release_url() const
+  releaseUrl() const
   {
-    return _config->release(_url);
+    return BgFetchState::getInstance().release(_url);
   }
 
   const char *
-  get_url() const
+  getUrl() const
   {
     return _url.c_str();
   }
+
   void
-  add_bytes(int64_t b)
+  addBytes(int64_t b)
   {
     _bytes += b;
   }
@@ -406,7 +201,6 @@ private:
   std::string _url;
   int64_t _bytes;
   TSCont _cont;
-  BGFetchConfig *_config;
 };
 
 
@@ -420,7 +214,7 @@ private:
 // Upon succesful completion, the struct should be ready to start a
 // background fetch.
 bool
-BGFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
+BgFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
 {
   TSAssert(TS_NULL_MLOC == hdr_loc);
   TSAssert(TS_NULL_MLOC == url_loc);
@@ -432,10 +226,10 @@ BGFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
     } else if (ip->sa_family == AF_INET6) {
       memcpy(&client_ip, ip, sizeof(sockaddr_in6));
     } else {
-      TSError("%s: Unknown address family %d", PLUGIN_NAME, ip->sa_family);
+      TSError("[%s] Unknown address family %d", PLUGIN_NAME, ip->sa_family);
     }
   } else {
-    TSError("%s: failed to get client host info", PLUGIN_NAME);
+    TSError("[%s] Failed to get client host info", PLUGIN_NAME);
     return false;
   }
 
@@ -474,10 +268,11 @@ BGFetchData::initialize(TSMBuffer request, TSMLoc req_hdr, TSHttpTxn txnp)
   return false;
 }
 
+static int cont_bg_fetch(TSCont contp, TSEvent event, void *edata);
 
 // Create, setup and schedule the background fetch continuation.
 void
-BGFetchData::schedule()
+BgFetchData::schedule()
 {
   TSAssert(NULL == _cont);
 
@@ -495,15 +290,14 @@ BGFetchData::schedule()
   TSContSchedule(_cont, 0, TS_THREAD_POOL_NET);
 }
 
-
 // Log format is:
 //    remap-tag bytes status url
 void
-BGFetchData::log(TSEvent event) const
+BgFetchData::log(TSEvent event) const
 {
-  BGFetchConfig *conf = _config ? _config : gConfig;
+  TSTextLogObject log = BgFetchState::getInstance().getLog();
 
-  if (conf->log) {
+  if (log || TSIsDebugTagSet(PLUGIN_NAME)) {
     const char *status;
 
     switch (event) {
@@ -525,7 +319,10 @@ BGFetchData::log(TSEvent event) const
     }
 
     // ToDo: Also deal with per-remap tagging
-    TSTextLogObjectWrite(conf->log, "%s %" PRId64 " %s %s", "-", _bytes, status, _url.c_str());
+    TSDebug(PLUGIN_NAME, "%s %" PRId64 " %s %s", "-", _bytes, status, _url.c_str());
+    if (log) {
+      TSTextLogObjectWrite(log, "%s %" PRId64 " %s %s", "-", _bytes, status, _url.c_str());
+    }
   }
 }
 
@@ -537,7 +334,7 @@ BGFetchData::log(TSEvent event) const
 static int
 cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
 {
-  BGFetchData *data = static_cast<BGFetchData *>(TSContDataGet(contp));
+  BgFetchData *data = static_cast<BgFetchData *>(TSContDataGet(contp));
   int64_t avail;
 
   switch (event) {
@@ -558,10 +355,10 @@ cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
         TSDebug(PLUGIN_NAME, "Client IPv6 = %s", buf);
         break;
       default:
-        TSError("%s: Unknown address family %d", PLUGIN_NAME, sockaddress->sa_family);
+        TSError("[%s] Unknown address family %d", PLUGIN_NAME, sockaddress->sa_family);
         break;
       }
-      TSDebug(PLUGIN_NAME, "Starting bg fetch on: %s", data->get_url());
+      TSDebug(PLUGIN_NAME, "Starting bg fetch on: %s", data->getUrl());
       dump_headers(data->mbuf, data->hdr_loc);
     }
 
@@ -576,16 +373,19 @@ cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
       data->w_vio = TSVConnWrite(data->vc, contp, data->req_io_buf_reader, TSIOBufferReaderAvail(data->req_io_buf_reader));
     } else {
       delete data;
-      TSError("%s: failed to connect to internal process, major malfunction", PLUGIN_NAME);
+      TSError("[%s] Failed to connect to internal process, major malfunction", PLUGIN_NAME);
     }
+    break;
 
   case TS_EVENT_VCONN_WRITE_COMPLETE:
+    // TSVConnShutdown(data->vc, 0, 1);
+    // TSVIOReenable(data->w_vio);
     TSDebug(PLUGIN_NAME, "Write Complete");
     break;
 
   case TS_EVENT_VCONN_READ_READY:
     avail = TSIOBufferReaderAvail(data->resp_io_buf_reader);
-    data->add_bytes(avail);
+    data->addBytes(avail);
     TSIOBufferReaderConsume(data->resp_io_buf_reader, avail);
     TSVIONDoneSet(data->r_vio, TSVIONDoneGet(data->r_vio) + avail);
     TSVIOReenable(data->r_vio);
@@ -604,13 +404,13 @@ cont_bg_fetch(TSCont contp, TSEvent event, void * /* edata ATS_UNUSED */)
 
     TSDebug(PLUGIN_NAME, "Closing down background transaction, event= %s(%d)", TSHttpEventNameLookup(event), event);
     avail = TSIOBufferReaderAvail(data->resp_io_buf_reader);
-    data->add_bytes(avail);
+    data->addBytes(avail);
     TSIOBufferReaderConsume(data->resp_io_buf_reader, avail);
     TSVIONDoneSet(data->r_vio, TSVIONDoneGet(data->r_vio) + avail);
-
     data->log(event);
-    data->vc = NULL;
+
     // Close, release and cleanup
+    data->vc = NULL;
     delete data;
     break;
 
@@ -644,10 +444,10 @@ cont_check_cacheable(TSCont contp, TSEvent /* event ATS_UNUSED */, void *edata)
 
       TSDebug(PLUGIN_NAME, "Testing: request / response is cacheable?");
       if (cacheable) {
-        BGFetchData *data = new BGFetchData();
+        BgFetchData *data = new BgFetchData();
 
         // Initialize the data structure (can fail) and acquire a privileged lock on the URL
-        if (data->initialize(request, req_hdr, txnp) && data->acquire_url()) {
+        if (data->initialize(request, req_hdr, txnp) && data->acquireUrl()) {
           data->schedule();
         } else {
           delete data; // Not sure why this would happen, but ok.
@@ -667,156 +467,6 @@ cont_check_cacheable(TSCont contp, TSEvent /* event ATS_UNUSED */, void *edata)
   return 0;
 }
 
-///////////////////////////////////////////////////////////////////////////
-// Check if a header excludes us from running the background fetch
-//
-static bool
-check_client_ip_configured(TSHttpTxn txnp, const char *cfg_ip)
-{
-  const sockaddr *client_ip = TSHttpTxnClientAddrGet(txnp);
-  char ip_buf[INET6_ADDRSTRLEN];
-
-  if (AF_INET == client_ip->sa_family) {
-    inet_ntop(AF_INET, &(reinterpret_cast<const sockaddr_in *>(client_ip)->sin_addr), ip_buf, INET_ADDRSTRLEN);
-  } else if (AF_INET6 == client_ip->sa_family) {
-    inet_ntop(AF_INET6, &(reinterpret_cast<const sockaddr_in6 *>(client_ip)->sin6_addr), ip_buf, INET6_ADDRSTRLEN);
-  } else {
-    TSError("%s: unknown family %d", PLUGIN_NAME, client_ip->sa_family);
-    return false;
-  }
-
-  TSDebug(PLUGIN_NAME, "cfg_ip %s, client_ip %s", cfg_ip, ip_buf);
-
-  if ((strlen(cfg_ip) == strlen(ip_buf)) && !strcmp(cfg_ip, ip_buf)) {
-    TSDebug(PLUGIN_NAME, "bg fetch for ip %s, configured ip %s", ip_buf, cfg_ip);
-    return true;
-  }
-
-  return false;
-}
-
-static bool
-check_content_length(const uint32_t len, const char *cfg_val)
-{
-  uint32_t cfg_cont_len = atoi(&cfg_val[1]);
-
-  if (cfg_val[0] == '<') {
-    return (len <= cfg_cont_len);
-  } else if (cfg_val[0] == '>') {
-    return (len >= cfg_cont_len);
-  } else {
-    TSError("%s: invalid content length condition %c", PLUGIN_NAME, cfg_val[0]);
-    return false;
-  }
-}
-
-///////////////////////////////////////////////////////////////////////////
-// Check if a header excludes us from running the background fetch
-//
-static bool
-check_field_configured(TSHttpTxn txnp, const char *field_name, const char *cfg_val)
-{
-  // check for client-ip first
-  if (!strcmp(field_name, "Client-IP")) {
-    if (!strcmp(cfg_val, "*")) {
-      TSDebug(PLUGIN_NAME, "Found client_ip wild card");
-      return true;
-    }
-    if (check_client_ip_configured(txnp, cfg_val)) {
-      TSDebug(PLUGIN_NAME, "Found client_ip match");
-      return true;
-    }
-  }
-
-  bool hdr_found = false;
-
-  TSMBuffer hdr_bufp;
-  TSMLoc resp_hdrs;
-
-  if (!strcmp(field_name, "Content-Length")) {
-    if (TS_SUCCESS == TSHttpTxnServerRespGet(txnp, &hdr_bufp, &resp_hdrs)) {
-      TSMLoc loc = TSMimeHdrFieldFind(hdr_bufp, resp_hdrs, field_name, -1);
-      if (TS_NULL_MLOC != loc) {
-        uint32_t content_len = TSMimeHdrFieldValueUintGet(hdr_bufp, resp_hdrs, loc, 0 /* index */);
-        if (check_content_length(content_len, cfg_val)) {
-          TSDebug(PLUGIN_NAME, "Found content-length match");
-          hdr_found = true;
-        }
-        TSHandleMLocRelease(hdr_bufp, resp_hdrs, loc);
-      } else {
-        TSDebug(PLUGIN_NAME, "No content-length field in resp");
-      }
-    } else {
-      TSError("%s: Failed to get resp headers", PLUGIN_NAME);
-    }
-    TSHandleMLocRelease(hdr_bufp, TS_NULL_MLOC, resp_hdrs);
-    return hdr_found;
-  }
-
-  TSMLoc req_hdrs;
-  TSReturnCode ret = TSHttpTxnClientReqGet(txnp, &hdr_bufp, &req_hdrs);
-
-  if (ret != TS_SUCCESS) {
-    // something wrong..
-    TSError("%s: Failed to get req headers", PLUGIN_NAME);
-    return false;
-  }
-
-  TSMLoc loc = TSMimeHdrFieldFind(hdr_bufp, req_hdrs, field_name, -1);
-
-  if (TS_NULL_MLOC != loc) {
-    if (!strcmp(cfg_val, "*")) {
-      TSDebug(PLUGIN_NAME, "Found %s wild card", field_name);
-      hdr_found = true;
-    } else {
-      int val_len = 0;
-      const char *val_str = TSMimeHdrFieldValueStringGet(hdr_bufp, req_hdrs, loc, 0, &val_len);
-
-      if (!val_str || val_len <= 0) {
-        TSDebug(PLUGIN_NAME, "invalid field");
-      } else {
-        TSDebug(PLUGIN_NAME, "comparing with %s", cfg_val);
-        if (NULL != strstr(val_str, cfg_val)) {
-          hdr_found = true;
-        }
-      }
-    }
-    TSHandleMLocRelease(hdr_bufp, req_hdrs, loc);
-  } else {
-    TSDebug(PLUGIN_NAME, "no field %s in request header", field_name);
-  }
-
-  TSHandleMLocRelease(hdr_bufp, TS_NULL_MLOC, req_hdrs);
-  return hdr_found;
-}
-
-///////////////////////////////////////////////////////////////////////////
-// Check the configuration (either per remap, or global), and decide if
-// this request is allowed to trigger a background fetch.
-//
-static bool
-is_background_fetch_allowed(TSHttpTxn txnp, BgFetchRuleMap *ri)
-{
-  TSDebug(PLUGIN_NAME, "Testing: request is internal?");
-  if (TSHttpIsInternalRequest(txnp) == TS_SUCCESS) {
-    return false;
-  }
-
-  bool allow_bg_fetch = true;
-
-  for (BgFetchRuleMap::iterator it = ri->begin(); it != ri->end(); ++it) {
-    BgFetchRuleStruct sRule = it->second;
-    const char *ruleField = sRule.ruleField.c_str();
-    if (check_field_configured(txnp, ruleField, sRule.ruleValue.c_str())) {
-      TSDebug(PLUGIN_NAME, "found field match %s, exclude %d", ruleField, sRule.exclude);
-      allow_bg_fetch = !sRule.exclude;
-      break;
-    }
-  }
-
-  return allow_bg_fetch;
-}
-
 
 //////////////////////////////////////////////////////////////////////////////
 // Main "plugin", which is a global READ_RESPONSE_HDR hook. Before
@@ -830,36 +480,46 @@ is_background_fetch_allowed(TSHttpTxn txnp, BgFetchRuleMap *ri)
 // there could be other plugins that modifies the response after us.
 //
 static int
-cont_handle_response(TSCont contp, TSEvent /* event ATS_UNUSED */, void *edata)
+cont_handle_response(TSCont contp, TSEvent event, void *edata)
 {
   // ToDo: If we want to support per-remap configurations, we have to pass along the data here
   TSHttpTxn txnp = static_cast<TSHttpTxn>(edata);
-  BgFetchRuleMap *ri = static_cast<BgFetchRuleMap *>(TSContDataGet(contp));
+  BgFetchConfig *config = static_cast<BgFetchConfig *>(TSContDataGet(contp));
 
-  if (ri == NULL) {
-    // something wrong..
-    TSError("%s: can't get BgFetchRule Map", PLUGIN_NAME);
-    ri = &gBgFetchRuleMap;
-  }
+  if (NULL == config) {
+    // something seriously wrong..
+    TSError("[%s] Can't get configurations", PLUGIN_NAME);
+  } else {
+    switch (event) {
+    case TS_EVENT_HTTP_READ_RESPONSE_HDR:
+      if (config->getRules()->bgFetchAllowed(txnp)) {
+        TSMBuffer response;
+        TSMLoc resp_hdr;
 
-  if (is_background_fetch_allowed(txnp, ri)) {
-    TSMBuffer response;
-    TSMLoc resp_hdr;
+        if (TS_SUCCESS == TSHttpTxnServerRespGet(txnp, &response, &resp_hdr)) {
+          // ToDo: Check the MIME type first, to see if it's a type we care about.
+          // ToDo: Such MIME types should probably be per remap rule.
 
-    if (TS_SUCCESS == TSHttpTxnServerRespGet(txnp, &response, &resp_hdr)) {
-      // ToDo: Check the MIME type first, to see if it's a type we care about.
-      // ToDo: Such MIME types should probably be per remap rule.
+          // Only deal with 206 responses from Origin
+          TSDebug(PLUGIN_NAME, "Testing: response is 206?");
+          if (TS_HTTP_STATUS_PARTIAL_CONTENT == TSHttpHdrStatusGet(response, resp_hdr)) {
+            // Everything looks good so far, add a TXN hook for SEND_RESPONSE_HDR
+            TSCont contp = TSContCreate(cont_check_cacheable, NULL);
 
-      // 2. Only deal with 206 responses from Origin
-      TSDebug(PLUGIN_NAME, "Testing: response is 206?");
-      if (TS_HTTP_STATUS_PARTIAL_CONTENT == TSHttpHdrStatusGet(response, resp_hdr)) {
-        // Everything looks good so far, add a TXN hook for SEND_RESPONSE_HDR
-        TSCont contp = TSContCreate(cont_check_cacheable, NULL);
-
-        TSHttpTxnHookAdd(txnp, TS_HTTP_SEND_RESPONSE_HDR_HOOK, contp);
+            TSHttpTxnHookAdd(txnp, TS_HTTP_SEND_RESPONSE_HDR_HOOK, contp);
+          }
+          // Release the response MLoc
+          TSHandleMLocRelease(response, TS_NULL_MLOC, resp_hdr);
+        }
       }
-      // Release the response MLoc
-      TSHandleMLocRelease(response, TS_NULL_MLOC, resp_hdr);
+      break;
+    case TS_EVENT_HTTP_TXN_CLOSE:
+      config->release(); // Release the configuration lease when txn closes
+      break;
+    default:
+      TSError("[%s] Unknown event for this plugin", PLUGIN_NAME);
+      TSDebug(PLUGIN_NAME, "unknown event for this plugin");
+      break;
     }
   }
 
@@ -883,23 +543,26 @@ TSPluginInit(int argc, const char *argv[])
   info.vendor_name = (char *)"Apache Software Foundation";
   info.support_email = (char *)"dev@trafficserver.apache.org";
 
-  if (TS_SUCCESS != TSPluginRegister(TS_SDK_VERSION_3_0, &info)) {
-    TSError("%s: plugin registration failed.\n", PLUGIN_NAME);
+  if (TS_SUCCESS != TSPluginRegister(&info)) {
+    TSError("[%s] Plugin registration failed.", PLUGIN_NAME);
   }
 
-  gConfig = new BGFetchConfig();
-  optind = 1;
+  TSCont cont = TSContCreate(cont_handle_response, NULL);
 
+  gConfig = new BgFetchConfig(cont);
+  gConfig->acquire(); // Inc refcount, although this global config should never go out of scope
+
+  optind = 1;
   while (true) {
     int opt = getopt_long(argc, (char *const *)argv, "lc", longopt, NULL);
 
     switch (opt) {
     case 'l':
-      gConfig->create_log(optarg);
+      BgFetchState::getInstance().createLog(optarg);
       break;
     case 'c':
       TSDebug(PLUGIN_NAME, "config file %s..", optarg);
-      read_config(optarg, &gBgFetchRuleMap);
+      gConfig->readConfig(optarg);
       break;
     }
 
@@ -909,9 +572,6 @@ TSPluginInit(int argc, const char *argv[])
   }
 
   TSDebug(PLUGIN_NAME, "Initialized");
-
-  TSCont cont = TSContCreate(cont_handle_response, NULL);
-  TSContDataSet(cont, static_cast<void *>(&gBgFetchRuleMap));
   TSHttpHookAdd(TS_HTTP_READ_RESPONSE_HDR_HOOK, cont);
 }
 
@@ -947,26 +607,18 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
 TSReturnCode
 TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf */, int /* errbuf_size */)
 {
-  BgFetchRuleMap *ri = new BgFetchRuleMap();
-  if (ri == NULL) {
-    TSError("%s:Unable to create remap instance", PLUGIN_NAME);
-    return TS_ERROR;
-  }
+  TSCont cont = TSContCreate(cont_handle_response, NULL);
+  BgFetchConfig *config = new BgFetchConfig(cont);
 
-  if (gConfig == NULL) {
-    TSDebug(PLUGIN_NAME, "creating gConfig");
-    gConfig = new BGFetchConfig();
-  }
+  config->acquire(); // Inc refcount
 
-  char *fileName = NULL;
+  // Parse the optional rules, wihch becomes a linked list of BgFetchRule's.
   if (argc > 2) {
-    fileName = argv[2];
-    TSDebug(PLUGIN_NAME, "config file %s", fileName);
+    TSDebug(PLUGIN_NAME, "config file %s", argv[2]);
+    config->readConfig(argv[2]);
   }
 
-  read_config(fileName, ri);
-
-  *ih = (void *)ri;
+  *ih = (void *)config;
 
   return TS_SUCCESS;
 }
@@ -974,8 +626,9 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf */, int /
 void
 TSRemapDeleteInstance(void *ih)
 {
-  BgFetchRuleMap *ri = static_cast<BgFetchRuleMap *>(ih);
-  delete ri;
+  BgFetchConfig *config = static_cast<BgFetchConfig *>(ih);
+
+  config->release();
 }
 
 
@@ -989,11 +642,24 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri */)
     return TSREMAP_NO_REMAP;
   }
 
-  TSDebug(PLUGIN_NAME, "background fetch TSRemapDoRemap...");
+  TSMBuffer bufp;
+  TSMLoc req_hdrs;
 
-  TSCont cont = TSContCreate(cont_handle_response, NULL);
-  TSContDataSet(cont, static_cast<void *>(ih));
-  TSHttpTxnHookAdd(txnp, TS_HTTP_READ_RESPONSE_HDR_HOOK, cont);
+  if (TS_SUCCESS == TSHttpTxnClientReqGet(txnp, &bufp, &req_hdrs)) {
+    TSMLoc field_loc = TSMimeHdrFieldFind(bufp, req_hdrs, TS_MIME_FIELD_RANGE, TS_MIME_LEN_RANGE);
+
+    if (field_loc) {
+      BgFetchConfig *config = static_cast<BgFetchConfig *>(ih);
+
+      config->acquire(); // Inc refcount
+      TSHttpTxnHookAdd(txnp, TS_HTTP_READ_RESPONSE_HDR_HOOK, config->getCont());
+      TSHttpTxnHookAdd(txnp, TS_HTTP_TXN_CLOSE_HOOK, config->getCont());
+
+      TSDebug(PLUGIN_NAME, "background fetch TSRemapDoRemap...");
+      TSHandleMLocRelease(bufp, req_hdrs, field_loc);
+    }
+    TSHandleMLocRelease(bufp, TS_NULL_MLOC, req_hdrs);
+  }
 
   return TSREMAP_NO_REMAP;
 }
