@@ -21,7 +21,7 @@
   limitations under the License.
  */
 
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -198,7 +198,7 @@ bool
 ContData::init()
 {
   if (initialized) {
-    TSError("[%s] ContData already initialized!", __FUNCTION__);
+    TSError("[esi][%s] ContData already initialized!", __FUNCTION__);
     return false;
   }
 
@@ -214,7 +214,7 @@ ContData::init()
     // Get upstream VIO
     input_vio = TSVConnWriteVIOGet(contp);
     if (!input_vio) {
-      TSError("[%s] Error while getting input vio", __FUNCTION__);
+      TSError("[esi][%s] Error while getting input vio", __FUNCTION__);
       goto lReturn;
     }
     input_reader = TSVIOReaderGet(input_vio);
@@ -223,7 +223,7 @@ ContData::init()
     TSVConn output_conn;
     output_conn = TSTransformOutputVConnGet(contp);
     if (!output_conn) {
-      TSError("[%s] Error while getting transform VC", __FUNCTION__);
+      TSError("[esi][%s] Error while getting transform VC", __FUNCTION__);
       goto lReturn;
     }
     output_buffer = TSIOBufferCreate();
@@ -265,7 +265,7 @@ ContData::getClientState()
   TSMBuffer req_bufp;
   TSMLoc req_hdr_loc;
   if (TSHttpTxnClientReqGet(txnp, &req_bufp, &req_hdr_loc) != TS_SUCCESS) {
-    TSError("[%s] Error while retrieving client request", __FUNCTION__);
+    TSError("[esi][%s] Error while retrieving client request", __FUNCTION__);
     return;
   }
 
@@ -281,7 +281,7 @@ ContData::getClientState()
     TSMBuffer bufp;
     TSMLoc url_loc;
     if (TSHttpTxnPristineUrlGet(txnp, &bufp, &url_loc) != TS_SUCCESS) {
-      TSError("[%s] Error while retrieving hdr url", __FUNCTION__);
+      TSError("[esi][%s] Error while retrieving hdr url", __FUNCTION__);
       return;
     }
     if (url_loc) {
@@ -437,12 +437,12 @@ ContData::getServerState()
       input_type = DATA_TYPE_PACKED_ESI;
       return;
     } else if (TSHttpTxnCachedRespGet(txnp, &bufp, &hdr_loc) != TS_SUCCESS) {
-      TSError("[%s] Could not get server response; set input type to RAW_ESI", __FUNCTION__);
+      TSError("[esi][%s] Could not get server response; set input type to RAW_ESI", __FUNCTION__);
       input_type = DATA_TYPE_RAW_ESI;
       return;
     }
   } else if (TSHttpTxnServerRespGet(txnp, &bufp, &hdr_loc) != TS_SUCCESS) {
-    TSError("[%s] Could not get server response; set input type to RAW_ESI", __FUNCTION__);
+    TSError("[esi][%s] Could not get server response; set input type to RAW_ESI", __FUNCTION__);
     input_type = DATA_TYPE_RAW_ESI;
     return;
   }
@@ -510,36 +510,36 @@ removeCacheKey(TSHttpTxn txnp)
   bool result = false;
 
   if (TSHttpTxnClientReqGet(txnp, &req_bufp, &req_hdr_loc) != TS_SUCCESS) {
-    TSError("[%s] Error while retrieving client request", __FUNCTION__);
+    TSError("[esi][%s] Error while retrieving client request", __FUNCTION__);
     return false;
   }
 
   do {
     if (TSHttpTxnPristineUrlGet(txnp, &req_bufp, &url_loc) != TS_SUCCESS) {
-      TSError("[%s] Error while retrieving hdr url", __FUNCTION__);
+      TSError("[esi][%s] Error while retrieving hdr url", __FUNCTION__);
       break;
     }
 
     contp = TSContCreate(removeCacheHandler, NULL);
     if (contp == NULL) {
-      TSError("[%s] Could not create continuation", __FUNCTION__);
+      TSError("[esi][%s] Could not create continuation", __FUNCTION__);
       break;
     }
 
     cacheKey = TSCacheKeyCreate();
     if (cacheKey == NULL) {
-      TSError("[%s] TSCacheKeyCreate fail", __FUNCTION__);
+      TSError("[esi][%s] TSCacheKeyCreate fail", __FUNCTION__);
       break;
     }
 
     if (TSCacheKeyDigestFromUrlSet(cacheKey, url_loc) != TS_SUCCESS) {
-      TSError("[%s] TSCacheKeyDigestFromUrlSet fail", __FUNCTION__);
+      TSError("[esi][%s] TSCacheKeyDigestFromUrlSet fail", __FUNCTION__);
       break;
     }
 
     TSCacheRemove(contp, cacheKey);
     result = true;
-    TSError("[%s] TSCacheRemoved", __FUNCTION__);
+    TSError("[esi][%s] TSCacheRemoved", __FUNCTION__);
   } while (0);
 
   if (cacheKey != NULL) {
@@ -645,7 +645,7 @@ transformData(TSCont contp)
     if (toread > 0) {
       avail = TSIOBufferReaderAvail(cont_data->input_reader);
       if (avail == TS_ERROR) {
-        TSError("[%s] Error while getting number of bytes available", __FUNCTION__);
+        TSError("[esi][%s] Error while getting number of bytes available", __FUNCTION__);
         return 0;
       }
 
@@ -742,7 +742,7 @@ transformData(TSCont contp)
         TSDebug(cont_data->debug_tag, "[%s] ESI processor output document of size %d starting with [%.10s]", __FUNCTION__,
                 out_data_len, (out_data_len ? out_data : "(null)"));
       } else {
-        TSError("[%s] ESI processor failed to process document; will return empty document", __FUNCTION__);
+        TSError("[esi][%s] ESI processor failed to process document; will return empty document", __FUNCTION__);
         out_data = "";
         out_data_len = 0;
       }
@@ -752,7 +752,7 @@ transformData(TSCont contp)
         string cdata;
         if (cont_data->gzip_output) {
           if (!gzip(out_data, out_data_len, cdata)) {
-            TSError("[%s] Error while gzipping content", __FUNCTION__);
+            TSError("[esi][%s] Error while gzipping content", __FUNCTION__);
             out_data_len = 0;
             out_data = "";
           } else {
@@ -767,7 +767,7 @@ transformData(TSCont contp)
         TSVConn output_conn;
         output_conn = TSTransformOutputVConnGet(contp);
         if (!output_conn) {
-          TSError("[%s] Error while getting transform VC", __FUNCTION__);
+          TSError("[esi][%s] Error while getting transform VC", __FUNCTION__);
           return 0;
         }
 
@@ -775,7 +775,7 @@ transformData(TSCont contp)
         output_vio = TSVConnWrite(output_conn, contp, cont_data->output_reader, out_data_len);
 
         if (TSIOBufferWrite(TSVIOBufferGet(output_vio), out_data, out_data_len) == TS_ERROR) {
-          TSError("[%s] Error while writing bytes to downstream VC", __FUNCTION__);
+          TSError("[esi][%s] Error while writing bytes to downstream VC", __FUNCTION__);
           return 0;
         }
 
@@ -806,7 +806,7 @@ transformData(TSCont contp)
       TSDebug(cont_data->debug_tag, "[%s] ESI processor output document of size %d starting with [%.10s]", __FUNCTION__,
               (int)out_data.size(), (out_data.size() ? out_data.data() : "(null)"));
     } else {
-      TSError("[%s] ESI processor failed to process document; will return empty document", __FUNCTION__);
+      TSError("[esi][%s] ESI processor failed to process document; will return empty document", __FUNCTION__);
       out_data.assign("");
 
       if (!cont_data->xform_closed) {
@@ -819,7 +819,7 @@ transformData(TSCont contp)
     if (!cont_data->xform_closed && out_data.size() > 0) {
       if (cont_data->gzip_output) {
         if (!cont_data->esi_gzip->stream_encode(out_data, cdata)) {
-          TSError("[%s] Error while gzipping content", __FUNCTION__);
+          TSError("[esi][%s] Error while gzipping content", __FUNCTION__);
         } else {
           TSDebug(cont_data->debug_tag, "[%s] Compressed document from size %d to %d bytes", __FUNCTION__, (int)out_data.size(),
                   (int)cdata.size());
@@ -828,12 +828,12 @@ transformData(TSCont contp)
 
       if (cont_data->gzip_output) {
         if (TSIOBufferWrite(TSVIOBufferGet(cont_data->output_vio), cdata.data(), cdata.size()) == TS_ERROR) {
-          TSError("[%s] Error while writing bytes to downstream VC", __FUNCTION__);
+          TSError("[esi][%s] Error while writing bytes to downstream VC", __FUNCTION__);
           return 0;
         }
       } else {
         if (TSIOBufferWrite(TSVIOBufferGet(cont_data->output_vio), out_data.data(), out_data.size()) == TS_ERROR) {
-          TSError("[%s] Error while writing bytes to downstream VC", __FUNCTION__);
+          TSError("[esi][%s] Error while writing bytes to downstream VC", __FUNCTION__);
           return 0;
         }
       }
@@ -845,15 +845,15 @@ transformData(TSCont contp)
           string cdata;
           int downstream_length;
           if (!cont_data->esi_gzip->stream_finish(cdata, downstream_length)) {
-            TSError("[%s] Error while finishing gzip", __FUNCTION__);
+            TSError("[esi][%s] Error while finishing gzip", __FUNCTION__);
             return 0;
           } else {
             if (TSVIOBufferGet(cont_data->output_vio) == NULL) {
-              TSError("[%s] Error while writing bytes to downstream VC", __FUNCTION__);
+              TSError("[esi][%s] Error while writing bytes to downstream VC", __FUNCTION__);
               return 0;
             }
             if (TSIOBufferWrite(TSVIOBufferGet(cont_data->output_vio), cdata.data(), cdata.size()) == TS_ERROR) {
-              TSError("[%s] Error while writing bytes to downstream VC", __FUNCTION__);
+              TSError("[esi][%s] Error while writing bytes to downstream VC", __FUNCTION__);
               return 0;
             }
             TSDebug(cont_data->debug_tag, "[%s] ESI processed overall/gzip: %d", __FUNCTION__, downstream_length);
@@ -887,7 +887,7 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
 
   if (!cont_data->initialized) {
     if (!cont_data->init()) {
-      TSError("[%s] Could not initialize continuation data; shutting down transformation", __FUNCTION__);
+      TSError("[esi][%s] Could not initialize continuation data; shutting down transformation", __FUNCTION__);
       goto lShutdown;
     }
     TSDebug(cont_data->debug_tag, "[%s] initialized continuation data", __FUNCTION__);
@@ -935,7 +935,7 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
       // doubt: what is this code doing?
       input_vio = TSVConnWriteVIOGet(contp);
       if (!input_vio) {
-        TSError("[%s] Error while getting upstream vio", __FUNCTION__);
+        TSError("[esi][%s] Error while getting upstream vio", __FUNCTION__);
       } else {
         TSContCall(TSVIOContGet(input_vio), TS_EVENT_ERROR, input_vio);
       }
@@ -974,7 +974,7 @@ transformHandler(TSCont contp, TSEvent event, void *edata)
             }
           }
         } else {
-          TSError("[%s] Could not handle fetch event!", __FUNCTION__);
+          TSError("[esi][%s] Could not handle fetch event!", __FUNCTION__);
         }
       } else {
         TSAssert(!"Unexpected event");
@@ -1021,17 +1021,17 @@ addMimeHeaderField(TSMBuffer bufp, TSMLoc hdr_loc, const char *name, int name_le
   TSMLoc field_loc = (TSMLoc)NULL;
   TSMimeHdrFieldCreate(bufp, hdr_loc, &field_loc);
   if (!field_loc) {
-    TSError("[%s] Error while creating mime field", __FUNCTION__);
+    TSError("[esi][%s] Error while creating mime field", __FUNCTION__);
   } else {
     if (TSMimeHdrFieldNameSet(bufp, hdr_loc, field_loc, name, name_len) != TS_SUCCESS) {
-      TSError("[%s] Error while setting name [%.*s] for MIME header field", __FUNCTION__, name_len, name);
+      TSError("[esi][%s] Error while setting name [%.*s] for MIME header field", __FUNCTION__, name_len, name);
     } else {
       if (TSMimeHdrFieldValueStringInsert(bufp, hdr_loc, field_loc, 0, value, value_len) != TS_SUCCESS) {
-        TSError("[%s] Error while inserting value [%.*s] string to MIME field [%.*s]", __FUNCTION__, value_len, value, name_len,
-                name);
+        TSError("[esi][%s] Error while inserting value [%.*s] string to MIME field [%.*s]", __FUNCTION__, value_len, value,
+                name_len, name);
       } else {
         if (TSMimeHdrFieldAppend(bufp, hdr_loc, field_loc) != TS_SUCCESS) {
-          TSError("[%s] Error while appending MIME field with name [%.*s] and value [%.*s]", __FUNCTION__, name_len, name,
+          TSError("[esi][%s] Error while appending MIME field with name [%.*s] and value [%.*s]", __FUNCTION__, name_len, name,
                   value_len, value);
         }
       }
@@ -1047,7 +1047,7 @@ modifyResponseHeader(TSCont contp, TSEvent event, void *edata)
   RespHdrModData *mod_data = static_cast<RespHdrModData *>(TSContDataGet(contp));
   TSHttpTxn txnp = static_cast<TSHttpTxn>(edata);
   if (event != TS_EVENT_HTTP_SEND_RESPONSE_HDR) {
-    TSError("[%s] Unexpected event (%d)", __FUNCTION__, event);
+    TSError("[esi][%s] Unexpected event (%d)", __FUNCTION__, event);
     goto lReturn;
   }
   TSMBuffer bufp;
@@ -1082,7 +1082,7 @@ modifyResponseHeader(TSCont contp, TSEvent event, void *edata)
         } else if (Utils::areEqual(name, name_len, TS_MIME_FIELD_CONTENT_LENGTH, TS_MIME_LEN_CONTENT_LENGTH)) {
           if (mod_data->head_only) {
             destroy_header = true;
-            TSError("[%s] remove Content-Length", __FUNCTION__);
+            TSDebug(DEBUG_TAG, "[%s] remove Content-Length", __FUNCTION__);
           }
         } else {
           int n_field_values = TSMimeHdrFieldValuesCount(bufp, hdr_loc, field_loc);
@@ -1133,7 +1133,7 @@ modifyResponseHeader(TSCont contp, TSEvent event, void *edata)
     TSDebug(DEBUG_TAG, "[%s] Inspected client-bound headers", __FUNCTION__);
     retval = 1;
   } else {
-    TSError("[%s] Error while getting response from txn", __FUNCTION__);
+    TSError("[esi][%s] Error while getting response from txn", __FUNCTION__);
   }
 
 lReturn:
@@ -1186,7 +1186,7 @@ maskOsCacheHeaders(TSHttpTxn txnp)
   TSMBuffer bufp;
   TSMLoc hdr_loc;
   if (TSHttpTxnServerRespGet(txnp, &bufp, &hdr_loc) != TS_SUCCESS) {
-    TSError("[%s] Couldn't get server response from txn", __FUNCTION__);
+    TSError("[esi][%s] Couldn't get server response from txn", __FUNCTION__);
     return;
   }
   int n_mime_headers = TSMimeHdrFieldsCount(bufp, hdr_loc);
@@ -1226,7 +1226,7 @@ maskOsCacheHeaders(TSHttpTxn txnp)
         masked_name.assign(HEADER_MASK_PREFIX);
         masked_name.append(name, name_len);
         if (TSMimeHdrFieldNameSet(bufp, hdr_loc, field_loc, masked_name.data(), masked_name.size()) != TS_SUCCESS) {
-          TSError("[%s] Couldn't rename header [%.*s]", __FUNCTION__, name_len, name);
+          TSError("[esi][%s] Couldn't rename header [%.*s]", __FUNCTION__, name_len, name);
         }
       }
     } // end if got header name
@@ -1250,7 +1250,7 @@ isTxnTransformable(TSHttpTxn txnp, bool is_cache_txn, bool *intercept_header, bo
   bool retval = false;
 
   if (TSHttpTxnClientReqGet(txnp, &bufp, &hdr_loc) != TS_SUCCESS) {
-    TSError("[%s] Couldn't get txn header", __FUNCTION__);
+    TSError("[esi][%s] Couldn't get txn header", __FUNCTION__);
     return false;
   }
 
@@ -1258,7 +1258,7 @@ isTxnTransformable(TSHttpTxn txnp, bool is_cache_txn, bool *intercept_header, bo
   const char *method;
   method = TSHttpHdrMethodGet(bufp, hdr_loc, &method_len);
   if (method == NULL) {
-    TSError("[%s] Couldn't get method", __FUNCTION__);
+    TSError("[esi][%s] Couldn't get method", __FUNCTION__);
     TSHandleMLocRelease(bufp, TS_NULL_MLOC, hdr_loc);
     return false;
   }
@@ -1275,7 +1275,7 @@ isTxnTransformable(TSHttpTxn txnp, bool is_cache_txn, bool *intercept_header, bo
 
   header_obtained = is_cache_txn ? TSHttpTxnCachedRespGet(txnp, &bufp, &hdr_loc) : TSHttpTxnServerRespGet(txnp, &bufp, &hdr_loc);
   if (header_obtained != TS_SUCCESS) {
-    TSError("[%s] Couldn't get txn header", __FUNCTION__);
+    TSError("[esi][%s] Couldn't get txn header", __FUNCTION__);
     return false;
   }
 
@@ -1318,7 +1318,7 @@ isCacheObjTransformable(TSHttpTxn txnp, bool *intercept_header, bool *head_only)
 {
   int obj_status;
   if (TSHttpTxnCacheLookupStatusGet(txnp, &obj_status) == TS_ERROR) {
-    TSError("[%s] Couldn't get cache status of object", __FUNCTION__);
+    TSError("[esi][%s] Couldn't get cache status of object", __FUNCTION__);
     return false;
   }
   if (obj_status == TS_CACHE_LOOKUP_HIT_FRESH) {
@@ -1339,7 +1339,7 @@ isCacheObjTransformable(TSHttpTxn txnp, bool *intercept_header, bool *head_only)
 static bool
 isInterceptRequest(TSHttpTxn txnp)
 {
-  if (TSHttpIsInternalRequest(txnp) != TS_SUCCESS) {
+  if (TSHttpTxnIsInternal(txnp) != TS_SUCCESS) {
     TSDebug(DEBUG_TAG, "[%s] Skipping external request", __FUNCTION__);
     return false;
   }
@@ -1347,7 +1347,7 @@ isInterceptRequest(TSHttpTxn txnp)
   TSMBuffer bufp;
   TSMLoc hdr_loc;
   if (TSHttpTxnClientReqGet(txnp, &bufp, &hdr_loc) != TS_SUCCESS) {
-    TSError("[%s] Could not get client request", __FUNCTION__);
+    TSError("[esi][%s] Could not get client request", __FUNCTION__);
     return false;
   }
 
@@ -1356,7 +1356,7 @@ isInterceptRequest(TSHttpTxn txnp)
   int method_len;
   const char *method = TSHttpHdrMethodGet(bufp, hdr_loc, &method_len);
   if (!method) {
-    TSError("[%s] Could not obtain method!", __FUNCTION__);
+    TSError("[esi][%s] Could not obtain method!", __FUNCTION__);
   } else {
     if ((method_len != TS_HTTP_LEN_POST) || (strncasecmp(method, TS_HTTP_METHOD_POST, TS_HTTP_LEN_POST))) {
       TSDebug(DEBUG_TAG, "[%s] Method [%.*s] invalid, [%s] expected", __FUNCTION__, method_len, method, TS_HTTP_METHOD_POST);
@@ -1399,7 +1399,7 @@ addSendResponseHeaderHook(TSHttpTxn txnp, const ContData *src_cont_data)
 {
   TSCont contp = TSContCreate(modifyResponseHeader, NULL);
   if (!contp) {
-    TSError("[%s] Could not create continuation", __FUNCTION__);
+    TSError("[esi][%s] Could not create continuation", __FUNCTION__);
     return false;
   }
   TSHttpTxnHookAdd(txnp, TS_HTTP_SEND_RESPONSE_HDR_HOOK, contp);
@@ -1421,7 +1421,7 @@ addTransform(TSHttpTxn txnp, const bool processing_os_response, const bool inter
 
   contp = TSTransformCreate(transformHandler, txnp);
   if (!contp) {
-    TSError("[%s] Error while creating a new transformation", __FUNCTION__);
+    TSError("[esi][%s] Error while creating a new transformation", __FUNCTION__);
     goto lFail;
   }
 
@@ -1450,7 +1450,7 @@ addTransform(TSHttpTxn txnp, const bool processing_os_response, const bool inter
   TSHttpTxnHookAdd(txnp, TS_HTTP_RESPONSE_TRANSFORM_HOOK, contp);
 
   if (!addSendResponseHeaderHook(txnp, cont_data)) {
-    TSError("[%s] Couldn't add send response header hook", __FUNCTION__);
+    TSError("[esi][%s] Couldn't add send response header hook", __FUNCTION__);
     goto lFail;
   }
 
@@ -1489,7 +1489,7 @@ globalHookHandler(TSCont contp, TSEvent event, void *edata)
     TSDebug(DEBUG_TAG, "[%s] handling read request header event...", __FUNCTION__);
     if (intercept_req) {
       if (!setupServerIntercept(txnp)) {
-        TSError("[%s] Could not setup server intercept", __FUNCTION__);
+        TSError("[esi][%s] Could not setup server intercept", __FUNCTION__);
       } else {
         TSDebug(DEBUG_TAG, "[%s] Setup server intercept", __FUNCTION__);
       }
@@ -1557,7 +1557,7 @@ loadHandlerConf(const char *file_name, Utils::KeyValueMap &handler_conf)
     Utils::parseKeyValueConfig(conf_lines, handler_conf);
     TSDebug(DEBUG_TAG, "[%s] Loaded handler conf file [%s]", __FUNCTION__, file_name);
   } else {
-    TSError("[%s] Failed to open handler config file [%s]", __FUNCTION__, file_name);
+    TSError("[esi][%s] Failed to open handler config file [%s]", __FUNCTION__, file_name);
   }
 }
 
@@ -1621,7 +1621,7 @@ esiPluginInit(int argc, const char *argv[], struct OptionInfo *pOptionInfo)
   if (threadKey == 0) {
     bKeySet = true;
     if ((result = pthread_key_create(&threadKey, NULL)) != 0) {
-      TSError("[%s] Could not create key", __FUNCTION__);
+      TSError("[esi][%s] Could not create key", __FUNCTION__);
       TSDebug(DEBUG_TAG, "[%s] Could not create key", __FUNCTION__);
     }
   } else {
@@ -1642,9 +1642,19 @@ esiPluginInit(int argc, const char *argv[], struct OptionInfo *pOptionInfo)
 void
 TSPluginInit(int argc, const char *argv[])
 {
+  TSPluginRegistrationInfo info;
+  info.plugin_name = (char *)"esi";
+  info.vendor_name = (char *)"Apache Software Foundation";
+  info.support_email = (char *)"dev@trafficserver.apache.org";
+
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[esi][%s] plugin registration failed.", __FUNCTION__);
+    return;
+  }
+
   struct OptionInfo *pOptionInfo = (struct OptionInfo *)TSmalloc(sizeof(struct OptionInfo));
   if (pOptionInfo == NULL) {
-    TSError("[%s] malloc %d bytes fail", __FUNCTION__, (int)sizeof(struct OptionInfo));
+    TSError("[esi][%s] malloc %d bytes fail", __FUNCTION__, (int)sizeof(struct OptionInfo));
     return;
   }
   if (esiPluginInit(argc, argv, pOptionInfo) != 0) {
@@ -1654,7 +1664,7 @@ TSPluginInit(int argc, const char *argv[])
 
   TSCont global_contp = TSContCreate(globalHookHandler, NULL);
   if (!global_contp) {
-    TSError("[%s] Could not create global continuation", __FUNCTION__);
+    TSError("[esi][%s] Could not create global continuation", __FUNCTION__);
     TSfree(pOptionInfo);
     return;
   }
@@ -1673,13 +1683,13 @@ TSRemapInit(TSRemapInterface *api_info, char *errbuf, int errbuf_size)
 {
   if (!api_info) {
     snprintf(errbuf, errbuf_size, "[TSRemapInit] - Invalid TSRemapInterface argument");
-    TSError("[TSRemapInit] - Invalid TSRemapInterface argument");
+    TSError("[esi][TSRemapInit] - Invalid TSRemapInterface argument");
     return TS_ERROR;
   }
 
   if (api_info->size < sizeof(TSRemapInterface)) {
     snprintf(errbuf, errbuf_size, "[TSRemapInit] - Incorrect size of TSRemapInterface structure");
-    TSError("[TSRemapInit] - Incorrect size of TSRemapInterface structure");
+    TSError("[esi][TSRemapInit] - Incorrect size of TSRemapInterface structure");
     return TS_ERROR;
   }
 
@@ -1694,7 +1704,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_s
     snprintf(errbuf, errbuf_size, "Unable to create remap instance, "
                                   "argc: %d < 2",
              argc);
-    TSError("Unable to create remap instance! argc: %d < 2", argc);
+    TSError("[esi]Unable to create remap instance! argc: %d < 2", argc);
     return TS_ERROR;
   }
 
@@ -1710,7 +1720,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char *errbuf, int errbuf_s
   struct OptionInfo *pOptionInfo = (struct OptionInfo *)TSmalloc(sizeof(struct OptionInfo));
   if (pOptionInfo == NULL) {
     snprintf(errbuf, errbuf_size, "malloc %d bytes fail", (int)sizeof(struct OptionInfo));
-    TSError("[%s] malloc %d bytes fail", __FUNCTION__, (int)sizeof(struct OptionInfo));
+    TSError("[esi][%s] malloc %d bytes fail", __FUNCTION__, (int)sizeof(struct OptionInfo));
     return TS_ERROR;
   }
   if (esiPluginInit(index, new_argv, pOptionInfo) != 0) {
@@ -1747,7 +1757,7 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri ATS_UNUSED
 
     if (isInterceptRequest(txnp)) {
       if (!setupServerIntercept(txnp)) {
-        TSError("[%s] Could not setup server intercept", __FUNCTION__);
+        TSError("[esi][%s] Could not setup server intercept", __FUNCTION__);
       } else {
         TSDebug(DEBUG_TAG, "[%s] Setup server intercept", __FUNCTION__);
       }
