@@ -19,8 +19,8 @@
   limitations under the License.
  */
 
-#include "ink_defs.h"
-#include "ink_platform.h"
+#include "ts/ink_defs.h"
+#include "ts/ink_platform.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -487,7 +487,7 @@ TSPluginInit(int argc, const char *argv[])
   }
 
   if (!pstate->config_file) {
-    TSError("Plugin requires a --config option along with a config file name.");
+    TSError("[regex_revalidate] Plugin requires a --config option along with a config file name.");
     free_plugin_state_t(pstate);
     return;
   }
@@ -503,15 +503,16 @@ TSPluginInit(int argc, const char *argv[])
   info.vendor_name = "Apache Software Foundation";
   info.support_email = "dev@trafficserver.apache.org";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
-    TSError("Plugin registration failed.");
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[regex_revalidate] Plugin registration failed.");
+
     free_plugin_state_t(pstate);
     return;
   } else
     TSDebug(LOG_PREFIX, "Plugin registration succeeded.");
 
   if (!check_ts_version()) {
-    TSError("Plugin requires Traffic Server %d.%d.%d", TS_VERSION_MAJOR, TS_VERSION_MINOR, TS_VERSION_MICRO);
+    TSError("[regex_revalidate] Plugin requires Traffic Server %d.%d.%d", TS_VERSION_MAJOR, TS_VERSION_MINOR, TS_VERSION_MICRO);
     free_plugin_state_t(pstate);
     return;
   }
