@@ -24,6 +24,7 @@
 #include <string.h>
 #include <getopt.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <limits.h>
 #include <ctype.h>
 
@@ -149,7 +150,7 @@ bool
 S3Config::parse_config(const char *config)
 {
   if (!config) {
-    TSError("%s: called without a config file, this is broken", PLUGIN_NAME);
+    TSError("[%s] called without a config file, this is broken", PLUGIN_NAME);
     return false;
   } else {
     char filename[PATH_MAX + 1];
@@ -163,7 +164,7 @@ S3Config::parse_config(const char *config)
     FILE *file = fopen(config, "r");
 
     if (NULL == file) {
-      TSError("%s: unable to open %s", PLUGIN_NAME, config);
+      TSError("[%s] unable to open %s", PLUGIN_NAME, config);
       return false;
     }
 
@@ -505,7 +506,7 @@ TSRemapNewInstance(int argc, char *argv[], void **ih, char * /* errbuf ATS_UNUSE
 
   // Make sure we got both the shared secret and the AWS secret
   if (!s3->valid()) {
-    TSError("%s: requires both shared and AWS secret configuration", PLUGIN_NAME);
+    TSError("[%s] requires both shared and AWS secret configuration", PLUGIN_NAME);
     delete s3;
     *ih = NULL;
     return TS_ERROR;
@@ -535,8 +536,8 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri */)
 {
   S3Config *s3 = static_cast<S3Config *>(ih);
 
-  TSAssert(s3->valid());
   if (s3) {
+    TSAssert(s3->valid());
     // Now schedule the continuation to update the URL when going to origin.
     // Note that in most cases, this is a No-Op, assuming you have reasonable
     // cache hit ratio. However, the scheduling is next to free (very cheap).
@@ -545,26 +546,10 @@ TSRemapDoRemap(void *ih, TSHttpTxn txnp, TSRemapRequestInfo * /* rri */)
     s3->schedule(txnp);
   } else {
     TSDebug(PLUGIN_NAME, "Remap context is invalid");
-    TSError("%s: No remap context available, check code / config", PLUGIN_NAME);
+    TSError("[%s] No remap context available, check code / config", PLUGIN_NAME);
     TSHttpTxnSetHttpRetStatus(txnp, TS_HTTP_STATUS_INTERNAL_SERVER_ERROR);
   }
 
   // This plugin actually doesn't do anything with remapping. Ever.
   return TSREMAP_NO_REMAP;
 }
-
-
-/*
-  local variables:
-  mode: C++
-  indent-tabs-mode: nil
-  c-basic-offset: 2
-  c-comment-only-line-offset: 0
-  c-file-offsets: ((statement-block-intro . +)
-  (label . 0)
-  (statement-cont . +)
-  (innamespace . 0))
-  end:
-
-  Indent with: /usr/bin/indent -ncs -nut -npcs -l 120 logstats.cc
-*/
