@@ -30,7 +30,7 @@
 #include <getopt.h>
 #include <arpa/inet.h>
 
-#include "ink_defs.h"
+#include "ts/ink_defs.h"
 #include "ts/ts.h"
 #include "ts/experimental.h"
 
@@ -268,6 +268,7 @@ consume_resource(TSCont cont, TSEvent event ATS_UNUSED, void *edata ATS_UNUSED)
   case TS_EVENT_VCONN_WRITE_READY:
     // We shouldn't get here because we specify the exact size of the buffer.
     TSDebug(PLUGIN_NAME, "Write Ready");
+    break;
   case TS_EVENT_VCONN_WRITE_COMPLETE:
     TSDebug(PLUGIN_NAME, "Write Complete");
     // TSDebug(PLUGIN_NAME, "TSVConnShutdown()");
@@ -352,7 +353,7 @@ consume_resource(TSCont cont, TSEvent event ATS_UNUSED, void *edata ATS_UNUSED)
     TSContDestroy(cont);
     break;
   default:
-    TSError("Unknown event %d.", event);
+    TSError("[stale_while_revalidate] Unknown event %d.", event);
     break;
   }
 
@@ -391,7 +392,7 @@ fetch_resource(TSCont cont, TSEvent event ATS_UNUSED, void *edata ATS_UNUSED)
 
   if (state) {
     TSDebug(PLUGIN_NAME, "Lets do the lookup");
-    consume_cont = TSContCreate(consume_resource, NULL);
+    consume_cont = TSContCreate(consume_resource, TSMutexCreate());
     TSContDataSet(consume_cont, (void *)state);
 
     if (state->async_req) {
@@ -473,7 +474,7 @@ main_plugin(TSCont cont, TSEvent event, void *edata)
   // Is this the proper event?
   case TS_EVENT_HTTP_READ_REQUEST_HDR:
 
-    if (TSHttpIsInternalRequest(txn) != TS_SUCCESS) {
+    if (TSHttpTxnIsInternal(txn) != TS_SUCCESS) {
       TSDebug(PLUGIN_NAME, "External Request");
       plugin_config = (config_t *)TSContDataGet(cont);
       state = TSmalloc(sizeof(StateInfo));
@@ -484,7 +485,7 @@ main_plugin(TSCont cont, TSEvent event, void *edata)
       TSHttpTxnHookAdd(txn, TS_HTTP_CACHE_LOOKUP_COMPLETE_HOOK, cont);
     } else {
       TSDebug(PLUGIN_NAME, "Internal Request"); // This is insufficient if there are other plugins using TSHttpConnect
-      TSHttpTxnConfigIntSet(txn, TS_CONFIG_HTTP_SHARE_SERVER_SESSIONS, 1);
+      TSHttpTxnConfigIntSet(txn, TS_CONFIG_HTTP_SERVER_SESSION_SHARING_MATCH, TS_SERVER_SESSION_SHARING_MATCH_NONE);
       // TSHttpTxnHookAdd(txn, TS_HTTP_CACHE_LOOKUP_COMPLETE_HOOK, cont);
       TSHttpTxnHookAdd(txn, TS_HTTP_READ_RESPONSE_HDR_HOOK, cont);
       // This might be needed in 3.2.0 to fix a timeout issue
@@ -527,7 +528,7 @@ main_plugin(TSCont cont, TSEvent event, void *edata)
           state->async_req = true;
           TSHttpTxnCacheLookupStatusSet(txn, TS_CACHE_LOOKUP_HIT_FRESH);
           // TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE);
-          fetch_cont = TSContCreate(fetch_resource, NULL);
+          fetch_cont = TSContCreate(fetch_resource, TSMutexCreate());
           TSContDataSet(fetch_cont, (void *)state);
           TSContSchedule(fetch_cont, 0, TS_THREAD_POOL_TASK);
           TSHttpTxnReenable(txn, TS_EVENT_HTTP_CONTINUE);
@@ -540,7 +541,7 @@ main_plugin(TSCont cont, TSEvent event, void *edata)
           state->async_req = false;
           state->txn = txn;
           state->main_cont = cont; // we need this for the warning header callback. not sure i like it, but it works.
-          fetch_cont = TSContCreate(fetch_resource, NULL);
+          fetch_cont = TSContCreate(fetch_resource, TSMutexCreate());
           TSContDataSet(fetch_cont, (void *)state);
           TSContSchedule(fetch_cont, 0, TS_THREAD_POOL_NET);
         } else {
@@ -615,8 +616,9 @@ TSPluginInit(int argc, const char *argv[])
   info.vendor_name = "Apache Software Foundation";
   info.support_email = "dev@trafficserver.apache.org";
 
-  if (TSPluginRegister(TS_SDK_VERSION_3_0, &info) != TS_SUCCESS) {
-    TSError("Plugin registration failed.\n");
+  if (TSPluginRegister(&info) != TS_SUCCESS) {
+    TSError("[%s] Plugin registration failed.\n", PLUGIN_NAME);
+
     return;
   } else {
     TSDebug(PLUGIN_NAME, "Plugin registration succeeded.\n");
