@@ -103,7 +103,7 @@ ts_lua_cached_response_get_status(lua_State *L)
   int status;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CACHED_RESPONSE_HDR(http_ctx);
 
@@ -123,7 +123,7 @@ ts_lua_cached_response_get_version(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CACHED_RESPONSE_HDR(http_ctx);
 
@@ -150,7 +150,7 @@ ts_lua_cached_response_header_get(lua_State *L)
   TSMLoc field_loc;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   /*   we skip the first argument that is the table */
   key = luaL_checklstring(L, 2, &key_len);
@@ -193,7 +193,7 @@ ts_lua_cached_response_get_headers(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CACHED_RESPONSE_HDR(http_ctx);
 
