@@ -17,7 +17,7 @@
 */
 
 
-#include "ink_platform.h"
+#include "ts/ink_platform.h"
 #include <netinet/in.h>
 #include "ts_lua_util.h"
 
@@ -63,7 +63,7 @@ static void ts_lua_inject_client_request_header_size_api(lua_State *L);
 static int ts_lua_client_request_client_addr_get_ip(lua_State *L);
 static int ts_lua_client_request_client_addr_get_port(lua_State *L);
 static int ts_lua_client_request_client_addr_get_addr(lua_State *L);
-
+static int ts_lua_client_request_client_addr_get_incoming_port(lua_State *L);
 
 void
 ts_lua_inject_client_request_api(lua_State *L)
@@ -105,6 +105,9 @@ ts_lua_inject_client_request_client_addr_api(lua_State *L)
   lua_pushcfunction(L, ts_lua_client_request_client_addr_get_addr);
   lua_setfield(L, -2, "get_addr");
 
+  lua_pushcfunction(L, ts_lua_client_request_client_addr_get_incoming_port);
+  lua_setfield(L, -2, "get_incoming_port");
+
   lua_setfield(L, -2, "client_addr");
 }
 
@@ -142,7 +145,7 @@ ts_lua_client_request_header_get(lua_State *L)
   TSMLoc field_loc;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   /*  we skip the first argument that is the table */
   key = luaL_checklstring(L, 2, &key_len);
@@ -178,7 +181,7 @@ ts_lua_client_request_header_set(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   remove = 0;
   val = NULL;
@@ -203,7 +206,7 @@ ts_lua_client_request_header_set(lua_State *L)
 
   } else if (TSMimeHdrFieldCreateNamed(http_ctx->client_request_bufp, http_ctx->client_request_hdrp, key, key_len, &field_loc) !=
              TS_SUCCESS) {
-    TSError("[%s] TSMimeHdrFieldCreateNamed error", __FUNCTION__);
+    TSError("[ts_lua][%s] TSMimeHdrFieldCreateNamed error", __FUNCTION__);
     return 0;
 
   } else {
@@ -236,7 +239,7 @@ ts_lua_client_request_get_headers(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   lua_newtable(L);
 
@@ -302,7 +305,7 @@ ts_lua_client_request_get_url(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   url = TSHttpTxnEffectiveUrlStringGet(http_ctx->txnp, &url_len);
 
@@ -328,7 +331,7 @@ ts_lua_client_request_get_pristine_url(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   if (TSHttpTxnPristineUrlGet(http_ctx->txnp, &bufp, &url_loc) != TS_SUCCESS)
     return 0;
@@ -356,7 +359,7 @@ ts_lua_client_request_get_url_host(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   host = TSUrlHostGet(http_ctx->client_request_bufp, http_ctx->client_request_url, &len);
 
@@ -395,7 +398,7 @@ ts_lua_client_request_set_url_host(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   host = luaL_checklstring(L, 1, &len);
 
@@ -411,7 +414,7 @@ ts_lua_client_request_get_url_port(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   port = TSUrlPortGet(http_ctx->client_request_bufp, http_ctx->client_request_url);
 
@@ -427,7 +430,7 @@ ts_lua_client_request_set_url_port(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   port = luaL_checkint(L, 1);
 
@@ -444,7 +447,7 @@ ts_lua_client_request_get_url_scheme(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   scheme = TSUrlSchemeGet(http_ctx->client_request_bufp, http_ctx->client_request_url, &len);
 
@@ -461,7 +464,7 @@ ts_lua_client_request_set_url_scheme(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   scheme = luaL_checklstring(L, 1, &len);
 
@@ -480,7 +483,7 @@ ts_lua_client_request_get_uri(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   path = TSUrlPathGet(http_ctx->client_request_bufp, http_ctx->client_request_url, &path_len);
 
@@ -503,7 +506,7 @@ ts_lua_client_request_set_uri(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   path = luaL_checklstring(L, 1, &path_len);
 
@@ -536,7 +539,7 @@ ts_lua_client_request_get_uri_args(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   param = TSUrlHttpQueryGet(http_ctx->client_request_bufp, http_ctx->client_request_url, &param_len);
 
@@ -557,7 +560,7 @@ ts_lua_client_request_set_uri_args(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   param = luaL_checklstring(L, 1, &param_len);
   TSUrlHttpQuerySet(http_ctx->client_request_bufp, http_ctx->client_request_url, param, param_len);
@@ -572,7 +575,7 @@ ts_lua_client_request_client_addr_get_ip(lua_State *L)
   char cip[128];
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   client_ip = TSHttpTxnClientAddrGet(http_ctx->txnp);
 
@@ -599,7 +602,7 @@ ts_lua_client_request_client_addr_get_port(lua_State *L)
   ts_lua_http_ctx *http_ctx;
   int port;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   client_ip = TSHttpTxnClientAddrGet(http_ctx->txnp);
 
@@ -619,6 +622,33 @@ ts_lua_client_request_client_addr_get_port(lua_State *L)
   return 1;
 }
 
+static int
+ts_lua_client_request_client_addr_get_incoming_port(lua_State *L)
+{
+  struct sockaddr const *incoming_addr;
+  ts_lua_http_ctx *http_ctx;
+  int port;
+
+  GET_HTTP_CONTEXT(http_ctx, L);
+
+  incoming_addr = TSHttpTxnIncomingAddrGet(http_ctx->txnp);
+
+  if (incoming_addr == NULL) {
+    lua_pushnil(L);
+
+  } else {
+    if (incoming_addr->sa_family == AF_INET) {
+      port = ((struct sockaddr_in *)incoming_addr)->sin_port;
+    } else {
+      port = ((struct sockaddr_in6 *)incoming_addr)->sin6_port;
+    }
+
+    lua_pushnumber(L, ntohs(port));
+  }
+
+  return 1;
+}
+
 static int
 ts_lua_client_request_client_addr_get_addr(lua_State *L)
 {
@@ -628,7 +658,7 @@ ts_lua_client_request_client_addr_get_addr(lua_State *L)
   int family;
   char cip[128];
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   client_ip = TSHttpTxnClientAddrGet(http_ctx->txnp);
 
@@ -674,7 +704,7 @@ ts_lua_client_request_get_method(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   method = TSHttpHdrMethodGet(http_ctx->client_request_bufp, http_ctx->client_request_hdrp, &method_len);
 
@@ -695,7 +725,7 @@ ts_lua_client_request_set_method(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   method = luaL_checklstring(L, 1, &method_len);
 
@@ -719,7 +749,7 @@ ts_lua_client_request_get_body_size(lua_State *L)
   int64_t body_size;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   body_size = TSHttpTxnClientReqBodyBytesGet(http_ctx->txnp);
   lua_pushnumber(L, body_size);
@@ -746,7 +776,7 @@ ts_lua_client_request_get_version(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   version = TSHttpHdrVersionGet(http_ctx->client_request_bufp, http_ctx->client_request_hdrp);
 
@@ -769,7 +799,7 @@ ts_lua_client_request_set_version(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   version = luaL_checklstring(L, 1, &len);
 
@@ -793,7 +823,7 @@ ts_lua_client_request_get_header_size(lua_State *L)
   int header_size;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   header_size = TSHttpTxnClientReqHdrBytesGet(http_ctx->txnp);
   lua_pushnumber(L, header_size);
