@@ -89,7 +89,7 @@ ts_lua_client_response_header_get(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   /*  we skip the first argument that is the table */
   key = luaL_checklstring(L, 2, &key_len);
@@ -132,7 +132,7 @@ ts_lua_client_response_header_set(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   remove = 0;
   val = NULL;
@@ -163,7 +163,7 @@ ts_lua_client_response_header_set(lua_State *L)
 
   } else if (TSMimeHdrFieldCreateNamed(http_ctx->client_response_bufp, http_ctx->client_response_hdrp, key, key_len, &field_loc) !=
              TS_SUCCESS) {
-    TSError("[%s] TSMimeHdrFieldCreateNamed error", __FUNCTION__);
+    TSError("[ts_lua][%s] TSMimeHdrFieldCreateNamed error", __FUNCTION__);
     return 0;
 
   } else {
@@ -196,7 +196,7 @@ ts_lua_client_response_get_headers(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CLIENT_RESPONSE_HDR(http_ctx);
 
@@ -247,7 +247,7 @@ ts_lua_client_response_get_status(lua_State *L)
   int status;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CLIENT_RESPONSE_HDR(http_ctx);
 
@@ -267,7 +267,7 @@ ts_lua_client_response_set_status(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CLIENT_RESPONSE_HDR(http_ctx);
 
@@ -291,7 +291,7 @@ ts_lua_client_response_get_version(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CLIENT_RESPONSE_HDR(http_ctx);
 
@@ -316,7 +316,7 @@ ts_lua_client_response_set_version(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_CLIENT_RESPONSE_HDR(http_ctx);
 
@@ -343,7 +343,8 @@ ts_lua_client_response_set_error_resp(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
+
   TS_LUA_CHECK_CLIENT_RESPONSE_HDR(http_ctx);
 
   n = lua_gettop(L);
