@@ -29,6 +29,12 @@ typedef struct {
 } ts_lua_package_path;
 
 
+static int path_cnt = 0;
+static ts_lua_package_path path[TS_LUA_MAX_PACKAGE_NUM];
+
+static int cpath_cnt = 0;
+static ts_lua_package_path cpath[TS_LUA_MAX_PACKAGE_NUM];
+
 static int g_path_cnt = 0;
 static ts_lua_package_path g_path[TS_LUA_MAX_PACKAGE_NUM];
 
@@ -88,20 +94,38 @@ ts_lua_add_package_path(lua_State *L)
     }
 
     if (item_len > 0) {
-      for (i = 0; i < g_path_cnt; i++) {
-        if (g_path[i].len == item_len && memcmp(g_path[i].name, ptr, item_len) == 0) // exist
-        {
-          break;
+      if (!conf->remap) {
+        for (i = 0; i < g_path_cnt; i++) {
+          if (g_path[i].len == item_len && memcmp(g_path[i].name, ptr, item_len) == 0) // exist
+          {
+            break;
+          }
         }
-      }
 
-      if (i >= g_path_cnt) {
-        if (n + i >= TS_LUA_MAX_PACKAGE_NUM)
-          return luaL_error(L, "extended package path number exceeds %d.", TS_LUA_MAX_PACKAGE_NUM);
+        if (i >= g_path_cnt) {
+          if (n + i >= TS_LUA_MAX_PACKAGE_NUM)
+            return luaL_error(L, "extended package path number exceeds %d.", TS_LUA_MAX_PACKAGE_NUM);
 
-        pp[n].name = (char *)ptr;
-        pp[n].len = item_len;
-        n++;
+          pp[n].name = (char *)ptr;
+          pp[n].len = item_len;
+          n++;
+        }
+      } else {
+        for (i = 0; i < path_cnt; i++) {
+          if (path[i].len == item_len && memcmp(path[i].name, ptr, item_len) == 0) // exist
+          {
+            break;
+          }
+        }
+
+        if (i >= path_cnt) {
+          if (n + i >= TS_LUA_MAX_PACKAGE_NUM)
+            return luaL_error(L, "extended package path number exceeds %d.", TS_LUA_MAX_PACKAGE_NUM);
+
+          pp[n].name = (char *)ptr;
+          pp[n].len = item_len;
+          n++;
+        }
       }
     }
 
@@ -112,7 +136,11 @@ ts_lua_add_package_path(lua_State *L)
     ts_lua_add_package_path_items(L, pp, n);
 
     if (conf->_last) {
-      elt = &g_path[g_path_cnt];
+      if (!conf->remap) {
+        elt = &g_path[g_path_cnt];
+      } else {
+        elt = &path[path_cnt];
+      }
 
       for (i = 0; i < n; i++) {
         elt->len = pp[i].len;
@@ -121,7 +149,11 @@ ts_lua_add_package_path(lua_State *L)
         elt++;
       }
 
-      g_path_cnt += n;
+      if (!conf->remap) {
+        g_path_cnt += n;
+      } else {
+        path_cnt += n;
+      }
     }
   }
 
@@ -154,7 +186,7 @@ ts_lua_add_package_path_items(lua_State *L, ts_lua_package_path *pp, int n)
 
   for (i = 0; i < n; i++) {
     if (new_path_len + pp[i].len + 1 >= sizeof(new_path)) {
-      TSError("extended package.path is too long.");
+      TSError("[ts_lua] Extended package.path is too long.");
       return -1;
     }
 
@@ -208,20 +240,38 @@ ts_lua_add_package_cpath(lua_State *L)
     }
 
     if (item_len > 0) {
-      for (i = 0; i < g_cpath_cnt; i++) {
-        if (g_cpath[i].len == item_len && memcmp(g_cpath[i].name, ptr, item_len) == 0) // exist
-        {
-          break;
+      if (!conf->remap) {
+        for (i = 0; i < g_cpath_cnt; i++) {
+          if (g_cpath[i].len == item_len && memcmp(g_cpath[i].name, ptr, item_len) == 0) // exist
+          {
+            break;
+          }
         }
-      }
 
-      if (i >= g_cpath_cnt) {
-        if (n + i >= TS_LUA_MAX_PACKAGE_NUM)
-          return luaL_error(L, "extended package cpath number exceeds %d.", TS_LUA_MAX_PACKAGE_NUM);
+        if (i >= g_cpath_cnt) {
+          if (n + i >= TS_LUA_MAX_PACKAGE_NUM)
+            return luaL_error(L, "extended package cpath number exceeds %d.", TS_LUA_MAX_PACKAGE_NUM);
 
-        pp[n].name = (char *)ptr;
-        pp[n].len = item_len;
-        n++;
+          pp[n].name = (char *)ptr;
+          pp[n].len = item_len;
+          n++;
+        }
+      } else {
+        for (i = 0; i < cpath_cnt; i++) {
+          if (cpath[i].len == item_len && memcmp(cpath[i].name, ptr, item_len) == 0) // exist
+          {
+            break;
+          }
+        }
+
+        if (i >= cpath_cnt) {
+          if (n + i >= TS_LUA_MAX_PACKAGE_NUM)
+            return luaL_error(L, "extended package cpath number exceeds %d.", TS_LUA_MAX_PACKAGE_NUM);
+
+          pp[n].name = (char *)ptr;
+          pp[n].len = item_len;
+          n++;
+        }
       }
     }
 
@@ -232,7 +282,11 @@ ts_lua_add_package_cpath(lua_State *L)
     ts_lua_add_package_cpath_items(L, pp, n);
 
     if (conf->_last) {
-      elt = &g_cpath[g_cpath_cnt];
+      if (!conf->remap) {
+        elt = &g_cpath[g_cpath_cnt];
+      } else {
+        elt = &cpath[cpath_cnt];
+      }
 
       for (i = 0; i < n; i++) {
         elt->len = pp[i].len;
@@ -241,7 +295,11 @@ ts_lua_add_package_cpath(lua_State *L)
         elt++;
       }
 
-      g_cpath_cnt += n;
+      if (!conf->remap) {
+        g_cpath_cnt += n;
+      } else {
+        cpath_cnt += n;
+      }
     }
   }
 
@@ -274,7 +332,7 @@ ts_lua_add_package_cpath_items(lua_State *L, ts_lua_package_path *pp, int n)
 
   for (i = 0; i < n; i++) {
     if (new_path_len + pp[i].len + 1 >= sizeof(new_path)) {
-      TSError("extended package.cpath is too long.");
+      TSError("[ts_lua] Extended package.cpath is too long.");
       return -1;
     }
 
