@@ -16,7 +16,7 @@
   limitations under the License.
 */
 
-#include "ink_platform.h"
+#include "ts/ink_platform.h"
 #include <netinet/in.h>
 #include "ts_lua_util.h"
 
@@ -137,7 +137,7 @@ ts_lua_server_request_header_get(lua_State *L)
   TSMLoc field_loc;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   /*  we skip the first argument that is the table */
   key = luaL_checklstring(L, 2, &key_len);
@@ -180,7 +180,7 @@ ts_lua_server_request_header_set(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   remove = 0;
   val = NULL;
@@ -211,7 +211,7 @@ ts_lua_server_request_header_set(lua_State *L)
 
   } else if (TSMimeHdrFieldCreateNamed(http_ctx->server_request_bufp, http_ctx->server_request_hdrp, key, key_len, &field_loc) !=
              TS_SUCCESS) {
-    TSError("[%s] TSMimeHdrFieldCreateNamed error", __FUNCTION__);
+    TSError("[ts_lua][%s] TSMimeHdrFieldCreateNamed error", __FUNCTION__);
     return 0;
 
   } else {
@@ -244,7 +244,7 @@ ts_lua_server_request_get_headers(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_SERVER_REQUEST_HDR(http_ctx);
 
@@ -282,7 +282,7 @@ ts_lua_server_request_get_header_size(lua_State *L)
   int header_size;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   header_size = TSHttpTxnServerReqHdrBytesGet(http_ctx->txnp);
   lua_pushnumber(L, header_size);
@@ -303,7 +303,7 @@ ts_lua_server_request_get_body_size(lua_State *L)
   int64_t body_size;
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   body_size = TSHttpTxnServerReqBodyBytesGet(http_ctx->txnp);
   lua_pushnumber(L, body_size);
@@ -331,7 +331,7 @@ ts_lua_server_request_get_uri(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_SERVER_REQUEST_URL(http_ctx);
 
@@ -356,7 +356,7 @@ ts_lua_server_request_set_uri(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_SERVER_REQUEST_URL(http_ctx);
 
@@ -390,7 +390,7 @@ ts_lua_server_request_set_uri_args(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_SERVER_REQUEST_URL(http_ctx);
 
@@ -408,7 +408,7 @@ ts_lua_server_request_get_uri_args(lua_State *L)
 
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   TS_LUA_CHECK_SERVER_REQUEST_URL(http_ctx);
 
@@ -431,7 +431,7 @@ ts_lua_server_request_server_addr_get_ip(lua_State *L)
   char sip[128];
   ts_lua_http_ctx *http_ctx;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   server_ip = TSHttpTxnServerAddrGet(http_ctx->txnp);
 
@@ -458,7 +458,7 @@ ts_lua_server_request_server_addr_get_port(lua_State *L)
   ts_lua_http_ctx *http_ctx;
   int port;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   server_ip = TSHttpTxnServerAddrGet(http_ctx->txnp);
 
@@ -485,7 +485,7 @@ ts_lua_server_request_server_addr_get_outgoing_port(lua_State *L)
   ts_lua_http_ctx *http_ctx;
   int port;
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   outgoing_addr = TSHttpTxnOutgoingAddrGet(http_ctx->txnp);
 
@@ -514,7 +514,7 @@ ts_lua_server_request_server_addr_get_addr(lua_State *L)
   int family;
   char sip[128];
 
-  http_ctx = ts_lua_get_http_ctx(L);
+  GET_HTTP_CONTEXT(http_ctx, L);
 
   server_ip = TSHttpTxnServerAddrGet(http_ctx->txnp);
 
