@@ -61,7 +61,7 @@ Condition::initialize(Parser &p)
 
   if (p.mod_exist("OR")) {
     if (p.mod_exist("AND")) {
-      TSError("%s: Can't have both AND and OR in mods", PLUGIN_NAME);
+      TSError("[%s] Can't have both AND and OR in mods", PLUGIN_NAME);
     } else {
       _mods = static_cast<CondModifiers>(_mods | COND_OR);
     }
