@@ -28,7 +28,7 @@
 
 ****************************************************************************/
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "Main.h"
 #include "P_EventSystem.h"
 #include "P_Cache.h"
@@ -293,7 +293,7 @@ ICPPeerReadCont::init(ICPProcessor *ICPpr, Peer *p, int lookup_local)
 {
   PeerReadData *s = PeerReadDataAllocator.alloc();
   s->init();
-  s->_start_time = ink_get_hrtime();
+  s->_start_time = Thread::get_hrtime();
   s->_peer = p;
   s->_next_state = READ_ACTIVE;
   s->_cache_lookup_local = lookup_local;
@@ -453,16 +453,21 @@ ICPPeerReadCont::ICPPeerQueryCont(int /* event ATS_UNUSED */, Event * /* e ATS_U
 
   SET_HANDLER((ICPPeerReadContHandler)&ICPPeerReadCont::ICPPeerQueryEvent);
   if (_state->_rICPmsg->un.query.URL && *_state->_rICPmsg->un.query.URL) {
+    HttpCacheKey key;
+    ICPConfigData *cfg = _ICPpr->GetConfig()->globalConfig();
+
+    Cache::generate_key(&key, &_state->_cachelookupURL, cfg->ICPCacheGeneration());
     _state->_queryResult = ~CACHE_EVENT_LOOKUP_FAILED;
-    _start_time = ink_get_hrtime();
-    if (pluginFreshnessCalcFunc && _ICPpr->GetConfig()->globalConfig()->ICPStaleLookup()) {
+    _start_time = Thread::get_hrtime();
+
+    if (pluginFreshnessCalcFunc && cfg->ICPStaleLookup()) {
       //////////////////////////////////////////////////////////////
       // Note: _cache_lookup_local is ignored in this case, since
       //       cache clustering is not used with stale lookup.
       //////////////////////////////////////////////////////////////
-      a = cacheProcessor.open_read(this, &_state->_cachelookupURL, false, &gclient_request, &global_cache_lookup_config, (time_t)0);
+      a = cacheProcessor.open_read(this, &key, false, &gclient_request, &global_cache_lookup_config, (time_t)0);
     } else {
-      a = cacheProcessor.lookup(this, &_state->_cachelookupURL, false, _state->_cache_lookup_local);
+      a = cacheProcessor.lookup(this, &key, false, _state->_cache_lookup_local);
     }
     if (!a) {
       a = ACTION_IO_ERROR;
@@ -906,7 +911,7 @@ ICPPeerReadCont::PeerReadStateMachine(PeerReadData *s, Event *e)
 
       MUTEX_UNTAKE_LOCK(ICPReqContMutex, ethread);
       if (request_start_time) {
-        ICP_SUM_DYN_STAT(total_icp_response_time_stat, (ink_get_hrtime() - request_start_time));
+        ICP_SUM_DYN_STAT(total_icp_response_time_stat, (Thread::get_hrtime() - request_start_time));
       }
       RECORD_ICP_STATE_CHANGE(s, 0, READ_NOT_ACTIVE);
       s->_next_state = READ_NOT_ACTIVE;
@@ -931,7 +936,7 @@ ICPPeerReadCont::PeerReadStateMachine(PeerReadData *s, Event *e)
       } else {
         // Last read was valid, see if any more read data before exiting
         s->reset();
-        s->_start_time = ink_get_hrtime();
+        s->_start_time = Thread::get_hrtime();
         s->_next_state = READ_ACTIVE;
         RECORD_ICP_STATE_CHANGE(s, 0, READ_ACTIVE);
         break; // restart
@@ -1294,7 +1299,7 @@ ICPRequestCont::ICPStateMachine(int event, void *d)
         _cont->handleEvent(_ret_status, (void *)&_ret_sockaddr);
       }
       MUTEX_UNTAKE_LOCK(mutex, this_ethread());
-      ICP_SUM_DYN_STAT(total_icp_request_time_stat, (ink_get_hrtime() - _start_time));
+      ICP_SUM_DYN_STAT(total_icp_request_time_stat, (Thread::get_hrtime() - _start_time));
 
       _next_state = ICP_WAIT_SEND_COMPLETE;
       break; // move to next_state
@@ -2204,8 +2209,10 @@ ICPProcessor::CancelPendingReads()
   r->_ICPmsg.h.version = ~r->_ICPmsg.h.version; // bogus message
 
   Peer *lp = GetLocalPeer();
-  r->_sendMsgHdr.msg_name = (caddr_t) & (lp->GetSendChan())->addr;
-  r->_sendMsgHdr.msg_namelen = sizeof((lp->GetSendChan())->addr);
+  IpEndpoint local_endpoint;
+  ats_ip_copy(&local_endpoint.sa, lp->GetIP());
+  r->_sendMsgHdr.msg_name = (caddr_t)&local_endpoint;
+  r->_sendMsgHdr.msg_namelen = sizeof(local_endpoint);
   udpNet.sendmsg_re(r, r, lp->GetSendFD(), &r->_sendMsgHdr);
 }
 
