@@ -35,20 +35,18 @@
 #include "ProxyConfig.h"
 #include "ControlBase.h"
 #include "ControlMatcher.h"
-
-#include "ink_apidefs.h"
-
 #include "P_RecProcess.h"
-
-#include "libts.h"
+#include "ts/ConsistentHash.h"
+#include "ts/Tokenizer.h"
+#include "ts/ink_apidefs.h"
 
 #define MAX_PARENTS 64
 
 struct RequestData;
-
 struct matcher_line;
 struct ParentResult;
 class ParentRecord;
+class ParentSelectionStrategy;
 
 enum ParentResultType {
   PARENT_UNDEFINED,
@@ -58,114 +56,13 @@ enum ParentResultType {
   PARENT_FAIL,
 };
 
-typedef ControlMatcher<ParentRecord, ParentResult> P_table;
-
-//
-// API to outside world
-//
-struct ParentResult {
-  ParentResult()
-    : r(PARENT_UNDEFINED), hostname(NULL), port(0), line_number(0), epoch(NULL), rec(NULL), last_parent(0), start_parent(0),
-      wrap_around(false), retry(false)
-  {
-    memset(foundParents, 0, sizeof(foundParents));
-  };
-
-  // For outside consumption
-  ParentResultType r;
-  const char *hostname;
-  int port;
-
-  // Internal use only
-  //   Not to be modified by HTTP
-  int line_number;
-  P_table *epoch; // A pointer to the table used.
-  ParentRecord *rec;
-  uint32_t last_parent;
-  uint32_t start_parent;
-  bool wrap_around;
-  bool retry;
-  // Arena *a;
-  ATSConsistentHashIter chashIter;
-  bool foundParents[MAX_PARENTS];
+enum ParentRR_t {
+  P_NO_ROUND_ROBIN = 0,
+  P_STRICT_ROUND_ROBIN,
+  P_HASH_ROUND_ROBIN,
+  P_CONSISTENT_HASH,
 };
 
-class HttpRequestData;
-
-struct ParentConfigParams : public ConfigInfo {
-  ParentConfigParams();
-  ~ParentConfigParams();
-
-  // void findParent(RequestData* rdata, ParentResult* result)
-  //
-  //   Does initial parent lookup
-  //
-  inkcoreapi void findParent(HttpRequestData *rdata, ParentResult *result);
-
-  // void markParentDown(ParentResult* rsult)
-  //
-  //    Marks the parent pointed to by result as down
-  //
-  inkcoreapi void markParentDown(ParentResult *result);
-
-  // void recordRetrySuccess
-  //
-  //    After a successful retry, http calls this function
-  //      to clear the bits indicating the parent is down
-  //
-  void recordRetrySuccess(ParentResult *result);
-
-  // void nextParent(RequestData* rdata, ParentResult* result);
-  //
-  //    Marks the parent pointed to by result as down and attempts
-  //      to find the next parent
-  //
-  inkcoreapi void nextParent(HttpRequestData *rdata, ParentResult *result);
-
-  // bool parentExists(HttpRequestData* rdata)
-  //
-  //   Returns true if there is a parent matching the request data and
-  //   false otherwise
-  bool parentExists(HttpRequestData *rdata);
-
-  // bool apiParentExists(HttpRequestData* rdata)
-  //
-  //   Retures true if a parent has been set through the api
-  bool apiParentExists(HttpRequestData *rdata);
-
-  P_table *ParentTable;
-  ParentRecord *DefaultParent;
-  int32_t ParentRetryTime;
-  int32_t ParentEnable;
-  int32_t FailThreshold;
-  int32_t DNS_ParentOnly;
-};
-
-struct ParentConfig {
-public:
-  static void startup();
-  static void reconfigure();
-  static void print();
-
-  inkcoreapi static ParentConfigParams *
-  acquire()
-  {
-    return (ParentConfigParams *)configProcessor.get(ParentConfig::m_id);
-  }
-  inkcoreapi static void
-  release(ParentConfigParams *params)
-  {
-    configProcessor.release(ParentConfig::m_id, params);
-  }
-
-
-  static int m_id;
-};
-//
-// End API to outside world
-//
-
-
 // struct pRecord
 //
 //    A record for an invidual parent
@@ -181,12 +78,7 @@ struct pRecord : ATSConsistentHashNode {
   float weight;
 };
 
-enum ParentRR_t {
-  P_NO_ROUND_ROBIN = 0,
-  P_STRICT_ROUND_ROBIN,
-  P_HASH_ROUND_ROBIN,
-  P_CONSISTENT_HASH,
-};
+typedef ControlMatcher<ParentRecord, ParentResult> P_table;
 
 // class ParentRecord : public ControlBase
 //
@@ -196,17 +88,22 @@ enum ParentRR_t {
 class ParentRecord : public ControlBase
 {
 public:
-  ParentRecord() : parents(NULL), num_parents(0), round_robin(P_NO_ROUND_ROBIN), rr_next(0), go_direct(true), chash(NULL) {}
+  ParentRecord()
+    : parents(NULL), secondary_parents(NULL), num_parents(0), num_secondary_parents(0), ignore_query(false), rr_next(0),
+      go_direct(true), selection_strategy(NULL)
+  {
+  }
 
   ~ParentRecord();
 
   config_parse_error Init(matcher_line *line_info);
   bool DefaultInit(char *val);
   void UpdateMatch(ParentResult *result, RequestData *rdata);
-  void FindParent(bool firstCall, ParentResult *result, RequestData *rdata, ParentConfigParams *config);
   void Print();
   pRecord *parents;
+  pRecord *secondary_parents;
   int num_parents;
+  int num_secondary_parents;
 
   bool
   bypass_ok() const
@@ -216,12 +113,152 @@ public:
 
   const char *scheme;
   // private:
-  const char *ProcessParents(char *val);
-  void buildConsistentHash(void);
-  ParentRR_t round_robin;
+  const char *ProcessParents(char *val, bool isPrimary);
+  bool ignore_query;
   volatile uint32_t rr_next;
   bool go_direct;
-  ATSConsistentHash *chash;
+  ParentSelectionStrategy *selection_strategy;
+};
+
+// If the parent was set by the external customer api,
+//   our HttpRequestData structure told us what parent to
+//   use and we are only called to preserve clean interface
+//   between HttpTransact & the parent selection code.  The following
+ParentRecord *const extApiRecord = (ParentRecord *)0xeeeeffff;
+
+struct ParentResult {
+  ParentResult()
+    : r(PARENT_UNDEFINED), hostname(NULL), port(0), retry(false), line_number(0), epoch(NULL), rec(NULL), last_parent(0),
+      start_parent(0), wrap_around(false), last_lookup(0)
+  {
+  }
+
+  // For outside consumption
+  ParentResultType r;
+  const char *hostname;
+  int port;
+  bool retry;
+
+  // Internal use only
+  //   Not to be modified by HTTP
+  int line_number;
+  P_table *epoch; // A pointer to the table used.
+  ParentRecord *rec;
+  uint32_t last_parent;
+  uint32_t start_parent;
+  bool wrap_around;
+  int last_lookup; // state for for consistent hash.
+};
+
+struct ParentSelectionPolicy {
+  int32_t ParentRetryTime;
+  int32_t ParentEnable;
+  int32_t FailThreshold;
+  int32_t DNS_ParentOnly;
+  ParentSelectionPolicy();
+};
+
+//
+// API definition.
+class ParentSelectionStrategy
+{
+public:
+  // void selectParent(const ParentSelectionPolicy, *policy, bool firstCall, ParentResult *result, RequestData *rdata)
+  //
+  // The implementation parent lookup.
+  //
+  virtual void selectParent(const ParentSelectionPolicy *policy, bool firstCall, ParentResult *result, RequestData *rdata) = 0;
+
+  // void markParentDown(const ParentSelectionPolicy, *policy, ParentResult* rsult)
+  //
+  //    Marks the parent pointed to by result as down
+  //
+  virtual void markParentDown(const ParentSelectionPolicy *policy, ParentResult *result) = 0;
+
+  // uint32_t numParents(ParentResult *result);
+  //
+  // Returns the number of parent records in a strategy.
+  //
+  virtual uint32_t numParents(ParentResult *result) const = 0;
+
+  // void markParentUp
+  //
+  //    After a successful retry, http calls this function
+  //      to clear the bits indicating the parent is down
+  //
+  virtual void markParentUp(ParentResult *result) = 0;
+
+  // virtual destructor.
+  virtual ~ParentSelectionStrategy(){};
+};
+
+class ParentConfigParams : public ConfigInfo
+{
+public:
+  explicit ParentConfigParams(P_table *_parent_table);
+  ~ParentConfigParams(){};
+
+  bool apiParentExists(HttpRequestData *rdata);
+  void findParent(HttpRequestData *rdata, ParentResult *result);
+  void nextParent(HttpRequestData *rdata, ParentResult *result);
+  bool parentExists(HttpRequestData *rdata);
+
+  // implementation of functions from ParentSelectionStrategy.
+  void
+  selectParent(bool firstCall, ParentResult *result, RequestData *rdata)
+  {
+    ink_release_assert(result->rec->selection_strategy != NULL);
+    return result->rec->selection_strategy->selectParent(&policy, firstCall, result, rdata);
+  }
+
+  void
+  markParentDown(ParentResult *result)
+  {
+    ink_release_assert(result->rec->selection_strategy != NULL);
+    result->rec->selection_strategy->markParentDown(&policy, result);
+  }
+
+  uint32_t
+  numParents(ParentResult *result)
+  {
+    ink_release_assert(result->rec->selection_strategy != NULL);
+    return result->rec->selection_strategy->numParents(result);
+  }
+
+  void
+  markParentUp(ParentResult *result)
+  {
+    ink_release_assert(result != NULL);
+    result->rec->selection_strategy->markParentUp(result);
+  }
+
+  P_table *parent_table;
+  ParentRecord *DefaultParent;
+  ParentSelectionPolicy policy;
+};
+
+class HttpRequestData;
+
+struct ParentConfig {
+public:
+  static void startup();
+  static void reconfigure();
+  static void print();
+  static void set_parent_table(P_table *pTable, ParentRecord *rec, int num_elements);
+
+  static ParentConfigParams *
+  acquire()
+  {
+    return (ParentConfigParams *)configProcessor.get(ParentConfig::m_id);
+  }
+
+  static void
+  release(ParentConfigParams *strategy)
+  {
+    configProcessor.release(ParentConfig::m_id, strategy);
+  }
+
+  static int m_id;
 };
 
 // Helper Functions
