@@ -38,7 +38,7 @@
 
 #include "Plugin.h"
 #include "P_Net.h"
-#include "ink_atomic.h"
+#include "ts/ink_atomic.h"
 
 class PluginVCCore;
 
@@ -60,7 +60,7 @@ enum PluginVC_t {
 
 // For the id in set_data/get_data
 enum {
-  PLUGIN_VC_DATA_LOCAL,
+  PLUGIN_VC_DATA_LOCAL = TS_API_DATA_LAST,
   PLUGIN_VC_DATA_REMOTE,
 };
 
@@ -93,8 +93,9 @@ public:
   virtual void set_inactivity_timeout(ink_hrtime timeout_in);
   virtual void cancel_active_timeout();
   virtual void cancel_inactivity_timeout();
-  virtual void add_to_keep_alive_lru();
-  virtual void remove_from_keep_alive_lru();
+  virtual void add_to_keep_alive_queue();
+  virtual void remove_from_keep_alive_queue();
+  virtual bool add_to_active_queue();
   virtual ink_hrtime get_active_timeout();
   virtual ink_hrtime get_inactivity_timeout();
 
