@@ -579,7 +579,7 @@ PrefetchTransform::redirect(HTTPHdr *resp)
       }
 
       Debug("PrefetchTransform", "Found embedded URL: %s", redirect_url);
-      entry->req_ip = m_sm->t_state.client_info.addr;
+      entry->req_ip = m_sm->t_state.client_info.src_addr;
 
       PrefetchBlaster *blaster = prefetchBlasterAllocator.alloc();
       blaster->init(entry, &m_sm->t_state.hdr_info.client_request, this);
@@ -602,7 +602,7 @@ PrefetchTransform::parse_data(IOBufferReader *reader)
       continue;
     }
     // Debug("PrefetchParserURLs", "Found embedded URL: %s", url_start);
-    ats_ip_copy(&entry->req_ip, &m_sm->t_state.client_info.addr);
+    ats_ip_copy(&entry->req_ip, &m_sm->t_state.client_info.src_addr);
 
     PrefetchBlaster *blaster = prefetchBlasterAllocator.alloc();
     blaster->init(entry, &m_sm->t_state.hdr_info.client_request, this);
@@ -645,7 +645,7 @@ check_n_attach_prefetch_transform(HttpSM *sm, HTTPHdr *resp, bool from_cache)
   INKVConnInternal *prefetch_trans;
   ip_text_buffer client_ipb;
 
-  IpEndpoint client_ip = sm->t_state.client_info.addr;
+  IpEndpoint client_ip = sm->t_state.client_info.src_addr;
 
   // we depend on this to setup @a client_ipb for all subsequent Debug().
   Debug("PrefetchParser", "Checking response for request from %s\n", ats_ip_ntop(&client_ip, client_ipb, sizeof(client_ipb)));
@@ -1446,12 +1446,16 @@ PrefetchBlaster::handleEvent(int event, void *data)
 
   switch (event) {
   case EVENT_IMMEDIATE: {
+    HttpCacheKey key;
+
     // Here, we need to decide if we need to prefetch based on whether it
     // is in the cache or not.
 
     // if (cache_lookup_necessary) do:
     initCacheLookupConfig();
-    cacheProcessor.open_read(this, request->url_get(), false, request, &cache_lookup_config, 0);
+
+    Cache::generate_key(&key, request->url_get()); // XXX choose a cache generation number ...
+    cacheProcessor.open_read(this, &key, false, request, &cache_lookup_config, 0);
 
     break;
   }
