@@ -27,7 +27,7 @@
 
  */
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include <dlfcn.h>
 #include "Main.h"
 #include "Error.h"
@@ -36,8 +36,8 @@
 #include "P_Cache.h"
 #include "ProxyConfig.h"
 #include "ReverseProxy.h"
-#include "MatcherUtils.h"
-#include "Tokenizer.h"
+#include "ts/MatcherUtils.h"
+#include "ts/Tokenizer.h"
 #include "api/ts/remap.h"
 #include "RemapPluginInfo.h"
 #include "RemapProcessor.h"
@@ -56,10 +56,8 @@ remap_plugin_info *remap_pi_list; // We never reload the remap plugins, just app
 #define FILE_CHANGED 0
 #define REVERSE_CHANGED 1
 #define TSNAME_CHANGED 2
-#define AC_PORT_CHANGED 3
+#define SYNTH_PORT_CHANGED 3
 #define TRANS_CHANGED 4
-#define DEFAULT_TO_PAC_CHANGED 5
-#define DEFAULT_TO_PAC_PORT_CHANGED 7
 #define URL_REMAP_MODE_CHANGED 8
 #define HTTP_DEFAULT_REDIRECT_CHANGED 9
 
@@ -86,11 +84,7 @@ init_reverse_proxy()
   REC_RegisterConfigUpdateFunc("proxy.config.url_remap.filename", url_rewrite_CB, (void *)FILE_CHANGED);
   REC_RegisterConfigUpdateFunc("proxy.config.proxy_name", url_rewrite_CB, (void *)TSNAME_CHANGED);
   REC_RegisterConfigUpdateFunc("proxy.config.reverse_proxy.enabled", url_rewrite_CB, (void *)REVERSE_CHANGED);
-  REC_RegisterConfigUpdateFunc("proxy.config.admin.autoconf_port", url_rewrite_CB, (void *)AC_PORT_CHANGED);
-  REC_RegisterConfigUpdateFunc("proxy.config.url_remap.default_to_server_pac", url_rewrite_CB, (void *)DEFAULT_TO_PAC_CHANGED);
-  REC_RegisterConfigUpdateFunc("proxy.config.url_remap.default_to_server_pac_port", url_rewrite_CB,
-                               (void *)DEFAULT_TO_PAC_PORT_CHANGED);
-  REC_RegisterConfigUpdateFunc("proxy.config.url_remap.url_remap_mode", url_rewrite_CB, (void *)URL_REMAP_MODE_CHANGED);
+  REC_RegisterConfigUpdateFunc("proxy.config.admin.synthetic_port", url_rewrite_CB, (void *)SYNTH_PORT_CHANGED);
   REC_RegisterConfigUpdateFunc("proxy.config.http.referer_default_redirect", url_rewrite_CB, (void *)HTTP_DEFAULT_REDIRECT_CHANGED);
   return 0;
 }
@@ -184,14 +178,12 @@ url_rewrite_CB(const char * /* name ATS_UNUSED */, RecDataT /* data_type ATS_UNU
     break;
 
   case TSNAME_CHANGED:
-  case DEFAULT_TO_PAC_CHANGED:
-  case DEFAULT_TO_PAC_PORT_CHANGED:
   case FILE_CHANGED:
   case HTTP_DEFAULT_REDIRECT_CHANGED:
     eventProcessor.schedule_imm(new UR_UpdateContinuation(reconfig_mutex), ET_TASK);
     break;
 
-  case AC_PORT_CHANGED:
+  case SYNTH_PORT_CHANGED:
     // The AutoConf port does not current change on manager except at restart
     break;
 
