@@ -27,7 +27,7 @@
 
 
 */
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "StatSystem.h"
 #include "P_Net.h"
 #include "I_OneWayTunnel.h"
@@ -101,7 +101,7 @@ SocksProxy::init(NetVConnection *netVC)
   buf = new_MIOBuffer();
   reader = buf->alloc_reader();
 
-  MUTEX_LOCK(lock, mutex, this_ethread());
+  SCOPED_MUTEX_LOCK(lock, mutex, this_ethread());
 
   SET_HANDLER((EventHandler)&SocksProxy::mainEvent);
 
@@ -467,9 +467,9 @@ SocksProxy::setupHttpRequest(unsigned char *p)
     // just use the last 4 octets as IPv4 addr:
     a->type = SOCKS_ATYPE_IPV4;
     a->addr.ipv4[0] = p[16];
-    a->addr.ipv4[0] = p[17];
-    a->addr.ipv4[0] = p[18];
-    a->addr.ipv4[0] = p[19];
+    a->addr.ipv4[1] = p[17];
+    a->addr.ipv4[2] = p[18];
+    a->addr.ipv4[3] = p[19];
 
     break;
   default:
