@@ -29,14 +29,17 @@
 #if !defined(_StatSystem_h_)
 #define _StatSystem_h_
 
-#include "ink_platform.h"
-#include "ink_hrtime.h"
-#include "ink_atomic.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_memory.h"
+#include "ts/ink_hrtime.h"
+#include "ts/ink_atomic.h"
+
 #ifdef USE_LOCKS_FOR_DYN_STATS
 #include "Lock.h"
 #endif
 
-#include "ink_apidefs.h"
+#include "ts/ink_apidefs.h"
+#include "ts/apidefs.h"
 
 #define STATS_MAJOR_VERSION 6 // increment when changing the stats!
 #define DEFAULT_SNAP_FILENAME "stats.snap"
@@ -49,61 +52,46 @@
 class TransactionMilestones
 {
 public:
-  TransactionMilestones()
-    : ua_begin(0), ua_read_header_done(0), ua_begin_write(0), ua_close(0), server_first_connect(0), server_connect(0),
-      server_connect_end(0), server_begin_write(0), server_first_read(0), server_read_header_done(0), server_close(0),
-      cache_open_read_begin(0), cache_open_read_end(0), cache_open_write_begin(0), cache_open_write_end(0), dns_lookup_begin(0),
-      dns_lookup_end(0), sm_start(0), sm_finish(0)
+  TransactionMilestones() { ink_zero(milestones); }
+
+  ink_hrtime &operator[](TSMilestonesType ms) { return milestones[ms]; }
+
+  ink_hrtime operator[](TSMilestonesType ms) const { return milestones[ms]; }
+
+  /**
+   * Takes two milestones and returns the difference.
+   * @param start The start time
+   * @param end The end time
+   * @return A double that is the time in seconds
+   */
+  int64_t
+  difference_msec(TSMilestonesType ms_start, TSMilestonesType ms_end) const
   {
+    if (milestones[ms_end] == 0) {
+      return -1;
+    }
+    return (milestones[ms_end] - milestones[ms_start]) / 1000000;
   }
 
+  double
+  difference(TSMilestonesType ms_start, TSMilestonesType ms_end) const
+  {
+    if (milestones[ms_end] == 0) {
+      return -1;
+    }
+    return (double)(milestones[ms_end] - milestones[ms_start]) / 1000000000;
+  }
 
-  ////////////////////////////////////////////////////////
-  // user agent:                                        //
-  // user_agent_begin represents the time this          //
-  // transaction started. If this is the first          //
-  // transaction in a connection, then user_agent_begin //
-  // is set to accept time. otherwise it is set to      //
-  // first read time.                                   //
-  ////////////////////////////////////////////////////////
-  ink_hrtime ua_begin;
-  ink_hrtime ua_read_header_done;
-  ink_hrtime ua_begin_write;
-  ink_hrtime ua_close;
+  ink_hrtime
+  elapsed(TSMilestonesType ms_start, TSMilestonesType ms_end) const
+  {
+    return milestones[ms_end] - milestones[ms_start];
+  }
 
-  ////////////////////////////////////////////////////////
-  // server (origin_server , parent, blind tunnnel //
-  ////////////////////////////////////////////////////////
-  ink_hrtime server_first_connect;
-  ink_hrtime server_connect;
-  ink_hrtime server_connect_end;
-  ink_hrtime server_begin_write;      //  http only
-  ink_hrtime server_first_read;       //  http only
-  ink_hrtime server_read_header_done; //  http only
-  ink_hrtime server_close;
-
-  ink_hrtime cache_open_read_begin;
-  ink_hrtime cache_open_read_end;
-  ink_hrtime cache_open_write_begin;
-  ink_hrtime cache_open_write_end;
-
-  ink_hrtime dns_lookup_begin;
-  ink_hrtime dns_lookup_end;
-
-  ///////////////////
-  // state machine //
-  ///////////////////
-  ink_hrtime sm_start;
-  ink_hrtime sm_finish;
-
-  // TODO: Should we instrument these at some point?
-  // ink_hrtime  cache_read_begin;
-  // ink_hrtime  cache_read_end;
-  // ink_hrtime  cache_write_begin;
-  // ink_hrtime  cache_write_end;
+private:
+  ink_hrtime milestones[TS_MILESTONE_LAST_ENTRY];
 };
 
-
 // Modularization Project: Build w/o thread-local-dyn-stats
 // temporarily until we switch over to librecords.  Revert to old
 // non-thread-local system so that TS will still build and run.
