@@ -61,7 +61,7 @@
 
 #include "ProxyConfig.h"
 #include "P_Net.h"
-#include "MimeTable.h"
+#include "ts/MimeTable.h"
 #include "TransformInternal.h"
 #include "HdrUtils.h"
 #include "Log.h"
