@@ -30,7 +30,7 @@
 
  ****************************************************************************/
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "HdrHeap.h"
 #include "URL.h"
 #include "MIME.h"
@@ -125,7 +125,7 @@ new_HdrHeap(int size)
 
   //    Debug("hdrs", "Allocated header heap in size %d", size);
 
-  // Patch vritual function table ptr
+  // Patch virtual function table ptr
   *((void **)h) = *((void **)&proto_heap);
 
   h->m_size = size;
@@ -973,6 +973,10 @@ HdrHeap::unmarshal(int buf_length, int obj_type, HdrHeapObjImpl **found_obj, Ref
 inline int
 HdrHeap::attach_str_heap(char *h_start, int h_len, RefCountObj *h_ref_obj, int *index)
 {
+  if (*index >= HDR_BUF_RONLY_HEAPS) {
+    return 0;
+  }
+
   // Loop over existing entries to see if this one is already present
   for (int z = 0; z < *index; z++) {
     if (m_ronly_heap[z].m_heap_start == h_start) {
@@ -988,10 +992,6 @@ HdrHeap::attach_str_heap(char *h_start, int h_len, RefCountObj *h_ref_obj, int *
     }
   }
 
-  if (*index >= HDR_BUF_RONLY_HEAPS) {
-    return 0;
-  }
-
   m_ronly_heap[*index].m_ref_count_ptr = h_ref_obj;
   m_ronly_heap[*index].m_heap_start = h_start;
   m_ronly_heap[*index].m_heap_len = h_len;
