@@ -26,9 +26,10 @@
 
 #include <sys/time.h>
 
-#include "ink_assert.h"
-#include "ink_apidefs.h"
-#include "ink_string++.h"
+#include "ts/ink_assert.h"
+#include "ts/ink_apidefs.h"
+#include "ts/ink_string++.h"
+#include "ts/ParseRules.h"
 #include "HdrHeap.h"
 #include "HdrToken.h"
 
@@ -164,6 +165,7 @@ struct MIMEField {
   int value_get_comma_list(StrList *list) const;
 
   void name_set(HdrHeap *heap, MIMEHdrImpl *mh, const char *name, int length);
+  bool name_is_valid() const;
 
   void value_set(HdrHeap *heap, MIMEHdrImpl *mh, const char *value, int length);
   void value_set_int(HdrHeap *heap, MIMEHdrImpl *mh, int32_t value);
@@ -175,6 +177,7 @@ struct MIMEField {
   // Other separators (e.g. ';' in Set-cookie/Cookie) are also possible
   void value_append(HdrHeap *heap, MIMEHdrImpl *mh, const char *value, int length, bool prepend_comma = false,
                     const char separator = ',');
+  bool value_is_valid() const;
   int has_dups() const;
 };
 
@@ -184,7 +187,7 @@ struct MIMEFieldBlockImpl : public HdrHeapObjImpl {
   MIMEFieldBlockImpl *m_next;
   MIMEField m_field_slots[MIME_FIELD_BLOCK_SLOTS];
   // mime_hdr_copy_onto assumes that m_field_slots is last --
-  // don't add any new fields afterit.
+  // don't add any new fields after it.
 
   // Marshaling Functions
   int marshal(MarshalXlate *ptr_xlate, int num_ptr, MarshalXlate *str_xlate, int num_str);
@@ -382,7 +385,7 @@ extern const char *MIME_FIELD_VIA;
 extern const char *MIME_FIELD_WARNING;
 extern const char *MIME_FIELD_WWW_AUTHENTICATE;
 extern const char *MIME_FIELD_XREF;
-extern const char *MIME_FIELD_INT_DATA_INFO;
+extern const char *MIME_FIELD_ATS_INTERNAL;
 extern const char *MIME_FIELD_X_ID;
 extern const char *MIME_FIELD_X_FORWARDED_FOR;
 extern const char *MIME_FIELD_SEC_WEBSOCKET_KEY;
@@ -485,7 +488,7 @@ extern int MIME_LEN_VIA;
 extern int MIME_LEN_WARNING;
 extern int MIME_LEN_WWW_AUTHENTICATE;
 extern int MIME_LEN_XREF;
-extern int MIME_LEN_INT_DATA_INFO;
+extern int MIME_LEN_ATS_INTERNAL;
 extern int MIME_LEN_X_ID;
 extern int MIME_LEN_X_FORWARDED_FOR;
 
@@ -588,7 +591,7 @@ extern int MIME_WKSIDX_VIA;
 extern int MIME_WKSIDX_WARNING;
 extern int MIME_WKSIDX_WWW_AUTHENTICATE;
 extern int MIME_WKSIDX_XREF;
-extern int MIME_WKSIDX_INT_DATA_INFO;
+extern int MIME_WKSIDX_ATS_INTERNAL;
 extern int MIME_WKSIDX_X_ID;
 extern int MIME_WKSIDX_SEC_WEBSOCKET_KEY;
 extern int MIME_WKSIDX_SEC_WEBSOCKET_VERSION;
@@ -762,6 +765,23 @@ MIMEField::name_set(HdrHeap *heap, MIMEHdrImpl *mh, const char *name, int length
   }
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+inline bool
+MIMEField::name_is_valid() const
+{
+  const char *name;
+  int length;
+
+  for (name = name_get(&length); length > 0; length--) {
+    if (ParseRules::is_control(name[length - 1])) {
+      return false;
+    }
+  }
+  return true;
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -852,6 +872,23 @@ MIMEField::value_append(HdrHeap *heap, MIMEHdrImpl *mh, const char *value, int l
   mime_field_value_append(heap, mh, this, value, length, prepend_comma, separator);
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+inline bool
+MIMEField::value_is_valid() const
+{
+  const char *value;
+  int length;
+
+  for (value = value_get(&length); length > 0; length--) {
+    if (ParseRules::is_control(value[length - 1])) {
+      return false;
+    }
+  }
+  return true;
+}
+
 inline int
 MIMEField::has_dups() const
 {
@@ -907,6 +944,7 @@ public:
   void field_delete(const char *name, int name_length);
 
   MIMEField *iter_get_first(MIMEFieldIter *iter);
+  MIMEField *iter_get(MIMEFieldIter *iter);
   MIMEField *iter_get_next(MIMEFieldIter *iter);
 
   uint64_t presence(uint64_t mask);
@@ -942,6 +980,7 @@ public:
   // Other separators (e.g. ';' in Set-cookie/Cookie) are also possible
   void field_value_append(MIMEField *field, const char *value, int value_length, bool prepend_comma = false,
                           const char separator = ',');
+  void field_combine_dups(MIMEField *field, bool prepend_comma = false, const char separator = ',');
   time_t get_age();
   int64_t get_content_length() const;
   time_t get_date();
@@ -1142,17 +1181,17 @@ inline MIMEField *
 MIMEHdr::iter_get_first(MIMEFieldIter *iter)
 {
   iter->m_block = &m_mime->m_first_fblock;
-  iter->m_slot = (unsigned int)-1;
-  return iter_get_next(iter);
+  iter->m_slot = 0;
+  return iter_get(iter);
 }
 
 inline MIMEField *
-MIMEHdr::iter_get_next(MIMEFieldIter *iter)
+MIMEHdr::iter_get(MIMEFieldIter *iter)
 {
   MIMEField *f;
   MIMEFieldBlockImpl *b = iter->m_block;
 
-  int slot = iter->m_slot + 1;
+  int slot = iter->m_slot;
 
   while (b) {
     for (; slot < (int)b->m_freetop; slot++) {
@@ -1171,6 +1210,13 @@ MIMEHdr::iter_get_next(MIMEFieldIter *iter)
   return NULL;
 }
 
+inline MIMEField *
+MIMEHdr::iter_get_next(MIMEFieldIter *iter)
+{
+  iter->m_slot++;
+  return iter_get(iter);
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -1328,6 +1374,24 @@ MIMEHdr::field_value_append(MIMEField *field, const char *value_str, int value_l
   field->value_append(m_heap, m_mime, value_str, value_len, prepend_comma, separator);
 }
 
+inline void
+MIMEHdr::field_combine_dups(MIMEField *field, bool prepend_comma, const char separator)
+{
+  MIMEField *current = field->m_next_dup;
+
+  while (current) {
+    int value_len = 0;
+    const char *value_str = current->value_get(&value_len);
+
+    if (value_len > 0) {
+      HdrHeap::HeapGuard guard(m_heap, value_str); // reference count the source string so it doesn't get moved
+      field->value_append(m_heap, m_mime, value_str, value_len, prepend_comma, separator);
+    }
+    field_delete(current, false); // don't delete duplicates
+    current = field->m_next_dup;
+  }
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
