@@ -106,7 +106,11 @@ HttpCacheSM::state_cache_open_read(int event, void *data)
   switch (event) {
   case CACHE_EVENT_OPEN_READ:
     HTTP_INCREMENT_DYN_STAT(http_current_cache_connections_stat);
-    ink_assert(cache_read_vc == NULL);
+    ink_assert((cache_read_vc == NULL) || master_sm->t_state.redirect_info.redirect_in_process);
+    if (cache_read_vc) {
+      // redirect follow in progress, close the previous cache_read_vc
+      close_read();
+    }
     open_read_cb = true;
     cache_read_vc = (CacheVConnection *)data;
     master_sm->handleEvent(event, data);
@@ -141,7 +145,7 @@ HttpCacheSM::state_cache_open_read(int event, void *data)
                         "retrying cache open read...",
           master_sm->sm_id, open_read_tries);
 
-    do_cache_open_read();
+    do_cache_open_read(cache_key);
     break;
 
   default:
@@ -168,10 +172,33 @@ HttpCacheSM::state_cache_open_write(int event, void *data)
     break;
 
   case CACHE_EVENT_OPEN_WRITE_FAILED:
-    // The cache is hosed or full or something.
-    // Forward the failure to the main sm
-    open_write_cb = true;
-    master_sm->handleEvent(event, data);
+    if (open_write_tries <= master_sm->t_state.txn_conf->max_cache_open_write_retries) {
+      // Retry open write;
+      open_write_cb = false;
+      do_schedule_in();
+    } else {
+      // The cache is hosed or full or something.
+      // Forward the failure to the main sm
+      Debug("http_cache", "[%" PRId64 "] [state_cache_open_write] cache open write failure %d. "
+                          "done retrying...",
+            master_sm->sm_id, open_write_tries);
+      open_write_cb = true;
+      master_sm->handleEvent(event, data);
+    }
+    break;
+
+  case EVENT_INTERVAL:
+    // Retry the cache open write if the number retries is less
+    // than or equal to the max number of open write retries
+    ink_assert(open_write_tries <= master_sm->t_state.txn_conf->max_cache_open_write_retries);
+    Debug("http_cache", "[%" PRId64 "] [state_cache_open_write] cache open write failure %d. "
+                        "retrying cache open write...",
+          master_sm->sm_id, open_write_tries);
+
+    open_write(
+      &cache_key, lookup_url, read_request_hdr, master_sm->t_state.cache_info.object_read,
+      (time_t)((master_sm->t_state.cache_control.pin_in_cache_for < 0) ? 0 : master_sm->t_state.cache_control.pin_in_cache_for),
+      retry_write, false);
     break;
 
   default:
@@ -196,7 +223,7 @@ HttpCacheSM::do_schedule_in()
 }
 
 Action *
-HttpCacheSM::do_cache_open_read()
+HttpCacheSM::do_cache_open_read(const HttpCacheKey &key)
 {
   open_read_tries++;
   ink_assert(pending_action == NULL);
@@ -207,7 +234,7 @@ HttpCacheSM::do_cache_open_read()
   }
   // Initialising read-while-write-inprogress flag
   this->readwhilewrite_inprogress = false;
-  Action *action_handle = cacheProcessor.open_read(this, this->lookup_url, master_sm->t_state.cache_control.cluster_cache_local,
+  Action *action_handle = cacheProcessor.open_read(this, &key, master_sm->t_state.cache_control.cluster_cache_local,
                                                    this->read_request_hdr, this->read_config, this->read_pin_in_cache);
 
   if (action_handle != ACTION_RESULT_DONE) {
@@ -226,10 +253,11 @@ HttpCacheSM::do_cache_open_read()
 }
 
 Action *
-HttpCacheSM::open_read(URL *url, HTTPHdr *hdr, CacheLookupHttpConfig *params, time_t pin_in_cache)
+HttpCacheSM::open_read(const HttpCacheKey *key, URL *url, HTTPHdr *hdr, CacheLookupHttpConfig *params, time_t pin_in_cache)
 {
   Action *act_return;
 
+  cache_key = *key;
   lookup_url = url;
   read_request_hdr = hdr;
   read_config = params;
@@ -240,7 +268,7 @@ HttpCacheSM::open_read(URL *url, HTTPHdr *hdr, CacheLookupHttpConfig *params, ti
   lookup_max_recursive++;
   current_lookup_level++;
   open_read_cb = false;
-  act_return = do_cache_open_read();
+  act_return = do_cache_open_read(cache_key);
   // the following logic is based on the assumption that the secnod
   // lookup won't happen if the HttpSM hasn't been called back for the
   // first lookup
@@ -263,11 +291,12 @@ HttpCacheSM::open_read(URL *url, HTTPHdr *hdr, CacheLookupHttpConfig *params, ti
 }
 
 Action *
-HttpCacheSM::open_write(URL *url, HTTPHdr *request, CacheHTTPInfo *old_info, time_t pin_in_cache, bool retry, bool allow_multiple)
+HttpCacheSM::open_write(const HttpCacheKey *key, URL *url, HTTPHdr *request, CacheHTTPInfo *old_info, time_t pin_in_cache,
+                        bool retry, bool allow_multiple)
 {
   SET_HANDLER(&HttpCacheSM::state_cache_open_write);
   ink_assert(pending_action == NULL);
-  ink_assert(cache_write_vc == NULL);
+  ink_assert((cache_write_vc == NULL) || master_sm->t_state.redirect_info.redirect_in_process);
   // INKqa12119
   open_write_cb = false;
   open_write_tries++;
@@ -280,6 +309,7 @@ HttpCacheSM::open_write(URL *url, HTTPHdr *request, CacheHTTPInfo *old_info, tim
   ink_assert(request == read_request_hdr || read_request_hdr == NULL);
   this->lookup_url = url;
   this->read_request_hdr = request;
+  cache_key = *key;
 
   // Make sure we are not stuck in a loop where the write
   //  fails but the retry read succeeds causing to issue
@@ -287,13 +317,13 @@ HttpCacheSM::open_write(URL *url, HTTPHdr *request, CacheHTTPInfo *old_info, tim
   //  that must be revalidated every time)
   // Changed by YTS Team, yamsat Plugin
   if (open_write_tries > master_sm->redirection_tries &&
-      open_write_tries > master_sm->t_state.http_config_param->max_cache_open_write_retries) {
+      open_write_tries > master_sm->t_state.txn_conf->max_cache_open_write_retries) {
     master_sm->handleEvent(CACHE_EVENT_OPEN_WRITE_FAILED, (void *)-ECACHE_DOC_BUSY);
     return ACTION_RESULT_DONE;
   }
 
   Action *action_handle =
-    cacheProcessor.open_write(this, 0, url, master_sm->t_state.cache_control.cluster_cache_local, request,
+    cacheProcessor.open_write(this, 0, key, master_sm->t_state.cache_control.cluster_cache_local, request,
                               // INKqa11166
                               allow_multiple ? (CacheHTTPInfo *)CACHE_ALLOW_MULTIPLE_WRITES : old_info, pin_in_cache);
 
