@@ -74,11 +74,11 @@ ServerSessionPool::match(HttpServerSession *ss, sockaddr const *addr, INK_MD5 co
          (TS_SERVER_SESSION_SHARING_MATCH_HOST == match_style || ats_ip_addr_port_eq(ss->server_ip, addr));
 }
 
-HttpServerSession *
-ServerSessionPool::acquireSession(sockaddr const *addr, INK_MD5 const &hostname_hash, TSServerSessionSharingMatchType match_style)
+HSMresult_t
+ServerSessionPool::acquireSession(sockaddr const *addr, INK_MD5 const &hostname_hash, TSServerSessionSharingMatchType match_style,
+                                  HttpServerSession *&to_return)
 {
-  HttpServerSession *zret = NULL;
-
+  HSMresult_t zret = HSM_NOT_FOUND;
   if (TS_SERVER_SESSION_SHARING_MATCH_HOST == match_style) {
     // This is broken out because only in this case do we check the host hash first.
     HostHashTable::Location loc = m_host_pool.find(hostname_hash);
@@ -86,9 +86,9 @@ ServerSessionPool::acquireSession(sockaddr const *addr, INK_MD5 const &hostname_
     while (loc && port != ats_ip_port_cast(loc->server_ip))
       ++loc; // scan for matching port.
     if (loc) {
-      zret = loc;
+      to_return = loc;
       m_host_pool.remove(loc);
-      m_ip_pool.remove(m_ip_pool.find(zret));
+      m_ip_pool.remove(m_ip_pool.find(loc));
     }
   } else if (TS_SERVER_SESSION_SHARING_MATCH_NONE != match_style) { // matching is not disabled.
     IPHashTable::Location loc = m_ip_pool.find(addr);
@@ -100,9 +100,9 @@ ServerSessionPool::acquireSession(sockaddr const *addr, INK_MD5 const &hostname_
         ++loc;
     }
     if (loc) {
-      zret = loc;
+      to_return = loc;
       m_ip_pool.remove(loc);
-      m_host_pool.remove(m_host_pool.find(zret));
+      m_host_pool.remove(m_host_pool.find(loc));
     }
   }
   return zret;
@@ -202,9 +202,16 @@ ServerSessionPool::eventHandler(int event, void *data)
 
   HttpConfig::release(http_config_params);
   if (!found) {
-    // We failed to find our session.  This can only be the result
-    //  of a programming flaw
-    Warning("Connection leak from http keep-alive system");
+    // We failed to find our session.  This can only be the result of a programming flaw. Since we only ever keep
+    // UnixNetVConnections and SSLNetVConnections in the session pool, the dynamic cast won't fail.
+    UnixNetVConnection *unix_net_vc = dynamic_cast<UnixNetVConnection *>(net_vc);
+    if (unix_net_vc) {
+      char peer_ip[INET6_ADDRPORTSTRLEN];
+      ats_ip_nptop(unix_net_vc->get_remote_addr(), peer_ip, sizeof(peer_ip));
+
+      Warning("Connection leak from http keep-alive system fd=%d closed=%d peer_ip_port=%s", unix_net_vc->con.fd,
+              unix_net_vc->closed, peer_ip);
+    }
     ink_assert(0);
   }
   return 0;
@@ -238,6 +245,7 @@ HttpSessionManager::acquire_session(Continuation * /* cont ATS_UNUSED */, sockad
   TSServerSessionSharingMatchType match_style =
     static_cast<TSServerSessionSharingMatchType>(sm->t_state.txn_conf->server_session_sharing_match);
   INK_MD5 hostname_hash;
+  HSMresult_t retval = HSM_NOT_FOUND;
 
   ink_code_md5((unsigned char *)hostname, strlen(hostname), (unsigned char *)&hostname_hash);
 
@@ -262,31 +270,65 @@ HttpSessionManager::acquire_session(Continuation * /* cont ATS_UNUSED */, sockad
     to_return = NULL;
   }
 
-  // Now check to see if we have a connection in our shared connection pool
-  EThread *ethread = this_ethread();
-
-  if (TS_SERVER_SESSION_SHARING_POOL_THREAD == sm->t_state.txn_conf->server_session_sharing_pool) {
-    to_return = ethread->server_session_pool->acquireSession(ip, hostname_hash, match_style);
-  } else {
-    MUTEX_TRY_LOCK(lock, m_g_pool->mutex, ethread);
+  // TS-3797 Adding another scope so the pool lock is dropped after it is removed from the pool and
+  // potentially moved to the current thread.  At the end of this scope, either the original
+  // pool selected VC is on the current thread or its content has been moved to a new VC on the
+  // current thread and the original has been deleted. This should adequately cover TS-3266 so we
+  // don't have to continue to hold the pool thread while we initialize the server session in the
+  // client session
+  {
+    // Now check to see if we have a connection in our shared connection pool
+    EThread *ethread = this_ethread();
+    ProxyMutex *pool_mutex = (TS_SERVER_SESSION_SHARING_POOL_THREAD == sm->t_state.http_config_param->server_session_sharing_pool) ?
+                               ethread->server_session_pool->mutex :
+                               m_g_pool->mutex;
+    MUTEX_TRY_LOCK(lock, pool_mutex, ethread);
     if (lock.is_locked()) {
-      to_return = m_g_pool->acquireSession(ip, hostname_hash, match_style);
-      Debug("http_ss", "[acquire session] pool search %s", to_return ? "successful" : "failed");
-    } else {
-      Debug("http_ss", "[acquire session] could not acquire session due to lock contention");
-      return HSM_RETRY;
+      if (TS_SERVER_SESSION_SHARING_POOL_THREAD == sm->t_state.http_config_param->server_session_sharing_pool) {
+        retval = ethread->server_session_pool->acquireSession(ip, hostname_hash, match_style, to_return);
+        Debug("http_ss", "[acquire session] thread pool search %s", to_return ? "successful" : "failed");
+      } else {
+        retval = m_g_pool->acquireSession(ip, hostname_hash, match_style, to_return);
+        Debug("http_ss", "[acquire session] global pool search %s", to_return ? "successful" : "failed");
+        // At this point to_return has been removed from the pool. Do we need to move it
+        // to the same thread?
+        if (to_return) {
+          UnixNetVConnection *server_vc = dynamic_cast<UnixNetVConnection *>(to_return->get_netvc());
+          if (server_vc) {
+            UnixNetVConnection *new_vc = server_vc->migrateToCurrentThread(sm, ethread);
+            // The VC moved, free up the original one
+            if (new_vc != server_vc) {
+              ink_assert(new_vc == NULL || new_vc->nh != NULL);
+              to_return->set_netvc(new_vc);
+              if (!new_vc) {
+                // Close out to_return, we were't able to get a connection
+                to_return->do_io_close();
+                to_return = NULL;
+                retval = HSM_NOT_FOUND;
+              } else {
+                // Keep things from timing out on us
+                new_vc->set_inactivity_timeout(new_vc->get_inactivity_timeout());
+              }
+            } else {
+              // Keep things from timing out on us
+              server_vc->set_inactivity_timeout(server_vc->get_inactivity_timeout());
+            }
+          }
+        }
+      }
+    } else { // Didn't get the lock.  to_return is still NULL
+      retval = HSM_RETRY;
     }
   }
 
   if (to_return) {
-    Debug("http_ss", "[%" PRId64 "] [acquire session] "
-                     "return session from shared pool",
-          to_return->con_id);
+    Debug("http_ss", "[%" PRId64 "] [acquire session] return session from shared pool", to_return->con_id);
     to_return->state = HSS_ACTIVE;
+    // the attach_server_session will issue the do_io_read under the sm lock
     sm->attach_server_session(to_return);
-    return HSM_DONE;
+    retval = HSM_DONE;
   }
-  return HSM_NOT_FOUND;
+  return retval;
 }
 
 HSMresult_t
