@@ -43,6 +43,12 @@ class HttpSM;
 
 void initialize_thread_for_http_sessions(EThread *thread, int thread_index);
 
+enum HSMresult_t {
+  HSM_DONE,
+  HSM_RETRY,
+  HSM_NOT_FOUND,
+};
+
 /** A pool of server sessions.
 
     This is a continuation so that it can get callbacks from the server sessions.
@@ -77,6 +83,10 @@ protected:
     static Key
     key(Value const *value)
     {
+      // Might be better to just fetch the comparing address from the
+      // netvc.  That is what the event_handler will be using to
+      // looking elements in the m_ip_pool
+      // return value->get_netvc()->get_remote_addr();
       return &value->server_ip.sa;
     }
     static bool
@@ -126,7 +136,8 @@ public:
 
       @return A pointer to the session or @c NULL if not matching session was found.
   */
-  HttpServerSession *acquireSession(sockaddr const *addr, INK_MD5 const &host_hash, TSServerSessionSharingMatchType match_style);
+  HSMresult_t acquireSession(sockaddr const *addr, INK_MD5 const &host_hash, TSServerSessionSharingMatchType match_style,
+                             HttpServerSession *&server_session);
   /** Release a session to to pool.
    */
   void releaseSession(HttpServerSession *ss);
@@ -140,12 +151,6 @@ public:
   HostHashTable m_host_pool;
 };
 
-enum HSMresult_t {
-  HSM_DONE,
-  HSM_RETRY,
-  HSM_NOT_FOUND,
-};
-
 class HttpSessionManager
 {
 public:
