@@ -25,7 +25,7 @@
 #if !defined(_HttpTransact_h_)
 #define _HttpTransact_h_
 
-#include "libts.h"
+#include "ts/ink_platform.h"
 #include "P_HostDB.h"
 #include "P_Net.h"
 #include "HttpConfig.h"
@@ -46,7 +46,6 @@
 #include "congest/Congestion.h"
 
 #define MAX_DNS_LOOKUPS 2
-#define NUM_SECONDS_IN_ONE_YEAR (31536000) // (365L * 24L * 3600L)
 
 #define HTTP_RELEASE_ASSERT(X) ink_release_assert(X)
 // #define ink_cluster_time(X) time(X)
@@ -110,7 +109,7 @@ struct HttpConfigParams;
 struct MimeTableEntry;
 class HttpSM;
 
-#include "InkErrno.h"
+#include "ts/InkErrno.h"
 #define UNKNOWN_INTERNAL_ERROR (INK_START_ERRNO - 1)
 
 enum ViaStringIndex_t {
@@ -301,6 +300,15 @@ public:
     TOTAL_CACHE_ACTION_TYPES
   };
 
+  enum CacheOpenWriteFailAction_t {
+    CACHE_WL_FAIL_ACTION_DEFAULT = 0x00,
+    CACHE_WL_FAIL_ACTION_ERROR_ON_MISS = 0x01,
+    CACHE_WL_FAIL_ACTION_STALE_ON_REVALIDATE = 0x02,
+    CACHE_WL_FAIL_ACTION_ERROR_ON_MISS_STALE_ON_REVALIDATE = 0x03,
+    CACHE_WL_FAIL_ACTION_ERROR_ON_MISS_OR_REVALIDATE = 0x04,
+    TOTAL_CACHE_WL_FAIL_ACTION_TYPES
+  };
+
   enum CacheWriteLock_t {
     CACHE_WL_INIT,
     CACHE_WL_SUCCESS,
@@ -645,19 +653,17 @@ public:
     /// zero means no failure (not attempted, succeeded).
     int connect_result;
     char *name;
-    bool dns_round_robin;
     TransferEncoding_t transfer_encoding;
 
-    IpEndpoint addr; // replaces 'ip' field
+    /** This is the source address of the connection from the point of view of the transaction.
+        It is the address of the source of the request.
+    */
+    IpEndpoint src_addr;
+    /** This is the destination address of the connection from the point of view of the transaction.
+        It is the address of the target of the request.
+    */
+    IpEndpoint dst_addr;
 
-    // port to connect to, except for client
-    // connection where it is port on proxy
-    // that client connected to.
-    // This field is managed separately from the port
-    // part of 'addr' above as in various cases the two
-    // are set/manipulated independently and things are
-    // clearer this way.
-    uint16_t port; // host order.
     ServerState_t state;
     AbortState_t abort;
     HttpProxyPort::TransportType port_attribute;
@@ -683,11 +689,11 @@ public:
 
     ConnectionAttributes()
       : http_version(), keep_alive(HTTP_KEEPALIVE_UNDEFINED), receive_chunked_response(false), pipeline_possible(false),
-        proxy_connect_hdr(false), connect_result(0), name(NULL), dns_round_robin(false), transfer_encoding(NO_TRANSFER_ENCODING),
-        port(0), state(STATE_UNDEFINED), abort(ABORT_UNDEFINED), port_attribute(HttpProxyPort::TRANSPORT_DEFAULT),
-        is_transparent(false)
+        proxy_connect_hdr(false), connect_result(0), name(NULL), transfer_encoding(NO_TRANSFER_ENCODING), state(STATE_UNDEFINED),
+        abort(ABORT_UNDEFINED), port_attribute(HttpProxyPort::TRANSPORT_DEFAULT), is_transparent(false)
     {
-      memset(&addr, 0, sizeof(addr));
+      memset(&src_addr, 0, sizeof(src_addr));
+      memset(&dst_addr, 0, sizeof(dst_addr));
     }
   };
 
@@ -792,10 +798,11 @@ public:
 
     HttpConfigParams *http_config_param;
     CacheLookupInfo cache_info;
-    bool force_dns;
     DNSLookupInfo dns_info;
+    bool force_dns;
     RedirectInfo redirect_info;
     unsigned int updated_server_version;
+    unsigned int cache_open_write_fail_action;
     bool is_revalidation_necessary; // Added to check if revalidation is necessary - YTS Team, yamsat
     bool request_will_not_selfloop; // To determine if process done - YTS Team, yamsat
     ConnectionAttributes client_info;
@@ -822,23 +829,23 @@ public:
     bool cdn_remap_complete;
     bool first_dns_lookup;
 
+    bool backdoor_request; // internal
+    bool cop_test_page;    // internal
+    HttpRequestData request_data;
     ParentConfigParams *parent_params;
     ParentResult parent_result;
-    HttpRequestData request_data;
     CacheControlResult cache_control;
     CacheLookupResult_t cache_lookup_result;
     // FilterResult             content_control;
-    bool backdoor_request; // internal
-    bool cop_test_page;    // internal
 
     StateMachineAction_t next_action;                      // out
     StateMachineAction_t api_next_action;                  // out
     void (*transact_return_point)(HttpTransact::State *s); // out
 
     // We keep this so we can jump back to the upgrade handler after remap is complete
-    bool is_upgrade_request;
     void (*post_remap_upgrade_return_point)(HttpTransact::State *s); // out
     const char *upgrade_token_wks;
+    bool is_upgrade_request;
 
     // Some WebSocket state
     bool is_websocket;
@@ -849,15 +856,15 @@ public:
     int64_t internal_msg_buffer_size; // out
     int64_t internal_msg_buffer_fast_allocator_size;
 
-    bool icp_lookup_success;          // in
     struct sockaddr_in icp_ip_result; // in
+    bool icp_lookup_success;          // in
 
     int scheme;          // out
     int next_hop_scheme; // out
     int orig_scheme;     // pre-mapped scheme
     int method;
-    HostDBInfo host_db_info;  // in
     int cause_of_death_errno; // in
+    HostDBInfo host_db_info;  // in
 
     ink_time_t client_request_time;    // internal
     ink_time_t request_sent_time;      // internal
@@ -873,6 +880,10 @@ public:
     StatBlock first_stats;
     StatBlock *current_stats;
 
+    // new ACL filtering result (calculated immediately after remap)
+    bool client_connection_enabled;
+    bool acl_filtering_performed;
+
     // for negative caching
     bool negative_caching;
     // for srv_lookup
@@ -880,15 +891,11 @@ public:
     // for authenticated content caching
     CacheAuth_t www_auth_content;
 
-    // new ACL filtering result (calculated immediately after remap)
-    bool client_connection_enabled;
-    bool acl_filtering_performed;
-
     // INK API/Remap API plugin interface
-    remap_plugin_info::_tsremap_os_response *fp_tsremap_os_response;
     void *remap_plugin_instance;
-    HTTPStatus http_return_code;
     void *user_args[HTTP_SSN_TXN_MAX_USER_ARG];
+    remap_plugin_info::_tsremap_os_response *fp_tsremap_os_response;
+    HTTPStatus http_return_code;
 
     int api_txn_active_timeout_value;
     int api_txn_connect_timeout_value;
@@ -910,11 +917,11 @@ public:
     bool api_req_cacheable;
     bool api_resp_cacheable;
     bool api_server_addr_set;
+    bool stale_icp_lookup;
     UpdateCachedObject_t api_update_cached_object;
     LockUrl_t api_lock_url;
     StateMachineAction_t saved_update_next_action;
     CacheAction_t saved_update_cache_action;
-    bool stale_icp_lookup;
 
     // Remap plugin processor support
     UrlMappingContainer url_map;
@@ -927,16 +934,16 @@ public:
     int congestion_congested_or_failed;
     int congestion_connection_opened;
 
+    unsigned int filter_mask;
+    char *remap_redirect;
     bool reverse_proxy;
     bool url_remap_success;
-    char *remap_redirect;
-    unsigned int filter_mask;
+
+    bool api_skip_all_remapping;
 
     bool already_downgraded;
     URL pristine_url; // pristine url is the url before remap
 
-    bool api_skip_all_remapping;
-
     // Http Range: related variables
     RangeSetup_t range_setup;
     int64_t num_range_fields;
@@ -960,30 +967,30 @@ public:
     // Constructor
     State()
       : m_magic(HTTP_TRANSACT_MAGIC_ALIVE), state_machine(NULL), http_config_param(NULL), force_dns(false),
-        updated_server_version(HostDBApplicationInfo::HTTP_VERSION_UNDEFINED), is_revalidation_necessary(false),
-        request_will_not_selfloop(false), // YTS Team, yamsat
+        updated_server_version(HostDBApplicationInfo::HTTP_VERSION_UNDEFINED), cache_open_write_fail_action(0),
+        is_revalidation_necessary(false), request_will_not_selfloop(false), // YTS Team, yamsat
         source(SOURCE_NONE), pre_transform_source(SOURCE_NONE), req_flavor(REQ_FLAVOR_FWDPROXY), pending_work(NULL),
         cdn_saved_next_action(SM_ACTION_UNDEFINED), cdn_saved_transact_return_point(NULL), cdn_remap_complete(false),
-        first_dns_lookup(true), parent_params(NULL), cache_lookup_result(CACHE_LOOKUP_NONE), backdoor_request(false),
-        cop_test_page(false), next_action(SM_ACTION_UNDEFINED), api_next_action(SM_ACTION_UNDEFINED), transact_return_point(NULL),
-        is_upgrade_request(false), post_remap_upgrade_return_point(NULL), upgrade_token_wks(NULL), is_websocket(false),
-        did_upgrade_succeed(false), internal_msg_buffer(NULL), internal_msg_buffer_type(NULL), internal_msg_buffer_size(0),
-        internal_msg_buffer_fast_allocator_size(-1), icp_lookup_success(false), scheme(-1), next_hop_scheme(scheme),
-        orig_scheme(scheme), method(0), cause_of_death_errno(-UNKNOWN_INTERNAL_ERROR), client_request_time(UNDEFINED_TIME),
-        request_sent_time(UNDEFINED_TIME), response_received_time(UNDEFINED_TIME), plugin_set_expire_time(UNDEFINED_TIME),
-        state_machine_id(0), first_stats(), current_stats(NULL), negative_caching(false), srv_lookup(false),
-        www_auth_content(CACHE_AUTH_NONE), client_connection_enabled(true), acl_filtering_performed(false),
-        fp_tsremap_os_response(NULL), remap_plugin_instance(0), http_return_code(HTTP_STATUS_NONE),
-        api_txn_active_timeout_value(-1), api_txn_connect_timeout_value(-1), api_txn_dns_timeout_value(-1),
-        api_txn_no_activity_timeout_value(-1), cache_req_hdr_heap_handle(NULL), cache_resp_hdr_heap_handle(NULL),
-        api_cleanup_cache_read(false), api_server_response_no_store(false), api_server_response_ignore(false),
-        api_http_sm_shutdown(false), api_modifiable_cached_resp(false), api_server_request_body_set(false),
-        api_req_cacheable(false), api_resp_cacheable(false), api_server_addr_set(false),
-        api_update_cached_object(UPDATE_CACHED_OBJECT_NONE), api_lock_url(LOCK_URL_FIRST),
-        saved_update_next_action(SM_ACTION_UNDEFINED), saved_update_cache_action(CACHE_DO_UNDEFINED), stale_icp_lookup(false),
-        url_map(), pCongestionEntry(NULL), congest_saved_next_action(SM_ACTION_UNDEFINED), congestion_control_crat(0),
-        congestion_congested_or_failed(0), congestion_connection_opened(0), reverse_proxy(false), url_remap_success(false),
-        remap_redirect(NULL), filter_mask(0), already_downgraded(false), pristine_url(), api_skip_all_remapping(false),
+        first_dns_lookup(true), backdoor_request(false), cop_test_page(false), parent_params(NULL),
+        cache_lookup_result(CACHE_LOOKUP_NONE), next_action(SM_ACTION_UNDEFINED), api_next_action(SM_ACTION_UNDEFINED),
+        transact_return_point(NULL), post_remap_upgrade_return_point(NULL), upgrade_token_wks(NULL), is_upgrade_request(false),
+        is_websocket(false), did_upgrade_succeed(false), internal_msg_buffer(NULL), internal_msg_buffer_type(NULL),
+        internal_msg_buffer_size(0), internal_msg_buffer_fast_allocator_size(-1), icp_lookup_success(false), scheme(-1),
+        next_hop_scheme(scheme), orig_scheme(scheme), method(0), cause_of_death_errno(-UNKNOWN_INTERNAL_ERROR),
+        client_request_time(UNDEFINED_TIME), request_sent_time(UNDEFINED_TIME), response_received_time(UNDEFINED_TIME),
+        plugin_set_expire_time(UNDEFINED_TIME), state_machine_id(0), first_stats(), current_stats(NULL),
+        client_connection_enabled(true), acl_filtering_performed(false), negative_caching(false), srv_lookup(false),
+        www_auth_content(CACHE_AUTH_NONE), remap_plugin_instance(0), fp_tsremap_os_response(NULL),
+        http_return_code(HTTP_STATUS_NONE), api_txn_active_timeout_value(-1), api_txn_connect_timeout_value(-1),
+        api_txn_dns_timeout_value(-1), api_txn_no_activity_timeout_value(-1), cache_req_hdr_heap_handle(NULL),
+        cache_resp_hdr_heap_handle(NULL), api_cleanup_cache_read(false), api_server_response_no_store(false),
+        api_server_response_ignore(false), api_http_sm_shutdown(false), api_modifiable_cached_resp(false),
+        api_server_request_body_set(false), api_req_cacheable(false), api_resp_cacheable(false), api_server_addr_set(false),
+        stale_icp_lookup(false), api_update_cached_object(UPDATE_CACHED_OBJECT_NONE), api_lock_url(LOCK_URL_FIRST),
+        saved_update_next_action(SM_ACTION_UNDEFINED), saved_update_cache_action(CACHE_DO_UNDEFINED), url_map(),
+        pCongestionEntry(NULL), congest_saved_next_action(SM_ACTION_UNDEFINED), congestion_control_crat(0),
+        congestion_congested_or_failed(0), congestion_connection_opened(0), filter_mask(0), remap_redirect(NULL),
+        reverse_proxy(false), url_remap_success(false), api_skip_all_remapping(false), already_downgraded(false), pristine_url(),
         range_setup(RANGE_NONE), num_range_fields(0), range_output_cl(0), ranges(NULL), txn_conf(NULL),
         transparent_passthrough(false), range_in_cache(false)
     {
@@ -1245,7 +1252,7 @@ public:
                                          int64_t user_agent_response_body_size, int origin_server_request_header_size,
                                          int64_t origin_server_request_body_size, int origin_server_response_header_size,
                                          int64_t origin_server_response_body_size, int pushed_response_header_size,
-                                         int64_t pushed_response_body_size);
+                                         int64_t pushed_response_body_size, const TransactionMilestones &milestones);
   static void histogram_request_document_size(State *s, int64_t size);
   static void histogram_response_document_size(State *s, int64_t size);
   static void user_agent_connection_speed(State *s, ink_hrtime transfer_time, int64_t nbytes);
