@@ -25,7 +25,7 @@
 #define _ACL_FILTERING_H_
 
 #include "Main.h"
-//#include "YAddr.h"
+#include "ts/ink_inet.h"
 
 #include <string>
 #include <set>
@@ -35,6 +35,7 @@
 // ACL like filtering defs (per one remap rule)
 
 static int const ACL_FILTER_MAX_SRC_IP = 128;
+static int const ACL_FILTER_MAX_IN_IP = 8;
 static int const ACL_FILTER_MAX_ARGV = 512;
 
 struct src_ip_info_t {
@@ -71,8 +72,9 @@ public:
   char *filter_name;           // optional filter name
   unsigned int allow_flag : 1, // action allow deny
     src_ip_valid : 1,          // src_ip range valid
-    active_queue_flag : 1,     // filter is in active state (used by .useflt directive)
-    internal : 1;              // filter internal HTTP requests
+    in_ip_valid : 1,
+    active_queue_flag : 1, // filter is in active state (used by .useflt directive)
+    internal : 1;          // filter internal HTTP requests
 
   // we need arguments as string array for directive processing
   int argc;                        // argument counter (only for filter defs)
@@ -88,6 +90,11 @@ public:
   // src_ip
   int src_ip_cnt; // how many valid src_ip rules we have
   src_ip_info_t src_ip_array[ACL_FILTER_MAX_SRC_IP];
+
+  // in_ip
+  int in_ip_cnt; // how many valid dst_ip rules we have
+  src_ip_info_t in_ip_array[ACL_FILTER_MAX_IN_IP];
+
   acl_filter_rule();
   ~acl_filter_rule();
   void name(const char *_name = NULL);
