@@ -1,6 +1,6 @@
 /** @file
 
-  A brief file description
+  [RFC 7541] HPACK: Header Compression for HTTP/2
 
   @section license License
 
@@ -37,14 +37,11 @@ const unsigned HPACK_LEN_AUTHORITY = countof(":authority") - 1;
 const unsigned HPACK_LEN_PATH = countof(":path") - 1;
 const unsigned HPACK_LEN_STATUS = countof(":status") - 1;
 
-// 5.1.  Maximum Table Size
-// The size of an entry is the sum of its name's length in octets (as
-// defined in Section 6.2), its value's length in octets (see
-// Section 6.2), plus 32.
+// [RFC 7541] 4.1. Calculating Table Size
+// The size of an entry is the sum of its name's length in octets (as defined in Section 5.2),
+// its value's length in octets, and 32.
 const static unsigned ADDITIONAL_OCTETS = 32;
 
-const static uint32_t HEADER_FIELD_LIMIT_LENGTH = 4096;
-
 typedef enum {
   TS_HPACK_STATIC_TABLE_0 = 0,
   TS_HPACK_STATIC_TABLE_AUTHORITY,
@@ -177,18 +174,62 @@ const static struct {
                     {"via", ""},
                     {"www-authenticate", ""}};
 
+Http2LookupIndexResult
+Http2IndexingTable::get_index(const MIMEFieldWrapper &field) const
+{
+  Http2LookupIndexResult result;
+  int target_name_len = 0, target_value_len = 0;
+  const char *target_name = field.name_get(&target_name_len);
+  const char *target_value = field.value_get(&target_value_len);
+  const int entry_num = TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table.get_current_entry_num();
+
+  for (int index = 1; index < entry_num; ++index) {
+    const char *table_name, *table_value;
+    int table_name_len = 0, table_value_len = 0;
+
+    if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
+      // static table
+      table_name = STATIC_TABLE[index].name;
+      table_value = STATIC_TABLE[index].value;
+      table_name_len = strlen(table_name);
+      table_value_len = strlen(table_value);
+    } else {
+      // dynamic table
+      const MIMEField *m_field = _dynamic_table.get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
+
+      table_name = m_field->name_get(&table_name_len);
+      table_value = m_field->value_get(&table_value_len);
+    }
+
+    // Check whether name (and value) are matched
+    if (ptr_len_casecmp(target_name, target_name_len, table_name, table_name_len) == 0) {
+      if (ptr_len_cmp(target_value, target_value_len, table_value, table_value_len) == 0) {
+        result.index = index;
+        result.value_is_indexed = true;
+        break;
+      } else if (!result.index) {
+        result.index = index;
+      }
+    }
+  }
+
+  return result;
+}
+
 int
-Http2DynamicTable::get_header_from_indexing_tables(uint32_t index, MIMEFieldWrapper &field) const
+Http2IndexingTable::get_header_field(uint32_t index, MIMEFieldWrapper &field) const
 {
   // Index Address Space starts at 1, so index == 0 is invalid.
   if (!index)
     return HPACK_ERROR_COMPRESSION_ERROR;
 
   if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM) {
+    // static table
     field.name_set(STATIC_TABLE[index].name, strlen(STATIC_TABLE[index].name));
     field.value_set(STATIC_TABLE[index].value, strlen(STATIC_TABLE[index].value));
-  } else if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM + get_current_entry_num()) {
-    const MIMEField *m_field = get_header(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM + 1);
+  } else if (index < TS_HPACK_STATIC_TABLE_ENTRY_NUM + _dynamic_table.get_current_entry_num()) {
+    // dynamic table
+    const MIMEField *m_field = _dynamic_table.get_header_field(index - TS_HPACK_STATIC_TABLE_ENTRY_NUM);
 
     int name_len, value_len;
     const char *name = m_field->name_get(&name_len);
@@ -197,7 +238,7 @@ Http2DynamicTable::get_header_from_indexing_tables(uint32_t index, MIMEFieldWrap
     field.name_set(name, name_len);
     field.value_set(value, value_len);
   } else {
-    // 3.3.3.  Index Address Space
+    // [RFC 7541] 2.3.3. Index Address Space
     // Indices strictly greater than the sum of the lengths of both tables
     // MUST be treated as a decoding error.
     return HPACK_ERROR_COMPRESSION_ERROR;
@@ -206,27 +247,34 @@ Http2DynamicTable::get_header_from_indexing_tables(uint32_t index, MIMEFieldWrap
   return 0;
 }
 
-// 5.2.  Entry Eviction when Header Table Size Changes
-// Whenever the maximum size for the header table is reduced, entries
-// are evicted from the end of the header table until the size of the
-// header table is less than or equal to the maximum size.
 void
-Http2DynamicTable::set_dynamic_table_size(uint32_t new_size)
+Http2IndexingTable::add_header_field_to_dynamic_table(const MIMEField *field)
 {
-  uint32_t old_size = _settings_dynamic_table_size;
-  while (old_size > new_size) {
-    int last_name_len, last_value_len;
-    MIMEField *last_field = _headers.last();
+  _dynamic_table.add_header_field(field);
+}
 
-    last_field->name_get(&last_name_len);
-    last_field->value_get(&last_value_len);
-    old_size -= ADDITIONAL_OCTETS + last_name_len + last_value_len;
+uint32_t
+Http2IndexingTable::get_dynamic_table_size() const
+{
+  return _dynamic_table.get_size();
+}
 
-    _headers.remove_index(_headers.length() - 1);
-    _mhdr->field_delete(last_field, false);
-  }
+bool
+Http2IndexingTable::set_dynamic_table_size(uint32_t new_size)
+{
+  return _dynamic_table.set_size(new_size);
+}
 
-  _settings_dynamic_table_size = new_size;
+bool
+Http2IndexingTable::is_header_in_dynamic_table(const char *target_name, const char *target_value) const
+{
+  return _dynamic_table.is_header_in(target_name, target_value);
+}
+
+const MIMEField *
+Http2DynamicTable::get_header_field(uint32_t index) const
+{
+  return _headers.get(index);
 }
 
 void
@@ -238,10 +286,13 @@ Http2DynamicTable::add_header_field(const MIMEField *field)
   uint32_t header_size = ADDITIONAL_OCTETS + name_len + value_len;
 
   if (header_size > _settings_dynamic_table_size) {
-    // 5.3. It is not an error to attempt to add an entry that is larger than the maximum size; an
-    // attempt to add an entry larger than the entire table causes the table to be emptied of all existing entries.
+    // [RFC 7541] 4.4. Entry Eviction When Adding New Entries
+    // It is not an error to attempt to add an entry that is larger than
+    // the maximum size; an attempt to add an entry larger than the entire
+    // table causes the table to be emptied of all existing entries.
     _headers.clear();
     _mhdr->fields_clear();
+    _current_size = 0;
   } else {
     _current_size += header_size;
     while (_current_size > _settings_dynamic_table_size) {
@@ -258,14 +309,77 @@ Http2DynamicTable::add_header_field(const MIMEField *field)
 
     MIMEField *new_field = _mhdr->field_create(name, name_len);
     new_field->value_set(_mhdr->m_heap, _mhdr->m_mime, value, value_len);
+    _mhdr->field_attach(new_field);
     // XXX Because entire Vec instance is copied, Its too expensive!
     _headers.insert(0, new_field);
   }
 }
 
+uint32_t
+Http2DynamicTable::get_size() const
+{
+  return _current_size;
+}
+
+//
+// [RFC 7541] 4.3. Entry Eviction when Header Table Size Changes
+//
+// Whenever the maximum size for the header table is reduced, entries
+// are evicted from the end of the header table until the size of the
+// header table is less than or equal to the maximum size.
+//
+bool
+Http2DynamicTable::set_size(uint32_t new_size)
+{
+  while (_current_size > new_size) {
+    if (_headers.n <= 0) {
+      return false;
+    }
+    int last_name_len, last_value_len;
+    MIMEField *last_field = _headers.last();
+
+    last_field->name_get(&last_name_len);
+    last_field->value_get(&last_value_len);
+    _current_size -= ADDITIONAL_OCTETS + last_name_len + last_value_len;
+
+    _headers.remove_index(_headers.length() - 1);
+    _mhdr->field_delete(last_field, false);
+  }
+
+  _settings_dynamic_table_size = new_size;
+  return true;
+}
+
+const uint32_t
+Http2DynamicTable::get_current_entry_num() const
+{
+  return _headers.length();
+}
+
+bool
+Http2DynamicTable::is_header_in(const char *target_name, const char *target_value) const
+{
+  const MIMEField *field = _mhdr->field_find(target_name, strlen(target_name));
+
+  if (field) {
+    do {
+      int target_value_len = strlen(target_value);
+      int table_value_len = 0;
+      const char *table_value = field->value_get(&table_value_len);
+      if (ptr_len_cmp(target_value, target_value_len, table_value, table_value_len) == 0) {
+        return true;
+      }
+    } while (field->has_dups() && (field = field->m_next_dup) != NULL);
+  }
+
+  return false;
+}
+
+//
 // The first byte of an HPACK field unambiguously tells us what
 // kind of field it is. Field types are specified in the high 4 bits
 // and all bits are defined, so there's no way to get an invalid field type.
+//
 HpackFieldType
 hpack_parse_field_type(uint8_t ftype)
 {
@@ -289,18 +403,9 @@ hpack_parse_field_type(uint8_t ftype)
   return HPACK_FIELD_NOINDEX_LITERAL;
 }
 
-/*
- * Pseudo code
- *
- * if I < 2^N - 1, encode I on N bits
- * else
- *   encode (2^N - 1) on N bits
- *   I = I - (2^N - 1)
- *   while I >= 128
- *     encode (I % 128 + 128) on 8 bits
- *     I = I / 128
- *   encode I on 8 bits
- */
+//
+// [RFC 7541] 5.1. Integer representation
+//
 int64_t
 encode_integer(uint8_t *buf_start, const uint8_t *buf_end, uint32_t value, uint8_t n)
 {
@@ -333,18 +438,52 @@ int64_t
 encode_string(uint8_t *buf_start, const uint8_t *buf_end, const char *value, size_t value_len)
 {
   uint8_t *p = buf_start;
+  bool use_huffman = true;
+  char *data = NULL;
+  int64_t data_len = 0;
+
+  // TODO Choose whether to use Huffman encoding wisely
+
+  if (use_huffman && value_len) {
+    data = static_cast<char *>(ats_malloc(value_len * 4));
+    if (data == NULL)
+      return -1;
+    data_len = huffman_encode(reinterpret_cast<uint8_t *>(data), reinterpret_cast<const uint8_t *>(value), value_len);
+  }
 
   // Length
-  const int64_t len = encode_integer(p, buf_end, value_len, 7);
-  if (len == -1)
+  const int64_t len = encode_integer(p, buf_end, data_len, 7);
+  if (len == -1) {
+    if (use_huffman) {
+      ats_free(data);
+    }
+
     return -1;
+  }
+
+  if (use_huffman) {
+    *p |= 0x80;
+  }
   p += len;
-  if (buf_end < p || static_cast<size_t>(buf_end - p) < value_len)
+
+  if (buf_end < p || buf_end - p < data_len) {
+    if (use_huffman) {
+      ats_free(data);
+    }
+
     return -1;
+  }
+
+  // Value
+  if (data_len) {
+    memcpy(p, data, data_len);
+    p += data_len;
+  }
+
+  if (use_huffman) {
+    ats_free(data);
+  }
 
-  // Value String
-  memcpy(p, value, value_len);
-  p += value_len;
   return p - buf_start;
 }
 
@@ -369,12 +508,13 @@ encode_indexed_header_field(uint8_t *buf_start, const uint8_t *buf_end, uint32_t
   *p |= 0x80;
   p += len;
 
+  Debug("http2_hpack_encode", "Encoded field: %d", index);
   return p - buf_start;
 }
 
 int64_t
-encode_literal_header_field(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header, uint32_t index,
-                            HpackFieldType type)
+encode_literal_header_field_with_indexed_name(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header,
+                                              uint32_t index, Http2IndexingTable &indexing_table, HpackFieldType type)
 {
   uint8_t *p = buf_start;
   int64_t len;
@@ -384,6 +524,7 @@ encode_literal_header_field(uint8_t *buf_start, const uint8_t *buf_end, const MI
 
   switch (type) {
   case HPACK_FIELD_INDEXED_LITERAL:
+    indexing_table.add_header_field_to_dynamic_table(header.field_get());
     prefix = 6;
     flag = 0x40;
     break;
@@ -419,11 +560,13 @@ encode_literal_header_field(uint8_t *buf_start, const uint8_t *buf_end, const MI
     return -1;
   p += len;
 
+  Debug("http2_hpack_encode", "Encoded field: %d: %.*s", index, value_len, value);
   return p - buf_start;
 }
 
 int64_t
-encode_literal_header_field(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header, HpackFieldType type)
+encode_literal_header_field_with_new_name(uint8_t *buf_start, const uint8_t *buf_end, const MIMEFieldWrapper &header,
+                                          Http2IndexingTable &indexing_table, HpackFieldType type)
 {
   uint8_t *p = buf_start;
   int64_t len;
@@ -433,6 +576,7 @@ encode_literal_header_field(uint8_t *buf_start, const uint8_t *buf_end, const MI
 
   switch (type) {
   case HPACK_FIELD_INDEXED_LITERAL:
+    indexing_table.add_header_field_to_dynamic_table(header.field_get());
     flag = 0x40;
     break;
   case HPACK_FIELD_NOINDEX_LITERAL:
@@ -473,29 +617,20 @@ encode_literal_header_field(uint8_t *buf_start, const uint8_t *buf_end, const MI
 
   p += len;
 
+  Debug("http2_hpack_encode", "Encoded field: %.*s: %.*s", name_len, name, value_len, value);
   return p - buf_start;
 }
 
-/*
- * 6.1.  Integer representation
- *
- * Pseudo code
- *
- * decode I from the next N bits
- *    if I < 2^N - 1, return I
- *    else
- *        M = 0
- *        repeat
- *            B = next octet
- *            I = I + (B & 127) * 2^M
- *            M = M + 7
- *        while B & 128 == 128
- *        return I
- *
- */
+//
+// [RFC 7541] 5.1. Integer representation
+//
 int64_t
 decode_integer(uint32_t &dst, const uint8_t *buf_start, const uint8_t *buf_end, uint8_t n)
 {
+  if (buf_start >= buf_end) {
+    return HPACK_ERROR_COMPRESSION_ERROR;
+  }
+
   const uint8_t *p = buf_start;
 
   dst = (*p & ((1 << n) - 1));
@@ -519,11 +654,17 @@ decode_integer(uint32_t &dst, const uint8_t *buf_start, const uint8_t *buf_end,
   return p - buf_start + 1;
 }
 
-// 6.2 return content from String Data (Length octets)
-// with huffman decoding if it is encoded
+//
+// [RFC 7541] 5.2. String Literal Representation
+// return content from String Data (Length octets) with huffman decoding if it is encoded
+//
 int64_t
 decode_string(Arena &arena, char **str, uint32_t &str_length, const uint8_t *buf_start, const uint8_t *buf_end)
 {
+  if (buf_start >= buf_end) {
+    return HPACK_ERROR_COMPRESSION_ERROR;
+  }
+
   const uint8_t *p = buf_start;
   bool isHuffman = *p & 0x80;
   uint32_t encoded_string_len = 0;
@@ -534,7 +675,7 @@ decode_string(Arena &arena, char **str, uint32_t &str_length, const uint8_t *buf
     return HPACK_ERROR_COMPRESSION_ERROR;
   p += len;
 
-  if (encoded_string_len > HEADER_FIELD_LIMIT_LENGTH || (p + encoded_string_len) > buf_end) {
+  if ((p + encoded_string_len) > buf_end) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -557,10 +698,12 @@ decode_string(Arena &arena, char **str, uint32_t &str_length, const uint8_t *buf
   return p + encoded_string_len - buf_start;
 }
 
-// 7.1. Indexed Header Field Representation
+//
+// [RFC 7541] 6.1. Indexed Header Field Representation
+//
 int64_t
 decode_indexed_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start, const uint8_t *buf_end,
-                            Http2DynamicTable &dynamic_table)
+                            Http2IndexingTable &indexing_table)
 {
   uint32_t index = 0;
   int64_t len = 0;
@@ -569,7 +712,7 @@ decode_indexed_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
   if (len == HPACK_ERROR_COMPRESSION_ERROR)
     return HPACK_ERROR_COMPRESSION_ERROR;
 
-  if (dynamic_table.get_header_from_indexing_tables(index, header) == HPACK_ERROR_COMPRESSION_ERROR) {
+  if (indexing_table.get_header_field(index, header) == HPACK_ERROR_COMPRESSION_ERROR) {
     return HPACK_ERROR_COMPRESSION_ERROR;
   }
 
@@ -580,17 +723,20 @@ decode_indexed_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
     const char *decoded_value = header.value_get(&decoded_value_len);
 
     Arena arena;
-    Debug("http2_hpack_decode", "Decoded field:  %s: %s\n", arena.str_store(decoded_name, decoded_name_len),
+    Debug("http2_hpack_decode", "Decoded field: %s: %s", arena.str_store(decoded_name, decoded_name_len),
           arena.str_store(decoded_value, decoded_value_len));
   }
 
   return len;
 }
 
-// 7.2.  Literal Header Field Representation
+//
+// [RFC 7541] 6.2. Literal Header Field Representation
+// Decode Literal Header Field Representation based on HpackFieldType
+//
 int64_t
 decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start, const uint8_t *buf_end,
-                            Http2DynamicTable &dynamic_table)
+                            Http2IndexingTable &indexing_table)
 {
   const uint8_t *p = buf_start;
   bool isIncremental = false;
@@ -599,14 +745,11 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
   HpackFieldType ftype = hpack_parse_field_type(*p);
 
   if (ftype == HPACK_FIELD_INDEXED_LITERAL) {
-    // 7.2.1. index extraction based on Literal Header Field with Incremental Indexing
     len = decode_integer(index, p, buf_end, 6);
     isIncremental = true;
   } else if (ftype == HPACK_FIELD_NEVERINDEX_LITERAL) {
-    // 7.2.3. index extraction Literal Header Field Never Indexed
     len = decode_integer(index, p, buf_end, 4);
   } else {
-    // 7.2.2. index extraction Literal Header Field without Indexing
     ink_assert(ftype == HPACK_FIELD_NOINDEX_LITERAL);
     len = decode_integer(index, p, buf_end, 4);
   }
@@ -620,7 +763,7 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
 
   // Decode header field name
   if (index) {
-    dynamic_table.get_header_from_indexing_tables(index, header);
+    indexing_table.get_header_field(index, header);
   } else {
     char *name_str = NULL;
     uint32_t name_str_len = 0;
@@ -651,10 +794,9 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
   p += len;
   header.value_set(value_str, value_str_len);
 
-
   // Incremental Indexing adds header to header table as new entry
   if (isIncremental) {
-    dynamic_table.add_header_field(header.field_get());
+    indexing_table.add_header_field_to_dynamic_table(header.field_get());
   }
 
   // Print decoded header field
@@ -664,16 +806,18 @@ decode_literal_header_field(MIMEFieldWrapper &header, const uint8_t *buf_start,
     int decoded_value_len;
     const char *decoded_value = header.value_get(&decoded_value_len);
 
-    Debug("http2_hpack_decode", "Decoded field:  %s: %s\n", arena.str_store(decoded_name, decoded_name_len),
+    Debug("http2_hpack_decode", "Decoded field: %s: %s", arena.str_store(decoded_name, decoded_name_len),
           arena.str_store(decoded_value, decoded_value_len));
   }
 
   return p - buf_start;
 }
 
-// 7.3. Header Table Size Update
+//
+// [RFC 7541] 6.3. Dynamic Table Size Update
+//
 int64_t
-update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Http2DynamicTable &dynamic_table)
+update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Http2IndexingTable &indexing_table)
 {
   if (buf_start == buf_end)
     return HPACK_ERROR_COMPRESSION_ERROR;
@@ -684,7 +828,9 @@ update_dynamic_table_size(const uint8_t *buf_start, const uint8_t *buf_end, Http
   if (len == HPACK_ERROR_COMPRESSION_ERROR)
     return HPACK_ERROR_COMPRESSION_ERROR;
 
-  dynamic_table.set_dynamic_table_size(size);
+  if (indexing_table.set_dynamic_table_size(size) == false) {
+    return HPACK_ERROR_COMPRESSION_ERROR;
+  }
 
   return len;
 }
