@@ -27,6 +27,7 @@
 #include "HTTP2.h"
 #include "HPACK.h"
 #include "FetchSM.h"
+#include "Http2Stream.h"
 
 class Http2ClientSession;
 
@@ -35,7 +36,8 @@ class Http2ConnectionSettings
 public:
   Http2ConnectionSettings()
   {
-    // 6.5.2.  Defined SETTINGS Parameters. These should generally not be modified,
+    // 6.5.2.  Defined SETTINGS Parameters. These should generally not be
+    // modified,
     // only if the protocol changes should these change.
     settings[indexof(HTTP2_SETTINGS_ENABLE_PUSH)] = 0; // Disabled for now
 
@@ -59,23 +61,25 @@ public:
   unsigned
   get(Http2SettingsIdentifier id) const
   {
-    ink_assert(id <= HTTP2_SETTINGS_MAX - 1);
-
-    if (id > HTTP2_SETTINGS_MAX - 1) {
-      return 0;
+    if (id < HTTP2_SETTINGS_MAX) {
+      return this->settings[indexof(id)];
+    } else {
+      ink_assert(!"Bad Settings Identifier");
     }
-    return this->settings[indexof(id)];
+
+    return 0;
   }
 
   unsigned
   set(Http2SettingsIdentifier id, unsigned value)
   {
-    ink_assert(id <= HTTP2_SETTINGS_MAX - 1);
-
-    if (id > HTTP2_SETTINGS_MAX - 1) {
-      return 0;
+    if (id < HTTP2_SETTINGS_MAX) {
+      return this->settings[indexof(id)] = value;
+    } else {
+      ink_assert(!"Bad Settings Identifier");
     }
-    return this->settings[indexof(id)] = value;
+
+    return 0;
   }
 
 private:
@@ -83,7 +87,7 @@ private:
   static unsigned
   indexof(Http2SettingsIdentifier id)
   {
-    ink_assert(id <= HTTP2_SETTINGS_MAX - 1);
+    ink_assert(id < HTTP2_SETTINGS_MAX);
 
     return id - 1;
   }
@@ -91,97 +95,10 @@ private:
   unsigned settings[HTTP2_SETTINGS_MAX - 1];
 };
 
-class Http2ConnectionState;
-
-class Http2Stream
-{
-public:
-  Http2Stream(Http2StreamId sid = 0, ssize_t initial_rwnd = Http2::initial_window_size)
-    : client_rwnd(initial_rwnd), server_rwnd(initial_rwnd), _id(sid), _state(HTTP2_STREAM_STATE_IDLE), _fetch_sm(NULL),
-      body_done(false), data_length(0)
-  {
-    _req_header.create(HTTP_TYPE_REQUEST);
-  }
-
-  ~Http2Stream()
-  {
-    _req_header.destroy();
-
-    if (_fetch_sm) {
-      _fetch_sm->ext_destroy();
-      _fetch_sm = NULL;
-    }
-  }
-
-  // Operate FetchSM
-  void init_fetcher(Http2ConnectionState &cstate);
-  void set_body_to_fetcher(const void *data, size_t len);
-  FetchSM *
-  get_fetcher()
-  {
-    return _fetch_sm;
-  }
-  bool
-  is_body_done() const
-  {
-    return body_done;
-  }
-  void
-  mark_body_done()
-  {
-    body_done = true;
-  }
-
-  const Http2StreamId
-  get_id() const
-  {
-    return _id;
-  }
-  const Http2StreamState
-  get_state() const
-  {
-    return _state;
-  }
-  bool change_state(uint8_t type, uint8_t flags);
-
-  int64_t
-  decode_request_header(const IOVec &iov, Http2DynamicTable &dynamic_table, bool cont)
-  {
-    return http2_parse_header_fragment(&_req_header, iov, dynamic_table, cont);
-  }
-
-  // Check entire DATA payload length if content-length: header is exist
-  void
-  increment_data_length(uint64_t length)
-  {
-    data_length += length;
-  }
-  bool
-  payload_length_is_valid() const
-  {
-    uint32_t content_length = _req_header.get_content_length();
-    return content_length == 0 || content_length == data_length;
-  }
-
-  // Stream level window size
-  ssize_t client_rwnd, server_rwnd;
-
-  LINK(Http2Stream, link);
-
-private:
-  Http2StreamId _id;
-  Http2StreamState _state;
-
-  HTTPHdr _req_header;
-  FetchSM *_fetch_sm;
-  bool body_done;
-  uint64_t data_length;
-};
-
-
 // Http2ConnectionState
 //
-// Capture the semantics of a HTTP/2 connection. The client session captures the frame layer, and the
+// Capture the semantics of a HTTP/2 connection. The client session captures the
+// frame layer, and the
 // connection state captures the connection-wide state.
 
 class Http2ConnectionState : public Continuation
@@ -189,14 +106,14 @@ class Http2ConnectionState : public Continuation
 public:
   Http2ConnectionState()
     : Continuation(NULL), ua_session(NULL), client_rwnd(Http2::initial_window_size), server_rwnd(Http2::initial_window_size),
-      stream_list(), latest_streamid(0), client_streams_count(0), continued_id(0)
+      stream_list(), latest_streamid(0), client_streams_count(0), continued_stream_id(0)
   {
     SET_HANDLER(&Http2ConnectionState::main_event_handler);
   }
 
   Http2ClientSession *ua_session;
-  Http2DynamicTable *local_dynamic_table;
-  Http2DynamicTable *remote_dynamic_table;
+  Http2IndexingTable *local_indexing_table;
+  Http2IndexingTable *remote_indexing_table;
 
   // Settings.
   Http2ConnectionSettings server_settings;
@@ -205,14 +122,11 @@ public:
   void
   init()
   {
-    local_dynamic_table = new Http2DynamicTable();
-    remote_dynamic_table = new Http2DynamicTable();
+    local_indexing_table = new Http2IndexingTable();
+    remote_indexing_table = new Http2IndexingTable();
 
     continued_buffer.iov_base = NULL;
     continued_buffer.iov_len = 0;
-
-    // Load the server settings from the records.config / RecordsConfig.cc settings.
-    server_settings.settings_from_configs();
   }
 
   void
@@ -220,8 +134,9 @@ public:
   {
     cleanup_streams();
 
-    delete local_dynamic_table;
-    delete remote_dynamic_table;
+    mutex = NULL; // magic happens - assigning to NULL frees the ProxyMutex
+    delete local_indexing_table;
+    delete remote_indexing_table;
 
     ats_free(continued_buffer.iov_base);
   }
@@ -239,19 +154,28 @@ public:
 
   void update_initial_rwnd(Http2WindowSize new_size);
 
+  Http2StreamId
+  get_latest_stream_id() const
+  {
+    return latest_streamid;
+  }
+
   // Continuated header decoding
   Http2StreamId
-  get_continued_id() const
+  get_continued_stream_id() const
   {
-    return continued_id;
+    return continued_stream_id;
   }
-  const IOVec &
-  get_continued_headers() const
+  void
+  set_continued_stream_id(Http2StreamId stream_id)
   {
-    return continued_buffer;
+    continued_stream_id = stream_id;
+  }
+  void
+  clear_continued_stream_id()
+  {
+    continued_stream_id = 0;
   }
-  void set_continued_headers(const char *buf, uint32_t len, Http2StreamId id);
-  void finish_continued_headers();
 
   // Connection level window size
   ssize_t client_rwnd, server_rwnd;
@@ -260,6 +184,7 @@ public:
   void send_data_frame(FetchSM *fetch_sm);
   void send_headers_frame(FetchSM *fetch_sm);
   void send_rst_stream_frame(Http2StreamId id, Http2ErrorCode ec);
+  void send_settings_frame(const Http2ConnectionSettings &new_settings);
   void send_ping_frame(Http2StreamId id, uint8_t flag, const uint8_t *opaque_data);
   void send_goaway_frame(Http2StreamId id, Http2ErrorCode ec);
   void send_window_update_frame(Http2StreamId id, uint32_t size);
@@ -274,14 +199,26 @@ private:
   Http2ConnectionState(const Http2ConnectionState &);            // noncopyable
   Http2ConnectionState &operator=(const Http2ConnectionState &); // noncopyable
 
+  // NOTE: 'stream_list' has only active streams.
+  //   If given Stream Identifier is not found in stream_list and it is less
+  //   than or equal to latest_streamid, the state of Stream
+  //   is CLOSED.
+  //   If given Stream Identifier is not found in stream_list and it is greater
+  //   than latest_streamid, the state of Stream is IDLE.
   DLL<Http2Stream> stream_list;
   Http2StreamId latest_streamid;
 
   // Counter for current acive streams which is started by client
   uint32_t client_streams_count;
 
-  // The buffer used for storing incomplete fragments of a header field which consists of multiple frames.
-  Http2StreamId continued_id;
+  // NOTE: Id of stream which MUST receive CONTINUATION frame.
+  //   - [RFC 7540] 6.2 HEADERS
+  //     "A HEADERS frame without the END_HEADERS flag set MUST be followed by a
+  //     CONTINUATION frame for the same stream."
+  //   - [RFC 7540] 6.10 CONTINUATION
+  //     "If the END_HEADERS bit is not set, this frame MUST be followed by
+  //     another CONTINUATION frame."
+  Http2StreamId continued_stream_id;
   IOVec continued_buffer;
 };
 
