@@ -22,7 +22,9 @@
  */
 
 #include "HuffmanCodec.h"
-#include "libts.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_memory.h"
+#include "ts/ink_defs.h"
 
 struct huffman_entry {
   uint32_t code_as_hex;
@@ -171,7 +173,7 @@ static const huffman_entry huffman_table[] = {{0x1ff8, 13},
                                               {0x7fffdc, 23},
                                               {0x7fffdd, 23},
                                               {0x7fffde, 23},
-                                              {0xffffeb, 23},
+                                              {0xffffeb, 24},
                                               {0x7fffdf, 23},
                                               {0xffffec, 24},
                                               {0xffffed, 24},
@@ -197,7 +199,7 @@ static const huffman_entry huffman_table[] = {{0x1ff8, 13},
                                               {0x7fffe8, 23},
                                               {0x7fffe9, 23},
                                               {0x1fffde, 21},
-                                              {0x7fffde, 23},
+                                              {0x7fffea, 23},
                                               {0x3fffdd, 22},
                                               {0x3fffde, 22},
                                               {0xfffff0, 24},
@@ -243,7 +245,7 @@ static const huffman_entry huffman_table[] = {{0x1ff8, 13},
                                               {0x7ffffe0, 27},
                                               {0x7ffffe1, 27},
                                               {0x3ffffe7, 26},
-                                              {0x3ffffe2, 27},
+                                              {0x7ffffe2, 27},
                                               {0xfffff2, 24},
                                               {0x1fffe4, 21},
                                               {0x1fffe5, 21},
@@ -290,6 +292,7 @@ static const huffman_entry huffman_table[] = {{0x1ff8, 13},
 typedef struct node {
   node *left, *right;
   char ascii_code;
+  bool leaf_node;
 } Node;
 
 Node *HUFFMAN_TREE_ROOT;
@@ -301,6 +304,7 @@ make_huffman_tree_node()
   n->left = NULL;
   n->right = NULL;
   n->ascii_code = '\0';
+  n->leaf_node = false;
   return n;
 }
 
@@ -327,6 +331,7 @@ make_huffman_tree()
       bit_len--;
     }
     current->ascii_code = i;
+    current->leaf_node = true;
   }
   return root;
 }
@@ -370,7 +375,7 @@ huffman_decode(char *dst_start, const uint8_t *src, uint32_t src_len)
       current = current->left;
     }
 
-    if (current->ascii_code) {
+    if (current->leaf_node == true) {
       *dst_end = current->ascii_code;
       ++dst_end;
       current = HUFFMAN_TREE_ROOT;
@@ -386,3 +391,51 @@ huffman_decode(char *dst_start, const uint8_t *src, uint32_t src_len)
 
   return dst_end - dst_start;
 }
+
+uint8_t *
+huffman_encode_append(uint8_t *dst, uint32_t src, int n = 0)
+{
+  for (int j = 3; j >= n; --j) {
+    *dst++ = ((src >> (8 * j)) & 255);
+  }
+  return dst;
+}
+
+int64_t
+huffman_encode(uint8_t *dst_start, const uint8_t *src, uint32_t src_len)
+{
+  uint8_t *dst = dst_start;
+  // NOTE: The maximum length of Huffman Code is 30, thus using uint32_t as buffer.
+  uint32_t buf = 0;
+  uint32_t remain_bits = 32;
+
+  for (uint32_t i = 0; i < src_len; ++i) {
+    const uint32_t hex = huffman_table[src[i]].code_as_hex;
+    const uint32_t bit_len = huffman_table[src[i]].bit_len;
+
+    if (remain_bits > bit_len) {
+      remain_bits = remain_bits - bit_len;
+      buf |= hex << remain_bits;
+    } else if (remain_bits == bit_len) {
+      buf |= hex;
+      dst = huffman_encode_append(dst, buf);
+      remain_bits = 32;
+      buf = 0;
+    } else {
+      buf |= hex >> (bit_len - remain_bits);
+      dst = huffman_encode_append(dst, buf);
+      remain_bits = (32 - (bit_len - remain_bits));
+      buf = hex << remain_bits;
+    }
+  }
+
+  dst = huffman_encode_append(dst, buf, remain_bits / 8);
+
+  // NOTE: Add padding w/ EOS
+  uint32_t pad_len = remain_bits % 8;
+  if (pad_len) {
+    *(dst - 1) |= 0xff >> (8 - pad_len);
+  }
+
+  return dst - dst_start;
+}
