@@ -31,7 +31,7 @@
   to provide support for marshalling and unmarshalling support for the other
   LogAccess derived classes.
  */
-#include "libts.h"
+#include "ts/ink_platform.h"
 
 #include "Error.h"
 #include "HTTP.h"
@@ -92,9 +92,21 @@ LogAccess::marshal_client_host_ip(char *buf)
   DEFAULT_IP_FIELD;
 }
 
+int
+LogAccess::marshal_host_interface_ip(char *buf)
+{
+  DEFAULT_IP_FIELD;
+}
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
+int
+LogAccess::marshal_cache_lookup_url_canon(char *buf)
+{
+  DEFAULT_STR_FIELD;
+}
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
 int
 LogAccess::marshal_client_host_port(char *buf)
 {
@@ -207,6 +219,15 @@ LogAccess::marshal_client_req_http_version(char *buf)
   return (2 * INK_MIN_ALIGN);
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_client_req_protocol_version(char *buf)
+{
+  DEFAULT_STR_FIELD;
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -225,6 +246,47 @@ LogAccess::marshal_client_req_body_len(char *buf)
   DEFAULT_INT_FIELD;
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_client_req_tcp_reused(char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
+
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_client_req_is_ssl(char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
+
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_client_req_ssl_reused(char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
+
+/*-------------------------------------------------------------------------
+-------------------------------------------------------------------------*/
+int
+LogAccess::marshal_client_security_protocol(char *buf)
+{
+  DEFAULT_STR_FIELD;
+}
+
+int
+LogAccess::marshal_client_security_cipher_suite(char *buf)
+{
+  DEFAULT_STR_FIELD;
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -342,6 +404,15 @@ LogAccess::marshal_proxy_req_server_ip(char *buf)
   DEFAULT_IP_FIELD;
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_proxy_req_server_port(char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
+
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
 
@@ -394,6 +465,14 @@ LogAccess::marshal_proxy_host_ip(char *buf)
   return marshal_ip(buf, &Machine::instance()->ip.sa);
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_proxy_req_is_ssl(char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
@@ -470,6 +549,13 @@ LogAccess::marshal_server_resp_time_s(char *buf)
   DEFAULT_INT_FIELD;
 }
 
+/*-------------------------------------------------------------------------
+-------------------------------------------------------------------------*/
+int
+LogAccess::marshal_server_transact_count(char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
 
 /*-------------------------------------------------------------------------
   -------------------------------------------------------------------------*/
@@ -636,6 +722,25 @@ LogAccess::marshal_config_str_var(char *config_var, char *buf)
   return len;
 }
 
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_milestone(TSMilestonesType ms, char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
+
+/*-------------------------------------------------------------------------
+  -------------------------------------------------------------------------*/
+
+int
+LogAccess::marshal_milestone_diff(TSMilestonesType ms1, TSMilestonesType ms2, char *buf)
+{
+  DEFAULT_INT_FIELD;
+}
+
+
 // To allow for a generic marshal_record function, rather than
 // multiple functions (one per data type) we always marshal a record
 // as a string of a fixed length.  We use a fixed length because the
