@@ -25,7 +25,7 @@
 #ifndef LOG_ACCESS_HTTP_H
 #define LOG_ACCESS_HTTP_H
 
-#include "Arena.h"
+#include "ts/Arena.h"
 #include "HTTP.h"
 #include "LogAccess.h"
 
@@ -56,6 +56,7 @@ public:
   // client -> proxy fields
   //
   virtual int marshal_client_host_ip(char *);                // STR
+  virtual int marshal_host_interface_ip(char *);             // STR
   virtual int marshal_client_host_port(char *);              // INT
   virtual int marshal_client_auth_user_name(char *);         // STR
   virtual int marshal_client_req_text(char *);               // STR
@@ -68,9 +69,15 @@ public:
   virtual int marshal_client_req_url_path(char *);           // STR
   virtual int marshal_client_req_url_scheme(char *);         // STR
   virtual int marshal_client_req_http_version(char *);       // INT
+  virtual int marshal_client_req_protocol_version(char *);   // STR
   virtual int marshal_client_req_header_len(char *);         // INT
   virtual int marshal_client_req_body_len(char *);           // INT
+  virtual int marshal_client_req_tcp_reused(char *);         // INT
+  virtual int marshal_client_req_is_ssl(char *);             // INT
+  virtual int marshal_client_req_ssl_reused(char *);         // INT
   virtual int marshal_client_finish_status_code(char *);     // INT
+  virtual int marshal_client_security_protocol(char *);      // STR
+  virtual int marshal_client_security_cipher_suite(char *);  // STR
 
   //
   // proxy -> client fields
@@ -91,8 +98,10 @@ public:
   virtual int marshal_proxy_req_body_len(char *);    // INT
   virtual int marshal_proxy_req_server_name(char *); // STR
   virtual int marshal_proxy_req_server_ip(char *);   // INT
+  virtual int marshal_proxy_req_server_port(char *); // INT
   virtual int marshal_proxy_hierarchy_route(char *); // INT
   virtual int marshal_proxy_host_port(char *);       // INT
+  virtual int marshal_proxy_req_is_ssl(char *);      // INT
 
   //
   // server -> proxy fields
@@ -105,6 +114,7 @@ public:
   virtual int marshal_server_resp_http_version(char *); // INT
   virtual int marshal_server_resp_time_ms(char *);      // INT
   virtual int marshal_server_resp_time_s(char *);       // INT
+  virtual int marshal_server_transact_count(char *);    // INT
 
   //
   // cache -> client fields
@@ -128,11 +138,12 @@ public:
   //
   // other fields
   //
-  virtual int marshal_transfer_time_ms(char *);    // INT
-  virtual int marshal_transfer_time_s(char *);     // INT
-  virtual int marshal_file_size(char *);           // INT
-  virtual int marshal_plugin_identity_id(char *);  // INT
-  virtual int marshal_plugin_identity_tag(char *); // STR
+  virtual int marshal_transfer_time_ms(char *);       // INT
+  virtual int marshal_transfer_time_s(char *);        // INT
+  virtual int marshal_file_size(char *);              // INT
+  virtual int marshal_plugin_identity_id(char *);     // INT
+  virtual int marshal_plugin_identity_tag(char *);    // STR
+  virtual int marshal_cache_lookup_url_canon(char *); // STR
 
   //
   // named fields from within a http header
@@ -140,6 +151,9 @@ public:
   virtual int marshal_http_header_field(LogField::Container container, char *field, char *buf);
   virtual int marshal_http_header_field_escapify(LogField::Container container, char *field, char *buf);
 
+  virtual int marshal_milestone(TSMilestonesType ms, char *buf);
+  virtual int marshal_milestone_diff(TSMilestonesType ms1, TSMilestonesType ms2, char *buf);
+
   virtual void set_client_req_url(char *, int);                // STR
   virtual void set_client_req_url_canon(char *, int);          // STR
   virtual void set_client_req_unmapped_url_canon(char *, int); // STR
@@ -173,10 +187,14 @@ private:
   int m_client_req_url_path_len;
   char *m_proxy_resp_content_type_str;
   int m_proxy_resp_content_type_len;
+  char *m_cache_lookup_url_canon_str;
+  int m_cache_lookup_url_canon_len;
 
   void validate_unmapped_url(void);
   void validate_unmapped_url_path(void);
 
+  void validate_lookup_url(void);
+
   // -- member functions that are not allowed --
   LogAccessHttp(const LogAccessHttp &rhs);
   LogAccessHttp &operator=(LogAccessHttp &rhs);
