@@ -25,7 +25,7 @@
 // include files
 //-------------------------------------------------------------------------
 
-#include "ink_config.h"
+#include "ts/ink_config.h"
 
 #include <stdio.h>
 #include <stdlib.h>
