@@ -21,18 +21,18 @@
   limitations under the License.
  */
 
-#include "libts.h"
-#include "I_Layout.h"
+#include "ts/ink_platform.h"
+#include "ts/I_Layout.h"
 
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 
-#include "ink_platform.h"
-#include "ink_file.h"
+#include "ts/ink_platform.h"
+#include "ts/ink_file.h"
 
 #include "Main.h"
-#include "List.h"
+#include "ts/List.h"
 #include "InkXml.h"
 
 #include "Log.h"
@@ -45,10 +45,9 @@
 #include "LogObject.h"
 #include "LogConfig.h"
 #include "LogUtils.h"
-#include "SimpleTokenizer.h"
+#include "ts/SimpleTokenizer.h"
 
 #include "LogCollationAccept.h"
-#include "LogPredefined.h"
 
 #define DISK_IS_CONFIG_FULL_MESSAGE                    \
   "Access logging to local log directory suspended - " \
@@ -82,29 +81,6 @@ LogConfig::setup_default_values()
   logfile_perm = 0644;
   logfile_dir = ats_strdup(".");
 
-  separate_icp_logs = 1;
-  separate_host_logs = false;
-
-  squid_log_enabled = true;
-  squid_log_is_ascii = true;
-  squid_log_name = ats_strdup("squid");
-  squid_log_header = NULL;
-
-  common_log_enabled = false;
-  common_log_is_ascii = true;
-  common_log_name = ats_strdup("common");
-  common_log_header = NULL;
-
-  extended_log_enabled = false;
-  extended_log_is_ascii = true;
-  extended_log_name = ats_strdup("extended");
-  extended_log_header = NULL;
-
-  extended2_log_enabled = false;
-  extended2_log_is_ascii = true;
-  extended2_log_name = ats_strdup("extended2");
-  extended2_log_header = NULL;
-
   collation_mode = Log::NO_COLLATION;
   collation_host = ats_strdup("none");
   collation_port = 0;
@@ -123,28 +99,6 @@ LogConfig::setup_default_values()
 
   custom_logs_enabled = false;
 
-  /* The default values for the search log                         */
-
-  search_log_enabled = false;
-  /* Comma separated filter names. These Filetrs are defined in    */
-  /* log_xml.config file. One such filter defines to reject URLs   */
-  /* with .gif extension. These filters are added to search log    */
-  /* object.                                                       */
-  search_log_filters = NULL;
-  search_rolling_interval_sec = 86400; // 24 hours
-  search_server_ip_addr = 0;
-  search_server_port = 8080;
-  search_top_sites = 100;
-  /* Comma separated filter names. These Filetrs are defined in    */
-  /* records.config or thru UI. These are applied to the search log*/
-  /* while parsing to generate top 'n' site summary file. The URLs */
-  /* containing these strings will not be parsed.                  */
-  search_url_filter = NULL;
-  /* Logging system captures all the URLs in this log file.        */
-  search_log_file_one = ats_strdup("search_log1");
-  /* Logging system captures only cache miss URLs to this log file.*/
-  search_log_file_two = ats_strdup("search_log2");
-
   sampling_frequency = 1;
   file_stat_frequency = 16;
   space_used_frequency = 900;
@@ -195,39 +149,11 @@ LogConfig::read_configuration_variables()
     max_space_mb_headroom = val;
   }
 
-  // TODO: We should mover this "parser" to lib/ts
   ptr = REC_ConfigReadString("proxy.config.log.logfile_perm");
-  if (ptr && strlen(ptr) == 9) {
-    logfile_perm = 0;
-    char *c = ptr;
-    if (*c == 'r')
-      logfile_perm |= S_IRUSR;
-    c++;
-    if (*c == 'w')
-      logfile_perm |= S_IWUSR;
-    c++;
-    if (*c == 'x')
-      logfile_perm |= S_IXUSR;
-    c++;
-    if (*c == 'r')
-      logfile_perm |= S_IRGRP;
-    c++;
-    if (*c == 'w')
-      logfile_perm |= S_IWGRP;
-    c++;
-    if (*c == 'x')
-      logfile_perm |= S_IXGRP;
-    c++;
-    if (*c == 'r')
-      logfile_perm |= S_IROTH;
-    c++;
-    if (*c == 'w')
-      logfile_perm |= S_IWOTH;
-    c++;
-    if (*c == 'x')
-      logfile_perm |= S_IXOTH;
-    ats_free(ptr);
-  }
+  int logfile_perm_parsed = ink_fileperm_parse(ptr);
+  if (logfile_perm_parsed != -1)
+    logfile_perm = logfile_perm_parsed;
+  ats_free(ptr);
 
   ptr = REC_ConfigReadString("proxy.config.log.hostname");
   if (ptr != NULL) {
@@ -245,110 +171,6 @@ LogConfig::read_configuration_variables()
     _exit(1);
   }
 
-  //
-  // for each predefined logging format, we need to know:
-  //   - whether logging in that format is enabled
-  //   - if we're logging to a file, what the name and format (ASCII,
-  //     BINARY) are
-  //   - what header should be written down at the start of each file for
-  //     this format
-  // this is accomplished with four config variables per format:
-  //   "proxy.config.log.<format>_log_enabled" INT
-  //   "proxy.config.log.<format>_log_is_ascii" INT
-  //   "proxy.config.log.<format>_log_name" STRING
-  //   "proxy.config.log.<format>_log_header" STRING
-  //
-
-
-  // SQUID
-  val = (int)REC_ConfigReadInteger("proxy.config.log.squid_log_enabled");
-  squid_log_enabled = (val > 0);
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.squid_log_is_ascii");
-  squid_log_is_ascii = (val > 0);
-
-  ptr = REC_ConfigReadString("proxy.config.log.squid_log_name");
-  if (ptr != NULL) {
-    ats_free(squid_log_name);
-    squid_log_name = ptr;
-  }
-
-  ptr = REC_ConfigReadString("proxy.config.log.squid_log_header");
-  if (ptr != NULL) {
-    ats_free(squid_log_header);
-    squid_log_header = ptr;
-  }
-
-  // COMMON
-  val = (int)REC_ConfigReadInteger("proxy.config.log.common_log_enabled");
-  common_log_enabled = (val > 0);
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.common_log_is_ascii");
-  common_log_is_ascii = (val > 0);
-
-  ptr = REC_ConfigReadString("proxy.config.log.common_log_name");
-  if (ptr != NULL) {
-    ats_free(common_log_name);
-    common_log_name = ptr;
-  }
-
-  ptr = REC_ConfigReadString("proxy.config.log.common_log_header");
-  if (ptr != NULL) {
-    ats_free(common_log_header);
-    common_log_header = ptr;
-  }
-
-  // EXTENDED
-  val = (int)REC_ConfigReadInteger("proxy.config.log.extended_log_enabled");
-  extended_log_enabled = (val > 0);
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.extended_log_is_ascii");
-  extended_log_is_ascii = (val > 0);
-
-  ptr = REC_ConfigReadString("proxy.config.log.extended_log_name");
-  if (ptr != NULL) {
-    ats_free(extended_log_name);
-    extended_log_name = ptr;
-  }
-
-  ptr = REC_ConfigReadString("proxy.config.log.extended_log_header");
-  if (ptr != NULL) {
-    ats_free(extended_log_header);
-    extended_log_header = ptr;
-  }
-
-  // EXTENDED2
-  val = (int)REC_ConfigReadInteger("proxy.config.log.extended2_log_enabled");
-  extended2_log_enabled = (val > 0);
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.extended2_log_is_ascii");
-  extended2_log_is_ascii = (val > 0);
-
-  ptr = REC_ConfigReadString("proxy.config.log.extended2_log_name");
-  if (ptr != NULL) {
-    ats_free(extended2_log_name);
-    extended2_log_name = ptr;
-  }
-
-  ptr = REC_ConfigReadString("proxy.config.log.extended2_log_header");
-  if (ptr != NULL) {
-    ats_free(extended2_log_header);
-    extended2_log_header = ptr;
-  }
-
-
-  // SPLITTING
-  // 0 means no splitting
-  // 1 means splitting
-  // for icp
-  //   -1 means filter out (do not log and do not create split file)
-  val = (int)REC_ConfigReadInteger("proxy.config.log.separate_icp_logs");
-  separate_icp_logs = (val > 0);
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.separate_host_logs");
-  separate_host_logs = (val > 0);
-
-
   // COLLATION
   val = (int)REC_ConfigReadInteger("proxy.local.log.collation_mode");
   // do not restrict value so that error message is logged if
@@ -441,61 +263,6 @@ LogConfig::read_configuration_variables()
   if (val > 0) {
     max_line_size = val;
   }
-
-  /* The following variables are initialized after reading the     */
-  /* variable values from records.config                           */
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.search_log_enabled");
-  if (Log::logging_mode == Log::LOG_MODE_FULL)
-    search_log_enabled = (val > 0);
-
-  /*                                                               */
-  /* The following collects the filter names to be added to search */
-  /* log object. User can define the filter to exclude URLs with   */
-  /* certain file extensions from being logged.                    */
-  /*                                                               */
-  ptr = REC_ConfigReadString("proxy.config.log.search_log_filters");
-  if (ptr != NULL) {
-    search_log_filters = ptr;
-  }
-
-  /*                                                               */
-  /* This filter information is used while parsing the search log  */
-  /* to generate top 'n' site sorted URL summary file.             */
-  /* This log file contains top 'n' number of frequently accessed  */
-  /* sites. This file is sent to search server mentioned by the    */
-  /* pair ip-address & port.                                       */
-  /*                                                               */
-  ptr = REC_ConfigReadString("proxy.config.log.search_url_filter");
-  if (ptr != NULL) {
-    search_url_filter = ptr;
-  }
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.search_top_sites");
-  if (val > 0) {
-    search_top_sites = val;
-  }
-
-  ptr = REC_ConfigReadString("proxy.config.log.search_server_ip_addr");
-  if (ptr != NULL) {
-    unsigned int ipaddr;
-    ipaddr = inet_addr(ptr);
-    if (ipaddr > 0) {
-      search_server_ip_addr = ipaddr;
-    } else
-      search_server_ip_addr = 0;
-  }
-
-  val = (int)REC_ConfigReadInteger("proxy.config.log.search_server_port");
-  if (val > 0) {
-    search_server_port = val;
-  }
-
-  /*  Rolling interval is taken care in LogObject.                 */
-  val = (int)REC_ConfigReadInteger("proxy.config.log.search_rolling_interval_sec");
-  if (val > 0) {
-    search_rolling_interval_sec = val;
-  }
 }
 
 /*-------------------------------------------------------------------------
@@ -534,18 +301,8 @@ LogConfig::~LogConfig()
 
   ats_free(hostname);
   ats_free(logfile_dir);
-  ats_free(squid_log_name);
-  ats_free(squid_log_header);
-  ats_free(common_log_name);
-  ats_free(common_log_header);
-  ats_free(extended_log_name);
-  ats_free(extended_log_header);
-  ats_free(extended2_log_name);
-  ats_free(extended2_log_header);
   ats_free(collation_host);
   ats_free(collation_secret);
-  ats_free(search_log_file_one);
-  ats_free(search_log_file_two);
   ats_free(m_dir_entry);
 }
 
@@ -634,13 +391,16 @@ LogConfig::init(LogConfig *prev_config)
   // ----------------------------------------------------------------------
   // Construct a new error log object candidate.
   if (Log::error_logging_enabled()) {
-    PreDefinedFormatInfo *info;
+    LogFormat *fmt;
 
     Debug("log", "creating predefined error log object");
-    info = MakePredefinedErrorLog(this);
-    errlog = this->create_predefined_object(info, 0, NULL);
+
+    fmt = MakeTextLogFormat("error");
+    this->global_format_list.add(fmt, false);
+    errlog = new LogObject(fmt, logfile_dir, "error.log", LOG_FILE_ASCII, NULL, (Log::RollingEnabledValues)rolling_enabled,
+                           collation_preproc_threads, rolling_interval_sec, rolling_offset_hr, rolling_size_mb);
+    log_object_manager.manage_object(errlog);
     errlog->set_fmt_timestamps();
-    delete info;
   } else {
     Log::error_log = NULL;
   }
@@ -693,24 +453,6 @@ LogConfig::display(FILE *fd)
   fprintf(fd, "   hostname = %s\n", hostname);
   fprintf(fd, "   logfile_dir = %s\n", logfile_dir);
   fprintf(fd, "   logfile_perm = 0%o\n", logfile_perm);
-  fprintf(fd, "   squid_log_enabled = %d\n", squid_log_enabled);
-  fprintf(fd, "   squid_log_is_ascii = %d\n", squid_log_is_ascii);
-  fprintf(fd, "   squid_log_name = %s\n", squid_log_name);
-  fprintf(fd, "   squid_log_header = %s\n", squid_log_header ? squid_log_header : "<no header defined>");
-  fprintf(fd, "   common_log_enabled = %d\n", common_log_enabled);
-  fprintf(fd, "   common_log_is_ascii = %d\n", common_log_is_ascii);
-  fprintf(fd, "   common_log_name = %s\n", common_log_name);
-  fprintf(fd, "   common_log_header = %s\n", common_log_header ? common_log_header : "<no header defined>");
-  fprintf(fd, "   extended_log_enabled = %d\n", extended_log_enabled);
-  fprintf(fd, "   extended_log_is_ascii = %d\n", extended_log_is_ascii);
-  fprintf(fd, "   extended_log_name = %s\n", extended_log_name);
-  fprintf(fd, "   extended_log_header = %s\n", extended_log_header ? extended_log_header : "<no header defined>");
-  fprintf(fd, "   extended2_log_enabled = %d\n", extended2_log_enabled);
-  fprintf(fd, "   extended2_log_is_ascii = %d\n", extended2_log_is_ascii);
-  fprintf(fd, "   extended2_log_name = %s\n", extended2_log_name);
-  fprintf(fd, "   extended2_log_header = %s\n", extended2_log_header ? extended2_log_header : "<no header defined>");
-  fprintf(fd, "   separate_icp_logs = %d\n", separate_icp_logs);
-  fprintf(fd, "   separate_host_logs = %d\n", separate_host_logs);
   fprintf(fd, "   collation_mode = %d\n", collation_mode);
   fprintf(fd, "   collation_host = %s\n", collation_host);
   fprintf(fd, "   collation_port = %d\n", collation_port);
@@ -737,313 +479,30 @@ LogConfig::display(FILE *fd)
   global_format_list.display(fd);
 }
 
-/*                                                               */
-/* The user defined filters are added to the search_one          */
-/* log object. These filters are defined to filter the images    */
-/* and vedio file etc., URLs. These filters depends on the       */
-/* end-user's search requirements.                               */
-/*                                                               */
-void
-LogConfig::add_filters_to_search_log_object(const char *format_name)
-{
-  LogObject *obj;
-
-  obj = log_object_manager.find_by_format_name(format_name);
-
-  /* Apply the user defined filters on to newly created LogObject */
-  SimpleTokenizer tok(search_log_filters, ',');
-  char *filter_name;
-  while (filter_name = tok.getNext(), filter_name != 0) {
-    LogFilter *f;
-    f = global_filter_list.find_by_name(filter_name);
-    if (!f) {
-      Warning("Filter %s not in the global filter list; cannot add to this LogObject", filter_name);
-    } else {
-      obj->add_filter(f);
-    }
-  }
-}
-
-//-----------------------------------------------------------------------------
-// Create one object for each of the entries in the pre-defined info list
-// and apply the specified filter(s) to it.
-//
-// Normally, only one pre-defined format has been selected on the config file
-// so this function creates a single object in such case.
-//
-// This function adds the pre-defined objects to the global_object_list.
-//
-
-LogObject *
-LogConfig::create_predefined_object(const PreDefinedFormatInfo *pdi, size_t num_filters, LogFilter **filter, const char *filt_name,
-                                    bool force_extension)
-{
-  const char *obj_fname;
-  char obj_filt_fname[PATH_NAME_MAX];
-
-  ink_release_assert(pdi != NULL);
-
-  if (filt_name) {
-    ink_string_concatenate_strings_n(obj_filt_fname, PATH_NAME_MAX, pdi->filename, "-", filt_name, NULL);
-    obj_fname = obj_filt_fname;
-  } else {
-    obj_fname = pdi->filename;
-  }
-
-  if (force_extension) {
-    switch (pdi->filefmt) {
-    case LOG_FILE_ASCII:
-      ink_string_append(obj_filt_fname, (char *)LOG_FILE_ASCII_OBJECT_FILENAME_EXTENSION, PATH_NAME_MAX);
-      break;
-    case LOG_FILE_BINARY:
-      ink_string_append(obj_filt_fname, (char *)LOG_FILE_BINARY_OBJECT_FILENAME_EXTENSION, PATH_NAME_MAX);
-      break;
-    default:
-      break;
-    }
-  }
-
-  // create object with filters
-  //
-  LogObject *obj;
-  obj = new LogObject(pdi->format, logfile_dir, obj_fname, pdi->filefmt, pdi->header, (Log::RollingEnabledValues)rolling_enabled,
-                      collation_preproc_threads, rolling_interval_sec, rolling_offset_hr, rolling_size_mb);
-
-  if (pdi->collatable) {
-    if (collation_mode == Log::SEND_STD_FMTS || collation_mode == Log::SEND_STD_AND_NON_XML_CUSTOM_FMTS) {
-      LogHost *loghost = new LogHost(obj->get_full_filename(), obj->get_signature());
-      ink_assert(loghost != NULL);
-
-      loghost->set_name_port(collation_host, collation_port);
-      obj->add_loghost(loghost, false);
-    }
-  }
-
-  for (size_t i = 0; i < num_filters; ++i) {
-    obj->add_filter(filter[i]);
-  }
-
-  // give object to object manager
-  if (log_object_manager.manage_object(obj) != LogObjectManager::NO_FILENAME_CONFLICTS) {
-    delete obj;
-    return NULL;
-  }
-
-  return obj;
-}
-
-void
-LogConfig::create_predefined_objects_with_filter(const PreDefinedFormatList &predef, size_t nfilters, LogFilter **filters,
-                                                 const char *filt_name, bool force_extension)
-{
-  PreDefinedFormatInfo *pdi;
-
-  for (pdi = predef.formats.head; pdi != NULL; pdi = (pdi->link).next) {
-    this->create_predefined_object(pdi, nfilters, filters, filt_name, force_extension);
-  }
-}
-
-
-//-----------------------------------------------------------------------------
-// split_by_protocol
-//
-// This function creates the objects needed to log different protocols on
-// their own file if any of the "separate_xxx_logs" config. variable is set.
-//
-// Upon return, the pf_list argument holds the filters that reject the
-// protocols for which objects have been created. This filter list is used
-// to create the rest of the pre defined log objects.
-//
-// As input, this function requires a list wilh all information regarding
-// pre-defined formats.
-//
-LogFilter *
-LogConfig::split_by_protocol(const PreDefinedFormatList &predef)
-{
-  if (!separate_icp_logs) {
-    return NULL;
-  }
-  // http MUST be last entry
-  enum {
-    icp = 0,
-    http,
-  };
-
-  int64_t value[] = {LOG_ENTRY_ICP, LOG_ENTRY_HTTP};
-  const char *name[] = {"icp", "http"};
-  const char *filter_name[] = {"__icp__", "__http__"};
-  int64_t filter_val[http]; // protocols to reject
-  size_t n = 0;
-
-  LogField *etype_field = Log::global_field_list.find_by_symbol("etype");
-  ink_assert(etype_field);
-
-  if (separate_icp_logs) {
-    if (separate_icp_logs == 1) {
-      LogFilter *filter[1];
-
-      filter[0] = new LogFilterInt(filter_name[icp], etype_field, LogFilter::ACCEPT, LogFilter::MATCH, value[icp]);
-      create_predefined_objects_with_filter(predef, countof(filter), filter, name[icp]);
-      delete filter[0];
-    }
-    filter_val[n++] = value[icp];
-  }
-
-  // At this point, separate objects for all protocols except http
-  // have been created if requested. We now add to the argument list
-  // the filters needed to reject the protocols that have already
-  // been taken care of. Note that we do not test for http since
-  // there is no "separate_http_logs" config variable and thus http
-  // could not have been taken care of at this point
-  //
-
-  if (n > 0) {
-    return new LogFilterInt("__reject_protocols__", etype_field, LogFilter::REJECT, LogFilter::MATCH, n, filter_val);
-  }
-  return NULL;
-}
-
-size_t
-LogConfig::split_by_hostname(const PreDefinedFormatList &predef, LogFilter *reject_protocol_filter)
-{
-  size_t n_hosts;
-  char **host = read_log_hosts_file(&n_hosts); // allocates memory for array
-
-  if (n_hosts) {
-    size_t num_filt = 0;
-    LogFilter *rp_ah[2]; // rejected protocols + accepted host
-    LogField *shn_field = Log::global_field_list.find_by_symbol("shn");
-    ink_assert(shn_field);
-
-    if (reject_protocol_filter) {
-      rp_ah[num_filt++] = reject_protocol_filter;
-    }
-
-    for (size_t i = 0; i < n_hosts; ++i) {
-      // add a filter that accepts the specified hostname
-      //
-      char filter_name[LOG_MAX_FORMAT_LINE + 12];
-      snprintf(filter_name, LOG_MAX_FORMAT_LINE, "__accept_%s__", host[i]);
-      rp_ah[num_filt] =
-        new LogFilterString(filter_name, shn_field, LogFilter::ACCEPT, LogFilter::CASE_INSENSITIVE_CONTAIN, host[i]);
-
-      create_predefined_objects_with_filter(predef, num_filt + 1, rp_ah, host[i], true);
-      delete rp_ah[num_filt];
-    }
-
-    LogFilter *rp_rh[2]; // rejected protocols + rejected hosts
-
-    num_filt = 0;
-
-    if (reject_protocol_filter) {
-      rp_rh[num_filt++] = reject_protocol_filter;
-    }
-
-    rp_rh[num_filt] =
-      new LogFilterString("__reject_hosts__", shn_field, LogFilter::REJECT, LogFilter::CASE_INSENSITIVE_CONTAIN, n_hosts, host);
-
-    // create the "catch-all" object that contains logs for all
-    // hosts other than those specified in the hosts file and for
-    // those protocols that do not have their own file
-    //
-    create_predefined_objects_with_filter(predef, num_filt + 1, rp_rh);
-    delete rp_rh[num_filt];
-
-    delete[] host; // deallocate memory allocated by
-    // read_log_hosts_file
-  }
-  // host is not allocated unless n_hosts > 0
-  // coverity[leaked_storage]
-  return n_hosts;
-}
-
 //-----------------------------------------------------------------------------
 // setup_log_objects
 //
-// Construct:
-//
-// -All objects necessary to log the pre defined formats considering
-//  protocol and host splitting.
-// -All custom objects.
+// Construct: All custom objects.
 //
 // Upon return from this function:
 // - global_object_list has the aforementioned objects
-// - global_format_list has all the pre-determined log formats (e.g. squid,
-//   common, etc.)
 // - global_filter_list has all custom filters
-//   Note that the filters necessary to do log splitting for the pre defined
-//   format are kept private (e.g., they do not go to the global_filter_list).
 //
 void
 LogConfig::setup_log_objects()
 {
   Debug("log", "creating objects...");
 
-  // ----------------------------------------------------------------------
-  // Construct the LogObjects for the pre-defined formats.
-
-  // gather the config information for the pre-defined formats
-  //
-  PreDefinedFormatList predef;
-
-  predef.init(this);
-
-  // do protocol splitting
-  //
-  LogFilter *reject_protocol_filter = split_by_protocol(predef);
-
-  // do host splitting
-  //
-  size_t num_hosts = 0;
-  if (separate_host_logs) {
-    num_hosts = split_by_hostname(predef, reject_protocol_filter);
-  }
-
-  if (num_hosts == 0) {
-    // if no host splitting was requested, or if host splitting
-    // was not successful (e.g. empty log_hosts.config file) then
-    // create the "catch-all" object that contains logs for all
-    // protocols that do not have their own file, and for all hosts
-    //
-    LogFilter *f[1];
-    f[0] = reject_protocol_filter;
-    create_predefined_objects_with_filter(predef, countof(f), f);
-  }
-
-  delete reject_protocol_filter;
-
   // ----------------------------------------------------------------------
   // Construct the LogObjects for the custom formats
 
   global_filter_list.clear();
 
-  /*                                                               */
-  /* create search Log Objects here.                               */
-  /* The following creates the search_one, search_two log objects  */
-  /* For search_one log object user defines filters in xml config  */
-  /* file. The names of these filters have to be given in          */
-  /* records.config file                                           */
-  /*                                                               */
-  if (search_log_enabled) {
-    Debug("log", "creating search log object");
-    /* Read xml configuration for search log from memory.            */
-    read_xml_log_config(1);
-  }
-
   if (custom_logs_enabled) {
     /* Read xml configuration from logs_xml.config file.             */
-    read_xml_log_config(0);
+    read_xml_log_config();
   }
 
-  /*                                                               */
-  /* Add the user defined filters to search_one log object.        */
-  /*                                                               */
-  if (search_log_enabled) {
-    if (search_log_filters) {
-      add_filters_to_search_log_object("m_search_one");
-      add_filters_to_search_log_object("m_search_two");
-    }
-  }
   // open local pipes so readers can see them
   //
   log_object_manager.open_local_pipes();
@@ -1085,21 +544,13 @@ LogConfig::register_config_callbacks()
   static const char *names[] = {
     "proxy.config.log.log_buffer_size", "proxy.config.log.max_secs_per_buffer", "proxy.config.log.max_space_mb_for_logs",
     "proxy.config.log.max_space_mb_for_orphan_logs", "proxy.config.log.max_space_mb_headroom", "proxy.config.log.logfile_perm",
-    "proxy.config.log.hostname", "proxy.config.log.logfile_dir", "proxy.config.log.squid_log_enabled",
-    "proxy.config.log.squid_log_is_ascii", "proxy.config.log.squid_log_name", "proxy.config.log.squid_log_header",
-    "proxy.config.log.common_log_enabled", "proxy.config.log.common_log_is_ascii", "proxy.config.log.common_log_name",
-    "proxy.config.log.common_log_header", "proxy.config.log.extended_log_enabled", "proxy.config.log.extended_log_is_ascii",
-    "proxy.config.log.extended_log_name", "proxy.config.log.extended_log_header", "proxy.config.log.extended2_log_enabled",
-    "proxy.config.log.extended2_log_is_ascii", "proxy.config.log.extended2_log_name", "proxy.config.log.extended2_log_header",
-    "proxy.config.log.separate_icp_logs", "proxy.config.log.separate_host_logs", "proxy.local.log.collation_mode",
+    "proxy.config.log.hostname", "proxy.config.log.logfile_dir", "proxy.local.log.collation_mode",
     "proxy.config.log.collation_host", "proxy.config.log.collation_port", "proxy.config.log.collation_host_tagged",
     "proxy.config.log.collation_secret", "proxy.config.log.collation_retry_sec", "proxy.config.log.collation_max_send_buffers",
     "proxy.config.log.rolling_enabled", "proxy.config.log.rolling_interval_sec", "proxy.config.log.rolling_offset_hr",
     "proxy.config.log.rolling_size_mb", "proxy.config.log.auto_delete_rolled_files", "proxy.config.log.custom_logs_enabled",
     "proxy.config.log.xml_config_file", "proxy.config.log.hosts_config_file", "proxy.config.log.sampling_frequency",
-    "proxy.config.log.file_stat_frequency", "proxy.config.log.space_used_frequency", "proxy.config.log.search_rolling_interval_sec",
-    "proxy.config.log.search_log_enabled", "proxy.config.log.search_top_sites", "proxy.config.log.search_server_ip_addr",
-    "proxy.config.log.search_server_port", "proxy.config.log.search_url_filter",
+    "proxy.config.log.file_stat_frequency", "proxy.config.log.space_used_frequency",
   };
 
 
@@ -1472,101 +923,25 @@ LogConfig::update_space_used()
 
   This is a new routine for reading the XML-based log config file.
   -------------------------------------------------------------------------*/
-
-static char xml_config_buffer[] = "<LogFilter> \
-                                  <Name = \"reject_gif\"/> \
-                                  <Action = \"REJECT\"/> \
-                                  <Condition = \"cqup CASE_INSENSITIVE_CONTAIN .gif\"/> \
-                                  </LogFilter>\
-                                              \
-                                  <LogFilter> \
-                                  <Name = \"reject_jpg\"/> \
-                                  <Action = \"REJECT\"/> \
-                                  <Condition = \"cqup CASE_INSENSITIVE_CONTAIN .jpg\"/> \
-                                  </LogFilter>\
-                                              \
-                                  <LogFilter> \
-                                  <Name      = \"only_cache_miss\"/> \
-                                  <Action    = \"ACCEPT\"/> \
-                                  <Condition = \"crc MATCH TCP_MISS\"/> \
-                                  </LogFilter> \
-                                              \
-                                  <LogFormat> \
-                                  <Name = \"m_search_one\"/> \
-                                  <Format = \"%%<cquc> %%<cqtt>\"/> \
-                                  </LogFormat> \
-                                               \
-                                  <LogFormat> \
-                                  <Name = \"m_search_two\"/> \
-                                  <Format = \"%%<cquc> %%<crc>\"/> \
-                                  </LogFormat> \
-                                               \
-                                  <LogObject> \
-                                  <Filename = \"search_log1\"/> \
-                                  <Format = \"m_search_one\"/> \
-                                  <Filters = \"reject_gif\", \"reject_jpg\"/> \
-                                  <RollingIntervalSec = \"%d\"/> \
-                                  </LogObject> \
-                                               \
-                                  <LogObject> \
-                                  <Filename = \"search_log2\"/> \
-                                  <Format = \"m_search_two\"/> \
-                                  <Filters = \"reject_gif\", \"reject_jpg\", \"only_cache_miss\"/> \
-                                  <RollingIntervalSec = \"%d\"/> \
-                                  </LogObject>";
 void
-LogConfig::read_xml_log_config(int from_memory)
+LogConfig::read_xml_log_config()
 {
   ats_scoped_str config_path;
 
-  if (!from_memory) {
-    config_path = RecConfigReadConfigPath("proxy.config.log.xml_config_file", "logs_xml.config");
-  }
-
+  config_path = RecConfigReadConfigPath("proxy.config.log.xml_config_file", "logs_xml.config");
   InkXmlConfigFile log_config(config_path ? (const char *)config_path : "memory://builtin");
 
-  if (!from_memory) {
-    Debug("log-config", "Reading log config file %s", (const char *)config_path);
-    Debug("xml", "%s is an XML-based config file", (const char *)config_path);
+  Debug("log-config", "Reading log config file %s", (const char *)config_path);
+  Debug("xml", "%s is an XML-based config file", (const char *)config_path);
 
-    if (log_config.parse() < 0) {
-      Note("Error parsing log config file %s; ensure that it is XML-based", (const char *)config_path);
-      return;
-    }
-
-    if (is_debug_tag_set("xml")) {
-      log_config.display();
-    }
-  } else {
-    int filedes[2];
-    int nbytes = sizeof(xml_config_buffer);
-    const size_t ptr_size = nbytes + 20;
-    char *ptr = (char *)ats_malloc(ptr_size);
-
-    if (pipe(filedes) != 0) {
-      Note("xml parsing: Error in Opening a pipe");
-      ats_free(ptr);
-      return;
-    }
-
-    snprintf(ptr, ptr_size, xml_config_buffer, search_rolling_interval_sec, search_rolling_interval_sec);
-    nbytes = strlen(ptr);
-    if (write(filedes[1], ptr, nbytes) != nbytes) {
-      Note("Error in writing to pipe.");
-      ats_free(ptr);
-      close(filedes[1]);
-      close(filedes[0]);
-      return;
-    }
-    ats_free(ptr);
-    close(filedes[1]);
-
-    if (log_config.parse(filedes[0]) < 0) {
-      Note("Error parsing log config info from memory.");
-      return;
-    }
+  if (log_config.parse() < 0) {
+    Note("Error parsing log config file %s; ensure that it is XML-based", (const char *)config_path);
+    return;
   }
 
+  if (is_debug_tag_set("xml")) {
+    log_config.display();
+  }
 
   //
   // At this point, the XMl file has been parsed into a list of
