@@ -26,9 +26,9 @@
 
 
  ***************************************************************************/
-#include "libts.h"
-
-
+#include "ts/ink_platform.h"
+#include "ts/CryptoHash.h"
+#include "ts/INK_MD5.h"
 #include "Error.h"
 #include "P_EventSystem.h"
 #include "LogUtils.h"
@@ -393,11 +393,7 @@ LogObject::_checkout_write(size_t *write_offset, size_t bytes_needed)
       INK_QUEUE_LD(h, m_log_buffer);
       head_p new_h;
       SET_FREELIST_POINTER_VERSION(new_h, FREELIST_POINTER(h), FREELIST_VERSION(h) + 1);
-#if TS_HAS_128BIT_CAS
-      result = ink_atomic_cas((__int128_t *)&m_log_buffer.data, h.data, new_h.data);
-#else
-      result = ink_atomic_cas((int64_t *)&m_log_buffer.data, h.data, new_h.data);
-#endif
+      result = ink_atomic_cas(&m_log_buffer.data, h.data, new_h.data);
     } while (!result);
     buffer = (LogBuffer *)FREELIST_POINTER(h);
     result_code = buffer->checkout_write(write_offset, bytes_needed);
@@ -430,11 +426,7 @@ LogObject::_checkout_write(size_t *write_offset, size_t bytes_needed)
         }
         head_p tmp_h;
         SET_FREELIST_POINTER_VERSION(tmp_h, new_buffer, 0);
-#if TS_HAS_128BIT_CAS
-        result = ink_atomic_cas((__int128_t *)&m_log_buffer.data, old_h.data, tmp_h.data);
-#else
-        result = ink_atomic_cas((int64_t *)&m_log_buffer.data, old_h.data, tmp_h.data);
-#endif
+        result = ink_atomic_cas(&m_log_buffer.data, old_h.data, tmp_h.data);
       } while (!result);
       if (FREELIST_POINTER(old_h) == FREELIST_POINTER(h)) {
         ink_atomic_increment(&buffer->m_references, FREELIST_VERSION(old_h) - 1);
@@ -472,11 +464,7 @@ LogObject::_checkout_write(size_t *write_offset, size_t bytes_needed)
           break;
         head_p tmp_h;
         SET_FREELIST_POINTER_VERSION(tmp_h, FREELIST_POINTER(h), FREELIST_VERSION(old_h) - 1);
-#if TS_HAS_128BIT_CAS
-        result = ink_atomic_cas((__int128_t *)&m_log_buffer.data, old_h.data, tmp_h.data);
-#else
-        result = ink_atomic_cas((int64_t *)&m_log_buffer.data, old_h.data, tmp_h.data);
-#endif
+        result = ink_atomic_cas(&m_log_buffer.data, old_h.data, tmp_h.data);
       } while (!result);
       if (FREELIST_POINTER(old_h) != FREELIST_POINTER(h))
         ink_atomic_increment(&buffer->m_references, -1);
@@ -970,7 +958,7 @@ LogObjectManager::_solve_filename_conflicts(LogObject *log_object, int maxConfli
     // file exists, try to read metafile to get object signature
     //
     uint64_t signature = 0;
-    MetaInfo meta_info(filename);
+    BaseMetaInfo meta_info(filename);
     bool conflicts = true;
 
     if (meta_info.file_open_successful()) {
@@ -1036,16 +1024,17 @@ LogObjectManager::_solve_filename_conflicts(LogObject *log_object, int maxConfli
                   "filename",
                   filename);
           LogFile logfile(filename, NULL, LOG_FILE_ASCII, 0);
-          long time_now = LogUtils::timestamp();
+          if (logfile.open_file() == LogFile::LOG_FILE_NO_ERROR) {
+            long time_now = LogUtils::timestamp();
 
-          if (logfile.roll(time_now - log_object->get_rolling_interval(), time_now) == 0) {
-            // an error happened while trying to roll the file
-            //
-            const char *msg = "Cannot roll log file %s to fix log "
-                              "filename conflicts";
-
-            Error(msg, filename);
-            LogUtils::manager_alarm(LogUtils::LOG_ALARM_ERROR, msg, filename);
+            if (logfile.roll(time_now - log_object->get_rolling_interval(), time_now) == 0) {
+              // an error happened while trying to roll the file
+              //
+              _filename_resolution_abort(filename);
+              retVal = CANNOT_SOLVE_FILENAME_CONFLICTS;
+            }
+          } else {
+            _filename_resolution_abort(filename);
             retVal = CANNOT_SOLVE_FILENAME_CONFLICTS;
           }
         }
@@ -1055,6 +1044,16 @@ LogObjectManager::_solve_filename_conflicts(LogObject *log_object, int maxConfli
   return retVal;
 }
 
+void
+LogObjectManager::_filename_resolution_abort(const char *filename)
+{
+  const char *msg = "Cannot roll log file %s to fix log "
+                    "conflicts (filename or log format): %s";
+  const char *err = strerror(errno);
+  Error(msg, filename, err);
+  LogUtils::manager_alarm(LogUtils::LOG_ALARM_ERROR, msg, filename, err);
+}
+
 
 bool
 LogObjectManager::_has_internal_filename_conflict(const char *filename, LogObjectList &objects)
@@ -1406,7 +1405,7 @@ REGRESSION_TEST(LogObjectManager_Transfer)(RegressionTest *t, int /* atype ATS_U
     rprintf(t, "mgr1 has %d objects, mgr2 has %d objects\n", (int)mgr1.get_num_objects(), (int)mgr2.get_num_objects());
 
     rprintf(t, "running Log::periodoc_tasks()\n");
-    Log::periodic_tasks(ink_get_hrtime() / HRTIME_SECOND);
+    Log::periodic_tasks(Thread::get_hrtime() / HRTIME_SECOND);
     rprintf(t, "Log::periodoc_tasks() done\n");
   }
 
