@@ -21,6 +21,8 @@
   limitations under the License.
  */
 
+#include "ts/ink_inet.h"
+#include "ts/ink_string.h"
 #include "P_EventSystem.h"
 #include "Error.h"
 
