@@ -22,6 +22,7 @@
  */
 
 #include "SpdyClientSession.h"
+#include "SpdySessionAccept.h"
 #include "I_Net.h"
 
 static ClassAllocator<SpdyClientSession> spdyClientSessionAllocator("spdyClientSessionAllocator");
@@ -42,10 +43,24 @@ static char const *const npnmap[] = {TS_NPN_PROTOCOL_SPDY_2, TS_NPN_PROTOCOL_SPD
 static int spdy_process_read(TSEvent event, SpdyClientSession *sm);
 static int spdy_process_write(TSEvent event, SpdyClientSession *sm);
 static int spdy_process_fetch(TSEvent event, SpdyClientSession *sm, void *edata);
-static int spdy_process_fetch_header(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm);
-static int spdy_process_fetch_body(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm);
+static int spdy_process_fetch_header(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm, SpdyRequest *req);
+static int spdy_process_fetch_body(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm, SpdyRequest *req);
 static uint64_t g_sm_id = 1;
 
+SpdyRequest *
+SpdyRequest::alloc()
+{
+  return spdyRequestAllocator.alloc();
+}
+
+void
+SpdyRequest::destroy()
+{
+  this->clear();
+  spdyRequestAllocator.free(this);
+}
+
+
 void
 SpdyRequest::init(SpdyClientSession *sm, int id)
 {
@@ -68,6 +83,10 @@ SpdyRequest::clear()
   SPDY_DECREMENT_THREAD_DYN_STAT(SPDY_STAT_CURRENT_CLIENT_STREAM_COUNT, spdy_sm->mutex->thread_holding);
 
   if (fetch_sm) {
+    // Clear the UserData just in case fetch_sm's
+    // death is delayed.  Don't want freed requests
+    // showing up in callbacks
+    TSFetchUserDataSet(fetch_sm, NULL);
     TSFetchDestroy(fetch_sm);
     fetch_sm = NULL;
   }
@@ -85,16 +104,15 @@ SpdyRequest::clear()
 }
 
 void
-SpdyClientSession::init(NetVConnection *netvc, spdy::SessionVersion vers)
+SpdyClientSession::init(NetVConnection *netvc)
 {
   int r;
 
-  this->mutex = new_ProxyMutex();
+  this->mutex = netvc->mutex;
   this->vc = netvc;
   this->req_map.clear();
-  this->version = vers;
 
-  r = spdylay_session_server_new(&session, versmap[vers], &spdy_callbacks, this);
+  r = spdylay_session_server_new(&session, versmap[this->version], &spdy_callbacks, this);
 
   // A bit ugly but we need a thread and I don't want to wait until the
   // session start event in case of a time out generating a decrement
@@ -109,13 +127,16 @@ SpdyClientSession::init(NetVConnection *netvc, spdy::SessionVersion vers)
   start_time = TShrtime();
 
   this->vc->set_inactivity_timeout(HRTIME_SECONDS(spdy_accept_no_activity_timeout));
-  vc->add_to_keep_alive_lru();
+  vc->add_to_keep_alive_queue();
   SET_HANDLER(&SpdyClientSession::state_session_start);
 }
 
 void
 SpdyClientSession::clear()
 {
+  if (!mutex)
+    return; // this object wasn't initialized.
+
   int last_event = event;
 
   SPDY_DECREMENT_THREAD_DYN_STAT(SPDY_STAT_CURRENT_CLIENT_SESSION_COUNT, this->mutex->thread_holding);
@@ -129,8 +150,7 @@ SpdyClientSession::clear()
   for (; iter != endIter; ++iter) {
     SpdyRequest *req = iter->second;
     if (req) {
-      req->clear();
-      spdyRequestAllocator.free(req);
+      req->destroy();
     } else {
       Error("req null in SpdSM::clear");
     }
@@ -140,6 +160,11 @@ SpdyClientSession::clear()
   this->mutex = NULL;
 
   if (vc) {
+    // clean up ssl's first byte iobuf
+    SSLNetVConnection *ssl_vc = dynamic_cast<SSLNetVConnection *>(vc);
+    if (ssl_vc) {
+      ssl_vc->set_ssl_iobuf(NULL);
+    }
     TSVConnClose(reinterpret_cast<TSVConn>(vc));
     vc = NULL;
   }
@@ -174,20 +199,29 @@ SpdyClientSession::clear()
 }
 
 void
-spdy_cs_create(NetVConnection *netvc, spdy::SessionVersion vers, MIOBuffer *iobuf, IOBufferReader *reader)
+SpdyClientSession::new_connection(NetVConnection *new_vc, MIOBuffer *iobuf, IOBufferReader *reader, bool backdoor)
 {
-  SpdyClientSession *sm;
+  // SPDY for the backdoor connections? Let's not deal woth that yet.
+  ink_release_assert(backdoor == false);
 
-  sm = spdyClientSessionAllocator.alloc();
-  sm->init(netvc, vers);
+  SpdyClientSession *sm = this;
 
-  sm->req_buffer = iobuf ? reinterpret_cast<TSIOBuffer>(iobuf) : TSIOBufferCreate();
+  sm->init(new_vc);
+
+  sm->req_buffer = iobuf ? reinterpret_cast<TSIOBuffer>(iobuf) : reinterpret_cast<TSIOBuffer>(new_empty_MIOBuffer());
   sm->req_reader = reader ? reinterpret_cast<TSIOBufferReader>(reader) : TSIOBufferReaderAlloc(sm->req_buffer);
 
-  sm->resp_buffer = TSIOBufferCreate();
+  sm->resp_buffer = reinterpret_cast<TSIOBuffer>(new_empty_MIOBuffer());
   sm->resp_reader = TSIOBufferReaderAlloc(sm->resp_buffer);
 
-  eventProcessor.schedule_imm(sm, ET_NET);
+  // Block on the mutex.  We just allocated the object, so the lock should be available.
+  EThread *thread(this_ethread());
+  MUTEX_TAKE_LOCK(sm->mutex, thread);
+  // Call state_session_start directly rather than scheduling the event
+  // and leaving a half-setup session around.  It seems like there are some
+  // degenerate cases when event re-ordering causes problems (TS-3957)
+  sm->state_session_start(ET_NET, NULL);
+  MUTEX_UNTAKE_LOCK(sm->mutex, thread);
 }
 
 int
@@ -198,13 +232,13 @@ SpdyClientSession::state_session_start(int /* event */, void * /* edata */)
     {SPDYLAY_SETTINGS_INITIAL_WINDOW_SIZE, SPDYLAY_ID_FLAG_SETTINGS_NONE, spdy_initial_window_size}};
   int r;
 
+  this->read_vio = (TSVIO) this->vc->do_io_read(this, INT64_MAX, reinterpret_cast<MIOBuffer *>(this->req_buffer));
+  this->write_vio = (TSVIO) this->vc->do_io_write(this, INT64_MAX, reinterpret_cast<IOBufferReader *>(this->resp_reader));
+
   if (TSIOBufferReaderAvail(this->req_reader) > 0) {
     spdy_process_read(TS_EVENT_VCONN_WRITE_READY, this);
   }
 
-  this->read_vio = (TSVIO) this->vc->do_io_read(this, INT64_MAX, reinterpret_cast<MIOBuffer *>(this->req_buffer));
-  this->write_vio = (TSVIO) this->vc->do_io_write(this, INT64_MAX, reinterpret_cast<IOBufferReader *>(this->resp_reader));
-
   SET_HANDLER(&SpdyClientSession::state_session_readwrite);
 
   r = spdylay_submit_settings(this->session, SPDYLAY_FLAG_SETTINGS_NONE, entries, countof(entries));
@@ -251,8 +285,7 @@ SpdyClientSession::state_session_readwrite(int event, void *edata)
   Debug("spdy-event", "++++SpdyClientSession[%" PRIu64 "], EVENT:%d, ret:%d", this->sm_id, event, ret);
 out:
   if (ret) {
-    this->clear();
-    spdyClientSessionAllocator.free(this);
+    this->do_io_close();
   } else if (!from_fetch) {
     this->vc->set_inactivity_timeout(HRTIME_SECONDS(spdy_no_activity_timeout_in));
   }
@@ -260,6 +293,20 @@ out:
   return EVENT_CONT;
 }
 
+void
+SpdyClientSession::destroy()
+{
+  this->clear();
+  spdyClientSessionAllocator.free(this);
+}
+
+
+SpdyClientSession *
+SpdyClientSession::alloc()
+{
+  return spdyClientSessionAllocator.alloc();
+}
+
 int64_t
 SpdyClientSession::getPluginId() const
 {
@@ -272,7 +319,6 @@ SpdyClientSession::getPluginTag() const
   return npnmap[this->version];
 }
 
-
 static int
 spdy_process_read(TSEvent /* event ATS_UNUSED */, SpdyClientSession *sm)
 {
@@ -309,22 +355,26 @@ spdy_process_fetch(TSEvent event, SpdyClientSession *sm, void *edata)
   int ret = -1;
   TSFetchSM fetch_sm = (TSFetchSM)edata;
   SpdyRequest *req = (SpdyRequest *)TSFetchUserDataGet(fetch_sm);
+  if (!req) {
+    Warning("spdy_process_fetch: stream already gone");
+    return ret;
+  }
 
   switch ((int)event) {
   case TS_FETCH_EVENT_EXT_HEAD_DONE:
     Debug("spdy", "----[FETCH HEADER DONE]");
-    ret = spdy_process_fetch_header(event, sm, fetch_sm);
+    ret = spdy_process_fetch_header(event, sm, fetch_sm, req);
     break;
 
   case TS_FETCH_EVENT_EXT_BODY_READY:
     Debug("spdy", "----[FETCH BODY READY]");
-    ret = spdy_process_fetch_body(event, sm, fetch_sm);
+    ret = spdy_process_fetch_body(event, sm, fetch_sm, req);
     break;
 
   case TS_FETCH_EVENT_EXT_BODY_DONE:
     Debug("spdy", "----[FETCH BODY DONE]");
     req->fetch_body_completed = true;
-    ret = spdy_process_fetch_body(event, sm, fetch_sm);
+    ret = spdy_process_fetch_body(event, sm, fetch_sm, req);
     break;
 
   default:
@@ -335,7 +385,6 @@ spdy_process_fetch(TSEvent event, SpdyClientSession *sm, void *edata)
       Debug("spdy_error",
             "spdy_process_fetch fetch error, fetch_sm %p, ret %d for sm_id %" PRId64 ", stream_id %u, req time %" PRId64 ", url %s",
             req->fetch_sm, ret, sm->sm_id, req->stream_id, req->start_time, req->url.c_str());
-      req->fetch_sm = NULL;
     }
     break;
   }
@@ -351,10 +400,9 @@ spdy_process_fetch(TSEvent event, SpdyClientSession *sm, void *edata)
 }
 
 static int
-spdy_process_fetch_header(TSEvent /*event*/, SpdyClientSession *sm, TSFetchSM fetch_sm)
+spdy_process_fetch_header(TSEvent /*event*/, SpdyClientSession *sm, TSFetchSM fetch_sm, SpdyRequest *req)
 {
   int ret = -1;
-  SpdyRequest *req = (SpdyRequest *)TSFetchUserDataGet(fetch_sm);
 
   SpdyNV spdy_nv(fetch_sm);
 
@@ -432,11 +480,10 @@ spdy_read_fetch_body_callback(spdylay_session * /*session*/, int32_t stream_id,
 }
 
 static int
-spdy_process_fetch_body(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm)
+spdy_process_fetch_body(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm, SpdyRequest *req)
 {
   int ret = 0;
   spdylay_data_provider data_prd;
-  SpdyRequest *req = (SpdyRequest *)TSFetchUserDataGet(fetch_sm);
   req->event = event;
 
   data_prd.source.ptr = (void *)req;
@@ -456,3 +503,19 @@ spdy_process_fetch_body(TSEvent event, SpdyClientSession *sm, TSFetchSM fetch_sm
   TSVIOReenable(sm->write_vio);
   return ret;
 }
+
+void
+SpdyClientSession::do_io_close(int alertno)
+{
+  if (vc) {
+    // vc is released (null'ed) within ProxyClientSession when handling
+    // TS_HTTP_SSN_CLOSE_HOOK, so, reset ssl_iobuf to prevent double free
+    SSLNetVConnection *ssl_vc = dynamic_cast<SSLNetVConnection *>(vc);
+    if (ssl_vc) {
+      ssl_vc->set_ssl_iobuf(NULL);
+    }
+  }
+  // The object will be cleaned up from within ProxyClientSession::handle_api_return
+  // This way, the object will still be alive for any SSN_CLOSE hooks
+  do_api_callout(TS_HTTP_SSN_CLOSE_HOOK);
+}
