@@ -43,7 +43,17 @@ SpdySessionAccept::mainEvent(int event, void *edata)
     NetVConnection *netvc = static_cast<NetVConnection *>(edata);
 
 #if TS_HAS_SPDY
-    spdy_cs_create(netvc, this->version, NULL, NULL);
+    SpdyClientSession *sm = SpdyClientSession::alloc();
+    sm->version = this->version;
+
+    SSLNetVConnection *ssl_vc = dynamic_cast<SSLNetVConnection *>(netvc);
+    MIOBuffer *iobuf = NULL;
+    IOBufferReader *reader = NULL;
+    if (ssl_vc) {
+      iobuf = ssl_vc->get_ssl_iobuf();
+      reader = ssl_vc->get_ssl_reader();
+    }
+    sm->new_connection(netvc, iobuf, reader, false);
 #else
     Error("accepted a SPDY session, but SPDY support is not available");
     netvc->do_io_close();
@@ -60,7 +70,9 @@ void
 SpdySessionAccept::accept(NetVConnection *netvc, MIOBuffer *iobuf, IOBufferReader *reader)
 {
 #if TS_HAS_SPDY
-  spdy_cs_create(netvc, this->version, iobuf, reader);
+  SpdyClientSession *sm = SpdyClientSession::alloc();
+  sm->version = this->version;
+  sm->new_connection(netvc, iobuf, reader, false);
 #else
   (void)netvc;
   (void)iobuf;
